@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1997 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1997-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -19,22 +20,31 @@ History:
 	970429	created
 //////////////////////////////////////////////////////////////////////#*/
 
-#include <fcntl.h>
-#include <stdio.h>
-#include <ctype.h>
-#include <string.h>
-extern void *malloc();
-
+/*
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <time.h>
+#include <fcntl.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <stdarg.h>
+#include <ctype.h>
+*/
+#include "yarg.h"
+#include "ystring.h"
+#include "file.h"
+
+int FS_withSymlink();
+int lock_exclusiveTO(int fd,int timeout,int *elapsedp);
+int stat_blocks(FileStat *stp);
 
 #ifndef S_ISDIR
 #define S_ISDIR(m)  (((m)&S_IFMT) == S_IFDIR)
 #endif
 
-#include "../include/vsocket.h"
+#include "vsocket.h"
 
-static char *USAGE = "\
+static const char *USAGE = "\
 Cafe -- cache file expirer\n\
 -----------------------------------------------------------------------\n\
 Usage: %s [options] directories\n\
@@ -55,22 +65,19 @@ Exmaple:\n\
   %s DIR -atime 12h -lslsu  -- list files not acceessed within 12 hours\n\
   %s DIR -atime 12h -rm  -- remove files not accessed within 12 hours\n\
 -----------------------------------------------------------------------\n\
-<ysato@etl.go.jp>\n\
 ";
-static put_usage(out,me)
-	FILE *out;
-	char *me;
+static void put_usage(FILE *out,PCStr(me))
 {
 	fprintf(out,USAGE,me,me,me,me);
 }
 
 static int ROOT_DEV;
 static int DIRLEV;
-static char *CWD;
+static const char *CWD;
 
 static int F_FOLLOW_MOUNT;
 static int F_FOLLOW_SYMLINK;
-static char *ls_opts;
+static const char *ls_opts;
 static int F_LSFORM;
 static int F_UATIME; /* update the atime of directory after scanning */
 static int F_VERBOSE;
@@ -83,6 +90,9 @@ static int F_MOVE;
 static int F_SUMMARY;
 static int REM_SIZE;
 
+static const char *F_TYPE;
+static int F_NAME_NOT;
+static const char *F_NAME;
 static int A_FROM;
 static int A_TILL;
 static int M_FROM;
@@ -97,26 +107,22 @@ static int N_ERR;
 static int N_BLK;
 static int N_BLKREM;
 
-static loghead(out)
-	FILE *out;
-{	char stime[128];
+static void loghead(FILE *out)
+{	CStr(stime,128);
 
-	StrftimeLocal(stime,sizeof(stime),"%Y/%m/%d-%H:%M:%S",time(0));
+	StrftimeLocal(AVStr(stime),sizeof(stime),"%Y/%m/%d-%H:%M:%S",time(0),0);
 	fseek(out,0,2);
 	fprintf(out,"%s [%d] ",stime,getpid());
 }
 
 static FILE *LOG;
-static errlog(fmt,a,b,c,d,e,f,g,h,i,j,k,l,m,n)
-	char *fmt,*a,*b,*c,*d,*e,*f,*g,*h,*i,*j,*k,*l,*m,*n;
-{
+static void errlog(PCStr(fmt),...)
+{	VARGS(16,fmt);
 	loghead(LOG);
-	fprintf(LOG,fmt,a,b,c,d,e,f,g,h,i,j,k,l,m,n);
+	fprintf(LOG,fmt,VA16);
 	fflush(LOG);
 }
-static summary(out,root)
-	FILE *out;
-	char *root;
+static void summary(FILE *out,PCStr(root))
 {
 	loghead(out);
 	fprintf(out,"%s: %d blk, %d dir, %d reg",root,N_BLK,N_DIR,N_REG);
@@ -131,7 +137,9 @@ static summary(out,root)
 typedef struct {
 	int	d_top;
 	FILE   *d_out;
-	char   *d_cwd;
+	int	d_Rlen;
+  const	char   *d_Root;
+  const	char   *d_cwd;
 	int	d_dev;
 	int	d_ino;
 	int	d_nch;
@@ -140,32 +148,35 @@ typedef struct {
 	int	d_blocks;
 } DA;
 
-static mkabspath(path,dir,file)
-	char *path,*dir,*file;
-{	char *dp,dc,*pp;
+static void mkabspath(PVStr(path),PCStr(dir),PCStr(file))
+{	const char *dp;
+	char dc;
+	refQStr(pp,path); /**/
 
-	pp = path;
+	cpyQStr(pp,path);
 	if( !isFullpath(file) ){
-		for( dp = dir; dc = *dp; dp++ )
-			*pp++ = dc;
-		if( path < pp &&  pp[-1] != '/' )
-			*pp++ = '/';
+		for( dp = dir; dc = *dp; dp++ ){
+			assertVStr(path,pp);
+			setVStrPtrInc(pp,dc);
+		}
+		if( path < pp &&  pp[-1] != '/' ){
+			setVStrPtrInc(pp,'/');
+		}
 	}
 	strcpy(pp,file);
 }
 
-static Stat(path,stp)
-	char *path;
-	struct stat *stp;
+static void Stat(PCStr(path),FileStat *stp)
 {
 }
 
-static char *filesys(path,root,parent)
-	char *path,*root,*parent;
+static char *filesys(PCStr(path),PVStr(root),PVStr(parent))
 {	int dev;
-	struct stat st;
-	char *dp,dc;
-	char cwd[1024],xpath[1024];
+	FileStat st;
+	const char *dp;
+	char dc;
+	CStr(cwd,1024);
+	CStr(xpath,1024);
 
 	if( stat(path,&st) != 0 ){
 		errlog("can't stat: %s\n",path);
@@ -176,7 +187,7 @@ static char *filesys(path,root,parent)
 	strcpy(xpath,path);
 	if( !fileIsdir(xpath) ){
 		if( dp = strrchr(xpath,'/') )
-			*dp = 0;
+			*(char*)dp = 0;
 		else	strcpy(xpath,".");
 	}
 	chdir(xpath);
@@ -185,59 +196,72 @@ static char *filesys(path,root,parent)
 
 	for( dp = strchr(xpath,'/'); dp; dp = strchr(dp+1,'/') ){
 		dc = dp[1];
-		dp[1] = 0;
+		((char*)dp)[1] = 0;
 		if( stat(xpath,&st) == 0 ){
 			if( st.st_dev == dev ){
 				if( dp != xpath )
-					*dp = 0;
+					*(char*)dp = 0;
 				strcpy(root,xpath);
 				break;
 			}
 		}
-		dp[1] = dc;
+		((char*)dp)[1] = dc;
 	}
 
 	chdir(cwd);
-	return root;
+	return (char*)root;
 }
-static put_icacheR(path)
-	char *path;
-{	char *dp,dc,*np;
+static int file_stats(int pdev,int pino,PVStr(path),PCStr(file),FileStat *stp,int *blocksp);
+static void put_icacheR(PCStr(path))
+{	const char *dp;
+	char dc;
 	int pdev,pino;
-	struct stat st;
-	char xpath[1024];
+	FileStat st;
+	CStr(xpath,1024);
 
 	strcpy(xpath,path);
 	pdev = pino = 0;
 
 	for( dp = strchr(xpath,'/'); dp; dp = strchr(dp+1,'/') ){
 		dc = dp[1];
-		dp[1] = 0;
-		if( file_stats(pdev,pino,xpath,xpath,&st,NULL) < 0 ){
-			dp[1] = dc;
+		((char*)dp)[1] = 0;
+		if( file_stats(pdev,pino,AVStr(xpath),xpath,&st,NULL) < 0 ){
+			((char*)dp)[1] = dc;
 			break;
 		}
-		dp[1] = dc;
+		((char*)dp)[1] = dc;
 		pdev = st.st_dev;
 		pino = st.st_ino;
 	}
-	file_stats(pdev,pino,xpath,xpath,&st,NULL);
+	file_stats(pdev,pino,AVStr(xpath),xpath,&st,NULL);
 }
 
 /*
  *	INODE CACHE
  */
-static char *icachedir = NULL;
-
 typedef struct {
 	int	 ic_dev;
-	char	*ic_rootpath;
+  const char	*ic_rootpath;
 	FILE	*ic_file;
 } IC;
-static IC ICs[32];
+typedef struct {
+  const char   *ie_icachedir;
+	IC	ie_ICs[32];
+	int	ie_put_force;
+	int	ie_inocnt;
+} ICacheEnv;
+static ICacheEnv *iCacheEnv;
+#define icachedir	iCacheEnv->ie_icachedir
+#define ICs		iCacheEnv->ie_ICs
+#define put_force	iCacheEnv->ie_put_force
+#define inocnt		iCacheEnv->ie_inocnt
+void minit_icache()
+{
+	if( iCacheEnv == 0 )
+		iCacheEnv = NewStruct(ICacheEnv);
+}
 
-static FILE *fopen_rw(path)
-	char *path;
+static FILE *fopen_rw(PCStr(path))
 {	FILE *fp;
 
 	if( (fp = fopen(path,"r+")) == NULL )
@@ -245,12 +269,8 @@ static FILE *fopen_rw(path)
 	return fp;
 }
 
-static int put_force;
-static int inoc;
-
-static FILE *ino_cache_sort(dev,key)
-	char *key;
-{	char path[1024];
+static FILE *ino_cache_sort(int dev,PCStr(key))
+{	CStr(path,1024);
 	FILE *ics;
 
 	sprintf(path,"%s/dev-%08x/ino-sort-by-%s",icachedir,dev,key);
@@ -259,11 +279,12 @@ static FILE *ino_cache_sort(dev,key)
 		ics = fopen(path,"w+");
 	return ics;
 }
-static FILE *ino_cache(rootpath,dev,ixp)
-	char *rootpath;
-	int *ixp;
-{	char icache[1024],path[1024],rootpathbuf[1024];
-	char parent[1024];
+static FILE *ino_cache(xPVStr(rootpath),int dev,int *ixp)
+{	CStr(icache,1024);
+	CStr(path,1024);
+	CStr(rootpathbuf,1024);
+	CStr(parent,1024);
+	CStr(tmp,1024);
 	FILE *ic;
 	int ix,dev1;
 
@@ -295,16 +316,16 @@ static FILE *ino_cache(rootpath,dev,ixp)
 			errlog("can't make inode cache file: %s\n",path);
 			return NULL;
 		}
-		rootpath = filesys(rootpath,rootpathbuf,parent);
+		rootpath = filesys(rootpath,AVStr(rootpathbuf),AVStr(parent));
 		fputs(rootpath,ic);
 		fclose(ic);
 
-		if( inoc == 0 ){
-			inoc++;
+		if( inocnt == 0 ){
+			inocnt++;
 			put_force = 1;
 			put_icacheR(parent);
 			put_force = 0;
-			inoc--;
+			inocnt--;
 		}
 
 	}else{
@@ -314,8 +335,9 @@ static FILE *ino_cache(rootpath,dev,ixp)
 			return NULL;
 		}
 		if( rootpath == NULL )
-			rootpath = rootpathbuf;
-		fscanf(ic,"%s",rootpath);
+			setPStr(rootpath,rootpathbuf,sizeof(rootpathbuf));
+		fgets(tmp,sizeof(tmp),ic);
+		Xsscanf(tmp,"%s",AVStr(rootpath));
 		fclose(ic);
 	}
 
@@ -328,7 +350,7 @@ static FILE *ino_cache(rootpath,dev,ixp)
 
 	ICs[ix].ic_dev = dev;
 	ICs[ix].ic_file = ic;
-	ICs[ix].ic_rootpath = strcpy(malloc(strlen(rootpath)+1),rootpath);
+	ICs[ix].ic_rootpath = stralloc(rootpath);
 	if( ixp ) *ixp = ix;
 
 	return ic;
@@ -341,9 +363,7 @@ typedef struct {
 	int	i_atime;
 } INO;
 
-static put_inocache(pdev,pino,path,stp,blocks)
-	char *path;
-	struct stat *stp;
+static void put_inocache(int pdev,int pino,PVStr(path),FileStat *stp,int blocks)
 {	FILE *ic;
 	int ix;
 	INO inoc;
@@ -358,7 +378,7 @@ static put_inocache(pdev,pino,path,stp,blocks)
 	if( pdev != dev )
 		return;
 
-	if( (ic = ino_cache(path,dev,&ix)) == NULL )
+	if( (ic = ino_cache(AVStr(path),dev,&ix)) == NULL )
 		return;
 
 	inoc.i_ino   = ino;
@@ -374,10 +394,7 @@ static put_inocache(pdev,pino,path,stp,blocks)
 	}
 }
 
-static int file_stats(pdev,pino,path,file,stp,blocksp)
-	char *path,*file;
-	struct stat *stp;
-	int *blocksp;
+static int file_stats(int pdev,int pino,PVStr(path),PCStr(file),FileStat *stp,int *blocksp)
 {       int rcode,blocks;
 
 	if( FS_withSymlink() ){
@@ -391,14 +408,13 @@ static int file_stats(pdev,pino,path,file,stp,blocksp)
         if( rcode == 0 ){
 		blocks = stat_blocks(stp);
 		if( blocksp ) *blocksp = blocks;
-		put_inocache(pdev,pino,path,stp,blocks);
+		put_inocache(pdev,pino,AVStr(path),stp,blocks);
 		return S_ISDIR(stp->st_mode);
         }else	return -1;
 }
 
 
-static cmpino(i1,i2)
-	INO **i1,**i2;
+static int cmpino(INO **i1,INO **i2)
 {	int diff;
 
 	if( diff = (*i1)->i_atime - (*i2)->i_atime )
@@ -409,8 +425,7 @@ static cmpino(i1,i2)
 }
 
 extern int scan_ino;
-static inofind1(file,ino,name)
-	char *file,*name;
+static scanDirFunc inofind1(PCStr(file),int ino,PVStr(name))
 {
 	if( scan_ino == ino ){
 		strcpy(name,file);
@@ -418,20 +433,16 @@ static inofind1(file,ino,name)
 	}
 	return 0;
 }
-static ino_name(dir,ino,name)
-	char *dir,*name;
+static void ino_name(PCStr(dir),int ino,PVStr(name))
 {
-	name[0] = 0;
-	Scandir(dir,inofind1,ino,name);
+	setVStrEnd(name,0);
+	Scandir(dir,scanDirCall inofind1,ino,AVStr(name));
 }
 
-static ino2path(icx,root,ic,ino,path)
-	char *root;
-	FILE *ic;
-	char *path;
+static void ino2path(int icx,PCStr(root),FILE *ic,int ino,PVStr(path))
 {	int inov[256],lev,li;
 	INO inob;
-	char *pp;
+	refQStr(pp,path); /**/
 	int cx;
 
 	strcpy(path,root);
@@ -444,26 +455,26 @@ static ino2path(icx,root,ic,ino,path)
 		ino = inob.i_pino;
 	}
 
-	pp = path + strlen(path);
+	pp = (char*)path + strlen(path);
 	for( li = lev-1; 0 <= li; li-- ){
 		if( path < pp && pp[-1] != '/' )
-			*pp++ = '/';
-		ino_name(path,inov[li],pp);
+			setVStrPtrInc(pp,'/');
+		ino_name(path,inov[li],QVStr(pp,path));
 		pp += strlen(pp);
 	}
 }
 
-static find_icache1(out,dev,nlist)
-	FILE *out;
+static void find_icache1(FILE *out,int dev,int nlist)
 {	int imax;
 	INO **inos,*inov;
 	int ino,ngot,icx,ix,blks,rblks;
 	FILE *ic,*ics;
-	char root[1024],path[2048];
-	char ctime[64];
+	CStr(root,1024);
+	CStr(path,2048);
+	CStr(ctime,64);
 
 	*root = 0;
-	ic = ino_cache(root,dev,&icx);
+	ic = ino_cache(AVStr(root),dev,&icx);
 	if( ic == NULL )
 		return;
 
@@ -490,7 +501,7 @@ static find_icache1(out,dev,nlist)
 		}
 	}
 
-	qsort(inos,ngot,sizeof(INO*),cmpino);
+	qsort(inos,ngot,sizeof(INO*),(sortFunc)cmpino);
 	ics = ino_cache_sort(dev,"atime");
 	for( ix = 0; ix < ngot; ix++ )
 		fwrite(inos[ix],sizeof(INO),1,ics);
@@ -502,21 +513,21 @@ static find_icache1(out,dev,nlist)
 		nlist = imax;
 	rblks = 0;
 	for( ix = 0; ix < ngot && ix < nlist; ix++ ){
-		ino2path(icx,root,ic,inos[ix]->i_ino,path);
+		ino2path(icx,root,ic,inos[ix]->i_ino,AVStr(path));
 		rblks += inos[ix]->i_blks;
 		blks -= inos[ix]->i_blks;
 
 		if( F_LSFORM ){
-			struct stat st;
+			FileStat st;
 			if( stat(path,&st) == 0 ){
 				printf("%8d ",st.st_ino);
-				ls_unix(out,ls_opts,NULL,path,&st);
+				ls_unix(out,ls_opts,CVStr(NULL),path,&st);
 			}else{
 				/* stale cache */
 				/* remove it */
 			}
 		}else{
-			rsctime(inos[ix]->i_atime,ctime);
+			rsctime(inos[ix]->i_atime,AVStr(ctime));
 			printf("%8d %8d - %8d %6d %6d %s %s\n",
 				rblks,blks,inos[ix]->i_blks,
 				inos[ix]->i_ino,inos[ix]->i_pino,
@@ -531,39 +542,42 @@ static find_icache1(out,dev,nlist)
 	free(inov);
 }
 
-static find_icache(out,path,nlist)
-	FILE *out;
-	char *path;
+static void find_icache(FILE *out,PCStr(path),int nlist)
 {	int dev;
-	struct stat st;
+	FileStat st;
 
 	if( stat(path,&st) == 0 )
 		find_icache1(out,st.st_dev,nlist);
 	else	errlog("unknown path: %s\n",path);
 }
 
-static find1(file,da)
-	char *file;
-	DA *da;
+/*
+static int dir1(PCStr(cwd),PCStr(dir),PCStr(rdir),int dev,int ino,FILE *out,int *blocks);
+*/
+static int dir1(DA *da,PCStr(cwd),PCStr(dir),PCStr(rdir),int dev,int ino,FILE *out,int *blocks);
+static scanDirFunc find1(PCStr(file),DA *da)
 {	int isdir,mtime,atime,size,blocks,dev,ino;
 	int cnt;
 	int match,rcode,cblocks;
-	char path[1024];
+	CStr(path,1024);
 	FILE *out = da->d_out;
-	struct stat st;
-	char *ftype;
+	FileStat st;
+	const char *ftype;
 
 	if( !da->d_top && file[0] == '.' )
 		if( file[1] == 0 || file[1] == '.' && file[2] == 0 )
 			return 0;
 	da->d_nch++;
-	mkabspath(path,da->d_cwd,file);
+	mkabspath(AVStr(path),da->d_cwd,file);
 
 	CNT++;
 	match = 0;
 	rcode = 0;
 
-	isdir = file_stats(da->d_dev,da->d_ino,path,file,&st,&blocks);
+	if( da->d_Root ){
+	isdir = file_stats(da->d_dev,da->d_ino,AVStr(path),path,&st,&blocks);
+	}else
+	isdir = file_stats(da->d_dev,da->d_ino,AVStr(path),file,&st,&blocks);
 	dev = st.st_dev;
 	ino = st.st_ino;
 	mtime = st.st_mtime;
@@ -585,7 +599,10 @@ static find1(file,da)
 
 	if( isdir ){
 		N_DIR++;
+		/*
 		match = 0 < dir1(da->d_cwd,path,file,dev,ino,out,&cblocks);
+		*/
+		match = 0 < dir1(da,da->d_cwd,path,file,dev,ino,out,&cblocks);
 		if( F_REMOVE ){
 			match = match && (0 < N_REM);
 		}else
@@ -600,6 +617,27 @@ static find1(file,da)
 			match = 1;
 	}
 
+	if( F_TYPE ){
+		switch( *F_TYPE ){
+			case 'f': if(  isdir ){ match = 0; } break;
+			case 'd': if( !isdir ){ match = 0; } break;
+		}
+	}
+	if( F_NAME ){
+		CStr(fnam,1024);
+		const char *np;
+		if( np = strrpbrk(file,"/") ){
+			np++;
+		}else{
+			np = file;
+		}
+		if( rexpmatch(F_NAME,np) ){
+			if(  F_NAME_NOT ) match = 0;
+		}else{
+			if( !F_NAME_NOT ) match = 0;
+		}
+	}
+
 	if( match && F_REMOVE && !da->d_top ){
 		if( isdir )
 			rcode = rmdir(file);
@@ -614,6 +652,9 @@ static find1(file,da)
 		da->d_blocks += blocks;
 		if( isdir ) da->d_blocks += cblocks;
 	}
+
+	if( !match && (F_TYPE || F_NAME) ){
+	}else
 	if( match && (F_PRINT || F_LSFORM)
 	 || da->d_top && !F_REMOVE
 	 || isdir && F_DIR
@@ -625,31 +666,32 @@ static find1(file,da)
 				fprintf(out,"%-7d ",(blocks+cblocks)/2);
 			else	fprintf(out,"%-7d ",blocks/2);
 		}
-/*
-		{
-			fprintf(out,"%d %6d %9d %9d %4d %4d ",
-				rcode,ino,NOW-mtime,NOW-atime,blocks,size);
-		}
-*/
 		if( F_LSFORM ){
-			ls_unix(out,ls_opts,NULL,path,&st);
+			ls_unix(out,ls_opts,CVStr(NULL),path,&st);
 		}else{
 			ftype = "";
 			if( isdir && path[strlen(path)-1] != '/' )
 				ftype = "/";
+			if( da->d_Root
+			 && strneq(path,da->d_Root,da->d_Rlen) ){
+				fprintf(out,"%s%s\n",path+da->d_Rlen,ftype);
+			}else
 			fprintf(out,"%s%s\n",path,ftype);
 		}
 	}
 	return 0;
 }
-static find0(out,top)
-	FILE *out;
-	char *top;
+/*
+static void find0(FILE *out,PCStr(top))
+*/
+static void find0(FILE *out,PCStr(Root),PCStr(top))
 {	DA dab;
-	struct stat st;
+	FileStat st;
 
 	dab.d_top = 1;
 	dab.d_out = out;
+	dab.d_Rlen = Root?strlen(Root):0;
+	dab.d_Root = Root;
 	dab.d_cwd = "";
 	dab.d_dev = 0;
 	dab.d_ino = 0;
@@ -660,24 +702,21 @@ static find0(out,top)
 	find1(top,&dab);
 }
 
-ScandirNCAT(path,rdir,func,arg)
-	char *path,*rdir;
-	int (*func)();
-	char *arg;
-{	struct stat st;
+static void ScandirNCAT(PCStr(path),PCStr(rdir),scanDirCallP func,PCStr(arg))
+{	FileStat st;
 
 	stat(path,&st);
 	Scandir(rdir,func,arg);
 	set_utimes(path,st.st_atime,st.st_mtime);
 }
 
-static dir1(cwd,dir,rdir,dev,ino,out,blocks)
-	char *cwd,*dir,*rdir;
-	FILE *out;
-	int *blocks;
+/*
+static int dir1(PCStr(cwd),PCStr(dir),PCStr(rdir),int dev,int ino,FILE *out,int *blocks)
+*/
+static int dir1(DA *da,PCStr(cwd),PCStr(dir),PCStr(rdir),int dev,int ino,FILE *out,int *blocks)
 {	DA dab;
-	char pcwd[1024];
-	char dirbuf[1024];
+	CStr(pcwd,1024);
+	CStr(dirbuf,1024);
 
 	*blocks = 0;
 	if( chdir(rdir) != 0 ){
@@ -685,10 +724,14 @@ static dir1(cwd,dir,rdir,dev,ino,out,blocks)
 		return 0;
 	}
 	if( !isFullpath(cwd) ){
-		mkabspath(pcwd,CWD,cwd);
+		mkabspath(AVStr(pcwd),CWD,cwd);
 		cwd = pcwd;
 	}
 
+	if( da ){
+		dab.d_Root = da->d_Root;
+		dab.d_Rlen = da->d_Rlen;
+	}
 	dab.d_top = 0;
 	dab.d_out = out;
 	dab.d_cwd = dir;
@@ -700,13 +743,13 @@ static dir1(cwd,dir,rdir,dev,ino,out,blocks)
 	dab.d_lev = DIRLEV;
 	DIRLEV++;
 	if( F_UATIME )
-		Scandir(".",find1,&dab);
+		Scandir(".",scanDirCall find1,&dab);
 	else{
 		if( !isFullpath(dir) ){
-			mkabspath(dirbuf,CWD,dir);
+			mkabspath(AVStr(dirbuf),CWD,dir);
 			dir = dirbuf;
 		}
-		ScandirNCAT(dir,".",find1,&dab);
+		ScandirNCAT(dir,".",scanDirCall find1,(char*)&dab);
 	}
 	DIRLEV--;
 
@@ -718,8 +761,7 @@ static dir1(cwd,dir,rdir,dev,ino,out,blocks)
 	return dab.d_nch == dab.d_nrm;
 }
 
-static get_period(pspec)
-	char *pspec;
+static int get_period(PCStr(pspec))
 {	int period;
 	int type;
 
@@ -734,8 +776,7 @@ static get_period(pspec)
 	else	return  period;
 }
 
-static FILE *setlog(path)
-	char *path;
+static FILE *setlog(PCStr(path))
 {	FILE *fp;
 
 	if( path[0] == '-' )
@@ -752,29 +793,36 @@ static FILE *setlog(path)
 	}
 	return fp;
 }
-static char *catarg1(args,ap,arg)
-	char *args,*ap,*arg;
+static char *catarg1(PCStr(args),xPVStr(ap),PCStr(arg))
 {
 	if( ap != args )
-		*ap++ = ' ';
+		setVStrPtrInc(ap,' ');
 	strcpy(ap,arg);
 	ap += strlen(ap);
-	return ap;
+	return (char*)ap;
 }
-cafe_main(ac,av)
-	char *av[];
-{	char *arg;
+int cafe_mainX(int ac,const char *av[],FILE *out,FILE *log)
+{	const char *arg;
 	int ai;
 	int period;
-	char cwd[1024];
-	char *tops[128],*top;
+	CStr(cwd,1024);
+	const char *Root = 0;
+	IStr(Rtop,1024);
+	const char *tops[1024]; /**/
+	const char *top;
 	int topx,tx;
 	int cacheonly = 0,wonly = 0;
-	char ls_optsb[128];
-	char args[1024],*ap=args,*pp;
+	CStr(ls_optsb,128);
+	CStr(args,64*1024);
+	refQStr(ap,args); /**/
+	const char *ax = &args[sizeof(args)-1];
+	const char *pp;
+	int fnot = 0;
+
+	minit_icache();
 
 	topx = 0;
-	LOG = stdout;
+	LOG = out;
 
 	if( ac == 1 ){
 		put_usage(LOG,av[0]);
@@ -786,16 +834,25 @@ cafe_main(ac,av)
 	for( ai = 1; ai < ac; ai++ ){
 		arg = av[ai];
 		pp = ap;
-		ap = catarg1(args,ap,arg);
+		ap = catarg1(args,AVStr(ap),arg);
+		if( ax <= ap ){
+			break;
+		}
 
+		if( strncmp(arg,"-root=",6) == 0 ){
+			Root = arg+6;
+		}else
+		if( strcmp(arg,"-not") == 0 ){
+			fnot = ai+1;
+		}else
 		if( strcmp(arg,"-ign") == 0 ){
 			++ai;
-			ap = pp;
+			ap = (char*)pp;
 		}else
 		if( strcmp(arg,"-log") == 0 || strcmp(arg,"-err") == 0 ){
 			if( ++ai < ac ){
 				LOG = setlog(av[ai]);
-				ap = pp;
+				ap = (char*)pp;
 			}
 		}else
 		if( strcmp(arg,"-utime") == 0 ){
@@ -803,7 +860,7 @@ cafe_main(ac,av)
 		}else
 		if( strcmp(arg,"-atime") == 0 ){
 			if( ++ai < ac ){
-				ap = catarg1(args,ap,av[ai]);
+				ap = catarg1(args,AVStr(ap),av[ai]);
 				period = get_period(av[ai]);
 				if( period < 0 )
 					A_FROM = NOW + period;
@@ -812,7 +869,7 @@ cafe_main(ac,av)
 		}else
 		if( strcmp(arg,"-mtime") == 0 ){
 			if( ++ai < ac ){
-				ap = catarg1(args,ap,av[ai]);
+				ap = catarg1(args,AVStr(ap),av[ai]);
 				period = get_period(av[ai]);
 				if( period < 0 )
 					M_FROM = NOW + period;
@@ -821,10 +878,21 @@ cafe_main(ac,av)
 		}else
 		if( strcmp(arg,"-asize") == 0 ){
 			if( ++ai < ac ){
-				ap = catarg1(args,ap,av[ai]);
+				ap = catarg1(args,AVStr(ap),av[ai]);
 				REM_SIZE = atoi(av[ai]);
 			}
 		}else
+		if( strcmp(arg,"-name") == 0 ){
+			if( ++ai < ac ){
+				F_NAME_NOT = (fnot == ai-1);
+				F_NAME = av[ai];
+			}
+		}else
+		if( strcmp(arg,"-type") == 0 ){
+			if( ++ai < ac ){
+				F_TYPE = av[ai];
+			}
+		}else
 		if( strncmp(arg,"-ls",3) == 0 ){
 			F_LSFORM = 1;
 			if( arg[3] == 0 )
@@ -868,12 +936,15 @@ cafe_main(ac,av)
 			if( atoi(&arg[1]) )
 				N_LIST = atoi(&arg[1]);
 		}else{
+			if( elnumof(tops)-2 <= topx ){
+				continue;
+			}
 			tops[topx++] = arg;
 			tops[topx] = 0;
-			ap = pp;
+			ap = (char*)pp;
 		}
 	}
-	*ap = 0;
+	XsetVStrEnd(AVStr(ap),0);
 
 	if( topx == 0 ){
 		tops[topx++] = ".";
@@ -897,12 +968,30 @@ cafe_main(ac,av)
 
 	for( tx = 0; tx < topx; tx++ ){
 		top = tops[tx];
+		if( Root ){
+			IStr(tmp,1024);
+			strcpy(tmp,"");
+			chdir_cwd(AVStr(tmp),top,0);
+			strcpy(Rtop,Root);
+			if( *tmp == '/' )
+				ovstrcpy(tmp,tmp+1);
+			if( *tmp ){
+				if( strtailchr(Rtop) != '/' )
+					strcat(Rtop,"/");
+				strcat(Rtop,tmp);
+			}
+			top = Rtop;
+		}
 
 		if( LOG != stdout )
+		if( LOG != out )
 		errlog("%s %s\n",top,args);
 
 		if( !icachedir || !cacheonly )
+			find0(LOG,Root,top);
+			/*
 			find0(LOG,top);
+			*/
 
 		if( icachedir != NULL )
 		if( !wonly )
@@ -912,6 +1001,12 @@ cafe_main(ac,av)
 			summary(LOG,top);
 	}
 
+	if( LOG != out )
 	fclose(LOG);
+	return 0;
+}
+int cafe_main(int ac,const char *av[]){
+	cafe_mainX(ac,av,stdout,stdout);
 	exit(0);
+	return 0;
 }
