@@ -1,13 +1,16 @@
 #include <stdio.h>
+#include <stdlib.h>
+#include "ystring.h"
+#include "fpoll.h"
 extern double Time();
+int CFI_init(int ac,const char *av[]);
 
-dping_main(ac,av)
-	char *av[];
+int dping_main(int ac,const char *av[])
 {	int oi,ox,oc,ix,ic,ifd,ofd,rcc;
 	int iwait;
 	char *outstr;
 	double ss,ds,rtt,min,max,total;
-	char buf[2];
+	CStr(buf,2);
 	FILE *fs;
 
 	if( ac <= 1 ){
@@ -15,7 +18,7 @@ dping_main(ac,av)
 		return -1;
 	}
 	CFI_init(ac,av);
-	outstr = av[1];
+	outstr = (char*)av[1];
 	iwait = 2;
 	ifd = 0;
 	ofd = 1;
@@ -49,6 +52,7 @@ dping_main(ac,av)
 			ix++;
 			ds = Time();
 			rtt = (ds - ss) * 1000;
+			buf[0] = ic;
 			fprintf(stderr,"[%f] put:%02X get:%02X %f ms\n",
 				ds,outstr[oi],buf[0],rtt);
 			total += rtt;
