@@ -15,6 +15,7 @@
  *        0,
  *      };
  */
+#ifndef UNDER_CE
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -38,12 +39,40 @@ int main(int ac,char *av[]){
 	int dorw = 0;
 	char libname[32];
 	char opt[128];
+	int skip = 0;
+	int lni,lno;
 
 	libname[0] = 0;
+	lni = lno = 0;
 	for(;;){
+		if( lni != lno ){
+			printf("#line %d\n",lni+1);
+			lno = lni;
+		}
 		if( fgets(line,sizeof(line),stdin) == NULL )
 			break;
+		lni++;
+
+		if( skip ){
+			if( strncmp(line,"#endif",6) == 0
+			 || strncmp(line,"#else",5) == 0
+			){
+				skip = 0;
+			}else{
+				printf("\n");
+				lno++;
+				continue;
+			}
+		}
+		if( strncmp(line,"#ifdef _MSC_VER",15) == 0 ){
+#ifdef _MSC_VER
+#else
+			skip = 1;
+#endif
+		}
+
 printf("%s",line);
+lno++;
 		if( sp = strstr(line,"/*BEGIN_STAB") ){
 			if( sp[12] == '(' ){
 				sscanf(sp+13,"%[^)]",libname);
@@ -54,6 +83,9 @@ printf("#define PCStr(s) const char *s /*DLST*/\n");
 printf("#define ISDLIB /*DLST*/\n");
 printf("/*DLST*/ typedef struct {char*name; void*addr; char*opt;} DLMap;\n");
 printf("/*DLST*/ extern DLMap dlmap_%s[];\n",libname);
+printf("/*DLST*/ void *dgdlsym(DLMap*,const char *sym);\n");
+printf("#define mydlsym(s) dgdlsym(dlmap_%s,s)\n",libname);
+lno += 7;
 		}else
 		if( strstr(line,"/*END_STAB*/") ){
 			dorw = 0;
@@ -120,6 +152,7 @@ printf("/*DLST*/ extern DLMap dlmap_%s[];\n",libname);
 printf("/*DLST*/ %s",line);
 printf("/*DLST*/ %s (*%s_PTR_)%s",type,name,sp);
 printf("#define %s (*%s_PTR_) /*DLST*/\n",name,name);
+lno += 3;
 		if( op = strstr(line,"/*OPT(") ){
 			opt[0] = 0;
 			sscanf(op+6,"%[^)]",opt);
@@ -130,6 +163,7 @@ printf("#define %s (*%s_PTR_) /*DLST*/\n",name,name);
 	names[namex] = 0;
 
 printf("/*DLST*/ DLMap dlmap_%s[] = {\n",libname);
+printf("/*DLST*/ {\"#%s\"}, /*handle*/\n",libname);
 	for( i = 0; i < namex; i++ ){
 		name1 = names[i];
 if( opts[i] )
@@ -138,4 +172,6 @@ else
 printf("/*DLST*/ {\"%s\",&%s_PTR_},\n",name1,name1);
 	}
 printf("/*DLST*/ 0};\n");
+	return 0;
 }
+#endif
