@@ -1,10 +1,11 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 2004 National Institute of Advanced Industrial Science and Technology (AIST)
+Copyright (c) 2004-2006 National Institute of Advanced Industrial Science and Technology (AIST)
 
-Permission to use this material for evaluation, copy this material for
-your own use, and distribute the copies via publically accessible on-line
-media, without fee, is hereby granted provided that the above copyright
-notice and this permission notice appear in all copies.
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
 AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
@@ -22,26 +23,8 @@ History:
 
 typedef struct DGCtx DGC;
 
-#ifdef MAIN
-#define TRACE	trace
-void trace(PCStr(fmt),...)
-{	VARGS(8,fmt);
-	fprintf(stderr,fmt,VA8);
-}
-void swft_main(int ac,char *av[]);
-int main(int ac,char *av[])
-{
-	swft_main(ac,av);
-	return 0;
-}
-int reverseMOUNT(DGC*ctx,PVStr(url),int siz)
-{
-	return 0;
-}
-#else
 #define TRACE	syslog_ERROR
 int reverseMOUNT(DGC*ctx,PVStr(url),int siz);
-#endif
 
 /*
 static char REC_DefineButton2[] = {34,UI16,UI8,UI16,BRREC,UI8,ACTIONS};
@@ -297,6 +280,7 @@ static int swf_tag(Swf *swf)
 
 #define SIZEINC	256 /* max. increased size of .swf by rewritig URL */
 
+FileSize copyfile1(FILE*in,FILE*out);
 void swfFilter(DGC*ctx,FILE *in,FILE *out,PCStr(arg))
 {	Swf swfb,*swf = &swfb;
 	int nt,ch,xleng;
@@ -317,6 +301,23 @@ void swfFilter(DGC*ctx,FILE *in,FILE *out,PCStr(arg))
 	if( swf_head(swf) < 0 )
 		return;
 
+	if( swf->s_ibuf[0] == 'C' ){
+		TRACE("Not supported Compressed SWF: %d\n",swf->s_hver);
+		if( 0 < swf->s_ibi ){
+			fwrite(swf->s_ibuf,1,swf->s_ibi,out);
+		}
+		copyfile1(in,out);
+		return;
+	}
+	if( swf->s_hver < 5 ){
+		TRACE("Not supported Ver? %d (%d)\n",swf->s_hver,swf->s_ibi);
+		if( 0 < swf->s_ibi ){
+			fwrite(swf->s_ibuf,1,swf->s_ibi,out);
+		}
+		copyfile1(in,out);
+		return;
+	}
+
 	/* the real-length should be rewriten after conversion,
 	 * if the output file is not stream (seekable)
 	 */
@@ -358,7 +359,8 @@ void swfFilter(DGC*ctx,FILE *in,FILE *out,PCStr(arg))
 		swf->s_leng,swf->s_ooff,swf->s_ninc,swf->s_nrew);
 }
 
-void swft_main(int ac,char *av[])
+int swft_main(int ac,const char *av[])
 {
 	swfFilter(NULL,stdin,stdout,"");
+	return 0;
 }
