@@ -27,19 +27,21 @@ History:
 #include <stdio.h>
 #undef getc
 #undef putc
-
-extern char *str_fgets();
+#undef fflush
+#undef fprintf
 
 #define getc(file)		str_fgetc(file)
 #define fgetc(file)		str_fgetc(file)
 #define ungetc(ch,file)		str_fungetc(ch,file)
-#define fgets(buf,size,file)	str_fgets(buf,size,file)
+#undef fgets
+#define fgets(buf,size,file)	str_fgets(AVStr(buf),size,file)
 #define putc(ch,file)		str_fputc(ch,file)
 #define fputc(ch,file)		str_fputc(ch,file)
 #define fputs(buf,file)		str_fputs(buf,file)
 #define fflush(file)		str_fflush(file)
 #define fseek(file,off,where)	str_fseek(file,off,where)
 #define ftell(file)		str_ftell(file)
+#undef fclose
 #define fclose(file)		str_fclose(file)
 
 #define fprintf			str_fprintf
