@@ -2,9 +2,6 @@
 #define _VSOCKET_H
 
 #include "ystring.h"
-/*
-#include "typedefs.h"
-*/
 
 typedef unsigned /* long -- bad for DEC-ALPHA */ INETADDRV4;
 extern INETADDRV4 _inet_addrV4(PCStr(addr));
@@ -14,49 +11,59 @@ extern INETADDRV4 _inet_addrV4(PCStr(addr));
 
 int xinet_pton(int af,const char *src,void *dst);
 
-#ifdef _MSC_VER
+#ifdef _MSC_VER /*{*/
 #include "ywinsock.h"
 #include "ysocket.h"
 #define SELF_AF_UNIX
 #define SELF_RESOLV_H
 
-#ifndef EISCONN
+#ifndef EISCONN /*{*/
 #define EISCONN		WSAEISCONN
+#define EALREADY	WSAEALREADY
+#define EMSGSIZE	WSAEMSGSIZE
+#define EWOULDBLOCK	WSAEWOULDBLOCK
 #define EINPROGRESS	WSAEINPROGRESS
 #define ECONNREFUSED	WSAECONNREFUSED
+#define EFAULT		WSAEFAULT
 #define ENETUNREACH	WSAENETUNREACH
 #define EHOSTUNREACH	WSAEHOSTUNREACH
 #define ETIMEDOUT	WSAETIMEDOUT
 #define EADDRNOTAVAIL	WSAEADDRNOTAVAIL
 #define EADDRINUSE	WSAEADDRINUSE
-#endif
+#endif /*}*/
 
-#else
+#else /*}else (UNIX){*/
 
 #include <unistd.h>
 #include <sys/types.h>
 #include <netdb.h>
+/*
+#ifdef __osf__
+*/
+#if defined(__osf__) || defined(sun)
+#define gethostbyname2(name,af) ((af==AF_INET)?gethostbyname(name):NULL)
+#endif
 
-#ifdef __EMX__
+#ifdef __EMX__ /*{*/
 #define TCPIPV4
 #ifndef MAXSOCKETS
 #define MAXSOCKETS 2048
 #endif
 #define SELECT_WIDTH(fd)	(fd+3)
-#endif
+#endif /*}*/
 
 
-#if defined(__CYGWIN__) || defined(__MINGW32__)
+#if defined(__CYGWIN__) || defined(__MINGW32__) /*{*/
 #define SO_SNDBUF	0x1001
 #define SO_RCVBUF	0x1002
 #define SO_TYPE		0x1008
 #define SELF_AF_UNIX
 #define SELF_RESOLV_H
-#else
+#else /*}{*/
 #ifndef IPPROTO_TCP
 #include <sys/param.h>
 #endif
-#endif
+#endif /*}*/
 
 #include <sys/time.h>
 #include <sys/uio.h>
@@ -90,7 +97,7 @@ int Xaccept(int s,struct sockaddr *a,int *l);
 #define recvfrom	Xrecvfrom
 /*#######################################################*/
 
-#endif /* _MSC_VER */
+#endif /* } _MSC_VER */
 
 #ifdef SELF_AF_UNIX
 struct sockaddr_un {
@@ -99,13 +106,27 @@ struct sockaddr_un {
 };
 #endif
 
+#if defined(AF_INET6) && !defined(IPPROTO_IPV6) \
+ && defined(__linux__) /* RedHat6 */
+#define IPPROTO_IPV6  IPPROTO_IPV6  /* defined only in enum */
+#define sin6_scope_id sin6_flowinfo /* without scope_id */
+#endif
+
+#if !defined(_MSC_VER) || _MSC_VER < 1400 /* before VC2005 {*/
+/*
 #if !defined(AF_INET6) || !defined(IPPROTO_IPV6)
+*/
+#if !defined(AF_INET6) || !defined(IPPROTO_IPV6) || !defined(NI_NUMERICHOST)
 #ifndef AF_INET6
-#define AF_INET6 -6
+#define AF_INET6 ((unsigned int)-6)
 #endif
 struct in6_addr {
 	unsigned char addr8[16];
 };
+
+/* "sin6_family" might be "short" in sockaddr_in for IPv4
+ * without "sin_len", so it should be not be referred for portability
+ */
 struct sockaddr_in6 { 
 	unsigned char   sin6_len;
 	unsigned char   sin6_family;
@@ -132,8 +153,11 @@ struct addrinfo *ai_next;
 #define freeaddrinfo(ai) -1
 #define AI_NUMERICHOST 4
 #define IPPROTO_IPV6 41
+#define NI_NUMERICHOST 1 /* 2 on BSD, but ignored anyway */
 #endif
 
+#endif /*}*/
+
 #ifndef IPV6_V6ONLY
 #define IPV6_V6ONLY -1
 #endif
@@ -210,13 +234,19 @@ struct state {
 #ifndef ECONNRESET
 #define ECONNRESET	-109
 #endif
+#ifndef EWOULDBLOCK
+#define EWOULDBLOCK	-110
+#endif
+#ifndef EFAULT
+#define EFAULT		-111
+#endif
 
 #ifndef _VADDR_H
 #ifndef _VSADDR_DEFINED_
 #define _VSADDR_DEFINED_
 typedef union {
 struct sockaddr	_sa;
-	char	_sab[32];
+	int	_sab[32]; /* enough size to hold sockaddr_un */
 } VSAddr;
 #endif
 #endif
@@ -232,6 +262,8 @@ int   VSA_cto_(char addr[]);
 const char *VSA_ntoa(VSAddr *sap);
 const char *VSA_htoa(struct hostent *hp);
 const char *VSA_ltoa(const unsigned char *addr,int len,int type);
+int   VSA_aptosa(VSAddr *sa,PCStr(addrport));
+int   VSA_satoap(VSAddr *sa,PVStr(addrport));
 int   VSA_port(VSAddr *sap);
 char *VSA_xtoap(VSAddr *sa,PVStr(buf),int siz);
 int   VSA_strisaddr(PCStr(addr));
@@ -242,7 +274,7 @@ void  VSA_setport(VSAddr *sap,int port);
 int   VSA_atosa(VSAddr *sa,int port,PCStr(addr));
 int   VSA_htosa(VSAddr *sap,int port,struct hostent *hp,int hi);
 int   VSA_size(VSAddr *sap);
-int   VSA_atob(PCStr(aaddr),char baddrb[],int *btypep);
+int   VSA_atob(PCStr(aaddr),PVStr(baddrb),int *btypep);
 int   VSA_decomp(VSAddr *sap,const char **baddr,int *btype,const char **bport);
 int   VSA_islocal(VSAddr *vsa);
 int   VSA_6to4(VSAddr *vsa);
@@ -257,7 +289,7 @@ int   isinetAddr(PCStr(saddr));
 int   sock_isv6(int sock);
 int   VSA_af(VSAddr *sap);
 
-int   SOCKS_recvfrom(int sock,char buf[],int len,int flags,SAP from,int *fromlen);
+int   SOCKS_recvfrom(int sock,PVStr(buf),int len,int flags,SAP from,int *fromlen);
 int   SOCKS_sendto(int sock,PCStr(buf),int len,int flags,SAP to,int tolen);
 void  SOCKS_udpclose(int msock);
 
