@@ -2,20 +2,20 @@
 #define _YALLOCA_H
 
 typedef struct {
-  unsigned int	s_sp0;
+	char   *s_sp0;
 	char   *s_what;
 	int   (*s_func)(const void*,...);
-	char   *s_av[4];
+	char   *s_av[6];
 	int	s_size;
 	int	s_unit;
 	int	s_count;
 	int	s_trace;
 	int	s_level;
+	char   *s_top;
 } AllocaArg;
 
 int alloca_call(AllocaArg *ap);
 
-extern int RAND_TRACE;
 #define STACK1	128
 
 #endif /* _YALLOCA_H */
