@@ -5,7 +5,7 @@
 #include <sys/types.h>
 #ifndef FD_SETSIZE
 #ifdef _MSC_VER
-#include <winsock.h>
+#include "ywinsock.h"
 #else
 #include <sys/time.h>
 #ifndef FD_SETSIZE
@@ -28,4 +28,9 @@ typedef int FdSet;
 #define FD_SETSIZE	32
 #endif
 
+#if defined(FD_SETSIZE) && 2048 < FD_SETSIZE
+#undef FD_SETSIZE
+#define FD_SETSIZE	2048
+#endif
+
 #endif /* _YSELECT_H */
