@@ -3,6 +3,7 @@
 
 #ifdef _MSC_VER
 #include <winsock2.h> /* for fd_set */
+#include "ywinsock.h"
 #endif
 
 typedef struct sockaddr *_SAP;
@@ -21,6 +22,11 @@ int _SELECT(int,fd_set*,fd_set*,fd_set*,struct timeval*);
 int _GETHOSTNAME(char*,unsigned int);
 int _SHUTDOWN(int,int);
 
+#ifdef __APPLE__
+int recvDarwin(int s,void *b,int l,int f,FL_PAR);
+#define recv(s,b,l,f) recvDarwin(s,b,l,f,FL_ARG)
+#endif
+
 #define STD_HOSTENT
 #define gethostbyname	_GETHOSTBYNAME
 #define gethostbyaddr	_GETHOSTBYADDR
@@ -32,10 +38,14 @@ int _GETSOCKNAME(int s,_SAP,int *l);
 int _GETPEERNAME(int s,_SAP,int *l);
 
 
+/*
 #define socket		_SOCKET
+*/
 #define bind		_BIND
 #define listen		_LISTEN
+/*
 #define accept		_ACCEPT
+*/
 #define connect		_CONNECT
 #define shutdown	_SHUTDOWN
 
