@@ -12,35 +12,14 @@ DO NOT USE ITS BUGGY POLL
 #include "yselect.h" /* FD_SETSIZE */
 
 static int _gotRESET = -1;
-connRESETbypeer(){ return _gotRESET; }
+int connRESETbypeer(){ return _gotRESET; }
 static int _gotHUP = -1;
-connHUP(){ return _gotHUP; }
+int connHUP(){ return _gotHUP; }
 
-static getmsg1(where,what,fd)
-	char *where,*what;
-{	int rcode;
-	struct strbuf cntl,data;
-	int flags;
-
-	cntl.maxlen = 0; cntl.len = -1; cntl.buf = 0;
-	data.maxlen = 0; data.len = -1; data.buf = 0;
-	flags = 0;
-	errno = 0;
-	rcode = getmsg(fd,&cntl,&data,&flags);
-	if( errno == ECONNRESET )
-		_gotRESET = fd;
-
-	syslog_ERROR("%s.%s getmsg(%d)=%d %d,%d flags=%x errno=%d\n",
-		where,what,fd,rcode,cntl.len,data.len,flags,errno);
-}
-static checkERR(where,fd,revents)
-	char *where;
+static void checkERR(const char *where,int fd,int revents)
 {	int xerrno;
 
 	xerrno = errno;
-	if( revents & POLLERR )
-		getmsg1(where,"POLLERR",fd);
-
 	if( revents & POLLHUP ){
 		_gotHUP = fd;
 		if( xerrno )
@@ -55,7 +34,7 @@ static checkERR(where,fd,revents)
 	errno = xerrno;
 }
 
-PollOut(fd,timeout)
+int PollOut(int fd,int timeout)
 {	struct pollfd pfd[FD_SETSIZE];
 	int nready;
 
@@ -73,15 +52,18 @@ PollOut(fd,timeout)
 }
 
 int pollIn_HUP = 1;
-PollIn_HUP(on){ int xon = pollIn_HUP; pollIn_HUP = on; return xon; }
+int PollIn_HUP(int on){ int xon = pollIn_HUP; pollIn_HUP = on; return xon; }
 #define POLLIN_MASK	(POLLIN|POLLPRI|(pollIn_HUP?POLLHUP:0))
 
-PollIn1(fd,timeout)
+int _gotOOB;
+
+int PollIn1(int fd,int timeout)
 {	struct pollfd pfd[FD_SETSIZE];
 	int nready;
 
 	_gotRESET = -1;
 	_gotHUP = -1;
+	_gotOOB = 0;
 
 	if( fd < 0 )
 		return -1;
@@ -100,17 +82,19 @@ PollIn1(fd,timeout)
 	if( pfd[0].revents == POLLHUP )
 		syslog_DEBUG("PollIn(%d,%d) = POLLHUP\n",fd,timeout);
 
+	if( pfd[0].revents & POLLPRI )
+		_gotOOB = fd+1;
 	return (pfd[0].revents & POLLIN_MASK) ? 1 : 0;
 }
 
-PollIns(timeout,size,mask,rmask)
-	int *mask,*rmask;
+int PollIns(int timeout,int size,int *mask,int *rmask)
 {	struct pollfd pfd[FD_SETSIZE];
 	int fd,nfd,rfd,nready,rready;
 	int fi;
 
 	_gotRESET = -1;
 	_gotHUP = -1;
+	_gotOOB = 0;
 
 	nfd = 0;
 	for(fi = 0; fi < size; fi++){
@@ -138,6 +122,12 @@ PollIns(timeout,size,mask,rmask)
 		syslog_DEBUG("PollIns(%d,%d) = POLLHUP\n",fd,timeout);
 
 			if( pfd[rfd].revents & POLLIN_MASK ){
+				if( pfd[rfd].revents & POLLPRI ){
+					_gotOOB = fd+1;
+				syslog_DEBUG("[%d] poll() detected OOB\n",fd);
+					if((pfd[rfd].revents & POLLIN)==0)
+					syslog_ERROR("[%d] OOB only\n",fd);
+				}
 				rready++;
 				rmask[fi] = 1;
 			}else	rmask[fi] = 0;
@@ -147,9 +137,22 @@ PollIns(timeout,size,mask,rmask)
 	return rready;
 }
 
+int withOOB(int fd){
+	struct pollfd pfd[1];
+	int ready;
 
-PollInsOuts(timeout,nfds,fdv,ev,rev)
-	int fdv[],ev[],rev[];
+	pfd[0].fd = fd;
+	pfd[0].events = POLLPRI;
+	ready = poll(pfd,1,0);
+	syslog_DEBUG("[%d] withOOB ? ready=%d %X\n",fd,ready,pfd[0].revents);
+	if( 0 < ready ){
+		if( pfd[0].revents & POLLPRI )
+			return 1;
+	}
+	return 0;
+}
+
+int PollInsOuts(int timeout,int nfds,int fdv[],int ev[],int rev[])
 {	struct pollfd fds[1024];
 	int fi,ev1,ev2,nready;
 
@@ -160,6 +163,9 @@ PollInsOuts(timeout,nfds,fdv,ev,rev)
 		ev2 = 0;
 		if( ev1 & PS_IN  ) ev2 |= POLLIN;
 		if( ev1 & PS_PRI ) ev2 |= POLLPRI;
+#ifdef POLLRDHUP
+		if( ev1 & PS_PRI ) ev2 |= POLLRDHUP;
+#endif
 		if( ev1 & PS_OUT ) ev2 |= POLLOUT;
 		fds[fi].events = ev2;
 	}
@@ -169,11 +175,18 @@ PollInsOuts(timeout,nfds,fdv,ev,rev)
 		ev1 = 0;
 		if( ev2 & POLLIN   ) ev1 |= PS_IN;
 		if( ev2 & POLLPRI  ) ev1 |= PS_PRI;
+#ifdef POLLRDHUP
+		if( ev2 & POLLRDHUP) ev1 |= PS_PRI;
+#endif
 		if( ev2 & POLLOUT  ) ev1 |= PS_OUT;
 		if( ev2 & POLLERR  ) ev1 |= PS_ERR;
 		if( ev2 & POLLHUP  ) ev1 |= PS_HUP;
 		if( ev2 & POLLNVAL ) ev1 |= PS_NVAL;
 		rev[fi] = ev1;
+		if( ev1 & PS_ERR ){
+		    syslog_ERROR("PollInsOuts(%d/%d) %d ERR %X %X err=%d\n",
+			fi,nfds,nready,ev1,ev2,errno);
+		}
 	}
 	return nready;
 }
