@@ -1,5 +1,4 @@
-#include <stdio.h>
-#include "yarg.h"
+#include "ystring.h"
 
 #include <lwp/lwp.h>
 #include <lwp/stackdep.h>
@@ -54,6 +53,13 @@ static int thread_yield()
 	return lwp_yield(SELF);
 }
 
-const char *WithThread = "lwp_create";
+const char *WithThread = "Lwp";
 caddr_t (*ThreadFork)() = thread_fork;
 int (*ThreadYield)() = thread_yield;
+
+static int thread_wait(int tid,int timeout){
+        return -1;
+}
+int (*ThreadWait)(int,int) = (int(*)(int,int))thread_wait;
+int (*ThreadId)() = 0;
+int (*ThreadExit)(void *code) = 0;
