@@ -37,44 +37,40 @@
 #define WCOREDUMP(stat)	-1
 #endif
 
-INHERENT_ptrace(){ return 1; }
+int INHERENT_ptrace(){ return 1; }
 
-ptraceTraceMe()
+int ptraceTraceMe()
 {
 	return PTrace(ptTRACEME,0,0,0);
 }
-ptraceContinue(pid,sig)
+int ptraceContinue(int pid,int sig)
 {
 	return PTrace(ptCONTINUE,pid,1,sig);
 }
-ptraceKill(pid)
+int ptraceKill(int pid)
 {
 	return PTrace(ptKILL,pid,0,0);
 }
 
-getWaitStopSig(statp)
-	int *statp;
+int getWaitStopSig(int *statp)
 {
 	if( WIFSTOPPED(*statp) )
 		return WSTOPSIG(*statp);
 	else	return -1;
 }
-getWaitExitSig(statp)
-	int *statp;
+int getWaitExitSig(int *statp)
 {
 	if( WIFSIGNALED(*statp) )
 		return WTERMSIG(*statp);
 	else	return -1;
 }
-getWaitExitCode(statp)
-	int *statp;
+int getWaitExitCode(int *statp)
 {
 	if( WIFEXITED(*statp) )
 		return WEXITSTATUS(*statp);
 	else	return -1;
 }
-getWaitExitCore(statp)
-	int *statp;
+int getWaitExitCore(int *statp)
 {
 	return WCOREDUMP(*statp);
 }
