@@ -11,7 +11,7 @@ int Scandir(const char *dirpath,int(*func)(const char*,...),...)
 	int nent,ei;
 	int rcode;
 
-	VARGS(3,func);
+	VARGS(8,func);
 
 	rcode = 0;
 	dirents = NULL;
@@ -21,7 +21,7 @@ int Scandir(const char *dirpath,int(*func)(const char*,...),...)
 			dir1->d_name[dir1->d_namlen] = 0;
 			scan_ino = dir1->d_fileno;
 
-			if( rcode = (*func)(dir1->d_name,va[0],va[1],va[2]) )
+			if( rcode = (*func)(dir1->d_name,VA8) )
 				break;
 		}
 		free(dirents);
