@@ -1,10 +1,23 @@
 #include <sys/utsname.h>
-static struct utsname un;
-static int un_got;
+#include <stdio.h>
+#include "ystring.h"
 
-Uname(name)
-	char *name;
+typedef struct {
+ struct utsname	ue_un;
+	int	ue_un_got;
+} UnEnv;
+static UnEnv *unEnv;
+#define un	unEnv->ue_un
+#define un_got	unEnv->ue_un_got
+void minit_uname(){
+	if( unEnv == 0 )
+		unEnv = NewStruct(UnEnv);
+}
+
+int Uname(PVStr(name))
 {
+	minit_uname();
+
 	if( un_got == 0 ){
 		if( 0 <= uname(&un) ){
 			un.sysname[sizeof(un.sysname)-1] = 0;
