@@ -1,16 +1,20 @@
-#if defined(hpux) \
+#if defined(hpux) || defined(__hpux__) \
  || defined(_nec_ews) \
  || defined(BUGGY_USLEEP)
 
 #include "usleep.c"
 
 #else
+#include "ystring.h"
+#include <unistd.h>
+void usleep_bypoll(int);
+
 static int buggy_USLEEP;
-Usleep(usec)
-{	char uname[128];
+void Usleep(int usec)
+{	CStr(uname,128);
 
 	if( buggy_USLEEP == 0 ){
-		Uname(uname);
+		Uname(AVStr(uname));
 		if( strncmp(uname,"HI-UX",5) == 0 )
 			buggy_USLEEP = 1;
 		else	buggy_USLEEP = -1;
