@@ -1,8 +1,8 @@
 #include <stdio.h>
 
-main(ac,av)
-	char *av[];
-{	char *CC,*sym;
+int main(int ac,char *av[])
+{	const char *CC;
+	const char *sym;
 	int avail;
 	FILE *tmp;
 
