@@ -1,6 +1,14 @@
+#include "ystring.h"
+
 int SUBST_bcopy = 1; /* bcopy(void*,void*,unsigned int*); */
 
-void bcopy(const void *b1,void *b2,unsigned int length)
+#if defined(__hpux__)
+#define TSZ unsigned long
+#else
+#define TSZ unsigned int
+#endif
+
+void bcopy(const void *b1,void *b2,TSZ length)
 {	int i;
 	const char *t1;
 	char *t2;
@@ -18,7 +26,7 @@ void bcopy(const void *b1,void *b2,unsigned int length)
 	}
 }
 
-int bcmp(const void *b1,const void *b2,unsigned int length)
+int bcmp(const void *b1,const void *b2,TSZ length)
 {	int i;
 
 	for( i = 0; i < length; i++) 
@@ -26,9 +34,3 @@ int bcmp(const void *b1,const void *b2,unsigned int length)
 			return 1;
 	return 0;
 }
-void bzero(void *b,unsigned int length)
-{	int i;
-
-	for( i = 0; i < length; i++ )
-		((char*)b)[i] = 0;
-}
