@@ -1,8 +1,9 @@
+int SUBST_fchown = 1;
+int INHERENT_fchown(){ return 0; }
+
 #include <stdio.h>
-
-INHERENT_fchown(){ return 0; }
-
-fchown(fd,uid,gid)
+int porting_dbg(const char *fmt,...);
+int fchown(int fd,int uid,int gid)
 {	int rcode;
 
 	if( uid == 0 && gid == 0 )
