@@ -2,9 +2,7 @@ int SUBST_strcasecmp = 1;
 
 #include <ctype.h>
 
-strncasecmp(a,b,n)
-	register char *a,*b;
-	register int n;
+int strncasecmp(const char *a, const char *b, unsigned int n)
 {	register char ac,bc;
 	register int i;
 
@@ -32,8 +30,7 @@ strncasecmp(a,b,n)
 	return ac - bc;
 }
 
-strcasecmp(a,b)
-	char *a,*b;
+int strcasecmp(const char *a,const char *b)
 {
 	return strncasecmp(a,b,0xFFFFFFF);
 }
