@@ -8,8 +8,8 @@
 #endif
 #endif
 
-Uname(name)
-	char *name;
+#include "ystring.h"
+int Uname(PVStr(name))
 {
 	strcpy(name,UNAME);
 	return -1;
