@@ -1,12 +1,11 @@
-INHERENT_utimes(){ return 0; }
+int SUBST_utimes = 1;
+int INHERENT_utimes(){ return 0; }
 
 #include <sys/types.h>
-#include "vsocket.h" /* <sys/time.h> */
 #include <utime.h>
+int utime(char*,void*);
 
-utimes(file,tvp)
-	char *file;
-	struct timeval *tvp;
+int utimes(char file[],struct timeval *tvp)
 {	struct utimbuf times;
 
 	if( tvp ){
