@@ -1,11 +1,14 @@
 /*///////////////////////////////////////////////////////////////////////
-Copyright (c) 1994-1999 Electrotechnical Laboratry (ETL), AIST, MITI
-Copyright (c) 1994-1999 Yutaka Sato
+Copyright (c) 1994-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, and distribute this material for any purpose
-and without fee is hereby granted, provided that the above copyright
-notice and this permission notice appear in all copies.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -18,7 +21,12 @@ History:
 	950312	encode/decode parts in a multipart message
 //////////////////////////////////////////////////////////////////////#*/
 
+#define DOFGSZ
 #include "mime.h"
+#ifdef MIMEKIT
+#define strid_alloc(s)	stralloc(s)
+#endif
+
 static int encodeMIMEpart(const char *boundaries[],FILE*fc,FILE*ts,FILE*cache,int filter,int _);
 static int decodeMIMEpart(const char *boundaries[],FILE*fs,FILE*tc,FILE*cache,int filter,int enHTML);
 
@@ -52,13 +60,16 @@ int MIME_CONV = MIMECONV;
 #define DECODE(flag)	(MIME_CONV & flag)
 #define ENCODE(flag)	(MIME_CONV & (flag<<16))
 
+/*
 static int maybe_MIME;
 static int maybe_NONASCII;
 static int got_EOR;
+*/
 static int addQY;
 
 #define DEBUG	syslog_DEBUG
 
+/*
 #define O_HEAD	1
 #define O_DELIM	2
 #define O_BODY	4
@@ -67,12 +78,15 @@ static int addQY;
 #define O_MIME2PGP	0x100
 #define O_TEXTONLY	0x200
 #define O_MULTIPART	0x400
+*/
 
 #define HEAD_ALSO(filter)	(filter & O_HEAD)
 #define DELIM_ALSO(filter)	(filter & O_DELIM)
 #define BODY_ALSO(filter)	(filter & O_BODY)
 #define EOR_ALSO(filter)	(filter & O_EOR)
 
+int MIME_FILTER;
+
 #define MIME2PGP(filter)	((filter& O_MIME2PGP) && (PGP_DECR()||PGP_VRFY()))
 #define TEXTONLY(filter)	(filter & O_TEXTONLY)
 #define MULTIPART(filter)	(filter & O_MULTIPART)
@@ -123,8 +137,24 @@ static int is8bitCharset(PCStr(charset))
 	return 0;
 }
 
+const char *MIME_transAddrSpec;
+int MC_scanAnons(PCStr(smask));
+int MC_setAnons(int mask);
+extern const char *MIME_nomapMailAddrs;
 static scanListFunc conv1(PCStr(spec))
 {
+	if( strneq(spec,"rewaddr:",8) ){
+		CStr(list,128);
+		int amask;
+		MIME_CONV = C_ENCODE(C_HEAD|C_BODY) | C_DECODE(C_HEAD|C_BODY);
+		wordScanY(spec+8,list,"^:");
+		amask = MC_scanAnons(list);
+		MC_setAnons(amask);
+		MIME_transAddrSpec = strid_alloc(spec+8);
+	}else
+	if( strneq(spec,"nomapemail:",11) ){
+		MIME_nomapMailAddrs = strid_alloc(spec+11);
+	}else
 	if( streq(spec,"thru")) MIME_CONV = 0; else
 	if( streq(spec,"all" )) MIME_CONV = C_ALL; else
 	if( streq(spec,"enc" )) MIME_CONV |= C_ENCODE(C_HEAD|C_BODY); else 
@@ -134,12 +164,43 @@ static scanListFunc conv1(PCStr(spec))
 				MIME_CONV &= ~C_ENCODE(C_HEAD_SPENC); else
 	if( streq(spec,"charcode") )
 				MIME_CONV = C_DECODE(C_CHAR)|C_ENCODE(C_CHAR);
+	else
+	if( streq(spec,"zero:none") ){
+		FGSZ_flags = FGSZ_NONE;
+	}else
+	if( streq(spec,"zero:utf8") ){
+		FGSZ_flags = FGSZ_UTF8;
+	}else
+	if( streq(spec,"zero:kill") ){
+		FGSZ_flags = FGSZ_KILL;
+	}
+	else
+	if( streq(spec,"textonly") ){
+		MIME_FILTER |= O_TEXTONLY;
+	}
 	else	syslog_ERROR("#### ERROR: unknown MIMECONV=%s\n",spec);
 	return 0;
 }
 extern int MIME_SPACE_ENCODING;
+#define HM_TOMAIL "Date,Subject,From,Cc,Reply-To,X-Seqno,Organization,Xref,Newsgroups,Distribution,Message-ID,References,In-Reply-To,Lines,Content-Type,Content-Transfer-Encoding,MIME-Version,X-Mailer,X-From-Fp"
+const char *MIME_headMask;
+const char *setHeadMask(PCStr(hmask)){
+	const char *ohm;
+	ohm = MIME_headMask;
+	if( hmask && streq(hmask,"") )
+		MIME_headMask = HM_TOMAIL;
+	else	MIME_headMask = hmask;
+	return ohm;
+}
+
 void scan_MIMECONV(PCStr(convspec))
 {
+	if( strneq(convspec,"headmask:",9) ){
+		if( streq(convspec+9,"-ng2ml") )
+			MIME_headMask = HM_TOMAIL;
+		else	MIME_headMask = stralloc(convspec+9);
+		return;
+	}
 	if( *convspec == 0 )
 		MIME_CONV = C_ENCODE(C_HEAD|C_BODY) | C_DECODE(C_HEAD|C_BODY);
 	else	scan_commaList(convspec,0,scanListCall conv1);
@@ -147,19 +208,219 @@ void scan_MIMECONV(PCStr(convspec))
 		MIME_SPACE_ENCODING = 0;
 }
 
-const char *MIME_transAddrSpec;
-static char *fgetsTee(PVStr(buf),int size,FILE *in,FILE *out,FILE *cache)
+int MC_MASK;
+
+int MC_scanMasks(PCStr(smask)){
+	int imask = 0;
+	if( isinList(smask,"_GECOS") )     imask |= MM_FROM;
+	if( isinList(smask,"_Body") )      imask |= MM_BODY;
+	if( isinList(smask,"_Signature") ) imask |= MM_SIGN;
+	return imask;
+}
+int MC_setMasks(int mask){
+	int omask;
+	omask = MC_MASK;
+	MC_MASK = (MC_MASK & 0xFFFF0000) | mask;
+	return omask;
+}
+int MC_getMasks(){
+	return MC_MASK & 0xFFFF;
+}
+int MC_scanAnons(PCStr(smask)){
+	int imask = 0;
+	if( streq(smask,"*") )
+		smask = "_Poster,_MessageID,_Email,_Phone";
+	if( isinList(smask,"Body") )	   imask |= MA_EMAIL;
+	if( isinList(smask,"_Poster") )    imask |= MA_FROM;
+	if( isinList(smask,"_MessageID") ) imask |= MA_MSGID;
+	if( isinList(smask,"_Email") )	   imask |= MA_EMAIL;
+	if( isinList(smask,"_Phone") )	   imask |= MA_PHONE;
+	return imask;
+}
+int MC_setAnons(int mask){
+	int omask;
+	omask = MC_MASK;
+	MC_MASK = (mask << 16) | (MC_MASK & 0xFFFF);
+	return omask;
+}
+int MC_getAnons(){
+	return (MC_MASK >> 16) & 0xFFFF;
+}
+static scanListFunc filt1(PCStr(l1),PVStr(lp),int *off){
+	if( strcaseeq(l1,"_Poster") ) return 0;
+	if( strcaseeq(l1,"_MessageID") ) return 0;
+	if( strcaseeq(l1,"_Email") ) return 0;
+	if( strcaseeq(l1,"_Phone") ) return 0;
+	if( *off ){
+		Xstrcat(DVStr(lp,*off),",");
+		*off += 1;
+	}
+	Xstrcat(DVStr(lp,*off),l1);
+	*off += strlen(l1);
+	return 0;
+}
+void MIME_anon2list(int mask,PVStr(list)){
+	int m1;
+	const char *sm1;
+	CStr(xlist,1024);
+	refQStr(lp,xlist);
+	int off = 0;
+
+	truncVStr(xlist);
+	scan_commaListL(list,0,scanListCall filt1,AVStr(xlist),&off);
+	lp = xlist + strlen(xlist);
+
+	for( m1 = 1; m1; m1 = m1 << 1 ){
+		if( (m1 & mask) == 0 )
+			continue;
+		switch( m1 ){
+			case MA_FROM:  sm1 = "_Poster"; break;
+			case MA_MSGID: sm1 = "_MessageID"; break;
+			case MA_EMAIL: sm1 = "_Email"; break;
+			case MA_PHONE: sm1 = "_Phone"; break;
+			default: continue;
+		}
+		if( xlist < lp ){
+			sprintf(lp,",%s",sm1);
+		}else{
+			sprintf(lp,"%s",sm1);
+		}
+		lp += strlen(lp);
+	}
+	if( strcmp(list,xlist) != 0 ){
+		strcpy(list,xlist);
+	}
+}
+extern int (*MIME_setPosterMasks)(PCStr(from));
+static char OrigFrom[256];
+int MIME_disableTransHead;
+
+
+int MIME_rewriteAddr(PVStr(buf),int inHead){
+	int anons = MC_getAnons();
+/*
+CStr(sb,1024); strcpy(sb,buf);
+*/
+/*
+if(strchr(buf,'@')) fprintf(stderr,"---H=%d U=%X [%s] %s",
+inHead,anons,
+MIME_transAddrSpec?MIME_transAddrSpec:"NULL",buf);
+*/
+
+	if( anons & MA_FROM )
+	if( MIME_setPosterMasks && inHead ){
+		if( strncaseeq(buf,"From:",5) ){
+			CStr(Fb,1024);
+			RFC822_addresspartX(buf+5,AVStr(Fb),sizeof(Fb));
+			(*MIME_setPosterMasks)(Fb);
+			Xstrcpy(FVStr(OrigFrom),Fb);
+/*
+ fprintf(stderr,"#### OrigFrom [%d] #### [%s]\n",getpid(),OrigFrom);
+*/
+		}
+	}
+	if( MIME_disableTransHead ){
+		return 0;
+	}
+
+	if( MIME_transAddrSpec ){
+		if( inHead ){
+			MIME_rewriteHeader("",MIME_transAddrSpec,AVStr(buf),NULL);
+/*
+if(strcmp(buf,sb)) fprintf(stderr,"#tH %X %X %s",MC_getAnons(),MC_MASK,buf);
+*/
+		}else{
+			const char *dp;
+			if( dp = strchr(MIME_transAddrSpec,':') ){
+				if( anons )
+				scanAddrInBody(anons,dp+1,AVStr(buf));
+			}
+/*
+if(strcmp(buf,sb)) fprintf(stderr,"#tB %X %X %s",MC_getAnons(),MC_MASK,buf);
+*/
+		}
+	}
+	return 0;
+}
+static int maskFrom(PVStr(head)){
+	CStr(from,256);
+	if( strncaseeq(head,"From:",5) ){
+		RFC822_addresspartX(head+5,AVStr(from),sizeof(from));
+		sprintf(head,"From: %s (((Masked-From)))\r\n",from);
+		return 1;
+	}
+	return 0;
+}
+static int maskBody(PVStr(tmpa)){
+	const char *t0;
+	refQStr(tp,tmpa); /* before signature */
+	char tc;
+
+	if( MC_MASK_BODY ){
+		strcpy(tmpa,"(((Masked-Body)))\r\n");
+		return 1;
+	}else
+	if( MC_MASK_SIGNATURE ){
+		t0 = tmpa + 4;
+		tp = tmpa+strlen(tmpa)-1;
+		tc = 0;
+		for(; t0 < tp; tp-- ){
+			tc = *tp;
+			if( tc!=' ' && tc!='\t' && tc!='\r' && tc!='\n' )
+				break;
+		}
+		for(; t0 < tp; tp-- ){
+			if( strneq(tp-4,"\r\n\r\n",4) ){
+				goto MASKBODY;
+			}
+			if( strneq(tp-2,"\n\n",2) ){
+				goto MASKBODY;
+			}
+			if( strneq(tp-4,"\n--\r",4)||strneq(tp-4,"\n--\n",4) ){
+				tp -= 3;
+				goto MASKBODY;
+			}
+		}
+	}
+	return 0;
+MASKBODY:
+	strcpy(tp,"(((Masked-Signature)))\r\n");
+	return 1;
+}
+
+static int xfputs(PCStr(wh),PCStr(buf),FILE *out){
+	int fcode;
+	fcode = fputs(buf,out);
+	if( fcode == EOF ){
+		syslog_ERROR("--MIME EOF on %s [%d/%X]\n",wh,fileno(out),out);
+	}
+	return fcode;
+}
+static char *fgetsTee(PVStr(buf),int size,FILE *in,FILE *out,FILE *cache,int inHead)
 {	const char *rcode;
 
 	setVStrEnd(buf,0);
-	if( rcode = fgets(buf,size,in) ){
+	if( MIME_transAddrSpec ){
+		/* 9.9.1 might be expanded in rewriteAddr() */
+		size = (size * 7) / 8;
+	}
+	if( inHead )
+		rcode = RFC822_fgetsHeaderField(BVStr(buf),size,in);
+	else	rcode = fgets(buf,size,in);
+	if( rcode ){
 		if( !isEOR(buf) ){
 			if( cache != NULL )
 				fputs(buf,cache);
-			if( MIME_transAddrSpec )
-				scanAddrInBody(MIME_transAddrSpec,AVStr(buf));
+			MIME_rewriteAddr(BVStr(buf),inHead);
 			if( out != NULL )
+			{
+				/*
 				fputs(buf,out);
+				*/
+				if( xfputs("fgetsTee",buf,out) == EOF ){
+					rcode = NULL;
+				}
+			}
 		}
 	}
 	return (char*)rcode;
@@ -202,9 +463,12 @@ static int pop_boundary(const char *boundaries[],PCStr(line))
 
 	if( line[0] == '-' && line[1] == '-' ){
 		for( bi = 0; b1 = boundaries[bi]; bi++ ){
+			/*
 			blen = strlen(b1);
 			if( strncmp(line+2,b1,blen) == 0 ){
 				tp = &line[2+blen];
+			*/
+			if( tp = strheadstrX(line+2,b1,0) ){
 				if( tp[0]=='-' && tp[1]=='-' ){
 					free((char*)b1);
 					boundaries[bi] = 0;
@@ -316,7 +580,8 @@ static void scan_ctype(PCStr(fvalue),PVStr(ctype),PVStr(ichset),PVStr(boundary),
 		scan_boundary(fvalue,AVStr(boundary),bsize);
 }
 
-static int readHeader(FILE *fs,PVStr(buf),int size,FILE *cache,int filter,PVStr(ctype),PVStr(ichset),const char *boundaries[],PVStr(encoding),int enHTML)
+extern int (*MIME_makeAdminKey)(PCStr(from),PVStr(key),int siz);
+static int readHeader(MimeConv *Mcv,FILE *fs,PVStr(buf),int size,FILE *cache,int filter,PVStr(ctype),PVStr(ichset),const char *boundaries[],PVStr(encoding),int enHTML)
 {	refQStr(hp,buf); /**/
 	const char *hs;
 	char hc;
@@ -325,6 +590,7 @@ static int readHeader(FILE *fs,PVStr(buf),int size,FILE *cache,int filter,PVStr(
 	int conv_field;
 	CStr(boundary,256);
 	CStr(tmp,LINESIZE);
+	int skipping = 0;
 
 	maybe_MIME = 0;
 	maybe_NONASCII = 0;
@@ -337,9 +603,10 @@ static int readHeader(FILE *fs,PVStr(buf),int size,FILE *cache,int filter,PVStr(
 	setVStrEnd(buf,0);
 	conv_field = 0;
 	cur_field[0] = 0;
+	OrigFrom[0] = 0;
 
 	for( rem = size; LINESIZE < rem; ){
-		if( fgetsTee(AVStr(hp),LINESIZE,fs,NULL,cache) == NULL )
+		if( fgetsTee(AVStr(hp),LINESIZE,fs,NULL,cache,1) == NULL )
 			break;
 		if( got_EOR = isEOR(hp) ){
 			if( !EOR_ALSO(filter) )
@@ -357,8 +624,27 @@ static int readHeader(FILE *fs,PVStr(buf),int size,FILE *cache,int filter,PVStr(
 				scan_ctype(hp+1,AVStr(ctype),AVStr(ichset),
 					AVStr(boundary),sizeof(boundary));
 			}
+			if( skipping )
+				continue;
 		}else{
+			if( MC_MASK_FROM ){
+				maskFrom(AVStr(hp));
+			}
+			if( MIME_headMask ){
+				CStr(fnam,64);
+				wordScanY(hp,fnam,"^:");
+				if( !isinListX(MIME_headMask,fnam,"wc") ){
+					skipping = 1;
+					continue;
+				}
+				
+			}
+			skipping = 0;
+
+			/*
 			wordscanY(hp,AVStr(cur_field),sizeof(cur_field),"^: \t\r\n");
+			*/
+			wordscanY(hp,AVStr(cur_field),sizeof(cur_field),"^: \300\t\r\n");
 			conv_field = strncasecmp(hp,"Subject:",8) == 0
 				  || strncasecmp(hp,"From:",5) == 0;
 
@@ -380,7 +666,12 @@ static int readHeader(FILE *fs,PVStr(buf),int size,FILE *cache,int filter,PVStr(
 
 		for(hs = buf; hc = *hs; hs++ ){
 			if( hc == '=' && hs[1] == '?' )
+			{
 				maybe_MIME = 1;
+				if( maybe_NONASCII == 0 ){
+					maybe_NONASCII = 2;
+				}
+			}
 			if( hc == 033 || hc & 0x80 )
 				maybe_NONASCII = 1;
 			if( hc == '$' && (hs[1] == 'B' || hs[1] == '@') )
@@ -392,12 +683,45 @@ static int readHeader(FILE *fs,PVStr(buf),int size,FILE *cache,int filter,PVStr(
 		rem -= len;
 		hp += len;
 	}
+
+	if( OrigFrom[0] && MIME_makeAdminKey ){
+		CStr(eline,128);
+		CStr(key,128);
+		(*MIME_makeAdminKey)(OrigFrom,AVStr(key),sizeof(key));
+		strcpy(eline,hp);
+		sprintf(hp,"X-From-Fp: %s\r\n",key+16);
+/*
+ fprintf(stderr,"***** ##### %X %s [%s][%s]\n",MC_MASK,hp,OrigFrom,key);
+*/
+		len = strlen(hp);
+		hp += len;
+		rem -= len;
+		strcpy(hp,eline);
+		rem -= strlen(hp);
+	}
+
 	if( boundary[0] )
 		push_boundary(boundaries,boundary);
 	if( ctype[0] == 0 )
 		strcpy(ctype,C_TEXT_PLAIN);
 	return rem;
 }
+int relayHeader(FILE *fs,FILE *tc,PCStr(mask),PVStr(buf),int siz){
+	int rem;
+	const char *boundaries[1];
+	CStr(cty,128);
+	CStr(ics,128);
+	CStr(enc,128);
+	const char *omask;
+	MimeConv Mcvb,*Mcv = &Mcvb;
+
+	boundaries[0] = 0;
+	omask = setHeadMask(mask);
+	bzero(Mcv,sizeof(MimeConv));
+	rem = readHeader(Mcv,fs,BVStr(buf),siz,tc,0,AVStr(cty),AVStr(ics),boundaries,AVStr(enc),0);
+	setHeadMask(omask);
+	return rem;
+}
 
 typedef int (*encdecFunc)(const char*[],FILE*,FILE*,FILE*,int,int);
 
@@ -408,7 +732,7 @@ int scan_multipart(const char *boundaries[],PVStr(endline),int size,encdecFunc f
 	int top;
 
 	/* preamble */
-	while( fgetsTee(AVStr(line),size,src,NULL,cache) != NULL ){
+	while( fgetsTee(AVStr(line),size,src,NULL,cache,0) != NULL ){
 		if( !MIME2PGP(filter) )
 		if( !TEXTONLY(filter) )
 		fputs(line,dst);
@@ -421,6 +745,8 @@ int scan_multipart(const char *boundaries[],PVStr(endline),int size,encdecFunc f
 	for(;;){
 		if( feof(src) )
 			rcode = MP_EOF;
+		if( rcode != MP_EOF && ferror(dst) )
+			rcode = MP_EOF;
 		if( rcode==MP_EOF || rcode==MP_EOR || rcode==MP_EOP )
 			break;
 		if( top != top_boundary(boundaries) )
@@ -439,7 +765,7 @@ int scan_multipart(const char *boundaries[],PVStr(endline),int size,encdecFunc f
 	/* epilogue */
 	if( rcode == MP_EOP ){
 		for(;;){
-			if( fgetsTee(AVStr(endline),size,src,NULL,cache) == NULL ){
+			if( fgetsTee(AVStr(endline),size,src,NULL,cache,0) == NULL ){
 				rcode = MP_EOF;
 				break;
 			}
@@ -449,26 +775,75 @@ int scan_multipart(const char *boundaries[],PVStr(endline),int size,encdecFunc f
 			}
 			if( !TEXTONLY(filter) )
 			if( !MIME2PGP(filter) )
+			{
+				/*
 				fputs(endline,dst);
+				*/
+				if( xfputs("scan_mp",endline,dst) == EOF ){
+					rcode = MP_EOF;
+					break;
+				}
+			}
 		}
 	}
 	return rcode;
 }
 
-static char *readPart(FILE*src,FILE*dst,FILE*cache,const char *boundaries[],PVStr(line),int *codep,int *lengp)
+static char *readPart(FILE*src,FILE*dst,FILE*cache,int filter,const char *boundaries[],PVStr(line),int *codep,int *lengp)
 {	int rcode;
 	int leng,len1,reqsize;
 	CStr(buff,0x10000);
+	refQStr(bp,buff);
+	const char *bx = &buff[sizeof(buff)-256];
 	FILE *bfile;
+	FILE *odst = 0;
 
 	rcode = 0;
 	leng = 0;
 	buff[0] = 0;
 	bfile = NULL;
 
+/*
+ fprintf(stderr,"#### readPart %X [%s]\n",MC_MASK,
+ MIME_transAddrSpec?MIME_transAddrSpec:"NONE");
+*/
+
+	if( TEXTONLY(filter) ){
+		/* to skip ending boundary */
+		odst = dst;
+		dst = 0;
+	}
+
+	if( TEXTONLY(filter) )
+	if( odst == 0 && cache == 0 && (filter & O_BODY_CTE) == 0 ){
+	  /* 9.6.2 it's too heavy to copy large data line by line.
+	   */
+	  for(;;){
+		if( rawfgets((char*)bp,sizeof(buff)-(bp-buff),src) == NULL ){
+			rcode = MP_EOF;
+			break;
+		}
+		len1 = strlen(bp);
+		if( rcode = pop_boundary(boundaries,bp) ){
+			truncVStr(bp);
+			break;
+		}
+		leng += len1;
+		if( filter & O_BODY_GET ){
+			bp += len1;
+			if( bx < bp ){
+				/* should switch to normal input ? */
+				bp = bx;
+			}
+		}
+	  }
+	  *lengp = leng;
+	  *codep = rcode;
+	  return stralloc(buff);
+	}
 	for(;;){
 		setVStrEnd(line,0);
-		if( fgetsTee(AVStr(line),LINESIZE,src,dst,cache) == NULL ){
+		if( fgetsTee(BVStr(line),LINESIZE,src,dst,cache,0) == NULL ){
 			/*DEBUG("GotEOF: length=%d\n",leng);*/
 			rcode = MP_EOF;
 			break;
@@ -479,6 +854,9 @@ static char *readPart(FILE*src,FILE*dst,FILE*cache,const char *boundaries[],PVSt
 		if( rcode = pop_boundary(boundaries,line) )
 			break;
 
+		if( odst ){
+			fputs(line,odst);
+		}
 		if( bfile == NULL && sizeof(buff) <= reqsize ){
 			bfile = TMPFILE("readPart");
 			fputs(buff,bfile);
@@ -505,6 +883,26 @@ static char *readPart(FILE*src,FILE*dst,FILE*cache,const char *boundaries[],PVSt
 	}
 }
 
+void MimeCodeconv(MimeConv *Mcv,PCStr(src),PVStr(dst),int dsz,PCStr(ctype),int repair){
+	const char *icode;
+	const char *ocode;
+	int ccx[32];
+	M17N m17n;
+
+	if( Mcv && (icode = Mcv->c_icode) && (ocode = Mcv->c_ocode) ){
+		if( m17n = Mcv->c_m17n ){
+			m17n_ccx_string(m17n,src,strlen(src),(char*)dst,dsz);
+		}else{
+			if( strcaseeq(icode,"ISO-2022-JP") )
+				icode = "*";
+			CCXcreate((char*)icode,ocode,(CCXP)ccx);
+			CCXexec((CCXP)ccx,src,strlen(src),AVStr(dst),dsz);
+		}
+	}else{
+		codeconv_line(src,BVStr(dst),ctype,repair);
+	}
+}
+
 #define JBSIZE(leng)	(leng*2+128)
 
 static void scan_charset(PCStr(header),PVStr(chset),int len)
@@ -513,16 +911,29 @@ static void scan_charset(PCStr(header),PVStr(chset),int len)
 	if( cp = strcasestr(header,"charset=") )
 		valuescanX(cp+8,AVStr(chset),len);
 }
-static void external_charcode(PCStr(src),PVStr(dst),int dlen,PCStr(ichset))
+static CCXP MIME_CCX = NULL;
+static void external_charcode(MimeConv *Mcv,PCStr(src),PVStr(dst),int dlen,PCStr(ichset))
 {	int ccx[32];
 
+	if( Mcv && Mcv->c_icode && Mcv->c_ocode ){
+		MimeCodeconv(Mcv,src,BVStr(dst),dlen,"text/plain",0);
+		return;
+	}
 	if( strcaseeq(ichset,"ISO-2022-JP") )
 		ichset = "*";
 	CCXcreate((char*)ichset,"ISO-2022-JP",(CCXP)ccx);
 	CCXexec((CCXP)ccx,src,strlen(src),AVStr(dst),dlen);
 }
+static void external_charcodeB(MimeConv *Mcv,PCStr(src),PVStr(dst),int dlen,PCStr(ichset)){
+	if( MIME_CCX ){
+		CCXexec((CCXP)MIME_CCX,src,strlen(src),BVStr(dst),dlen);
+		CCXexec((CCXP)MIME_CCX,"",0,TVStr(dst),dlen-strlen(dst));
+	}else{
+		external_charcode(Mcv,src,BVStr(dst),dlen,ichset);
+	}
+}
 
-static int encodeBODYpart(FILE *fc,FILE *ts,PCStr(ctype),PCStr(ichset),const char *boundaries[],PCStr(encoding),const char **charsetp)
+static int encodeBODYpart(MimeConv *Mcv,FILE *fc,FILE *ts,PCStr(ctype),PCStr(ichset),int filter,const char *boundaries[],PCStr(encoding),const char **charsetp)
 {
 	CStr(endline,LINESIZE);
 	const char *charset = 0;
@@ -531,7 +942,7 @@ static int encodeBODYpart(FILE *fc,FILE *ts,PCStr(ctype),PCStr(ichset),const cha
 	defQStr(tmpb); /*alloc*/
 	defQStr(tmpc); /*alloc*/
 
-	tmpa = readPart(fc,NULL,NULL,boundaries,AVStr(endline),&rcode,&leng);
+	tmpa = readPart(fc,NULL,NULL,filter,boundaries,AVStr(endline),&rcode,&leng);
 	setQStr(tmpb,(char*)malloc(JBSIZE(leng)),JBSIZE(leng));
 	setQStr(tmpc,(char*)malloc(JBSIZE(leng)),JBSIZE(leng));
 
@@ -539,7 +950,7 @@ DEBUG("BODY-LENG1:%d+%d\n",strlen(tmpa),strlen(endline));
 
 	if( !ENCODE(C_BODY_CHAR) ){
 		strcpy(tmpc,tmpa);
-		goto OUT;
+		goto OUTx;
 	}
 
 	/* Decode C-T-E, convert charset, then identify charset.
@@ -552,15 +963,38 @@ DEBUG("BODY-LENG1:%d+%d\n",strlen(tmpa),strlen(endline));
 		strcpy(tmpb,tmpa);
 
 	if( ENCODE(C_BODY_CHAR) )
+		/*
 		external_charcode(tmpb,AVStr(tmpc),JBSIZE(leng),ichset);
+		*/
+		external_charcodeB(Mcv,tmpb,AVStr(tmpc),JBSIZE(leng),ichset);
+		if( Mcv->c_ocode ){
+			charset = Mcv->c_ocode;
+		}
 	else	strcpy(tmpc,tmpb);
+	if( Mcv ){
+		const char *sp;
+		for( sp = tmpc; *sp; sp++ ){
+			if( *sp & 0x80 ){
+				Mcv->c_8bits = 1;
+				break;
+			}
+		}
+	}
 	if( charset == 0 ){
 		if( charset = guess_charset(tmpc) )
 			DEBUG("encodeBDOY[charset=%s]\n",charset);
+		else
+		if( MIME_CCX && CCXactive(MIME_CCX) ){
+			const char *xcs;
+			if( CCXoutcharset(MIME_CCX,&xcs) ){
+				syslog_ERROR("#encodeBODY[charset=%s]\n",xcs);
+				charset = xcs;
+			}
+		}
 	}
 	if( charset == NULL ) /* charset cannot be in "C-T-E: 7bit" */
 		strcpy(tmpc,tmpa);
-OUT:
+OUTx:
 /*
 	if( strcmp(tmpb,tmpc) == 0 )
 		fputs(tmpa,ts);
@@ -572,6 +1006,7 @@ OUT:
 			fputs("\r\n",ts);
 	}
 
+	if( !TEXTONLY(filter) )
 	fputs(endline,ts);
 	free((char*)tmpa);
 	free((char*)tmpb);
@@ -585,7 +1020,7 @@ int relayBODYpart(FILE *src,FILE *dst,const char *boundaries[],int extract,PVStr
 	const char *ep;
 	int rcode,leng;
 
-	tmpa = readPart(src,NULL,NULL,boundaries,AVStr(endline),&rcode,&leng);
+	tmpa = readPart(src,NULL,NULL,0,boundaries,AVStr(endline),&rcode,&leng);
 	if( tmpa[0] && extract ){
 		ep = &tmpa[strlen(tmpa)-1];
 		if( *ep == '\n' ){
@@ -599,7 +1034,7 @@ int relayBODYpart(FILE *src,FILE *dst,const char *boundaries[],int extract,PVStr
 	return rcode;
 }
 
-static int decodeBODYpart(FILE *fs,FILE *tc,FILE *cache,PCStr(ctype),const char *boundaries[],PCStr(encoding),PCStr(decodeto),int do_enHTML,PVStr(endline))
+static int decodeBODYpart(MimeConv *Mcv,FILE *fs,FILE *tc,FILE *cache,PCStr(ctype),const char *boundaries[],int filter,PCStr(encoding),PCStr(decodeto),int do_enHTML,PVStr(endline))
 {	int rcode,leng;
 	defQStr(tmpa); /*alloc*/
 	defQStr(tmpb); /*alloc*/
@@ -614,14 +1049,40 @@ static int decodeBODYpart(FILE *fs,FILE *tc,FILE *cache,PCStr(ctype),const char
 	do_conv = codeconv_get(ctype,&xcharset,&plain2html);
 
 	setVStrEnd(endline,0);
+	if( TEXTONLY(filter) ){
+		out_thru = NULL;
+		syslog_DEBUG("-- decodeBODYpart[%s] [%s]decodeto[%s]\n",
+			ctype,encoding?encoding:"",decodeto?decodeto:"");
+		if( strheadstrX(ctype,"text/",1) ){
+			if( encoding
+			 && !streq(encoding,"7bits")
+			 && !streq(encoding,"8bit") ){
+				if( decodeto == NULL || *decodeto == 0 ){
+					decodeto = "8bit";
+				}
+				filter |= O_BODY_CTE;
+			}else{
+				filter |= O_BODY_GET;
+			}
+		}
+	}else
+	if( MC_MASK_SIGNATURE || MC_MASK_BODY ){
+		out_thru = NULL;
+	}else
 	if( DECODE(C_BODY) == 0 )
 		out_thru = tc;
 	else	out_thru = NULL;
-	tmpa = readPart(fs,out_thru,cache,boundaries,AVStr(endline),&rcode,&leng);
+	tmpa = readPart(fs,out_thru,cache,filter,boundaries,AVStr(endline),&rcode,&leng);
 	if( tmpa == NULL )
 		return -1;
 	if( leng == 0 )
+	{
+		if( out_thru && rcode != MP_EOR ){
+			/* it is put already */
+			truncVStr(endline);
+		}
 		goto EXIT;
+	}
 
 	/* guess the end-of-line character(string) of the message ... */
 	if( beol = strpbrk(tmpa,"\r\n") ){
@@ -645,13 +1106,20 @@ static int decodeBODYpart(FILE *fs,FILE *tc,FILE *cache,PCStr(ctype),const char
 	setQStr(tmpa,(char*)realloc((char*)tmpa,tsiz),tsiz);
 	setQStr(tmpb,(char*)malloc(tsiz),tsiz);
 
+	/*
 	if( DECODE(C_BODY_CTE) && encoding != NULL && decodeto != NULL ){
+	*/
+	if( DECODE(C_BODY_CTE) || TEXTONLY(filter) )
+	if( encoding != NULL && decodeto != NULL ){
 		DEBUG("decode: %s -> %s\n",encoding,decodeto);
 		if( decodeLine(tmpa,AVStr(tmpb),encoding) )
 			strcpy(tmpa,tmpb);
 	}
 	if( DECODE(C_BODY_CHAR) && do_conv ){
+		/*
 		codeconv_line(tmpa,AVStr(tmpb),ctype,0);
+		*/
+		MimeCodeconv(Mcv,tmpa,AVStr(tmpb),tsiz,ctype,0);
 		strcpy(tmpa,tmpb);
 	}
 	if( do_enHTML ){
@@ -660,6 +1128,7 @@ static int decodeBODYpart(FILE *fs,FILE *tc,FILE *cache,PCStr(ctype),const char
 	}
 	free((char*)tmpb);
 EXIT:
+	maskBody(AVStr(tmpa));
 	fputs(tmpa,tc);
 
 PUTEOL:
@@ -693,10 +1162,12 @@ int decodeBODY(FILE *fs,FILE *tc,int filter,PCStr(encoding),PCStr(decodeto),int
 	const char *ctype = "text/plain";
 	CStr(endline,LINESIZE);
 	int rcode;
+	MimeConv Mcvb,*Mcv = &Mcvb;
 
 	boundaries[0] = 0;
-	rcode = decodeBODYpart(fs,tc,NULL,ctype,
-			boundaries,encoding,decodeto,do_enHTML,AVStr(endline));
+	bzero(Mcv,sizeof(MimeConv));
+	rcode = decodeBODYpart(Mcv,fs,tc,NULL,ctype,boundaries,
+			filter,encoding,decodeto,do_enHTML,AVStr(endline));
 	if( EOR_ALSO(filter) || endline[0] != '.' )
 		fputs(endline,tc);
 	return rcode;
@@ -730,12 +1201,38 @@ int decodeMIMEpart(const char *boundaries[],FILE*fs,FILE*tc,FILE*cache,int filte
 	const char *xcharset;
 	FILE *ntc;
 	int ismainpart;
+	MimeConv Mcvb,*Mcv = &Mcvb;
+
+	filter |= MIME_FILTER;
 
 	ismainpart = boundaries[0] == 0;
 	do_conv = codeconv_get(NULL,&xcharset,&plain2html);
-	readHeader(fs,FVStr(head[0]),sizeof(head[0]),
+	bzero(Mcv,sizeof(MimeConv));
+	readHeader(Mcv,fs,FVStr(head[0]),sizeof(head[0]),
 		cache,filter,AVStr(ctype),AVStr(ichset),boundaries,AVStr(encoding),enHTML);
 
+	if( DECODE(C_HEAD_EW) )
+	if( ichset[0] == 0 && do_conv && xcharset ){
+		MIME_strHeaderDecodeX(Mcv,head[0],FVStr(head[1]),sizeof(head[1]));
+		if( Mcv->c_ewicode[0] ){
+			/* conv. non-ASCII in the raw code */
+			strcpy(ichset,Mcv->c_ewicode);
+			Mcv->c_ewiccxeach = 1;
+			if( strcaseeq(ichset,"ISO-2022-JP") ){
+				/* to escape the bug in m17n-lib <= 1.5.3 */
+				strcpy(ichset,"EUC-JP");
+			}
+		}
+	}
+	if( DECODE(C_HEAD_EW) )
+	if( ichset[0] != 0 && do_conv && xcharset ){
+		Mcv->c_icode = ichset;
+		Mcv->c_ocode = xcharset;
+		if( m17n_known_code(ichset) ){
+			Mcv->c_m17n = m17n_ccx_new(ichset,xcharset,0,0);
+		}
+	}
+
 	if( got_EOR )
 		rcode = MP_EOR;
 	else	rcode = 0;
@@ -756,9 +1253,15 @@ int decodeMIMEpart(const char *boundaries[],FILE*fs,FILE*tc,FILE*cache,int filte
 
 	hi = 0;
 	if( DECODE(C_HEAD_EW) )
+	if( do_conv && Mcv->c_ewiccxeach ){
+		nhi = (hi + 1 ) % 2;
+		MimeCodeconv(Mcv,head[hi],FVStr(head[nhi]),sizeof(head[nhi]),C_TEXT_PLAIN,1);
+		hi = nhi;
+	}
+	if( DECODE(C_HEAD_EW) )
 	if( maybe_MIME ){
 		nhi = (hi + 1 ) % 2;
-		MIME_strHeaderDecode(head[hi],FVStr(head[nhi]),sizeof(head[1]));
+		MIME_strHeaderDecodeX(Mcv,head[hi],FVStr(head[nhi]),sizeof(head[1]));
 		hi = nhi;
 		maybe_NONASCII = 1;
 	}
@@ -768,9 +1271,15 @@ int decodeMIMEpart(const char *boundaries[],FILE*fs,FILE*tc,FILE*cache,int filte
 	/*
 	if( maybe_NONASCII && do_conv ){
 	*/
+	if( Mcv->c_ewiccxeach ){
+		/* conv. done already above and in strHeaderDecodeX() */
+	}else
 	if( maybe_NONASCII && do_conv || (do_conv & 0xFFFE) ){
 		nhi = (hi + 1 ) % 2;
+		MimeCodeconv(Mcv,head[hi],FVStr(head[nhi]),sizeof(head[nhi]),C_TEXT_PLAIN,1);
+		/*
 		codeconv_line(head[hi],FVStr(head[nhi]),C_TEXT_PLAIN,1);
+		*/
 		hi = nhi;
 	}
 
@@ -871,19 +1380,34 @@ int decodeMIMEpart(const char *boundaries[],FILE*fs,FILE*tc,FILE*cache,int filte
 	 && MULTIPART(filter)
 	 && strncasecmp(ctype,"text/plain",10) != 0 ){
 		DEBUG("## TEXTONLY skip non text/plain: %s\n",ctype);
-		rcode = decodeBODYpart(fs,NULLFP(),cache,ctype,boundaries,
+		rcode = decodeBODYpart(Mcv,fs,NULLFP(),cache,ctype,boundaries,
+				filter,
 				encoding,decodeto,do_enHTML,AVStr(endline));
 	}else
 	if( BODY_ALSO(filter) ){
 		if( putPRE ) fprintf(tc,"<PRE>\n");
+/*
+		if( TEXTONLY(filter) && strncaseeq(ctype,"text/html",9) ){
+			int HTMLtoTEXT(FILE *ain,FILE *aout,int toHtml);
+			FILE *tmp;
 
+			tmp = TMPFILE("HTMLtoTEXT");
+			rcode = decodeBODYpart(Mcv,fs,tmp,cache,ctype,boundaries,
+					filter,
+					encoding,decodeto,do_enHTML,AVStr(endline));
+			fflush(tmp);
+			fseek(tmp,0,0);
+			HTMLtoTEXT(tmp,tc,0);
+			fclose(tmp);
+		}else
+*/
 		if( MIME2PGP(filter) )
 			if( substr(ctype,C_PGPSIGN) )
 				rcode = relay_pgpSIGN(fs,tc,boundaries,AVStr(endline));
 			else	rcode = relay_pgpSIGNED(fs,tc,boundaries,AVStr(endline));
 		else
-		rcode = decodeBODYpart(fs,tc,cache,ctype,boundaries,
-				encoding,decodeto,do_enHTML,AVStr(endline));
+		rcode = decodeBODYpart(Mcv,fs,tc,cache,ctype,boundaries,
+			filter,encoding,decodeto,do_enHTML,AVStr(endline));
 		if( putPRE ) fprintf(tc,"</PRE>\n");
 	}else{
 		if( fseek(fs,0,0) == -1 ){
@@ -978,47 +1502,109 @@ int encodeMIMEpart(const char *boundaries[],FILE*fc,FILE*ts,FILE*cache,int filte
 	CStr(encoding,128);
 	int rcode;
 	CStr(endline,LINESIZE);
+	int isroot;
+	MimeConv Mcvb,*Mcv = &Mcvb;
 
-	readHeader(fc,FVStr(head[0]),sizeof(head[0]),
+	isroot = boundaries[0] == 0;
+	bzero(Mcv,sizeof(MimeConv));
+	readHeader(Mcv,fc,FVStr(head[0]),sizeof(head[0]),
 		cache,filter,AVStr(ctype),AVStr(ichset),boundaries,AVStr(encoding),0);
 	if( got_EOR )
 		rcode = MP_EOR;
 	else	rcode = 0;
 
+	if( ichset[0] ){
+		int do_conv,plain2html;
+		int iknown = 0;
+		const char *xchset = 0;
+
+		iknown = m17n_known_code(ichset);
+		if( MIME_CCX ){
+			/* CHARCODE=code:tosv with SERVER=smtp */
+			do_conv = 1;
+			CCXoutcharset(MIME_CCX,&xchset);
+			syslog_ERROR("#M17N%s#MIME_CCX (%s <= %s)\n",
+				iknown?"+":"-",xchset?xchset:"",ichset);
+		}else{
+			/* CHARCODE=code with -FenMime */
+			do_conv = codeconv_get(NULL,&xchset,&plain2html);
+			syslog_ERROR("#M17N%s#CHARCODE (%s <= %s)\n",
+				iknown?"+":"-",xchset?xchset:"",ichset);
+		}
+		if( do_conv && xchset ){
+			Mcv->c_icode = ichset;
+			Mcv->c_ocode = xchset;
+			if( iknown && m17n_known_code(xchset) ){
+				Mcv->c_m17n = m17n_ccx_new(ichset,xchset,0,0);
+			}else{
+			}
+		}
+	}
+
 	hi = 0;
 	if( maybe_NONASCII ){
 		if( ENCODE(C_HEAD_CHAR) )
-			external_charcode(head[0],FVStr(head[1]),0x10000,ichset);
+		{
+			external_charcode(Mcv,head[0],FVStr(head[1]),0x10000,ichset);
+			if( Mcv->c_ocode && strcmp(head[0],head[1]) != 0 ){
+				/* set the charset name to be shown
+				 * in =?charset?e?x?= in strHeaderEncode()
+				 */
+				Mcv->c_icode = Mcv->c_ocode;
+			}
+		}
 		else	Xstrcpy(EVStr(head[1]),head[0]);
 		if( strcmp(head[0],head[1]) != 0 ){
 			DEBUG("POST: code converted.\n");
 		}
 		if( ENCODE(C_HEAD_EW) )
+		{
+			MIME_strHeaderEncodeX(Mcv,head[1],FVStr(head[0]),sizeof(head[0]));
+			/*
 			MIME_strHeaderEncode(head[1],FVStr(head[0]),sizeof(head[0]));
+			*/
+		}
+		else{
+			hi = 1;
+		}
 	}
 
 	if( strncasecmp(ctype,"multipart/",10) == 0 ){
+		if( TEXTONLY(filter) && isroot ){
+			replaceContentType(FVStr(head[hi]),"text/plain");
+		}
+		if( !TEXTONLY(filter) || isroot )
 		fputs(head[hi],ts);
 		rcode = scan_multipart(boundaries,AVStr(endline),sizeof(endline),
 				encodeMIMEpart,fc,ts,cache,filter,0);
+
+		if( !TEXTONLY(filter) )
 		fputs(endline,ts);
 		return rcode;
 	}
 
 	if( rcode == MP_EOR ){
 		fputs(head[hi],ts);
+		return rcode;
+	}else
+	if( TEXTONLY(filter) && !is_text(ctype) ){
+		rcode = relayBODYpart(fc,NULLFP(),boundaries,0,AVStr(endline));
 	}else
 	if( is_text(ctype) || tobe_charconv(ctype) ){
 		tmp = TMPFILE("encodeMIMEpart");
 		charset = 0;
-		rcode = encodeBODYpart(fc,tmp,ctype,ichset,boundaries,encoding,&charset);
+		rcode = encodeBODYpart(Mcv,fc,tmp,ctype,ichset,filter,boundaries,encoding,&charset);
 		if( charset ){
 			replace_charset(FVStr(head[hi]),charset);
 			if( findField(head[hi],FN_CTE,NULL) )
+				if( Mcv->c_8bits ){
+			replaceFieldValue(FVStr(head[hi]),FN_CTE,"8bit");
+				}else
 			replaceFieldValue(FVStr(head[hi]),FN_CTE,"7bit");
 		}
 		fflush(tmp);
 		fseek(tmp,0,0);
+		if( !TEXTONLY(filter) )
 		fputs(head[hi],ts);
 		copyfile1(tmp,ts);
 		fclose(tmp);
@@ -1135,10 +1721,25 @@ void decodeTERM(FILE *fs,FILE *tc,int enHTML)
 }
 
 void encodeMIME(FILE *fc,FILE *ts)
+{
+	encodeMIMEX(fc,ts,O_ALL);
+}
+void encodeMIMEX(FILE *fc,FILE *ts,int filter)
 {	const char *boundaries[MAXDEPTH]; /**/
 
 	boundaries[0] = 0;
-	encodeMIMEpart(boundaries,fc,ts,NULL,O_ALL,0);
+	encodeMIMEpart(boundaries,fc,ts,NULL,filter,0);
+}
+int encodeMIMEXX(MimeEnv *me,FILE *fc,FILE *ts){
+	const char *boundaries[MAXDEPTH]; /**/
+	CCXP sav_ccx;
+
+	sav_ccx = MIME_CCX;
+	MIME_CCX = me->me_ccx;
+	boundaries[0] = 0;
+	encodeMIMEpart(boundaries,fc,ts,NULL,me->me_filter,0);
+	MIME_CCX = sav_ccx;
+	return 0;
 }
 
 void decodeMIME(FILE*fs,FILE*tc,FILE*cache, int filter,int codeconv,int enHTML)
@@ -1157,5 +1758,5 @@ void deMime(int ac,const char *av[])
 }
 void enMime(int ac,const char *av[])
 {
-	encodeMIME(stdin,stdout);
+	encodeMIMEX(stdin,stdout,O_ALL);
 }
