@@ -1,10 +1,16 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1998-1999 Electrotechnical Laboratry (ETL), AIST, MITI
-Copyright (c) 1998-1999 Yutaka Sato
+Copyright (c) 1998-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, and distribute this material for any purpose
-and without fee is hereby granted, provided that the above copyright
-notice and this permission notice appear in all copies.
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
+OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
 Content-Type:	program/C; charset=US-ASCII
 Program:	mimecodes.c
@@ -13,8 +19,7 @@ Description:
 History:
 	980910	created
 //////////////////////////////////////////////////////////////////////#*/
-#include <ctype.h>
-#include <stdio.h>
+#include "mime.h"
 
 #define CR	'\r'
 #define LF	'\n'
@@ -28,11 +33,12 @@ static char B64CH[] =
 #define NON64	((char)-1)
 #define WHITE	((char)-2)
 #define PADDCH	((char)-3)
-static char B64IX[0x100];
+static char *B64IX;
 
-static init_64()
+static void init_64()
 {	int ix;
 
+	B64IX = (char*)StructAlloc(0x100);
 	for( ix = 0; ix < 0x100; ix++ )
 		B64IX[ix] = NON64;
 	B64IX[SP] = B64IX[TAB] = B64IX[CR] = B64IX[LF] = WHITE;
@@ -42,19 +48,19 @@ static init_64()
 }
 
 #define QP_THRU	 1
-#define QP_ESC	-1
-static char QPIX[0x100];
-static init_QP()
+#define QP_ESC	(char)-1
+static char *QPIX;
+static void init_QP()
 {	int ix;
 
+	QPIX = (char*)StructAlloc(0x100);
 	for( ix = 0; ix < 0x100; ix++ ){
 		if( 0x80 <= ix || iscntrl(ix) && !isspace(ix) || ix == EQ )
 			QPIX[ix] = QP_ESC;
 		else	QPIX[ix] = QP_THRU;
 	}
 }
-MIME_setQP(qpchars)
-	char *qpchars;
+void MIME_setQP(PCStr(qpchars))
 {	int ch;
 
 	init_QP();
@@ -62,13 +68,12 @@ MIME_setQP(qpchars)
 		QPIX[ch] = QP_ESC;
 }
 
-MIME_to64(in,out)
-	FILE *in,*out;
+void MIME_to64X(FILE *in, FILE *out, int leng);
+void MIME_to64(FILE *in, FILE *out)
 {
 	MIME_to64X(in,out,0);
 }
-MIME_to64X(in,out,leng)
-	FILE *in,*out;
+void MIME_to64X(FILE *in, FILE *out, int leng)
 {	int gotEOF,ich,ocol,ix,oint,shifts,o1,ox,icc;
 
 	gotEOF = 0;
@@ -110,11 +115,107 @@ MIME_to64X(in,out,leng)
 		putc(LF,out);
 	fflush(out);
 }
-MIME_from64(in,out)
-	FILE *in,*out;
+
+#ifndef MIME_to64
+int strtoB64(PCStr(str),int slen,PVStr(b64),int bsiz,int withnl){
+	refQStr(bp,b64);
+	int gotEOF = 0;
+	int ich,ocol,ix,oint,shifts,o1,ox,icc;
+
+	ocol = 0;
+	for( icc = 0; !gotEOF && icc < slen; ){
+		oint = 0;
+		shifts = 16;
+		for( ix = 0; ix < 3; ix++ ){
+			if( slen <= icc ){
+				gotEOF = 1;
+				break;
+			}
+			ich = 0xFF & str[icc++];
+			oint |= (ich << shifts);
+			shifts -= 8;
+		}
+		if( ix == 0 )
+			break;
+		if( 71 < ocol ){
+			if( withnl )
+				setVStrPtrInc(bp,LF);
+			ocol = 0;
+		}
+		shifts = 18;
+		for( ox = 0; ox <= ix; ox++ ){
+			o1 = (oint >> shifts) & 0x3F;
+			shifts -= 6;
+			setVStrPtrInc(bp,B64CH[o1]);
+		}
+		for(; ox < 4; ox++ )
+			setVStrPtrInc(bp,EQ);
+		ocol += 4;
+	}
+	if( withnl && 0 < ocol )
+		setVStrPtrInc(bp,LF);
+	setVStrEnd(bp,0);
+	return bp - b64;
+}
+int B64tostr(PCStr(b64),int blen,PVStr(str),int slen){
+	refQStr(sp,str);
+	int icc;
+	int gotEOF,ich,ix,ii,i1,ox,oint,o1,shifts;
+
+	if( B64IX == 0 )
+		init_64();
+
+	gotEOF = 0;
+	for( icc = 0; !gotEOF && icc < blen; ){
+		ix = 0;
+		oint = 0;
+		shifts = 18;
+		for( ii = 0; ii < 4; ){
+			if( blen <= icc ){
+				if( ix != 0 )
+					syslog_ERROR("BASE64 premature EOF\n");
+				gotEOF = 1;
+				break;
+			}
+			ich = b64[icc++];
+			i1 = B64IX[ich];
+			if( i1 == NON64 ){
+				syslog_ERROR("BASE64 unrecognized CHAR(0x%x)\n",
+					ich);
+				gotEOF = 1;
+				break;
+			}
+			if( i1 == WHITE )
+				continue;
+
+			if( i1 == PADDCH ){
+				ii++;
+				continue;
+			}
+			oint |= (i1 << shifts);
+			shifts -= 6;
+			ix++;
+			ii++;
+		}
+		if( ix == 0 )
+			break;
+
+		shifts = 16;
+		for( ox = 1; ox < ix; ox++ ){
+			o1 = (oint >> shifts) & 0xFF;
+			shifts -= 8;
+			setVStrPtrInc(sp,o1);
+		}
+	}
+	setVStrEnd(sp,0);
+	return sp - str;
+}
+#endif
+
+void MIME_from64(FILE *in, FILE *out)
 {	int gotEOF,ich,ix,ii,i1,ox,oint,o1,shifts;
 
-	if( B64IX[0] == 0 )
+	if( B64IX == 0 )
 		init_64();
 
 	gotEOF = 0;
@@ -166,11 +267,10 @@ MIME_from64(in,out)
 #define GETC1(ch,in,exit)	{ch = getc(in); if( ch == EOF ) goto exit;}
 static char hexd[] = "0123456789ABCDEF";
 
-MIME_toQP(in,out)
-	FILE *in,*out;
+void MIME_toQP(FILE *in, FILE *out)
 {	int ich,ocol;
 
-	if( QPIX[0] == 0 )
+	if( QPIX == 0 )
 		init_QP();
 
 	ocol = 0;
@@ -205,10 +305,9 @@ EXIT:
 	return;
 }
 
-MIME_fromQP(in,out)
-	FILE *in,*out;
+void MIME_fromQP(FILE *in, FILE *out)
 {	int ch1,ch2,softnl,och;
-	char buf[3];
+	CStr(buf,3);
 
 	softnl = 0;
 	buf[2] = 0;
