@@ -1,11 +1,14 @@
 /*///////////////////////////////////////////////////////////////////////
-Copyright (c) 1992-1999 Electrotechnical Laboratry (ETL), AIST, MITI
-Copyright (c) 1992-1999 Yutaka Sato
+Copyright (c) 1992-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, and distribute this material for any purpose
-and without fee is hereby granted, provided that the above copyright
-notice and this permission notice appear in all copies.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -128,6 +131,9 @@ typedef struct {
 #define B_JP_KANJI1	3
 #define B_JP_KANJI2	4
 #define B_UCS_2		5
+#define B_EUC_JP	6
+#define B_SHIFT_JIS	7
+#define B_M17N		8
 
 static Charset BasicCharsets[] = {
 	{"",       "UNKNOWN"	},
@@ -160,6 +166,9 @@ char M_US_ASCII[]	= "US-ASCII";
 char M_ISO_8859_8[]	= "ISO-8859-8";
 char M_ISO_2022_JP[]	= "ISO-2022-JP";
 char M_UTF_8[]		= "UTF-8";
+char M_EUC_JP[]		= "EUC-JP";
+char M_SHIFT_JIS[]	= "Shift_JIS";
+char M_M17N[]           = "M17N";
 
 typedef struct {
 	int	local;
@@ -178,6 +187,8 @@ static MimeCharset Codes2[16] = {
 	{1, '@', M_ISO_2022_JP,	B_JP_KANJI1, ENCODE_BASE64	},
 	{1, 'B', M_ISO_2022_JP,	B_JP_KANJI2, ENCODE_BASE64	},
 	{1, 0,   M_UTF_8,       B_UCS_2,     ENCODE_BASE64	},
+	{1, 0,   M_EUC_JP,      B_EUC_JP,    ENCODE_BASE64	},
+	{1, 0,   M_SHIFT_JIS,   B_SHIFT_JIS, ENCODE_BASE64	},
 	0
 };
 
@@ -202,15 +213,15 @@ typedef struct {
 	int	c_ch;		/* character code value */
   const char*	c_mcharset;	/* MIME charset */
 	int	c_bcharset;	/* basic charset */
-} CHAR;
-static CHAR NULL_CHAR = { 0,  M_US_ASCII, B_US_ASCII };
+} CHARx;
+static CHARx NULL_CHAR = { 0,  M_US_ASCII, B_US_ASCII };
 
 /*
  *	ROUND ROBBIN BUFFER
  */
 #define RRBUFF_SIZE 8
 typedef struct {
-	CHAR	b_BUFF[RRBUFF_SIZE];
+	CHARx	b_BUFF[RRBUFF_SIZE];
 	int	b_putx;
 	int	b_getx;
 } RRBUFF;
@@ -219,6 +230,10 @@ static RRBUFF NULL_RRBUFF = {0};
 typedef struct {
 	FILE	*in_file;
 	int	 in_column;
+	MStr(	 in_ewcharset,64);	/* =?charset?e?x?= */
+	int	 in_ewicodemix;		/* mixed charset enc-word */
+	int	 in_ewiccxeach;		/* decode and CCX each enc-word */
+  const char	*in_charcode8B;		/* default charset in 8bits */
   const char	*in_mcharset;		/* current MIME charset */
 	int	 in_bcharset;		/* current basic charset */
 	int	 in_bcharset_got;	/* bcharset was got explicitly */
@@ -229,11 +244,13 @@ typedef struct {
 
 	FILE	*out_file;
 	int	 out_column;
+	int	 out_noccx;		/* disable CCX */
+   const char	*out_charcode;		/* output charset */
 	int	 out_lastputch;		/* EN_FPUTC() -> encode_word() */
 	int	 out_whichASCII;	/* disp_word() -> DE_FPUTC(),C1() */
 	int	 out_enLWSP;		/* EN_FPUTC() local */
-	CHAR	 out_prevCHAR;		/* EN_FPUTC1() local */
-	CHAR	 out_deLWSP[4];		/* DE_FPUTCX() local */
+	CHARx	 out_prevCHAR;		/* EN_FPUTC1() local */
+	CHARx	 out_deLWSP[4];		/* DE_FPUTCX() local */
 	int	 out_prev_bcharset;	/* DE_FPUTC1X() local */
 
 	union { int all; struct { unsigned int
@@ -253,6 +270,10 @@ static void INOUT_init(INOUT *io,FILE *in,FILE *out)
 {
 	io->in_file           = in;
 	io->in_column         = 0;
+	clearVStr(io->in_ewcharset);
+	io->in_ewicodemix     = 0;
+	io->in_ewiccxeach     = 0;
+	io->in_charcode8B     = 0;
 	io->in_mcharset       = M_US_ASCII;
 	io->in_bcharset       = B_US_ASCII;
 	io->in_bcharset_got   = 0;
@@ -263,6 +284,8 @@ static void INOUT_init(INOUT *io,FILE *in,FILE *out)
 
 	io->out_file          = out;
 	io->out_column        = 0;
+	io->out_noccx         = 0;
+	io->out_charcode      = 0;
 	io->out_lastputch     = 0;
 	io->out_whichASCII    = B_US_ASCII;
 	io->out_enLWSP        = 0;
@@ -331,18 +354,18 @@ static void NLfputc(int ch,FILE *out)
 	fputc(ch,out);
 }
 
-static void EN_UNGETC(CHAR *CH,INOUT *io)
+static void EN_UNGETC(CHARx *CH,INOUT *io)
 {
 	PUT_RRBUFF(&io->in_PUSHED,CH);
 }
 
-static CHAR *EN_FGETC(INOUT *io)
+static CHARx *EN_FGETC(INOUT *io)
 {	int ch;
 	MimeCharset *csw;
 	int ci;
 	const char *mcharset;
 	FILE *infile = io->in_file;
-	CHAR *CH;
+	CHARx *CH;
 	RRBUFF *BP;
 
 	BP = &io->in_BUFF;
@@ -359,7 +382,24 @@ GET1:
 	ch = NLfgetc(infile);
 GOT1:
 	if( ch != ESC )
+	{
+		if( ch & 0x80 ){
+			if( io->in_mcharset != M_UTF_8 )
+			if( io->in_charcode8B == M_UTF_8 )
+			{
+				io->in_mcharset = M_UTF_8;
+				io->in_bcharset = B_UCS_2;
+				io->in_encoding = ENCODE_BASE64;
+				goto exit;
+			}
+			if( io->in_charcode8B ){
+				io->in_mcharset = io->in_charcode8B;
+				io->in_bcharset = B_M17N;
+				io->in_encoding = ENCODE_BASE64;
+			}
+		}
 		goto exit;
+	}
 
 	/* got ESC character */
 	if( (ch = fgetc(infile)) == EOF )
@@ -524,7 +564,7 @@ static void EN_FPUTC(int ch,INOUT *io,PCStr(charset),PCStr(encoding))
  *	PASS THROUGH AN ASCII WORD
  */
 static int noencode_word(INOUT *io)
-{	CHAR *CH,*NCH;
+{	CHARx *CH,*NCH;
 	int ch,inx;
 	int canbe_folded;
 	MsgLine line;
@@ -607,7 +647,7 @@ static int encode_word(INOUT *io)
 	int prech = 0;
 	int postch = 0;
 	int delop = 0;
-	CHAR *CH;
+	CHARx *CH;
 
 	charset = io->in_mcharset;
 	char_bytes = 2;
@@ -647,6 +687,14 @@ static int encode_word(INOUT *io)
 		CH = EN_FGETC(io);
 		ch = CH->c_ch;
 		if( ch == EOF || ch == NL || CH->c_mcharset != charset ){
+			if( nchar == 0 && CH->c_mcharset != charset ){
+				if( CH->c_mcharset != M_US_ASCII ){
+					/* this can cause infinite loop */
+					/* so discard the character */
+					setVStrEnd(ins,inx);
+					break;
+				}
+			}
 			if( io->in_mcharset == M_US_ASCII )
 				Xstrcpy(QVStr(&ins[inx],Ins),in_CODESW_SEQ(io));
 				/* ASCII family like JIS-X0201-Roman */
@@ -655,7 +703,7 @@ static int encode_word(INOUT *io)
 			/* ENCODE A LWSP BETWEEN ENCODED-WORDS */
 			/*if( 4 <= (remlen-outlen) )*/
 			if( LWSP_CHAR(ch) ){
-				CHAR *NCH;
+				CHARx *NCH;
 				CStr(lwspb,4);
 				int lwspx,cx;
 
@@ -752,12 +800,30 @@ break;
  *  shorter than MAXCOL, but will be splitted in current line.  */
 
 
+int MIME_headerEncode0X(MimeConv *Mcv,FILE *in,FILE *out);
 int MIME_headerEncode0(FILE *in,FILE *out)
+{
+	return MIME_headerEncode0X(0,in,out);
+}
+int MIME_headerEncode0X(MimeConv *Mcv,FILE *in,FILE *out)
 {	INOUT iob,*io = &iob;
-	CHAR *CH;
+	CHARx *CH;
 	int ch;
 
 	INOUT_init(io,in,out);
+	if( Mcv ){
+		io->out_noccx = Mcv->c_noccx;
+		if( Mcv->c_ocode ){
+			io->out_charcode = Mcv->c_ocode;
+		}
+		if( Mcv->c_icode ){
+			if( strcaseeq(Mcv->c_icode,M_UTF_8) ){
+				io->in_charcode8B = M_UTF_8;
+			}else{
+				io->in_charcode8B = Mcv->c_icode;
+			}
+		}
+	}
 	for(;;){
 		CH = EN_FGETC(io);
 		EN_UNGETC(CH,io);
@@ -785,23 +851,32 @@ int MIME_headerEncode0(FILE *in,FILE *out)
 		EN_FPUTC(ch,io,M_US_ASCII,ENCODE_NONE);
 	return ch;
 }
+int MIME_headerEncodeX(MimeConv *Mcv,FILE *in,FILE *out);
 void MIME_headerEncode(FILE *in,FILE *out)
+{
+	MIME_headerEncodeX(0,in,out);
+}
+int MIME_headerEncodeX(MimeConv *Mcv,FILE *in,FILE *out)
 {	int ch;
 	CStr(line,MAX_LNSIZE);
 
+	/*
 	ch = MIME_headerEncode0(in,out);
+	*/
+	ch = MIME_headerEncode0X(Mcv,in,out);
 	if( ch != EOF ){
 		NLfputc(NL,out);
 		while( fgets(line,sizeof(line),in) != NULL )
 			fputs(line,out);
 	}
+	return 0;
 }
 
 
 /*
  *	FINAL OUTPUT WITH ISO-2022 CHARACTER SET SWITCH SEQUENCE
  */
-static void DE_FPUTC1X(CHAR *CH,INOUT *Out)
+static void DE_FPUTC1X(CHARx *CH,INOUT *Out)
 {	FILE *out;
 	int cset;
 	int ch;
@@ -831,14 +906,14 @@ static void DE_FPUTC1X(CHAR *CH,INOUT *Out)
  *	PUT ASCII (or CONTROL) CHARACTER IN A CURRENT ASCII FAMILY
  */
 static void DE_FPUTC1(int ch,INOUT *io)
-{	CHAR CH;
+{	CHARx CH;
 
 	CH.c_bcharset = io->out_whichASCII;
 	CH.c_ch = ch;
 	DE_FPUTC1X(&CH,io);
 }
 
-static void dumpDECODER(CHAR *CH,INOUT *io)
+static void dumpDECODER(CHARx *CH,INOUT *io)
 {	int i,ch;
 	CStr(sym,16);
 
@@ -877,9 +952,9 @@ static void dumpDECODER(CHAR *CH,INOUT *io)
  */
 #define CLEAR_LWSP(io)	(io->out_deLWSP[0].c_ch = 0)
 
-static void DE_FPUTCX(CHAR *CH,INOUT *io)
+static void DE_FPUTCX(CHARx *CH,INOUT *io)
 {	int ch;
-	CHAR PCH;
+	CHARx PCH;
 
 	if( io == 0 )
 		return;
@@ -991,7 +1066,7 @@ static void DE_FPUTCX(CHAR *CH,INOUT *io)
  *	PUT ASCII (or CONTROL) CHARACTER IN A CURRENT ASCII FAMILY
  */
 static void DE_FPUTC(int ch,INOUT *io)
-{	CHAR CH;
+{	CHARx CH;
 
 	CH.c_bcharset = io->out_whichASCII;
 	CH.c_ch = ch;
@@ -1028,7 +1103,10 @@ int scan_eword(FILE *in,xPVStr(reads),PVStr(charset),PVStr(encoding),PVStr(text)
 		setVStrEnd(encoding,i+1);
 	}
 	for(i = 0;;i++){
+		/*
 		if( 80 <= i ){
+		*/
+		if( 120 <= i ){
 			cs = XC_DE_TOOLONG;
 			goto error;
 		}
@@ -1060,7 +1138,7 @@ static int disp_word(INOUT *Out,PCStr(dtext),int len)
 	INOUT tmpInb,*tmpIn = &tmpInb;
 	int sdlen,dlen;
 	int eat_space = 0;
-	CHAR *CH;
+	CHARx *CH;
 
 	if( len <= 0 )
 		return 0;
@@ -1114,12 +1192,14 @@ static int disp_word(INOUT *Out,PCStr(dtext),int len)
 	return dlen;
 }
 
+CCXP MIMEHEAD_CCX;
 int bad_etext(PCStr(charset),PCStr(dtext),int len);
 static int decode_word(INOUT *io)
 {	MsgLine reads,charset,encoding,itext;
 	CStr(dtext,MAX_LNSIZE);
 	int ilen,dsize,len,pad,dlen;
 	int eow;
+	int m17n_known = 0;
 
 	*charset = *encoding = *itext = 0;
 
@@ -1133,6 +1213,10 @@ static int decode_word(INOUT *io)
 		return eow;
 	}
 
+	if( m17n_known_code((const char*)charset) ){
+		m17n_known = 1;
+		goto DECODE;
+	}
 	if( !MIME_localCharset((char*)charset) ){
 BAD_EWORD:
 		DE_FPUTC(XC_DE_OFLUSH,io);
@@ -1142,6 +1226,7 @@ BAD_EWORD:
 		return 0;
 	}
 
+DECODE:
 	ilen = strlen((char*)itext);
 	dsize = sizeof(dtext);
 	if( strcasecmp((char*)encoding,ENCODE_QP) == 0 ){
@@ -1157,6 +1242,52 @@ BAD_EWORD:
 		strcpy(dtext,(char*)itext);
 		len = ilen;
 	}
+
+	if( io->in_ewcharset[0] && !strcaseeq(io->in_ewcharset,(char*)charset) ){
+		io->in_ewicodemix++;
+	}
+	strcpy(io->in_ewcharset,(char*)charset);
+	if( io->out_noccx ){
+	}else
+	if( io->in_ewiccxeach || MIMEHEAD_CCX ){
+	    const char *ocset = "ISO-2022-JP";
+	    const char *oc;
+	    M17N m17n = 0;
+
+	    if( MIMEHEAD_CCX
+	     && !CCXguessing(MIMEHEAD_CCX)&&CCXoutcharset(MIMEHEAD_CCX,&oc)){
+		ocset = oc;
+	    }else
+	    if( io->out_charcode ){
+		ocset = io->out_charcode;
+	    }
+	    if( MIMEHEAD_CCX == 0 && m17n_known ){
+		m17n = m17n_ccx_new((const char*)charset,ocset,0,0);
+	    }
+	    if( m17n ){
+		CStr(mtext,MAX_LNSIZE);
+		m17n_ccx_string(m17n,(const char*)dtext,len,(char*)mtext,sizeof(mtext));
+		strcpy(dtext,mtext);
+		len = strlen(dtext);
+	    }else
+	    if( strcaseeq((char*)charset,"Shift_JIS")
+	     || strcaseeq((char*)charset,"EUC-JP")
+	     || strcaseeq((char*)charset,"ISO-2022-JP")
+	     || strcaseeq((char*)charset,"UTF-8")
+	    )
+	    /* it should be coverted to the charset of message body */
+	    {
+		int ccx[16];
+		/* the body encoding might be different from the head encoding.
+		 * but the encoding is not known here, so use ISO-2022-JP
+		 * because it can be uniquely identified from others.
+		 */
+		CCXcreate((char*)charset,ocset,(CCXP)ccx);
+		len = CCXexec((CCXP)ccx,dtext,len,AVStr(dtext),sizeof(dtext));
+		len += CCXexec((CCXP)ccx,"",0,TVStr(dtext),sizeof(dtext)-len);
+	    }
+	}
+
 	disp_word(io,dtext,len);
 	return 0;
 }
@@ -1236,12 +1367,23 @@ static int DE_FGETC(INOUT *io)
 	io->in_prevch = ch;
 	return ch;
 }
+int MIME_headerDecodeX(MimeConv *Mcv,FILE *in,FILE *out,int bodytoo);
 void MIME_headerDecode(FILE *in,FILE *out,int bodytoo)
+{
+	MIME_headerDecodeX(0,in,out,bodytoo);
+}
+int MIME_headerDecodeX(MimeConv *Mcv,FILE *in,FILE *out,int bodytoo)
 {	int ch,prev_ch,next_ch;
 	INOUT iob,*io = &iob;
 
 	INOUT_init(io,in,out);
 	io->UNFOLD_LINE = 1;
+	if( Mcv ){
+		io->in_ewicodemix = Mcv->c_ewicodemix;
+		io->in_ewiccxeach = Mcv->c_ewiccxeach;
+		io->out_charcode = Mcv->c_ocode; 
+		io->out_noccx = Mcv->c_noccx;
+	}
 
 	for(;;){
 		prev_ch = io->in_prevch;
@@ -1282,31 +1424,95 @@ void MIME_headerDecode(FILE *in,FILE *out,int bodytoo)
 		while( (ch = NLfgetc(in)) != EOF )
 			DE_FPUTC(ch,io);
 	DE_FPUTC(EOF,io);
+
+	if( Mcv ){
+		strcpy(Mcv->c_ewicode,io->in_ewcharset);
+		Mcv->c_ewicodemix = io->in_ewicodemix;
+	}
+	return 0;
 }
 
+int MIME_strHeaderDecodeX(MimeConv *Mcv,PCStr(ins),PVStr(outs),int osize);
 void MIME_strHeaderDecode(PCStr(ins),PVStr(outs),int osize)
+{
+	MIME_strHeaderDecodeX(0,ins,BVStr(outs),osize);
+}
+int MIME_strHeaderDecodeX(MimeConv *Mcv,PCStr(ins),PVStr(outs),int osize)
 {	FILE *In,*Out;
 	int oi;
 
 	In = str_fopen((char*)ins,strlen(ins),"r");
 	Out = str_fopen((char*)outs,osize,"w");
+	/*
 	MIME_headerDecode(In,Out,1);
+	*/
+	MIME_headerDecodeX(Mcv,In,Out,1);
 	fflush(Out);
 	for(oi = 0; outs[oi]; oi++)
 		if((outs[oi] & 0xFF) == 0xFF)
 			ovstrcpy((char*)&outs[oi],&outs[oi+1]);
 	str_fclose(In);
 	str_fclose(Out);
+	return 0;
 }
+void MimeCodeconv(MimeConv *Mcv,PCStr(src),PVStr(dst),int dsz,PCStr(ctype),int repair);
+int MIME_strHeaderEncodeX(MimeConv *Mcv,PCStr(ins),PVStr(outs),int osize);
 void MIME_strHeaderEncode(PCStr(ins),PVStr(outs),int osize)
+{
+	MIME_strHeaderEncodeX(0,ins,BVStr(outs),osize);
+}
+int MIME_strHeaderEncodeX(MimeConv *Mcv,PCStr(ins),PVStr(outs),int osize)
 {	FILE *In,*Out;
 
+	int xsiz = 0;
+	char *xout = 0;
+	MimeConv dMcvb,*dMcv=&dMcvb;
+
+	if( Mcv && Mcv->c_ocode && strstr(ins,"=?") ){
+		bzero(dMcv,sizeof(MimeConv));
+		xsiz = strlen(ins)*2+1;
+		xout = (char*)malloc(xsiz);
+		dMcv->c_noccx = 1;
+		MIME_strHeaderDecodeX(dMcv,ins,BVStr(outs),osize);
+		dMcv->c_noccx = 0;
+		if( dMcv->c_ewicode[0] ){
+			dMcv->c_icode = dMcv->c_ewicode;
+		}else{
+			dMcv->c_icode = Mcv->c_icode;
+		}
+		dMcv->c_ocode = Mcv->c_ocode;
+		if( m17n_known_code(dMcv->c_icode)
+		 && m17n_known_code(dMcv->c_ocode) ){
+			dMcv->c_m17n = m17n_ccx_new(dMcv->c_icode,dMcv->c_ocode,0,0);
+		}
+		MimeCodeconv(dMcv,outs,ZVStr(xout,xsiz),xsiz,"text/plain",0);
+		if( xout[0] == 0
+		 || streq(xout,"?")
+		 || strchr(xout,'\n')==0 && strchr(ins,'\n')!=0
+		){
+			/* code conversion error in M17N */
+			free(xout);
+			xout = 0;
+		}else{
+			ins = xout;
+			*dMcv = *Mcv;
+			dMcv->c_icode = Mcv->c_ocode;
+			Mcv = dMcv;
+		}
+	}
 	In = str_fopen((char*)ins,strlen(ins),"r");
 	Out = str_fopen((char*)outs,osize,"w");
+	/*
 	MIME_headerEncode(In,Out);
+	*/
+	MIME_headerEncodeX(Mcv,In,Out);
 	fflush(Out);
 	str_fclose(In);
 	str_fclose(Out);
+	if( xout ){
+		free(xout);
+	}
+	return 0;
 }
 
 int is_MIME_header(FILE *fp)
@@ -1382,7 +1588,7 @@ int MIME_localStrColumns(PCStr(str))
 {	INOUT iob,*io = &iob;
 	FILE *sfp;
 	int len;
-	CHAR *CH;
+	CHARx *CH;
 
 	sfp = str_fopen((char*)str,strlen(str),"r");
 	INOUT_init(io,sfp,NULL);
