@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1997 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1997-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -18,13 +19,10 @@ Description:
 History:
 	970208	created
 //////////////////////////////////////////////////////////////////////#*/
-#include <stdio.h>
-#include <string.h>
-extern FILE *TMPFILE();
-extern FILE *NULLFP();
-extern int MIME_CONV;
-
-extern char *MimeKit_Version;
+#include "ystring.h"
+#include "mime.h"
+static void putENCRIPTED(FILE *src,FILE *dst,PCStr(boundary),PCStr(EOL));
+static void putSIGNED(FILE *src,FILE *sign,FILE *dst,PCStr(boundary),PCStr(EOL));
 
 int PGP_MODE;
 #define _PGP_SIGN	1
@@ -33,11 +31,11 @@ int PGP_MODE;
 #define _PGP_DECR	8
 #define _PGP_MIME	0x10	/* encode in PGP/MIME format */
 
-PGP_SIGN(){ return PGP_MODE & _PGP_SIGN; }
-PGP_ENCR(){ return PGP_MODE & _PGP_ENCR; }
-PGP_VRFY(){ return PGP_MODE & _PGP_VRFY; }
-PGP_DECR(){ return PGP_MODE & _PGP_DECR; }
-PGP_MIME(){ return PGP_MODE & _PGP_MIME; }
+int PGP_SIGN(){ return PGP_MODE & _PGP_SIGN; }
+int PGP_ENCR(){ return PGP_MODE & _PGP_ENCR; }
+int PGP_VRFY(){ return PGP_MODE & _PGP_VRFY; }
+int PGP_DECR(){ return PGP_MODE & _PGP_DECR; }
+int PGP_MIME(){ return PGP_MODE & _PGP_MIME; }
 
 #define CRLF	"\r\n"
 
@@ -48,8 +46,7 @@ PGP_MIME(){ return PGP_MODE & _PGP_MIME; }
 #define LEOLINE	"\n"
 #endif
 
-static pgp1(arg)
-	char *arg;
+static scanListFunc pgp1(PCStr(arg))
 {
 	if( strcasecmp(arg,"sign") == 0 ) PGP_MODE |= _PGP_SIGN; else
 	if( strcasecmp(arg,"encr") == 0 ) PGP_MODE |= _PGP_ENCR; else
@@ -61,15 +58,12 @@ static pgp1(arg)
 		arg);
 	return 0;
 }
-scan_PGP(_,args)
-	char *_;
-	char *args;
+void scan_PGP(void*_,PCStr(args))
 {
-	scan_commaList(args,0,pgp1);
+	scan_commaList(args,0,scanListCall pgp1);
 }
 
-static FILE *xpopen(shcom,mode,dstfd)
-	char *shcom,*mode;
+static FILE *xpopen(PCStr(shcom),PCStr(mode),int dstfd)
 {	int stdout_fd;
 	FILE *pfp;
 
@@ -81,20 +75,18 @@ static FILE *xpopen(shcom,mode,dstfd)
 	return pfp;
 }
 
-FILE *TMPFILEX();
-
-PGPverify(src,sign,vrfy)
-	FILE *src,*sign,*vrfy;
-{	char datafile[1024],signfile[1024];
+void PGPverify(FILE *src,FILE *sign,FILE *vrfy)
+{	CStr(datafile,1024);
+	CStr(signfile,1024);
 	FILE *pfp,*ofp,*fp;
-	char shcom[1024];
+	CStr(shcom,1024);
 	int io[2],ch,nc,unlink_err;
 
-	fp = TMPFILEX(datafile);
+	fp = TMPFILEX(AVStr(datafile));
 	copyLEOLINE(src,fp);
 	fclose(fp);
 
-	fp = TMPFILEX(signfile);
+	fp = TMPFILEX(AVStr(signfile));
 	copyfile1(sign,fp);
 	fclose(fp);
 
@@ -130,13 +122,11 @@ PGPverify(src,sign,vrfy)
 		unlink(signfile);
 	}
 }
-PGPcodec(com,pass,src,sign,dst)
-	char *com,*pass;
-	FILE *src,*sign,*dst;
-{	char shcom[1024];
-	char line[1024];
+void PGPcodec(PCStr(com),PCStr(pass),FILE*src,FILE*sign,FILE*dst)
+{	CStr(shcom,1024);
+	CStr(line,1024);
 	FILE *pfp,*tmp;
-	char pgppassfd[128];
+	CStr(pgppassfd,128);
 	int passfd[2];
 
 	sprintf(shcom,"pgp -f");
@@ -165,7 +155,7 @@ PGPcodec(com,pass,src,sign,dst)
 		pfp = xpopen(shcom,"w",fileno(dst));
 	}else{
 		tmp = TMPFILE("PGPcodec");
-		pfp = xpopen(shcom,"w",tmp);
+		pfp = xpopen(shcom,"w",fileno(tmp));
 	}
 
 	if( pass[0] ){
@@ -189,28 +179,25 @@ PGPcodec(com,pass,src,sign,dst)
 /*
  * RFC 2015
  */
-isPGPMIME(art)
-	FILE *art;
-{	char ctype[512];
+int isPGPMIME(FILE *art)
+{	CStr(ctype,512);
 
-	fgetsHeaderField(art,"Content-Type",ctype,sizeof(ctype));
+	fgetsHeaderField(art,"Content-Type",AVStr(ctype),sizeof(ctype));
 	if( strncasecmp(ctype,"multipart/encrypted",19) == 0 )
 		return 1;
 	if( strncasecmp(ctype,"multipart/signed",16) == 0 )
 		return 1;
 	return 0;
 }
-static replaceCtype(src,dst,ctype,EOL,noEOH)
-	FILE *src,*dst;
-	char *ctype,*EOL;
-{	char line[1024];
+static void replaceCtype(FILE *src, FILE *dst,PCStr(ctype),PCStr(EOL),int noEOH)
+{	CStr(line,1024);
 	int putit,putctype,putmmver,EOH;
-	int mmver[128];
+	CStr(mmver,128);
 
 	putit = 0;
 	putmmver = 0;
 	putctype = 0;
-	myMIMEversion(mmver);
+	myMIMEversion(AVStr(mmver));
 
 	while( fgets(line,sizeof(line),src) ){
 		if( line[0] != ' ' && line[0] != '\t' ){
@@ -241,19 +228,19 @@ static replaceCtype(src,dst,ctype,EOL,noEOH)
 	}
 }
 
-static putPGPMIME(src,dst,pcom,EOL)
-	FILE *src,*dst;
-	char *pcom,*EOL;
-{	char ctype[512],nctype[1024];
+static void putPGPMIME(FILE *src,FILE *dst,PCStr(pcom),PCStr(EOL))
+{	CStr(ctype,512);
+	CStr(nctype,1024);
 	int off;
-	char *crtyp,*proto;
-	char boundary[128];
+	const char *crtyp;
+	const char *proto;
+	CStr(boundary,128);
 	FILE *body1,*body2;
 
 	off = ftell(src);
 	fseek(src,0,2);
 	sprintf(boundary,"PGP-MIME-MimeKit%s-%x-%x-%x",
-		MimeKit_Version,time(0),ftell(src),getpid());
+		MimeKit_Version,time(NULL),ftell(src),getpid());
 	fseek(src,off,0);
 
 	if( PGP_ENCR() ){
@@ -264,7 +251,7 @@ static putPGPMIME(src,dst,pcom,EOL)
 		proto = "application/pgp-signature";
 	}
 
-	fgetsHeaderField(src,"Content-Type",ctype,sizeof(ctype));
+	fgetsHeaderField(src,"Content-Type",AVStr(ctype),sizeof(ctype));
 
 	sprintf(nctype,"%s; boundary=\"%s\";%s  micalg=%s; protocol=\"%s\"",
 			crtyp, boundary, EOL, "pgp-md5", proto);
@@ -304,9 +291,7 @@ static putPGPMIME(src,dst,pcom,EOL)
 #define BGN_SIGN	"-----BEGIN PGP SIGNATURE-----"
 #define END_SIGN	"-----END PGP SIGNATURE-----"
 
-static putENCRIPTED(src,dst,boundary,EOL)
-	FILE *src,*dst;
-	char *boundary,*EOL;
+static void putENCRIPTED(FILE *src,FILE *dst,PCStr(boundary),PCStr(EOL))
 {
 	fprintf(dst,"--%s%s",boundary,EOL);
 	fprintf(dst,"Content-Type: application/pgp-encrypted%s",EOL);
@@ -321,9 +306,7 @@ static putENCRIPTED(src,dst,boundary,EOL)
 	fprintf(dst,"--%s--%s",boundary,EOL);
 }
 
-static putSIGNED(src,sign,dst,boundary,EOL)
-	FILE *src,*sign,*dst;
-	char *boundary,*EOL;
+static void putSIGNED(FILE *src,FILE *sign,FILE *dst,PCStr(boundary),PCStr(EOL))
 {
 	fprintf(dst,"--%s%s",boundary,EOL);
 	copyfile1(src,dst);
@@ -338,22 +321,18 @@ static putSIGNED(src,sign,dst,boundary,EOL)
 	fprintf(dst,"--%s--%s",boundary,EOL);
 }
 
-static FILE *getEOR(msg,eor,size)
-	FILE *msg;
-	char *eor;
+static FILE *getEOR(FILE *msg,PVStr(eor),int size)
 {	FILE *nmsg;
 
 	nmsg = TMPFILE("PGPMIME");
-	eor[0] = 0;
-	RFC821_skipbody(msg,nmsg,eor,size);
+	setVStrEnd(eor,0);
+	RFC821_skipbody(msg,nmsg,AVStr(eor),size);
 	fclose(msg);
 	fflush(nmsg);
 	fseek(nmsg,0,0);
 	return nmsg;
 }
-getEOLINE(msg,eol)
-	FILE *msg;
-	char *eol;
+void getEOLINE(FILE *msg,PVStr(eol))
 {	int off,ch;
 
 	off = ftell(msg);
@@ -372,10 +351,9 @@ getEOLINE(msg,eol)
 	}
 	fseek(msg,off,0);
 }
-copyLEOLINE(src,dst)
-	FILE *src,*dst;
-{	char line[1024];
-	char *np;
+void copyLEOLINE(FILE *src,FILE *dst)
+{	CStr(line,1024);
+	refQStr(np,line); /**/
 
 	while( fgets(line,sizeof(line),src) ){
 		if( np = strpbrk(line,CRLF) )
@@ -383,29 +361,27 @@ copyLEOLINE(src,dst)
 		fputs(line,dst);
 	}
 }
-toJISjaca(src,dst,EOL)
-	FILE *src,*dst;
-	char *EOL;
-{	char line[1024],*np;
+void toJISjaca(FILE *src,FILE *dst,PCStr(EOL))
+{	CStr(line,1024);
+	CStr(xline,2048);
 
 	RFC821_skipheader(src,dst,NULL);
 	fprintf(dst,EOL);
 
 	while( fgets(line,sizeof(line),src) ){
-		TO_JIS(line,line,"text/plain");
-		fputs(line,dst);
+		TO_JIS(line,AVStr(xline),"text/plain");
+		fputs(xline,dst);
 	}
 }
 
 /*
  *  RFC 1991
  */
-isPGPFORMAT(art)
-	FILE *art;
-{	char line[1024];
+int isPGPFORMAT(FILE *art)
+{	CStr(line,1024);
 	int ispgp;
 	int aoff;
-	char *pstr;
+	const char *pstr;
 	int plen;
 
 	ispgp = 0;
@@ -423,38 +399,61 @@ isPGPFORMAT(art)
 	return ispgp;
 }
 
-static isTEXT(art)
-	FILE *art;
-{	char ctype[1024];
+static int isTEXT(FILE *art)
+{	CStr(ctype,1024);
 
-	fgetsHeaderField(art,"Content-Type",ctype,sizeof(ctype));
+	fgetsHeaderField(art,"Content-Type",AVStr(ctype),sizeof(ctype));
 	if( ctype[0] == 0 || strncasecmp(ctype,"text/",5) == 0 )
 		return 1;
 	else	return 0;
 }
 
-PGPencodeMIME(src,dst)
-	FILE *src,*dst;
-{	FILE *tmp1,*tmp2;
-	char *pcom;
-	char EOL[4],EOR[128];
+void PGPencodeMIME(FILE *src,FILE *dst)
+{
+	PGPencodeMIMEX(src,dst,O_ALL);
+}
+void PGPencodeMIMEX(FILE *src,FILE *dst,int filter)
+{	MimeEnv me;
 
-	if( PGP_ENCR() == 0 && PGP_SIGN() == 0 )
-		return encodeMIME(src,dst);
+	me.me_filter = filter;
+	me.me_ccx = NULL;
+	PGPencodeMIMEXX(&me,src,dst);
+}
+int PGPencodeMIMEXX(MimeEnv *me,FILE *src,FILE *dst)
+{	FILE *tmp1,*tmp2;
+	const char *pcom;
+	CStr(EOL,4);
+	CStr(EOR,128);
+	int filter = me->me_filter;
+
+	if( PGP_ENCR() == 0 && PGP_SIGN() == 0 ){
+		/*
+		encodeMIMEX(src,dst,filter);
+		return;
+		*/
+		encodeMIMEXX(me,src,dst);
+		return 0;
+	}
 
 	tmp1 = TMPFILE("PGPMIME");
-	encodeMIME(src,tmp1);
+	/*
+	encodeMIMEX(src,tmp1,filter);
+	*/
+	encodeMIMEXX(me,src,tmp1);
 	fflush(tmp1);
 	fseek(tmp1,0,0);
 
 	if( !isTEXT(tmp1) ){
 		copyfile1(tmp1,dst);
 		fclose(tmp1);
+		/*
 		return ;
+		*/
+		return 0;
 	}
 
-	tmp1 = getEOR(tmp1,EOR,sizeof(EOR));
-	getEOLINE(tmp1,EOL);
+	tmp1 = getEOR(tmp1,AVStr(EOR),sizeof(EOR));
+	getEOLINE(tmp1,FVStr(EOL));
 
 	if( PGP_SIGN() )
 		pcom = "SIGN";
@@ -478,40 +477,33 @@ PGPencodeMIME(src,dst)
 	fputs(EOR,dst);
 	fflush(dst);
 	fclose(tmp1);
+	return 1;
 }
 
-relay_pgpSIGNED(fs,tc,boundaries,endline)
-	FILE *fs,*tc;
-	char *boundaries[];
-	char *endline;
+int relay_pgpSIGNED(FILE *fs,FILE *tc,const char *boundaries[],PVStr(endline))
 {	int rcode;
 
-	rcode = relayBODYpart(fs,tc,boundaries,1,endline);
+	rcode = relayBODYpart(fs,tc,boundaries,1,AVStr(endline));
 	return rcode;
 }
 static FILE *SIGN;
-relay_pgpSIGN(fs,tc,boundaries,endline)
-	FILE *fs,*tc;
-	char *boundaries[];
-	char *endline;
+int relay_pgpSIGN(FILE *fs,FILE *tc,const char *boundaries[],PVStr(endline))
 {	int rcode;
 	FILE *sign;
 
 	if( SIGN )
 		sign = SIGN;
 	else	sign = tc;
-	rcode = relayBODYpart(fs,sign,boundaries,1,endline);
+	rcode = relayBODYpart(fs,sign,boundaries,1,AVStr(endline));
 	fseek(sign,0,0);
 	return rcode;
 }
 
-static copyVerify(vrfy,out,outs)
-	FILE *vrfy,*out;
-	char *outs;
-{	char line[1024];
+static int copyVerify(FILE *vrfy, FILE *out,PVStr(outs))
+{	CStr(line,1024);
 	int verified;
 
-	*outs = 0;
+	setVStrEnd(outs,0);
 	verified = -1;
 	while( fgets(line,sizeof(line),vrfy) ){
 		if( strncmp(line,"Good signature",14) == 0 ){
@@ -531,27 +523,22 @@ static copyVerify(vrfy,out,outs)
 	}
 	return verified;
 }
-static putVerify(out,vrfys,EOL)
-	FILE *out;
-	char *EOL;
+static void putVerify(FILE *out,PCStr(vrfys),PCStr(EOL))
 {
 	fprintf(out,"X-PGP-Verified: by MimeKit/%s;%s%s",
 		MimeKit_Version,EOL,vrfys);
 }
 
-FILE *PGPMIMEverify(art,verified,EOL)
-	FILE *art;
-	int *verified;
-	char *EOL;
-{	char ctype[512];
+FILE *PGPMIMEverify(FILE *art,int *verified,PCStr(EOL))
+{	CStr(ctype,512);
 	FILE *jaca,*tmp,*out;
 	int oconv;
 	FILE *vrfy;
-	char vrfys[1024];
+	CStr(vrfys,1024);
 
 	*verified = 0;
 
-	fgetsHeaderField(art,"Content-Type",ctype,sizeof(ctype));
+	fgetsHeaderField(art,"Content-Type",AVStr(ctype),sizeof(ctype));
 	if( strncasecmp(ctype,"multipart/encrypted",19) == 0 )
 		return art;
 
@@ -580,14 +567,14 @@ FILE *PGPMIMEverify(art,verified,EOL)
 	fclose(SIGN);
 
 	fprintf(stderr,"\r\n==> VERIFICATION\n");
-	*verified = copyVerify(vrfy,stderr,vrfys);
+	*verified = copyVerify(vrfy,stderr,AVStr(vrfys));
 	fseek(vrfy,0,0);
 
 	out = TMPFILE("MIMEtoPGP");
 
 	if( PGP_DECR() ){
 		RFC821_skipheader(tmp,NULL,NULL);
-		fgetsHeaderField(tmp,"Content-Type",ctype,sizeof(ctype));
+		fgetsHeaderField(tmp,"Content-Type",AVStr(ctype),sizeof(ctype));
 		fseek(tmp,0,0);
 
 		replaceCtype(tmp,out,ctype,EOL,1);
@@ -613,11 +600,9 @@ FILE *PGPMIMEverify(art,verified,EOL)
 	return out;
 }
 
-FILE *PGPdecode(art,was_mime,EOL)
-	FILE *art;
-	char *EOL;
+FILE *PGPdecode(FILE *art,int was_mime,PCStr(EOL))
 {	FILE *jaca,*tmp1,*tmp2;
-	char ctype[512];
+	CStr(ctype,512);
 	int is_mime;
 
 	tmp1 = TMPFILE("PGPMIME");
@@ -631,7 +616,7 @@ FILE *PGPdecode(art,was_mime,EOL)
 	fseek(art,0,0);
 
 	if( is_mime )
-		fgetsHeaderField(tmp2,"Content-Type",ctype,sizeof(ctype));
+		fgetsHeaderField(tmp2,"Content-Type",AVStr(ctype),sizeof(ctype));
 
 	if( is_mime || was_mime ){
 		/* Skip Content-Type of Body Part */
@@ -652,19 +637,21 @@ FILE *PGPdecode(art,was_mime,EOL)
 	return tmp1;
 }
 
-PGPdecodeMIME(src,dst,cache,filter,codeconv,enHTML)
-	FILE *src,*dst,*cache;
+void PGPdecodeMIME(FILE*src,FILE*dst,FILE*cache, int filter,int codeconv,int enHTML)
 {	FILE *tmp;
 	int is_mime;
 	int omode;
 	int verified;
-	char EOL[4],EOR[1024];
+	CStr(EOL,4);
+	CStr(EOR,1024);
 
-	if( !PGP_DECR() && !PGP_VRFY() )
-		return decodeMIME(src,dst,cache,filter,codeconv,enHTML);
+	if( !PGP_DECR() && !PGP_VRFY() ){
+		decodeMIME(src,dst,cache,filter,codeconv,enHTML);
+		return;
+	}
 
 	tmp = TMPFILE("PGPMIME");
-	RFC821_skipbody(src,tmp,EOR,sizeof(EOR));
+	RFC821_skipbody(src,tmp,AVStr(EOR),sizeof(EOR));
 	fseek(tmp,0,0);
 	if( cache != NULL ){
 		copyfile1(tmp,cache);
@@ -672,7 +659,7 @@ PGPdecodeMIME(src,dst,cache,filter,codeconv,enHTML)
 	}
 
 	is_mime = isPGPMIME(tmp);
-	getEOLINE(tmp,EOL);
+	getEOLINE(tmp,FVStr(EOL));
 
 	if( !is_mime && !isPGPFORMAT(tmp) ){
 		copyfile1(tmp,dst);
