@@ -7,7 +7,7 @@
 static char HEXCH[] = "0123456789abcdef";
 
 void MD5toa(const char *digest,char md5a[])
-{	char *mp;
+{	char *mp; /*QA(QF)*/
 	unsigned char dg1;
 	unsigned int i;
 
@@ -33,6 +33,17 @@ void toMD5X(const char *str,int len,char digest[])
 	MD5Update(&context, (unsigned char*)str, len);
 	MD5Final((unsigned char*)digest, &context);
 }
+int fMD5(FILE *fp,char digest[])
+{	MD5_CTX context;
+	char buf[1024];
+	int rcc;
+
+	MD5Init(&context);
+	while( 0 < (rcc = fread(buf,1,sizeof(buf),fp)) )
+		MD5Update(&context,(unsigned char*)buf,rcc);
+	MD5Final((unsigned char*)digest, &context);
+	return 0;
+}
 void ftoMD5(FILE *fp,char md5[])
 {	MD5_CTX context;
 	char digest[16];
