@@ -26,11 +26,13 @@ History:
 #include <stdio.h>
 #include "ystring.h"
 #include <stdlib.h>
+#include "log.h"
 
 #ifndef NULL
 #define NULL 0L
 #endif
 
+#undef ctos /* in OSF/1 sys/wait.h ? */
 static char *ctos(int ch,PVStr(str))
 {
 	if( 0x20 < ch && ch < 0x7F )
@@ -398,8 +400,8 @@ static FaStat *fa_stins(FaStat *ofsp,FaStat *fsp,FaStat *nfsp,unsigned char in)
 /*##############################*
  *	FA GENERATOR		*
  *##############################*/
-static int fa_rexp(FaStat *ofsp,unsigned char *regexp,int xclass);
-static int fa_gen(unsigned char *regexp,int xclass)
+static long int fa_rexp(FaStat *ofsp,unsigned char *regexp,int xclass);
+static long int fa_gen(unsigned char *regexp,int xclass)
 {
 	return fa_rexp(fa_root,regexp,xclass);
 }
@@ -469,7 +471,7 @@ static int expset(const unsigned char *lrep,const unsigned char *rrep,unsigned c
 }
 
 static int fa_nosetrep;
-static int fa_rexp(FaStat *ofsp,unsigned char *regexp,int xclass)
+static long int fa_rexp(FaStat *ofsp,unsigned char *regexp,int xclass)
 {	const unsigned char *rep;
 	const unsigned char *lrep;
 	CStr(set,512);
@@ -482,6 +484,8 @@ static int fa_rexp(FaStat *ofsp,unsigned char *regexp,int xclass)
 	lastany = 0;
 	fsp = ofsp;
 	for(rep = regexp; *rep; rep++){
+	  if( 2 < LOGLEVEL )
+		syslog_ERROR("-- fa_rexp: %s\n",rep);
 
 #ifdef DEBUG
 printf("#### %X %X %s\n",ofsp,fsp,rep);
@@ -489,7 +493,7 @@ printf("#### %X %X %s\n",ofsp,fsp,rep);
 
 	  if(*rep == '$' && rep[1] == 0 ){
 		fsp->eclass = xclass;
-		return (int)fsp;
+		return (long int)fsp;
 	  }
 /*
 	  if(*rep == '$')
@@ -543,6 +547,11 @@ printf("#### %X %X %s\n",ofsp,fsp,rep);
 			if(rep[1] == '*'&& rep[2] != '*' )
 				fsp = fa_stins(ofsp,fsp,fsp,*rep);
 			else	fsp = fa_stins(ofsp,fsp, 0 ,*rep);
+			if( fsp == 0 ){
+				syslog_ERROR("fa_rexp: %X[%s] -> NULL [%s]\n",
+					ofsp,rep,regexp);
+				return 0;
+			}
 			fsp->other = lastany;
 	    }
 	}
@@ -550,7 +559,7 @@ printf("#### %X %X %s\n",ofsp,fsp,rep);
 /*
 fa_ls();
 */
-	return (int)fsp;
+	return (long int)fsp;
 }
 
 FaStat *frex_append(FaStat *fsp,PCStr(rexp))
