@@ -21,8 +21,7 @@ History:
 #include <stdio.h>
 #include "vsocket.h"
 
-long Gettimeofday(usec)
-	int *usec;
+long Gettimeofday(int *usec)
 {	struct timeval tv;
 
 	gettimeofday(&tv, NULL);
@@ -39,5 +38,37 @@ double Time()
 }
 
 static double StartTime;
-SetStartTime(){ StartTime = Time(); }
+void SetStartTime(){ StartTime = Time(); }
 double GetStartTime(){ return StartTime; }
+
+int scanTime(PCStr(sdate));
+int date_main(int ac,const char *av[]){
+	struct timeval tv;
+	int itime = -1;
+	int fset = 0;
+	int ai;
+	const char *a1;
+	CStr(date,128);
+
+	for( ai = 1; ai < ac; ai++ ){
+		a1 = av[ai];
+		if( streq(a1,"-s") ){
+			fset = 1;
+			continue;
+		}
+		itime = scanTime(a1);
+		StrftimeLocal(AVStr(date),sizeof(date),TIMEFORM_RFC822,itime,0);
+		printf("%s\n",date);
+		StrftimeGMT(AVStr(date),sizeof(date),TIMEFORM_RFC822,itime,0);
+		printf("%s\n",date);
+		printf("%u %X\n",itime,itime);
+	}
+	if( fset && itime != -1 ){
+		//settimeofday(&tv,NULL);
+	}
+	StrftimeLocal(AVStr(date),sizeof(date),TIMEFORM_RFC822,time(0),0);
+	printf("%s\n",date);
+	StrftimeGMT(AVStr(date),sizeof(date),TIMEFORM_RFC822,time(0),0);
+	printf("%s\n",date);
+	return 0;
+}
