@@ -17,25 +17,38 @@ History:
 	981119	extracted from ccxmain.c
 //////////////////////////////////////////////////////////////////////#*/
 #include <stdio.h>
-extern void *CCXnew();
-extern char *CCXident();
+#include <stdlib.h>
+#include "ystring.h"
 
-ccx_main(ac,av)
-	char *av[];
-{	void *ccx;
+int UCSinit();
+void dumpCharMapping(PCStr(code));
+void loadCharMapping(PCStr(code),FILE *ifp);
+int PollIn(int,int);
+void CCXcounts(CCXP);
+void scan_CHARMAPs(void *ctx,PCStr(maps));
+
+int ccx_main(int ac,const char *av[])
+{	CCXP ccx;
 	int rcc,len;
 	/* to test boundary condition; char in[8]; */
-	char in[1024];
-	char out[4096];
-	char *incode;
-	char *outcode;
-	char *arg,*infile;
+	/*
+	CStr(in,1024);
+	CStr(out,4096);
+	*/
+	CStr(in,4*1024);
+	CStr(out,16*1024);
+	const char *incode;
+	const char *outcode;
+	const char *arg;
+	const char *infile;
 	int ai;
 	FILE *ifp,*ofp;
 	int nready;
 	int ident = 0;
 	int dump = 0;
 	int load = 0;
+	int verbose = 0;
+	int charmap = 0;
 
 	incode = "*";
 	outcode = "j";
@@ -46,19 +59,40 @@ ccx_main(ac,av)
 	for( ai = 1; ai < ac; ai++ ){
 		arg = av[ai];
 		if( *arg == '-' ){
-			if( strcmp(arg,"--ident") == 0 )
+			if( strncmp(arg,"--ident",4) == 0 )
+			{
+				outcode = "guess";
 				ident = 1;
+			}
 			else
-			if( strcmp(arg,"--dump") == 0 )
+			if( strncmp(arg,"--dump",3) == 0 )
 				dump = 1;
 			else
-			if( strcmp(arg,"--load") == 0 )
+			if( strncmp(arg,"--load",3) == 0 )
 				load = 1;
 			else
+			if( strncmp(arg,"--verbose",3) == 0 )
+				verbose = 1;
+			else
 			if( strncmp(arg,"--",2) == 0 )
 				incode = arg+2;
 			else
+			if( streq(arg,"-o") ){
+				if( ai+1 < ac ){
+					ai++;
+					ofp = fopen(av[ai],"w");
+					if( ofp == NULL ){
+						fprintf(stderr,"Cannot open: %s\n",
+							av[ai]);
+						exit(-1);
+					}
+				}
+			}else
 			outcode = arg+1;
+		}else
+		if( strneq(arg,"CHARMAP=",8) ){
+			charmap = 1;
+			scan_CHARMAPs(0,arg+8);
 		}else{
 			infile = arg;
 		}
@@ -82,6 +116,8 @@ ccx_main(ac,av)
 	}
 
 	ccx = CCXnew(incode,outcode);
+	if( strcmp(incode,"*") != 0 )
+		CCX_setincode(ccx,incode);
 	if( ccx == NULL ){
 		fprintf(stderr,"unknown output code: %s\n",outcode);
 		exit(-1);
@@ -98,15 +134,20 @@ ccx_main(ac,av)
 		rcc = read(fileno(ifp),in,sizeof(in));
 		if( rcc <= 0 )
 			break;
-		len = CCXexec(ccx,in,rcc,out,sizeof(out));
+		len = CCXexec(ccx,in,rcc,AVStr(out),sizeof(out));
 		if( !ident )
 		fwrite(out,1,len,ofp);
 	}
-	len = CCXexec(ccx,"",0,out,sizeof(out));
+	len = CCXexec(ccx,"",0,AVStr(out),sizeof(out));
 	if( ident )
+	{
 		printf("%s\n",CCXident(ccx));
+	}
 	else
 	fwrite(out,1,len,ofp);
 
-	exit(0);
+	if( verbose ){
+		CCXcounts(ccx);
+	}
+	exit(0); return 0;
 }
