@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1994 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1994-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -28,10 +29,23 @@ History:
 #include <errno.h>
 #include <time.h>
 #include "file.h"
+/*
+#ifdef MIMEKIT
+#define lPATHFIND() 0
+#define lFILEOPEN() 0
+#define lDEBUGMSG() 0
+#define ENBUG_WINSOCKFD 0
+#else
+*/
+#include "log.h"
+/*
+#endif
+*/
 
 int getNonblockingIO(int fd);
 int setCloseOnExec(int fd);
 int clearCloseOnExec(int fd);
+extern int FS_dosPath;
 
 
 #ifdef S_ISUID /* set UID on execution */
@@ -43,12 +57,6 @@ int FS_maybeUnix(){ return 0; }
 int FS_withSetuid(){ return 0; }
 #endif
 
-#if defined(_MSC_VER) || defined(__CYGWIN__) || defined(__EMX__)
-int FS_dosPath = 1;
-#else
-int FS_dosPath = 0;
-#endif
-
 #ifdef S_IFLNK
 int FS_withSymlink(){ return 1; }
 #else
@@ -64,7 +72,7 @@ int FS_withSymlink(){ return 0; }
 #endif
 
 int File_stats(PCStr(path),int link,int *mode,int *uid,int *gid,int *size,int *mtime)
-{	struct stat st;
+{	FileStat st;
 	int code;
 
 	if( link )
@@ -80,7 +88,7 @@ int File_stats(PCStr(path),int link,int *mode,int *uid,int *gid,int *size,int *m
 	return code;
 }
 int File_ident(PCStr(path),PVStr(ident))
-{	struct stat st;
+{	FileStat st;
 
 	if( stat(path,&st) == 0 ){
 		sprintf(ident,"%x:%x:%x",st.st_dev,st.st_ino,st.st_ctime);
@@ -90,56 +98,70 @@ int File_ident(PCStr(path),PVStr(ident))
 	return -1;
 }
 int File_is(PCStr(path))
-{	struct stat st;
+{	FileStat st;
 
 	if( stat(path,&st) == 0 )
 		return 1;
 	else	return 0;
 }
 int File_isreg(PCStr(path))
-{	struct stat st;
+{	FileStat st;
 
 	if( stat(path,&st) == 0 )
 		return S_ISREG(st.st_mode);
 	else	return 0;
 }
 int File_ctime(PCStr(path))
-{	struct stat st;
+{	FileStat st;
 
 	if( stat(path,&st) == 0 )
 		return st.st_ctime;
 	else	return -1;
 }
 int File_mtime(PCStr(path))
-{	struct stat st;
+{	FileStat st;
 
 	if( stat(path,&st) == 0 )
 		return st.st_mtime;
 	else	return -1;
 }
+FileSize File_sizeX(PCStr(path))
+{	FileStat st;
+
+	if( stat(path,&st) == 0 )
+		return st.st_size;
+	else	return -1;
+}
 int File_size(PCStr(path))
-{	struct stat st;
+{	FileStat st;
 
 	if( stat(path,&st) == 0 )
 		return st.st_size;
 	else	return -1;
 }
 int File_uid(PCStr(path))
-{	struct stat st;
+{	FileStat st;
 
 	if( stat(path,&st) == 0 )
 		return st.st_uid;
 	else	return 0;
 }
+int File_gid(PCStr(path))
+{	FileStat st;
+
+	if( stat(path,&st) == 0 )
+		return st.st_gid;
+	else	return 0;
+}
 int File_ino(PCStr(path))
-{	struct stat st;
+{	FileStat st;
 
 	if( stat(path,&st) == 0 )
 		return st.st_ino;
 	else	return 0;
 }
 int file_cmp(int fd1,int fd2)
-{	struct stat st1,st2;
+{	FileStat st1,st2;
 
 	bzero(&st1,sizeof(st1));
 	bzero(&st2,sizeof(st2));
@@ -150,7 +172,7 @@ int file_cmp(int fd1,int fd2)
 	else	return 1;
 }
 int File_cmp(PCStr(path1),PCStr(path2))
-{	struct stat st1,st2;
+{	FileStat st1,st2;
 
 	bzero(&st1,sizeof(st1));
 	bzero(&st2,sizeof(st2));
@@ -168,7 +190,7 @@ int File_cmp(PCStr(path1),PCStr(path2))
 		return 0;
 	else	return 1;
 /*
-return bcmp(&st1,&st2,sizeof(struct stat));
+return bcmp(&st1,&st2,sizeof(FileStat));
 */
 }
 
@@ -202,21 +224,6 @@ int fcompare(FILE *fp1,FILE *fp2)
 	return diff;
 }
 
-int Ftruncate(FILE *fp,int offset,int whence)
-{	long savoff;
-	int rcode;
-
-	savoff = ftell(fp);
-	fseek(fp,offset,whence);
-	rcode = ftruncate(fileno(fp),(off_t)ftell(fp));
-	fseek(fp,savoff,0);
-	return rcode;
-}
-int Lseek(int fd,int off,int wh)
-{
-	return lseek(fd,(off_t)off,wh);
-}
-
 static const char *_pathsep;
 static const char *_PATHSEP(){
 	CStr(cwd,1024);
@@ -334,69 +341,6 @@ int isBoundpath(PCStr(path))
 	}
 	return 0;
 }
-void chdir_cwd(PVStr(cwd),PCStr(go),int userdir)
-{	refQStr(tp,cwd); /**/
-	const char *cp;
-	int uplen,len;
-
-	for( cpyQStr(tp,cwd); *tp; tp++ ){}
-	if( len = isFullpath(go) ){
-		tp = (char*)cwd;
-		while( 0 < len-- ){
-			assertVStr(cwd,tp+1);
-			setVStrPtrInc(tp,*go++);
-		}
-	}else
-	if( userdir && go[0] == '~' ){
-		if( go[1] == 0   ){ tp = (char*)cwd; go += 1; }else
-		if( go[1] == '/' ){ tp = (char*)cwd; go += 2; }
-	}
-
-	while( *go ){
-		uplen = 0;
-		if( go[0] == '.' ){
-			if( go[1] == 0   ){ go += 1; break; }
-			if( go[1] == '/' ){ go += 2; continue; }
-			if( go[1] == '.' ){
-				if( go[2] == 0   ){ uplen = 2; } else
-				if( go[2] == '/' ){ uplen = 3; }
-			}
-		}
-		if( uplen ){
-			if( cwd < tp ){
-				while( cwd+1 < tp ){
-					if( tp[-1] == '/' )
-						tp--;
-					else	break;
-				}
-				setVStrEnd(tp,0);
-				if( cp = strrchr(cwd,'/') ){
-					if( cp == cwd  )
-						tp = (char*)cp + 1;
-					else	tp = (char*)cp;
-				}else	tp = (char*)cwd;
-			}
-			go += uplen;
-		}else{
-			if( cp = strchr(go,'/') )
-				len = (cp - go) + 1;
-			else	len = strlen(go);
-
-			if( cwd < tp && tp[-1] != '/' ){
-				setVStrPtrInc(tp,'/');
-			}
-			strncpy(tp,go,len);
-			tp += len;
-			go += len;
-		}
-	}
-	while( cwd+1 < tp ){
-		if( tp[-1] == '/' )
-			tp--;
-		else	break;
-	}
-	setVStrEnd(tp,0);
-}
 
 static int _TMPSEQ;
 static int _TMPDIR_set;
@@ -461,7 +405,10 @@ void setOWNER(PCStr(dir),int uid,int gid)
 }
 static int bemine(PCStr(path),int *uidp,int *gidp)
 {
+/*
 	if( OWNER_DIR == 0 || OWNER_UID < 0 )
+*/
+	if( OWNER_DIR == 0 || OWNER_UID == -1 )
 		return 0;
 	/*if( strstr(path,OWNER_DIR) )*/
 	{
@@ -520,19 +467,26 @@ FILE *fopentmpfileShared(PCStr(path),int remove)
 int mkdirShared(PCStr(dir),int mode)
 {	int uid,gid,rcode;
 	int share,mine;
+	int err;
 
 	if( mode == 0 )
 		mode = 0755;
 	share = beshared(dir,1);
 	mine = bemine(dir,&uid,&gid);
 	rcode = mkdir(dir,mode);
+	err = errno;
 	if( rcode != 0 && mine ){
 		if( getuid() == 0 && uid != 0 ){
 			seteuid(uid);
 			rcode = mkdir(dir,mode);
+			err = errno;
 			seteuid(0);
 		}
 	}
+	if( rcode != 0 ){
+		syslog_ERROR("mkdirShared FALED errno=%d: %s\n",err,dir);
+		return rcode;
+	}
 	if( mine )
 		chown(dir,uid,gid);
 	if( share )
@@ -549,9 +503,16 @@ int chmodIfShared(PCStr(file),int mode)
 	return chmod(file,mode);
 }
 
+void setTMPDIRX(PCStr(dir),int ovw);
 void setTMPDIR(PCStr(dir))
+{
+	setTMPDIRX(dir,1);
+}
+void setTMPDIRX(PCStr(dir),int ovw)
 {	CStr(buff,1024);
 
+	if( _TMPDIR_set && ovw == 0 )
+		return;
 	if( dir ){
 		if( _TMPDIR && _TMPDIR_set )
 			free((char*)_TMPDIR);
@@ -565,6 +526,10 @@ FILE *TMPFILEXX(PCStr(what),xPVStr(path))
 	CStr(pathb,1024);
 	CStr(buff,1024);
 	FILE *fp;
+	int seq;
+	int now;
+	int pid;
+	_heapLock lock;
 
 	tmpdir = _TMPDIR;
 	if( tmpdir == NULL ){
@@ -578,16 +543,32 @@ FILE *TMPFILEXX(PCStr(what),xPVStr(path))
 
 	if( path == NULL )
 		setPStr(path,pathb,sizeof(pathb));
-	sprintf(path,"%s/dg%d.%d.%d",tmpdir,getpid(),++_TMPSEQ,time(NULL));
 
+	pid = getpid();
+	now = time(NULL);
+	heapLock(FL_ARG,lock);
+	seq = ++_TMPSEQ;
+	if( isWindowsCE() )
+		sprintf(path,"%s/dg%X.%04X.%X",tmpdir,now,seq,pid);
+	else	sprintf(path,"%s/dg%d.%06d.%d",tmpdir,pid,seq,now);
 	errno = 0;
 	fp = fopentmpfileShared(fullPath(path,AVStr(buff)),path==pathb);
+	heapUnLock(FL_ARG,lock);
 
 	if( fp != NULL )
 		syslog_DEBUG("TMPFILE(%s) = (%d) %s\n",what,fileno(fp),path);
 	else	syslog_ERROR("TMPFILE(%s): cannot create (%d) = %s\n",what,errno,path);
 	return fp;
 }
+
+#if isWindowsCE()
+FILE *XX_TMPFILEXX(PCStr(what),xPVStr(path));
+static FILE *TMPFILEXXX(PCStr(what),PVStr(path)){
+	return XX_TMPFILEXX(what,BVStr(path));
+}
+#define TMPFILEXX(what,path) TMPFILEXXX(what,path)
+#endif
+
 FILE *TMPFILEX(PVStr(path))
 {
 	return TMPFILEXX("*",AVStr(path));
@@ -662,18 +643,66 @@ FILE *Tmpfile()
 #undef NULLFP /* defined in <sys/file.h> of Solaris */
 #endif
 
+int Ofclose(FILE *fp);
+static FILE *NullFP;
+int isNULLFP(FILE *fp){
+	return fp == NullFP;
+}
+void closeNULLFP(){
+	if( NullFP ){
+		Ofclose(NullFP);
+		NullFP = 0;
+	}
+}
+int daemonlog(PCStr(what),PCStr(fmt),...);
 FILE *NULLFP(){
-	static FILE *NULLFP;
 	FILE *tmp;
 
-	if( NULLFP == NULL ){
+	if( NullFP == NULL ){
 		tmp = TMPFILE("NULLFP");
+		if( tmp == NULL || fileno(tmp) < 0 ){
+			daemonlog("F","--FATAL: NULLFP ERROR A\n");
+			_exit(-1);
+		}
 		/*close(fileno(NULLFP)); should hold to avoid reuse */
-		NULLFP = fdopen(fileno(tmp),"r");
+		NullFP = fdopen(fileno(tmp),"r");
+		if( NullFP == NULL ){
+			daemonlog("F","--FATAL: NULLFP ERROR B\n");
+			_exit(-1);
+		}
 	}
-	return NULLFP;
+	return NullFP;
+}
+static FILE *WrNullFP;
+FILE *WRNULLFP(){
+	if( WrNullFP == 0 ){
+		if( isWindows() )
+			WrNullFP = fopen("nul:","w");
+		else	WrNullFP = fopen("/dev/null","w");
+	}
+	return WrNullFP;
 }
 
+int nullFd = -1;
+int setNullFd(int fd){
+	int ofd = nullFd;
+	nullFd = fd;
+	return ofd;
+}
+int getNullFd(PCStr(what)){
+	int nfd;
+
+	if( 0 <= nullFd ){
+		nfd = nullFd;
+	}else{
+		if( isWindows() )
+			nfd = open("nul",0);
+		else	nfd = open("/dev/null",0);
+		if( 0 <= nfd )
+			nullFd = nfd;
+	}
+	return nfd;
+}
 int openNull(int rw){
 	int xfd;
 	FILE *xfp;
@@ -692,9 +721,33 @@ int openNull(int rw){
 	return -1;
 }
 
-int copyfile2(FILE *sfp,FILE *dfp,int doflush)
+int dupclosed_FL(FL_PAR,int fd){
+	int nfd;
+
+	/*
+	if( isWindows() )
+		nfd = open("nul",0);
+	else	nfd = open("/dev/null",0);
+	*/
+	nfd = getNullFd("dupclosed");
+	if( nfd < 0 ){
+		porting_dbg("--dupclosed() no NULL device");
+		Xclose_FL(FL_BAR,fd);
+		return -1;
+	}
+	if( Xdup2_FL(FL_BAR,nfd,fd) < 0 ){
+		porting_dbg("--dupclosed() cannot dup2(%d,%d) errno=%d <= %s:%d",
+			nfd,fd,errno,FL_BAR);
+		return -1;
+	}
+	return 0;
+}
+
+FileSize copyfile2(FILE *sfp,FILE *dfp,int doflush)
 {	CStr(buff,8*1024);
-	int totalc,rcc;
+	FileSize totalc;
+	int rcc;
+	int wcc;
 
 	totalc = 0;
 	for(;;){
@@ -702,15 +755,23 @@ int copyfile2(FILE *sfp,FILE *dfp,int doflush)
 		if( rcc == 0 )
 			break;
 		totalc += rcc;
+		/*
 		if( fwrite(buff,1,rcc,dfp) == 0 )
+		*/
+		wcc = fwrite(buff,1,rcc,dfp);
+		if( wcc <= 0 )
 			break;
+		if( ferror(dfp) ){
+			syslog_ERROR("copyfile() detected ferror()\n");
+			break;
+		}
 	}
 	if( doflush )
 		fflush(dfp);
 	return totalc;
 }
 
-int copyfile1(FILE *sfp,FILE *dfp)
+FileSize copyfile1(FILE *sfp,FILE *dfp)
 {
 	return copyfile2(sfp,dfp,1);
 }
@@ -728,10 +789,17 @@ void path_escchar(xPVStr(path))
 	if( isalpha(path[0]) && path[1] == ':' )
 		path += 2;
 
+	/*
+	 * escape "//", "\\", and one of ':*?"<>|' on Windows
+	 */
 	if( strpbrk(path,":") == NULL && strstr(path+2,"./") == NULL )
+	if( strstr(path,"//") == 0 )
+	if( strstr(path,"\\\\") == 0 )
+	if( strpbrk(path,":*?\"<>|") == 0 )
 		return;
 
 	buff = stralloc(path);
+	dp = path;
 	for( sp = buff; sc = *sp; sp++ ){
 		assertVStr(path,dp+3);
 		if( sc == '.' && sp[1] == '/' ){
@@ -748,6 +816,21 @@ void path_escchar(xPVStr(path))
 				setVStrPtrInc(dp,'.');
 				setVStrPtrInc(dp,'.');
 			}
+		}else
+		if( isWindows() && sc == '/' && sp[1] == '/' ){
+			setVStrPtrInc(dp,'%');
+			setVStrPtrInc(dp,'2');
+			setVStrPtrInc(dp,'F');
+		}else
+		if( isWindows() && sc == '?' ){
+			setVStrPtrInc(dp,'%');
+			setVStrPtrInc(dp,'3');
+			setVStrPtrInc(dp,'F');
+		}else
+		if( isWindows() && sc == '|' ){
+			setVStrPtrInc(dp,'%');
+			setVStrPtrInc(dp,'7');
+			setVStrPtrInc(dp,'C');
 		}else	setVStrPtrInc(dp,sc);
 	}
 	setVStrEnd(dp,0);
@@ -755,47 +838,68 @@ void path_escchar(xPVStr(path))
 }
 
 int file_is(int fd)
-{	struct stat stat;
+{	FileStat stat;
 
 	if( fstat(fd,&stat) == 0 )
 		return 1;
 	else	return 0;
 }
 int file_isreg(int fd)
-{	struct stat status;
+{	FileStat status;
 
 	if( fstat(fd,&status) == 0 )
 		return S_ISREG(status.st_mode);
 	return 0;
 }
 int file_isregular(int fd)
-{	struct stat st;
+{	FileStat st;
 
 	if( fstat(fd,&st) == 0 )
 		return S_ISREG(st.st_mode) || S_ISDIR(st.st_mode);
 	else	return 0;
 }
 int file_isdir(int fd)
-{	struct stat stat;
+{	FileStat stat;
 
 	if( fstat(fd,&stat) == 0 )
 		return S_ISDIR(stat.st_mode);
 	return 0;
 }
 int fileIsdir(PCStr(path))
-{	struct stat status;
+{	FileStat status;
 
 	if( stat(path,&status) == 0 )
 		return S_ISDIR(status.st_mode);
 	return 0;
 }
 int fileIsflat(PCStr(path))
-{	struct stat status;
+{	FileStat status;
 
 	if( stat(path,&status) == 0 )
 		return S_ISDIR(status.st_mode) == 0;
 	return 0;
 }
+int file_ino(int fd)
+{	FileStat stat;
+
+	if( fstat(fd,&stat) == 0 )
+		return stat.st_ino;
+	else	return 0;
+}
+int file_size(int fd)
+{	FileStat stat;
+
+	if( fstat(fd,&stat) == 0 )
+		return stat.st_size;
+	else	return -1;
+}
+int file_mtime(int fd)
+{	FileStat stat;
+
+	if( fstat(fd,&stat) == 0 )
+		return stat.st_mtime;
+	else	return -1;
+}
 
 char *freadfile(FILE *fp,int *sizep)
 {	FILE *tmp;
@@ -804,14 +908,14 @@ char *freadfile(FILE *fp,int *sizep)
 
 	if( size = *sizep ){
 		data = (char*)malloc(size+1);
-		rcc = fread((char*)data,1,size,fp);
+		rcc = fread((char*)data,1,size,fp); /**/
 	}else{
 		tmp = TMPFILE("freadfile");
 		size = copyfile1(fp,tmp);
 		fflush(tmp);
 		fseek(tmp,0,0);
 		data = (char*)malloc(size+1);
-		rcc = fread((char*)data,1,size,tmp);
+		rcc = fread((char*)data,1,size,tmp); /**/
 		fclose(tmp);
 	}
 	((char*)data)[rcc] = 0; /**/
@@ -819,25 +923,6 @@ char *freadfile(FILE *fp,int *sizep)
 	return (char*)data;
 }
 
-void ftouch(FILE *fp,int time)
-{	int off;
-
-	off = ftell(fp);
-	fseek(fp,0,2);
-	fwrite("",1,1,fp);
-	fflush(fp);
-	fseek(fp,-1,2);
-	Ftruncate(fp,0,1);
-	fseek(fp,off,0);
-}
-void File_touch(PCStr(path),int clock)
-{	FILE *fp;
-
-	if( fp = fopen(path,"a") ){
-		ftouch(fp,clock);
-		fclose(fp);
-	}
-}
 char *Fgets(PVStr(str),int size,FILE *fp)
 {	const char *rstr;
 	const char *dp;
@@ -898,14 +983,24 @@ int Fwrite(PCStr(buf),int esize,int ecount,FILE *fp)
 
 static int tmpFileFd = -1;
 static int tmpFilePid = -1;
+static int tmpFileIno = -1;
 static const char *tmpFileWhat = "";
 
 void freeTmpFile(PCStr(what))
 {
+	if( lMULTIST() ){
+		int SocketOf(int);
+		porting_dbg("-- NO freeTmpFile(%s)[%d] S[%d] H[%d]",what,
+			tmpFileFd,SocketOf(tmpFileFd),
+			fd2handle(tmpFileFd));
+		tmpFileFd = -1;
+		return;
+	}
 	if( 0 <= tmpFileFd ){
 		close(tmpFileFd);
 		tmpFileFd = -1;
 		tmpFilePid = -1;
+		tmpFileIno = -1;
 		tmpFileWhat = "";
 	}
 }
@@ -914,13 +1009,33 @@ FILE *getTmpFile(PCStr(what))
 	int pid;
 	CStr(msg,1024);
 
+	if( lMULTIST() ){
+		if( tmpFileFd && tmpFileFd != -1 ){
+			int doDeleteOnClose(int fd,int fh);
+			doDeleteOnClose(tmpFileFd,0);
+		}
+		fp = TMPFILE("getTmpFile");
+		tmpFileFd = fileno(fp);
+		return fp;
+	}
+
 	pid = getpid();
 	sprintf(msg,"getTmpFile: fd=%d [%d]%s->[%d]%s",
 		tmpFileFd,tmpFilePid,tmpFileWhat,pid,what);
 
 	if( 0 < tmpFileFd ){
+		int ino;
+		ino = file_ino(tmpFileFd);
+		/*
 		if( tmpFilePid != pid || tmpFileWhat != what ){
+		*/
+		if( tmpFilePid != pid || tmpFileWhat != what
+		 || tmpFileIno != ino
+		){
+			/*
 			syslog_ERROR("#### %s\n",msg);
+			*/
+			syslog_ERROR("#### %s ino=%d->%d\n",msg,tmpFileIno,ino);
 			tmpFileFd = -1;
 			freeTmpFile(what);
 		}
@@ -930,9 +1045,16 @@ FILE *getTmpFile(PCStr(what))
 		fp = TMPFILE(what);
 		tmpFileFd = dup(fileno(fp));
 		tmpFilePid = pid;
+		tmpFileIno = file_ino(tmpFileFd);
 		tmpFileWhat = what;
 	}else{
 		fp = fdopen(dup(tmpFileFd),"r+");
+		if( fp == NULL ){
+			syslog_ERROR("-- getTmpFile: fd=%d ino=%d errno=%d\n",
+				tmpFileFd,file_ino(tmpFileFd),errno);
+			syslog_ERROR("-- CANNOT OPEN: %s\n",msg);
+			abort();
+		}
 		fseek(fp,0,0);
 	}
 	return fp;
@@ -965,9 +1087,23 @@ FILE *fopen_PATH(const char *pathv[],PCStr(file),PCStr(mode),PVStr(xpath))
 	FILE *fp;
 
 	for( li = 0; dir1 = pathv[li]; li++ ){
+		if( isWindows() && (streq(dir1,"/") || streq(dir1,"\\")) ){
+			/* 9.9.1 retrieving //path might cause delay */
+			if( LOG_VERBOSE )
+			fprintf(stderr,"##ABS fopen_PATH(%s)\n",file);
+			if( *file == '/' || *file == '\\' )
+				strcpy(path1,file);
+			else	sprintf(path1,"/%s",file);
+		}else
 		if( *dir1 == 0 || strcmp(dir1,".") == 0 )
 			strcpy(path1,file);
 		else
+		/* should be like this in 9.2.4 ...
+		 * >>> 9.8.1 done by deltrailslash() in Xfopen()
+		if( *file == 0 ){
+			sprintf(path1,"%s",dir1);
+		}else
+		*/
 		if( strtailstr(dir1,_PATHSEP()) )
 			sprintf(path1,"%s%s",dir1,file);
 		else	sprintf(path1,"%s%s%s",dir1,_PATHSEP(),file);
@@ -981,11 +1117,19 @@ FILE *fopen_PATH(const char *pathv[],PCStr(file),PCStr(mode),PVStr(xpath))
 			fp = TMPFILE("fopen_PATH-DIR");/* dummy for Win32 */
 		}
 
+		if( lPATHFIND() ){
+			fprintf(stderr,"### [%d] %X \"%s\"\n",li,fp,path1);
+		}
 		syslog_DEBUG("### [%d] %s %x\n",li,path1,fp);
 		if( fp != NULL ){
 			if( xpath ){
 				if( *dir1 == 0 || strcmp(dir1,".") == 0 ){
 					getcwd((char*)xpath,256);
+					if( strtailchr(xpath) == '/'
+					 || strtailchr(xpath) == '\\'
+					){
+					 /* 9.8.1 don't make duplicate "//" */
+					}else
 					strcat(xpath,_PATHSEP());
 					strcat(xpath,path1);
 				}
@@ -1003,7 +1147,10 @@ FILE *fopen_PATHLIST(PCStr(pathlist),PCStr(file),PCStr(mode),PVStr(xpath))
 
 	lpathv = vect_PATH(pathlist);
 	fp = fopen_PATH(lpathv,file,mode,AVStr(xpath));
+	freev((char**)lpathv);
+	/*
 	free(lpathv);
+	*/
 	return fp;
 }
 FILE *fopen_LIBPATH(PCStr(file),PCStr(mode),PVStr(xpath))
@@ -1016,7 +1163,13 @@ FILE *fopen_LIBPATH(PCStr(file),PCStr(mode),PVStr(xpath))
 		else	setVStrEnd(xpath,0);
 	}else
 	if( libpath = getenv("LIBPATH") )
+	{
+		if( lPATHFIND() ){
+			fprintf(stderr,"### find '%s' in LIBPATH='%s'\n",
+				file,libpath);
+		}
 		fp = fopen_PATHLIST(libpath,file,mode,AVStr(xpath));
+	}
 	else	fp = 0;
 	return fp;
 }
@@ -1043,3 +1196,391 @@ int LIBFILE_IS(PCStr(file),PVStr(rxfile))
 	}
 	return 0;
 }
+
+/*
+ * Normalize a path name to be compatible between Unix and Windows:
+ *
+ *   ""      -> "."    (?)
+ *   "X:"    -> "X:/"
+ *   "path/" -> "path" stati64()/Win rejects this even for a directory
+ *                     stat()/Win and Solaris allows this even for a flat file
+ */
+int deltrailslash(PCStr(ipath),PVStr(pb)){
+	int del = 0;
+	int len;
+
+	if( strtailchr(ipath) != '/' && strtailchr(ipath) != '\\' )
+		return 0;
+
+	strcpy(pb,ipath);
+	while( strtailchr(pb) == '/' || strtailchr(pb) == '\\' ){
+		len = strlen(pb);
+		if( isalpha(pb[0]) && pb[1] == ':' ){ /* full path */
+			if( len <= 3 ){
+				break;
+			}
+		}else
+		if( pb[0] == '/' || pb[0] == '\\' ){  /* absolute path */
+			if( len <= 1 ){
+				break;
+			}
+		}else{                                /* relative path */
+			if( len <= 1 ){
+				/* never happen */
+				break;
+			}
+		}
+		del++;
+		setVStrEnd(pb,len-1);
+	}
+	if( del ){
+		if( fileIsdir(pb) ){
+			syslog_ERROR("--deltrailslash(%s) %d\n",pb,del);
+			return del;
+		}
+	}
+	return 0;
+}
+int pathnorm(PCStr(what),PCStr(path),PVStr(xpath)){
+	const char *npath = 0;
+	if( isWindows() ){
+		if( path[0] == 0 ){
+			/*
+			strcpy(xpath,".");
+			npath = xpath;
+			*/
+		}else
+		if( isalpha(path[0]) && path[1] == ':' && path[2] == 0 ){
+			sprintf(xpath,"%s/",path);
+			npath = xpath;
+		}else
+		if( deltrailslash(path,AVStr(xpath)) ){
+			npath = xpath;
+		}else
+		if( strtailchr(path) == '/' ){
+		/* should be like this in 9.2.4 ...
+		 * >>> 9.8.1 done by deltrailslash()
+		if( strtailchr(path) == '/' || strtailchr(path) == '\\' ){
+		*/
+			if( isalpha(path[0]) && path[1]==':' && 3 < strlen(path)
+			 || (path[0]=='/' || path[0]=='\\') && 2 < strlen(path)
+			){ 
+				strcpy(xpath,path);
+				setVStrEnd(xpath,strlen(xpath)-1);
+				npath = xpath;
+			}
+		}
+	}
+	if( npath != 0 ){
+		if( lDEBUGMSG() )
+		porting_dbg("pathnorm(%s)[%s]->[%s]",what,path,xpath);
+		return 1;
+	}else	return 0;
+}
+
+#ifdef statX
+int Xstat(PCStr(path),FileStat *st){
+	int rcode;
+	CStr(xpath,1024);
+
+	if( pathnorm("Xstat",path,AVStr(xpath)) )
+		path = xpath;
+	rcode = statX(path,st);
+	return rcode;
+}
+#endif
+
+/*
+ * opening large-file
+ * checking off-limit access
+ * "/dev/null" / "nul" selection
+ * freopen() should also be redirected
+ * ":" to ".." conversion on DOS file system
+ * bare open() should not used
+ */
+FILE *XXfopen(const char *F,int L,const char *path,const char *mode);
+#ifdef fopen
+#undef fopen
+#endif
+int fopen_debug = 0;
+#ifdef O_LARGEFILE
+FILE *large_fopen(PCStr(path),PCStr(mode)){
+	int fd;
+	int flag;
+	FILE *fp;
+
+	if( strchr(mode,'a') ){
+		if( strchr(mode,'+') )
+			flag = O_CREAT|O_APPEND|O_RDWR;
+		else	flag = O_CREAT|O_APPEND|O_WRONLY;
+	}else
+	if( strchr(mode,'w') ){
+		if( strchr(mode,'+') )
+			flag = O_CREAT|O_RDWR;
+		else	flag = O_CREAT|O_WRONLY|O_TRUNC;
+	}else{
+		if( strchr(mode,'+') )
+			flag = O_RDWR;
+		else	flag = O_RDONLY;
+	}
+	fd = open(path,O_LARGEFILE|flag,0666);
+	if( 0 <= fd ){
+		fp = fdopen(fd,mode);
+	}else{
+		fp = NULL;
+	}
+	return fp;
+}
+#else
+FILE *large_fopen(PCStr(path),PCStr(mode)){
+	return NULL;
+}
+#endif
+FILE *Xfopen(const char *F,int L,PCStr(path),PCStr(mode)){
+	FILE *fp;
+	CStr(xpath,1024);
+
+	/*
+	 * 9.8.1 enabled. this pathnorm() is added but disabled (why?)
+	 * in 9.2.2-pre3 (060526)
+	 */
+	if( pathnorm("Xfopen",path,AVStr(xpath)) ){
+		syslog_ERROR("Xfopen(%s)->(%s)\n",path,xpath);
+		path = xpath;
+	}
+
+#if defined(O_LARGEFILE) && defined(STAT64)
+	if( fp = large_fopen(path,mode) ){
+		return fp;
+	}
+#endif
+	/*
+	if( isWindowsCE() ){
+		fp = XX_fopen(path,mode);
+	}else
+	fp = fopen(path,mode);
+	*/
+	fp = XXfopen(F,L,path,mode);
+#if defined(O_LARGEFILE) && defined(EFBIG)
+	if( fopen_debug ){
+		fprintf(stderr,"---- %X (%s) %s\n",fp,mode,path);
+	}
+	if( fp == NULL ){
+		if( errno == EFBIG || errno == EOVERFLOW ){
+			fp = large_fopen(path,mode);
+		}
+	}
+#endif
+	/* should be like this in 9.2.4 ...
+	if( fp == NULL && isWindows() && strchr(mode,'r') ){
+		if( strtailstr(path) == '/' || strtailstr(path) == '\\' ){
+			if( fileIsdir(path) ){
+				errno = EACCES;
+			}
+		}
+	}
+	*/
+	if( lFILEOPEN() ){
+		fprintf(stderr,"-- -dO %8X fopen(%s,%s)\n",fp,path,mode);
+	}
+	return fp;
+}
+
+#ifdef fdopen
+#undef fdopen
+#endif
+FILE *XXfdopen(const char *F,int L,int fd,const char *mode);
+int daemonlog(const char *what,const char *fmt,...);
+FILE *Xfdopen(PCStr(F),int L,int fd,PCStr(mode)){
+	FILE *fp;
+	fp = XXfdopen(F,L,fd,mode);
+	if( fp == NULL ){
+		daemonlog("F","--FATAL(%s:%d) Xfdopen(%d,%s)=0\n",F,L,fd,mode);
+	}
+	return fp;
+}
+
+#ifdef fwrite
+#undef fwrite
+#endif
+int Xfwrite(const char *F,int L,const void *b,int z,int n,FILE *f){
+	int ne;
+	int pfe = ferror(f);
+
+	if( isWindowsCE() ){
+		ne = XX_fwrite(F,L,b,z,n,f);
+	}else
+	ne = fwrite(b,z,n,f);
+
+	if( lDEBUGMSG() )
+	if( (0 != n && ne <= 0) || ferror(f) )
+	if( f == NullFP ){
+	}else
+	{
+		fprintf(stderr,
+		"-dD[%d] Xfwrite(%d,%d,%d)=%d, ferror()=%d (%d), errno=%d\n",
+			getpid(),z,n,fileno(f),ne,ferror(f),pfe,errno);
+	}
+
+	if( 0 < ne ){
+		if( ferror(f) ){
+			if( lDEBUGMSG() )
+			fprintf(stderr,"(Win) fwrite()=%d -> %d, errno=%d\n",
+				ne,0,errno);
+			ne = 0;
+		}
+	}
+	return ne;
+}
+
+#ifdef fputs
+#undef fputs
+#endif
+int Xfputs(const char *s,FILE *f){
+	int rcode;
+	rcode = fputs(s,f);
+
+	if( rcode != EOF ){
+		if( ferror(f) ){
+			if( lDEBUGMSG() )
+			fprintf(stderr,"(Win) fputs()=%d -> %d\n",rcode,EOF);
+			rcode = EOF;
+		}
+	}
+	return rcode;
+}
+
+#ifdef fclose
+#undef fclose
+#endif
+int Ofclose(FILE *fp){
+	return fclose(fp);
+}
+void del_FILEY(FL_PAR,const char *wh,FILE *fp);
+int XXfclose(const char *F,int L,FILE *fp);
+int Xfclose(const char *F,int L,FILE *fp){
+	if( NullFP ){
+		if( lDEBUGMSG() )
+		if( fp == NullFP ){
+			fprintf(stderr,"[%d] %X %s:%d -- Xfclose(%X)\n",
+				getpid(),NullFP,F,L,fp);
+		}
+		/* should suppress fclose(NULLFP()) ... */
+	}
+	if( isWindows()  /* any file must be closed by XX_fclose() */
+	 || numthreads() /* must be done under mutex */
+	){
+		int rcode;
+		rcode = XXfclose(F,L,fp);
+		return rcode;
+	}
+	del_FILEY(F,L,"Xfclose",fp);
+	return fclose(fp);
+}
+
+
+#if defined(_MSC_VER) || defined(__CYGWIN__) || defined(__EMX__)
+int FS_dosPath = 1;
+#define IsSep(ch)		(ch == '/' || ch == '\\')
+#define FindDirSep(path)	strpbrk(path,"/\\")
+#define FindRDirSep(path)	strrpbrk(path,"/\\")
+#else
+int FS_dosPath = 0;
+#define IsSep(ch)		(ch == '/')
+#define FindDirSep(path)	strchr(path,'/')
+#define FindRDirSep(path)	strrchr(path,'/')
+#endif
+
+int isFullpath(PCStr(path));
+void chdir_cwd(PVStr(cwd),PCStr(go),int userdir)
+{	refQStr(tp,cwd); /**/
+	const char *cp;
+	int uplen,len;
+
+	for( cpyQStr(tp,cwd); *tp; tp++ ){}
+	if( len = isFullpath(go) ){
+		tp = (char*)cwd;
+		while( 0 < len-- ){
+			assertVStr(cwd,tp+1);
+			setVStrPtrInc(tp,*go++);
+		}
+	}else
+	if( userdir && go[0] == '~' ){
+		if( go[1] == 0   ){ tp = (char*)cwd; go += 1; }else
+		if( IsSep(go[1]) ){ tp = (char*)cwd; go += 2; }
+	}
+
+	while( *go ){
+		uplen = 0;
+		if( go[0] == '.' ){
+			if( go[1] == 0   ){ go += 1; break; }
+			if( IsSep(go[1]) ){ go += 2; continue; }
+			if( go[1] == '.' ){
+				if( go[2] == 0   ){ uplen = 2; } else
+				if( IsSep(go[2]) ){ uplen = 3; }
+			}
+		}
+		if( uplen ){
+			if( cwd < tp ){
+				while( cwd+1 < tp ){
+					if( IsSep(tp[-1]) )
+						tp--;
+					else	break;
+				}
+				setVStrEnd(tp,0);
+				if( cp = FindRDirSep(cwd) ){
+					if( cp == cwd  )
+						tp = (char*)cp + 1;
+					else	tp = (char*)cp;
+				}else	tp = (char*)cwd;
+			}
+			go += uplen;
+		}else{
+			if( cp = FindDirSep(go) )
+				len = (cp - go) + 1;
+			else	len = strlen(go);
+
+			if( cwd < tp && !IsSep(tp[-1]) ){
+				setVStrPtrInc(tp,'/');
+			}
+			strncpy(tp,go,len);
+			tp += len;
+			go += len;
+		}
+	}
+	while( cwd+1 < tp ){
+		if( IsSep(tp[-1]) )
+			tp--;
+		else	break;
+	}
+	setVStrEnd(tp,0);
+}
+
+#if !isWindowsCE()
+int usedFDX(PCStr(F),int L,int usedfd){
+	int fd;
+
+	fd = dup(0);
+	close(fd);
+	if( fd != usedfd ){
+		/*fprintf(stderr,"SETFD[%d]\n",usedfd);*/
+		syslog_DEBUG("usedFD():setCloseOnExec(%d)\n",fd);
+
+		/*
+		fprintf(stderr,"----[%d] %s:%d usedFD():setCloseOnExec(%d)\n",
+			getpid(),F,L,fd);
+		dumpFds(stderr);
+		*/
+		setCloseOnExec(usedfd);
+		return 1;
+	}
+	return 0;
+}
+int nextFDX(PCStr(F),int L){
+	int fd;
+
+	fd = dup(0);
+	close(fd);
+	return fd;
+}
+#endif
