@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1995 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1995-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -53,12 +54,20 @@ int _fPollIns(int timeout,int fpc,FILE *fps[],int rdv[])
 				if( ch != EOF ){
 					ungetc(ch,fps[fi]);
 				}else{
+					/*
 					if( errno = EAGAIN ){
+					*/
+					if( errno == EAGAIN ){
 syslog_ERROR("## _fPollIns(%d) pop_fd %d\n",fd,nready);
 						clearerr(fp);
 						rdv[fi] = 0;
 						nready--;
 					}
+					else
+					if( fpop_fd(fp) ){
+syslog_ERROR("## _fPollIns(%d) fpop_fd %d\n",fd,nready);
+					return _fPollIns(timeout,fpc,fps,rdv);
+					}
 				}
 			}
 		}
