@@ -93,6 +93,17 @@ void Hdestroy(int htid)
 {
 }
 
+unsigned long elfhash(unsigned char *key){
+	unsigned long kx = 0;
+	unsigned long g;
+	while( *key ){
+		kx = (kx << 4) + *key++;
+		g = kx & 0xF0000000L;
+		if( g ) kx ^= g >> 24;
+		kx &= ~g;
+	}
+	return kx;
+}
 static int index1(unsigned char *key)
 {	const unsigned char *ks;
 	int kc,kx;
@@ -105,6 +116,7 @@ static int index1(unsigned char *key)
 /*
 #define index1(key)	FQDN_hash(key)
 */
+#define index1(key)	elfhash(key)
 
 const char *Hsearch(int htid,PCStr(key),PCStr(data))
 {	HTable *ht;
