@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1997 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1997-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -18,8 +19,12 @@ Description:
 History:
 	970514	extracted from misc.c
 //////////////////////////////////////////////////////////////////////#*/
+#include <errno.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include "ystring.h"
+int setCloseOnExec(int fd);
+int pam_auth1(PCStr(service),PCStr(user),PCStr(pass));
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -37,28 +42,8 @@ static struct group group0 = { "?", "", 0, 0 };
 #define getgrnam(gid)	(&group0)
 #endif
 
-
-usedFD(usedfd)
-{	int fd;
-
-	fd = dup(0);
-	close(fd);
-	if( fd != usedfd ){
-		/*fprintf(stderr,"SETFD[%d]\n",usedfd);*/
-		setCloseOnExec(usedfd);
-	}
-}
-nextFD(){
-	int fd;
-
-	fd = dup(0);
-	close(fd);
-	return fd;
-}
-
-
 /* Solaris2.X leaves NIS+? relevant fd be opened after exec() */
-struct passwd *GETpwuid(uid)
+struct passwd *GETpwuid(int uid)
 {	struct passwd *pw;
 	int fd;
 
@@ -67,7 +52,7 @@ struct passwd *GETpwuid(uid)
 	usedFD(fd);
 	return pw;
 }
-struct group *GETgrgid(gid)
+struct group *GETgrgid(int gid)
 {	struct group *gr;
 	int fd;
 
@@ -76,7 +61,7 @@ struct group *GETgrgid(gid)
 	usedFD(fd);
 	return gr;
 }
-struct passwd *GETpwnam(name) char *name;
+struct passwd *GETpwnam(PCStr(name))
 {	struct passwd *pw;
 	int fd;
 
@@ -85,7 +70,7 @@ struct passwd *GETpwnam(name) char *name;
 	usedFD(fd);
 	return pw;
 }
-struct group *GETgrnam(group) char *group;
+struct group *GETgrnam(PCStr(group))
 {	struct group *gr;
 	int fd;
 
@@ -95,41 +80,48 @@ struct group *GETgrnam(group) char *group;
 	return gr;
 }
 
-char *getUsername(uid,name)
-	char *name;
+char *getUsername(int uid,PVStr(name))
 {	struct passwd *passwd;
 
 	if( passwd = GETpwuid(uid) )
 		strcpy(name,passwd->pw_name);
 	else	sprintf(name,"#%d",uid);
-	return name;
+	return (char*)name;
 }
 
-static int LastUid = -1;
-static char LastUname[32];
-char *getUsernameCached(uid,name)
-	char *name;
+typedef struct {
+	int ue_LastUid;
+	MStr(ue_LastUname,32);
+} UidnameCache;
+static UidnameCache *uidnameCache;
+#define LastUid   uidnameCache->ue_LastUid
+#define LastUname uidnameCache->ue_LastUname
+/**/
+
+char *getUsernameCached(int uid,PVStr(name))
 {
+	if( uidnameCache == 0 ){
+		uidnameCache = NewStruct(UidnameCache);
+		LastUid = -1;
+	}
 	if( uid == LastUid && LastUname[0] != 0 )
 		strcpy(name,LastUname);
 	else{
-		getUsername(uid,name);
+		getUsername(uid,AVStr(name));
 		strcpy(LastUname,name);
 		LastUid = uid;
 	}
-	return name;
+	return (char*)name;
 }
 
-getHOME(uid,home)
-	char *home;
+void getHOME(int uid,PVStr(home))
 {	struct passwd *passwd;
 
 	if( passwd = GETpwuid(uid) )
 		strcpy(home,passwd->pw_dir);
 	else	strcpy(home,"/");
 }
-getSHELL(uid,shell)
-	char *shell;
+int getSHELL(int uid,PVStr(shell))
 {	struct passwd *passwd;
 
 	if( passwd = GETpwuid(uid) ){
@@ -138,39 +130,79 @@ getSHELL(uid,shell)
 	}
 	return 0;
 }
-char *getGroupname(gid,name)
-	char *name;
+char *getGroupname(int gid,PVStr(name))
 {	struct group *group;
 
 	if( group = GETgrgid(gid) )
 		strcpy(name,group->gr_name);
 	else	sprintf(name,"#%d",gid);
-	return name;
+	return (char*)name;
 }
-char *getusernames(names)
-	char *names;
-{	char uname[32],euname[32];
-	char gname[32],egname[32];
+char *getusernames(PVStr(names))
+{	CStr(uname,32);
+	CStr(euname,32);
+	CStr(gname,32);
+	CStr(egname,32);
 
 	uname[0] = euname[0] = 0;
-	getUsername(getuid(),uname);
-	getUsername(geteuid(),euname);
+	getUsername(getuid(),AVStr(uname));
+	getUsername(geteuid(),AVStr(euname));
 
 	gname[0] = egname[0] = 0;
-	getGroupname(getgid(),gname);
-	getGroupname(getegid(),egname);
+	getGroupname(getgid(),AVStr(gname));
+	getGroupname(getegid(),AVStr(egname));
 
 	sprintf(names,"%s/%s(%s/%s)",uname,gname,euname,egname);
-	return names;
+	return (char*)names;
+}
+
+int getUserId(PCStr(user))
+{	struct passwd *passwd;
+
+	if( passwd = GETpwnam(user) )
+		return passwd->pw_uid;
+	else	return -1;
+}
+int getGroupId(PCStr(gname))
+{	struct group *group;
+
+	if( group = GETgrnam(gname) )
+		return group->gr_gid;
+	else	return -1;
+}
+
+int owner_main(int ac,const char *av[]){
+	struct passwd *pw;
+	int id;
+	IStr(name,128);
+
+	id = getuid();
+	errno = 0;
+	pw = getpwuid(id);
+	printf("getpwuid(%d)=%X, errno=%d\n",id,pw,errno);
+	if( pw == 0 && errno == 0 ){
+		endpwent(); /* for MacOSX ? */
+		setpwent();
+		pw = getpwuid(id);
+		endpwent();
+		printf(">>> getpwuid(%d)=%X, errno=%d\n",id,pw,errno);
+	}
+
+	printf("User: %s (%d)\n",getUsername(id,AVStr(name)),id);
+	id = geteuid();
+	printf("E-User: %s (%d)\n",getUsername(id,AVStr(name)),id);
+	id = getgid();
+	printf("Group: %s (%d)\n",getGroupname(id,AVStr(name)),id);
+	return 0;
 }
 
 /*
  *	user_group:	user / group ( euser / egroup )
  */
-scan_guid(user_group,uidp,gidp)
-	char *user_group;
-	int *uidp,*gidp;
-{	char *dp,user[256],group[256];
+int scan_guid(PCStr(user_group),int *uidp,int *gidp)
+{	const char *dp;
+	CStr(user,256);
+	CStr(group,256);
 	struct passwd *pw;
 	struct group *gr;
 	int len;
@@ -178,9 +210,8 @@ scan_guid(user_group,uidp,gidp)
 
 	if( dp = strchr(user_group,'/') ){
 		if( len = dp - user_group ){
-			strncpy(user,user_group,len);
-			user[len] = 0;
-		}else	getUsername(getuid(),user);
+			strncpy(user,user_group,len); setVStrEnd(user,len);
+		}else	getUsername(getuid(),AVStr(user));
 		strcpy(group,dp+1);
 	}else{
 		strcpy(user,user_group);
@@ -210,18 +241,24 @@ scan_guid(user_group,uidp,gidp)
 	return 0;
 }
 
-static struct passwd _pwuid;
-struct passwd *getpwuid_nis(map,domain,uid)
-	char *map,*domain;
-{	char *ypdomain,*key,*val,*vp;
-	char uids[32];
+static struct passwd *_pwuids;
+#define _pwuid	_pwuids[0]
+struct passwd *getpwuid_nis(PCStr(map),PCStr(domain),int uid)
+{	const char *ypdomain;
+	const char *key;
+	const char *val;
+	const char *vp;
+	CStr(uids,32);
 	int vlen;
 	int rcode;
 
+	if( _pwuids == 0 )
+		_pwuids = NewStruct(struct passwd);
+
 	if( _pwuid.pw_name ){
 		if( _pwuid.pw_uid == uid )
 			return &_pwuid;
-		free(_pwuid.pw_name);
+		free((char*)_pwuid.pw_name);
 		bzero(&_pwuid,sizeof(struct passwd));
 	}
 
@@ -233,7 +270,7 @@ struct passwd *getpwuid_nis(map,domain,uid)
 		ypdomain = domain;
 	else{
 		ypdomain = NULL;
-		rcode = yp_get_default_domain(&ypdomain);
+		rcode = yp_get_default_domain((char**)&ypdomain);
 		if( rcode != 0 || ypdomain == NULL || *ypdomain == 0 )
 			return NULL;
 		if( strcmp(ypdomain,"(none)") == 0 )
@@ -241,36 +278,36 @@ struct passwd *getpwuid_nis(map,domain,uid)
 	}
 
 	sprintf(uids,"%d",uid);
-	rcode = yp_match(ypdomain,map,uids,strlen(uids),&val,&vlen);
+	rcode = yp_match(ypdomain,(char*)map,uids,strlen(uids),(char**)&val,&vlen);
 	if( rcode != 0 )
 		return NULL;
 
-	val[vlen] = 0;
+	((char*)val)[vlen] = 0; /* is this safe ? */
 
-	_pwuid.pw_name = vp = strdup(val);
+	_pwuid.pw_name = (char*)(vp = stralloc(val));
 	if( vp = strchr(vp,':') ){
-		*vp++ = 0;
-		_pwuid.pw_passwd = vp;
+		truncVStr(vp); vp++;
+		_pwuid.pw_passwd = (char*)vp;
 	if( vp = strchr(vp,':') ){
-		*vp++ = 0;
+		truncVStr(vp); vp++;
 		_pwuid.pw_uid = atoi(vp);
 	if( vp = strchr(vp,':') ){
-		*vp++ = 0;
+		truncVStr(vp); vp++;
 		_pwuid.pw_gid = atoi(vp);
 	if( vp = strchr(vp,':') ){
-		*vp++ = 0;
-		_pwuid.pw_gecos = vp;
+		truncVStr(vp); vp++;
+		_pwuid.pw_gecos = (char*)vp;
 	if( vp = strchr(vp,':') ){
-		*vp++ = 0;
-		_pwuid.pw_dir = vp;
+		truncVStr(vp); vp++;
+		_pwuid.pw_dir = (char*)vp;
 	if( vp = strchr(vp,':') ){
-		*vp++ = 0;
-		_pwuid.pw_shell = vp;
+		truncVStr(vp); vp++;
+		_pwuid.pw_shell = (char*)vp;
 	}}}}}}
 
 	return &_pwuid;
 }
-struct passwd *Getpwuid(uid)
+struct passwd *Getpwuid(int uid)
 {	struct passwd *pw;
 
 	if( pw = getpwuid(uid) )
@@ -279,8 +316,7 @@ struct passwd *Getpwuid(uid)
 }
 
 #ifdef MAIN
-main(ac,av)
-	char *av[];
+main(int ac,char *av[])
 {	struct passwd *pp;
 
 	if( pp = Getpwuid(atoi(av[1])) ){
@@ -294,3 +330,6 @@ main(ac,av)
 	}
 }
 #endif
+
+#define PAM_BY_MAIN_ONLY
+#include "pam.c"
