@@ -20,17 +20,28 @@ History:
 	991205	simplified and no exec version
 //////////////////////////////////////////////////////////////////////#*/
 #include <stdio.h>
-#include <string.h>
+#include "ystring.h"
+int IsBOW1_5();
+int FS_maybeUnix();
+
 int PSTITLE_DISABLE;
-char *pstitle_area;
+struct { defQStr(p); } pstitle_area;
 int   pstitle_size;
 int   pstitle_leng;
 int   pstitle_lengmax = 2048;
 
-#ifdef hpux
+#if defined(__hpux__)
 #include <sys/pstat.h>
-static p_title(buf)
-	char *buf;
+static int p_title(PCStr(buf)){
+	union pstun un;
+	un.pst_command = (char*)buf;
+	pstat(PSTAT_SETCMD,un,strlen(buf),0,0);
+	return 0;
+}
+#else
+#if defined(hpux)
+#include <sys/pstat.h>
+static int p_title(PCStr(buf))
 {
 	pstat(PSTAT_SETCMD,buf,0,0,0);
 	return 0;
@@ -39,27 +50,28 @@ static p_title(buf)
 #if defined(__sony_news) && defined(_SYSTYPE_SYSV)
 #include <sys/sysnews.h>
 #include <sys/sysmips.h>
-static p_title(buf)
-	char *buf;
+static int p_title(PCStr(buf))
 {
 	sysmips(SONY_SYSNEWS,NEWS_SETPSARGS,buf);
 	return 0;
 }
 #else
-static p_title(buf)
-	char *buf;
+static int p_title(PCStr(buf))
 {
-	if( pstitle_area ){
-		Strncpy(pstitle_area,buf,pstitle_size);
+	if( Setproctitle("-%s",buf) == 0 )
+		return 0;
+	if( pstitle_area.p ){
+		XStrncpy(AVStr(pstitle_area.p),buf,pstitle_size); /**/
 		return 0;
 	}else	return -1;
 }
 #endif
 #endif
+#endif
 
-proc_title(fmt,a,b,c,d,e,f,g)
-	char *fmt,*a,*b,*c,*d,*e,*f,*g;
-{	char buf[2048];
+int proc_title(PCStr(fmt),...)
+{	CStr(buf,2048);
+	VARGS(8,fmt);
 
 	if( PSTITLE_DISABLE )
 		return 0;
@@ -69,12 +81,12 @@ proc_title(fmt,a,b,c,d,e,f,g)
 		return 0;
 	}
 
-	sprintf(buf,fmt,a,b,c,d,e,f,g);
+	sprintf(buf,fmt,VA8);
 	if( sizeof(buf) <= strlen(buf) ){
-		sv1tlog("PSTITLE overflow: %d / %d / %d > %d\n",
+		syslog_ERROR("PSTITLE overflow: %d / %d / %d > %d\n",
 			strlen(buf),pstitle_size,pstitle_leng,sizeof(buf));
 		buf[sizeof(buf)-1] = 0;
-		sv1tlog("PSTITLE overflow: %s\n",buf);
+		syslog_ERROR("PSTITLE overflow: %s\n",buf);
 		Finish(-1);
 	}
 	return p_title(buf);
