@@ -1,11 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1999 Yutaka Sato
-Copyright (c) 1999 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1999-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, and distribute this material for any purpose
-and without fee is hereby granted, provided that the above copyright
-notice and this permission notice appear in all copies.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -23,21 +26,40 @@ ToDo:
 #include "yalloca.h"
 #include "ystring.h"
 #include "fcntl.h"
+#include "log.h"
 long Gettimeofday(int *usec);
 int IsOS2EMX();
 
-int RAND_TRACE = 0;
 int RANDHEAP_SIZE = 64*1024;
 int RANDSTACK_RANGE = 64;
 int RANDSTACK_UNIT = 128;
 static unsigned int prev_rand;
+int strCRC32(PCStr(str),int len);
 
+/*
 static unsigned int trand1(unsigned int max)
+*/
+unsigned int trand1(unsigned int max)
 {	unsigned int sec,usec;
 	int nrand;
+	struct {
+		int prand;
+		int sec;
+		int usec;
+		int pid;
+	} seed;
+	unsigned int nrando;
 
+	/*
 	sec = Gettimeofday((int*)&usec);
 	nrand = ((prev_rand<<8) ^ (usec >> 10)) % max;
+	*/
+	bzero(&seed,sizeof(seed));
+	seed.prand = prev_rand;
+	seed.sec = Gettimeofday((int*)&seed.usec);
+	seed.pid = getpid();
+	nrando = strCRC32((const char*)&seed,sizeof(seed));
+	nrand = nrando % max;
 	prev_rand = nrand;
 	return nrand;
 }
@@ -55,16 +77,17 @@ int randstack_call(int strg,iFUNCP func, ...)
 {	AllocaArg arg;
 	unsigned int size;
 	int rcode;
-	VARGS(4,func);
+	VARGS(8,func);
 
 	if( RANDSTACK_RANGE == 0 )
 		size = 0;
 	else	size = trand1(RANDSTACK_RANGE);
 	arg.s_what = "#### RANDSTACK";
-	arg.s_sp0 = (unsigned int)&func;
+	arg.s_sp0 = (char*)&func;
 	arg.s_func = func;
 	arg.s_av[0] = va[0]; arg.s_av[1] = va[1];
 	arg.s_av[2] = va[2]; arg.s_av[3] = va[3];
+	arg.s_av[4] = va[4]; arg.s_av[5] = va[5];
 	arg.s_size = size;
 	arg.s_unit = RANDSTACK_UNIT;
 	arg.s_count = size;
@@ -74,6 +97,29 @@ int randstack_call(int strg,iFUNCP func, ...)
 	delStrBuffer(strg);
 	return rcode;
 }
+int allocaCall(PCStr(what),int size,iFUNCP func,...){
+	AllocaArg aa;
+	int rcode;
+	VARGS(8,func);
+
+	aa.s_what = (char*)what;
+	aa.s_sp0 = (char*)&func;
+	aa.s_func = func;
+	aa.s_av[0] = (char*)va[0];
+	aa.s_av[1] = (char*)va[1];
+	aa.s_av[2] = (char*)va[2];
+	aa.s_av[3] = (char*)va[3];
+	aa.s_av[4] = (char*)va[4];
+	aa.s_av[5] = (char*)va[5];
+	aa.s_size = size / STACK1;
+	aa.s_unit = STACK1;
+	aa.s_count = size / STACK1;
+	aa.s_trace = RAND_TRACE;
+	aa.s_level = SB_SERV;
+	rcode = alloca_call(&aa);
+	delStrBuffer(SB_SERV);
+	return rcode;
+}
 
 int RANDENV_RANGE = 1024;
 struct { defQStr(randenv_buff); } randenv_buff;
@@ -83,6 +129,12 @@ void randenv()
 {	int randen,ex;
 	char *rande; /**/
 
+	if( isWindowsCE() ){
+		return;
+	}
+	if( numthreads() ){
+		return;
+	}
 	if( RANDENV_RANGE <= 0 )
 		return;
 
@@ -134,11 +186,24 @@ static int getrandfd(int tfd,int rand)
 	}
 	return rfd;
 }
+int getNullFd(PCStr(what));
 int randfd(int fd)
 {	unsigned int foff;
 	int tfd,rfd;
 	FILE *tmp;
 
+	if( isWindowsCE() ){
+		/* dup(), dup2() for socket is NG ? with "nul" ?
+		 * and randfd() is realized more effecitivry in WinCE 
+		 */
+		return fd;
+	}
+	if( numthreads() ){
+		return fd;
+	}
+	if( isWindows95() ){
+		return fd;
+	}
 	if( fd < 0 )
 		return fd;
 
@@ -148,7 +213,20 @@ int randfd(int fd)
 /*
 	tmp = fopen("/dev/null","r");
 */
+/*
 	tfd = open("/dev/null",0); tmp = fdopen(tfd,"r");
+*/
+	/*
+	if( isWindows() )
+		tfd = open("nul",0);
+	else	tfd = open("/dev/null",0);
+	*/
+	tfd = getNullFd("randfd");
+	if( 0 <= tfd )
+		tfd = dup(tfd);
+	if( 0 <= tfd )
+		tmp = fdopen(tfd,"r");
+	else	tmp = NULL;
 
 	if( tmp == NULL )
 		tmp = tmpfile();
@@ -196,6 +274,10 @@ char **move_envarg(int ac,const char *av[],const char **areap,int *lengp,int *si
 		 */
 		return (char**)av;
 	}
+	if( isWindowsCE() ){
+		/* no environ */
+		return (char**)av;
+	}
 
 	/*
 	 * move environ[0-] to heap area
