@@ -1,16 +1,16 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1992 Electrotechnical Laboratry (ETL)
+Copyright (c) 1992-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material
-for any purpose and without fee is hereby granted, provided
-that the above copyright notice and this permission notice
-appear in all copies, and that the name of ETL not be
-used in advertising or publicity pertaining to this
-material without the specific, prior written permission
-of an authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY
-OF THIS MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS",
-WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES.
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
+OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
 Content-Type: program/C; charset=US-ASCII
 Program:      str_stdio.h
@@ -31,17 +31,12 @@ typedef unsigned char Uchar;
 char *raw_Strncpy(PVStr(s1),PCStr(s2),int n);
 
 #define Str_MAGIC 0x12345678
-typedef struct _StrHead {
-	int	sh_magic;
-	MStr(	sh_id,16);
-	MStr(	sh_mode,16);
-	defQStr(sh_base);
-	int	sh_peak;
-	int	sh_maxsize;		/* limit of auto expansion */
-	int	sh_size;
-} StrHead;
 typedef struct _String {
+#if isWindowsCE()
+	int	s_FILE[32];
+#else
 	FILE	s_FILE;
+#endif
 	StrHead	s_STRING;
 } String;
 #define s_magic	s_STRING.sh_magic
@@ -303,7 +298,7 @@ int Str_sprintf(StrHead *StrH,PCStr(fmt), ...)
 					&dp[leng] - (Uchar*)StrH->sh_base);
 				leng = xp - dp - 4;
 				if( 0 < leng ){
-					bcopy(v1,dp,leng);
+					bcopy(v1,dp,leng); /**/
 					Xstrcpy(QVStr(dp+leng,StrH->sh_base),"=-=\n");
 					dp += leng + 4;
 				}
