@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1995 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1995-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -59,23 +60,38 @@ int strid_create(int nelem)
 	htid = Hcreate(nelem,NULLVAL);
 	return htid;
 }
-int strid(int tab,PCStr(str),int id)
+long int stridX(int tab,PCStr(str),long int id,const char **rkey);
+long int strid(int tab,PCStr(str),long int id)
+{
+	return stridX(tab,str,id,NULL);
+}
+long int stridX(int tab,PCStr(str),long int id,const char **rkey)
 {	const char *sid;
+	const char *key;
 
 	if( tab <= 0 )
+	{
+		if( rkey ) *rkey = 0;
 		return id;
+	}
 
 	sid = Hsearch(tab,str,NULLVAL);
 	if( sid != NULLVAL ){
-		if( (int)sid == id )
+		if( (long int)sid == id )
 			s_uni++;
 		else	s_dup++;
-		return (int)sid;
+		if( rkey ) *rkey = 0;
+		return (long int)sid;
 	}
+/*
 	if( Hsearch(tab,newStr(str),(char*)id) != NULLVAL )
+*/
+	key = newStr(str);
+	if( Hsearch(tab,key,(char*)id) != NULLVAL )
 		s_new++;
 	else	s_full++;
 
+	if( rkey ) *rkey = key;
 	return id;
 }
 int strid_next(int tab,int hx,const char **strp,int *idp)
@@ -98,3 +114,17 @@ const char *strid_find(int tab,int hx,int id)
 	}
 	return 0;
 }
+
+static int _strid;
+const char *strid_alloc(PCStr(str)){
+	const char *key;
+
+	if( _strid == 0 )
+		_strid = strid_create(128);
+	key = Hsearch(_strid,str,NULLVAL);
+	if( key == NULLVAL ){
+		key = newStr(str);
+		Hsearch(_strid,(char*)key,(char*)key);
+	}
+	return key;
+}
