@@ -24,6 +24,7 @@ History:
 #include "yarg.h"
 #include "ystring.h"
 #include "vsignal.h"
+#include "proc.h"
 void Usleep(int);
 
 typedef struct {
@@ -32,6 +33,8 @@ typedef struct {
 	vfuncp	func;
 	vfuncp	osig;
 	time_t	time;
+	int	tid;  /* the thread to receive the SIGALRM */
+	int	stid; /* original receiver thread of the SIGALRM */
 } Timer;
 
 typedef struct {
@@ -60,6 +63,18 @@ static void sigALRM(int a)
 	if( 0 < timeSp ){
 		tp = &timeStack[timeSp-1];
 dprintf(stderr,"!!sigALRM: %s\n",tp->what);
+		if( actthreads() ){
+			if( tp->tid != getthreadid() ){
+				tp->stid = getthreadid();
+				Vsignal(SIGALRM,sigALRM);
+				thread_kill(tp->tid,SIGALRM);
+				return;
+			}
+			if( tp->stid ){
+				syslog_ERROR("[%X] SIGALRM forwarded from %X\n",
+					getthreadid(),tp->stid);
+			}
+		}
 		if( tp->func != NULL )
 			(*tp->func)(a);
 	}
@@ -86,6 +101,8 @@ int pushTimer(PCStr(what),vfuncp func,int intvl)
 
 	sp = timeSp++;
 	tp = &timeStack[sp];
+	tp->tid = getthreadid();
+	tp->stid = 0;
 	tp->what = what;
 	tp->func = func;
 	tp->intvl = intvl;
