@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1994 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1994-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -124,6 +125,7 @@ int url_escapeY(PCStr(src),PVStr(dst),int siz,PCStr(escs),PCStr(sbrk))
 	int nodflt,escspc,escbin,eschtm,escdel;
 	int escqpt; /* %= ... escape with =XX (quoted printable) */
 	int escctl;
+	int escurl = 0;
 	const char *xp;
 	CStr(buf,32);
 	int len;
@@ -140,6 +142,7 @@ int url_escapeY(PCStr(src),PVStr(dst),int siz,PCStr(escs),PCStr(sbrk))
 		if( escs[1] == 'C' ){ escctl = 1; escs += 2; }else
 		if( escs[1] == 'H' ){ escbin = 1; escs += 2; }else
 		if( escs[1] == 'S' ){ escspc = 1; escs += 2; }else
+		if( escs[1] == 'U' ){ escurl = 1; escs += 2; }else
 		if( escs[1] == '&' ){ eschtm = 1; escs += 2; }else
 		if( escs[1] == '=' ){ escqpt = 1; escs += 2; }else
 		if( escs[1] == '#' ){ escdel = 1; escs += 2; }else
@@ -171,7 +174,10 @@ int url_escapeY(PCStr(src),PVStr(dst),int siz,PCStr(escs),PCStr(sbrk))
 		 || escctl && iscntrl(ch)
 		 || escbin && (ch & 0x80)
 		 || escspc && isspace(ch)
+		 || (in2byte == 0||escurl) && strchr(escs,ch) != 0
+		/*
 		 || in2byte == 0 && strchr(escs,ch) != 0
+		*/
 		){
 			if( dp == 0 ){
 				len = sp - src;
@@ -312,3 +318,197 @@ int url_strstrX(PCStr(url),PCStr(pat),int nocase)
 		return ulen;
 	else	return 0;
 }
+
+#define isRESERVED(ch)	(strchr(":/@.?&=+#",ch)!=0)
+/*
+#define isHEX(c) ('0'<=c&&c<='9'  || 'A'<=c&&c<='Z'|| 'a'<=c&&c<='z')
+*/
+#define isHEX(c) ('0'<=c&&c<='9'  || 'A'<=c&&c<='F'|| 'a'<=c&&c<='f')
+static int dec2X(unsigned PCStr(str)){
+	if( isHEX(str[0]) && isHEX(str[1]) ){
+		char xbuf[3];
+		int ch;
+		xbuf[0] = str[0];
+		xbuf[1] = str[1];
+		xbuf[2] = 0;
+		sscanf(xbuf,"%x",&ch);
+		return ch;
+	}else{
+		return -1;
+	}
+}
+static int set2B(PCStr(src),PCStr(sp),int in2byte){
+	if( src+3 <= sp ){
+		if( sp[-3] == 033 )
+		switch( sp[-2] ){
+			case '$': in2byte = 1; break;
+			case '(': in2byte = 0; break;
+		}
+	}
+	return in2byte;
+}
+/*
+ * if "escrsvd" is ON, then reserved characters are escaped with %XX
+ * to be restored in URL_reescape()
+ */
+int URL_unescape(PCStr(src),PVStr(dst),int isform,int escrsvd){
+	int nesc = 0;
+	const unsigned char *sp;
+	refQStr(dp,dst);
+	int ch;
+	int in2byte = 0;
+	int out2byte = 0;
+
+	for( sp = (const unsigned char*)src; ch = *sp; sp++ ){
+		in2byte = set2B(src,(char*)sp,in2byte);
+		out2byte = set2B(dst,(char*)dp,out2byte);
+
+/*
+  if( 3 <= dp-dst ) fprintf(stderr,
+  "--URLunesc %d %d out[%2X %2X %2X] in[%2X %2X %2X][%2X %2X %2X] %2X %c\n",
+    in2byte,out2byte,
+    0xFF&dp[-3],0xFF&dp[-2],0xFF&dp[-1],
+    0xFF&sp[-3],0xFF&sp[-2],0xFF&sp[-1],
+    0xFF&sp[ 0],0xFF&sp[ 1],0xFF&sp[ 2],
+    ch,ch);
+*/
+
+		if( in2byte ){
+		}else
+		if( ch == '%' && isHEX(sp[1]) && isHEX(sp[2]) ){
+			ch = dec2X(sp+1);
+			if( escrsvd && ch == '%' ){
+				/* 9.8.4 thru original "%25" in ASCII */
+				setVStrPtrInc(dp,ch);
+				continue;
+			}
+			nesc++;
+			sp += 2;
+		}
+		else
+		if( out2byte ){
+			/* ESC seq. might be esacped with %XX in the URL
+			 * while reserved character in 2Bytes of JIS is not
+			 * escaped. (ex. 0x222E by "%1B%24%42%22.%1B ...")
+			 * It must not be escaped with %XX as RESERVED.
+			 */
+		}else
+		if( escrsvd && (ch == '%' || isRESERVED(ch)) ){
+			sprintf(dp,"%%%02X",ch);
+			dp += 3;
+			nesc++;
+			continue;
+		}else
+		if( !escrsvd && ch == '+' && isform ){
+			ch = ' ';
+			nesc++;
+		}
+		setVStrPtrInc(dp,ch);
+	}
+	setVStrEnd(dp,0);
+	return nesc;
+}
+int URL_reescape(PCStr(src),PVStr(dst),int isform,int rstrsvd){
+	int nesc = 0;
+	const unsigned char *sp;
+	refQStr(dp,dst);
+	unsigned char ch;
+	int in2byte = 0;
+	for( sp = (const unsigned char*)src; ch = *sp; sp++ ){
+		in2byte = set2B(src,(char*)sp,in2byte);
+		if( !in2byte )
+		if( rstrsvd && ch == '%' && isHEX(sp[1]) && isHEX(sp[2]) ){
+			int xch;
+			xch = dec2X(sp+1);
+			if( isRESERVED(xch) ){
+				setVStrPtrInc(dp,xch);
+				sp += 2;
+				continue;
+			}
+			if( rstrsvd && xch == '%' ){
+				/* 9.8.4 thru original "%25" in ASCII */
+				setVStrPtrInc(dp,ch);
+				continue;
+			}
+		}
+		/* even if the ch is reserved, if it was not esaped to
+		 * %XX in the original URL like above, maybe it is a part
+		 * of multi-byte character generated by code-conversion
+		 */
+		if( !rstrsvd && !in2byte && isRESERVED(ch) ){
+			setVStrPtrInc(dp,ch);
+		}else
+		if( isalnum(ch) || ch == '_' || ch == '-' ){
+			setVStrPtrInc(dp,ch);
+		}else
+		if( !rstrsvd && !in2byte && ch == ' ' && isform ){
+			setVStrPtrInc(dp,'+');
+			nesc++;
+		}else{
+			sprintf(dp,"%%%02X",ch);
+			nesc++;
+			dp += 3;
+		}
+	}
+	setVStrEnd(dp,0);
+	return nesc;
+}
+int URL_unescape2B(PCStr(src),PVStr(dst)){
+	int nesc = 0;
+	const unsigned char *sp;
+	refQStr(dp,dst);
+	int ch;
+	int xch;
+	int in2byte = 0;
+	int out2byte = 0;
+
+	for( sp = (const unsigned char*)src; ch = *sp; sp++ ){
+		if( ch == '%' && isHEX(sp[1]) && isHEX(sp[2]) ){
+			xch = dec2X(sp+1);
+			if( xch & 0x80 ){
+				setVStrPtrInc(dp,xch);
+				sp += 2;
+				continue;
+			}
+		}
+		setVStrPtrInc(dp,ch);
+	}
+	setVStrEnd(dp,0);
+	return nesc;
+}
+int URL_escape2B(PCStr(src),PVStr(dst)){
+	int nesc = 0;
+	const unsigned char *sp;
+	refQStr(dp,dst);
+	unsigned char ch;
+	int in2byte = 0;
+	for( sp = (const unsigned char*)src; ch = *sp; sp++ ){
+		if( ch & 0x80 ){
+			sprintf(dp,"%%%02X",ch);
+			nesc++;
+			dp += 3;
+		}else{
+			setVStrPtrInc(dp,ch);
+		}
+	}
+	setVStrEnd(dp,0);
+	return nesc;
+}
+int urlescape_main(int ac,const char *av[]){
+	CStr(line,1024);
+	CStr(xline,1024);
+	while( fgets(line,sizeof(line),stdin) != NULL ){
+		URL_reescape(line,AVStr(xline),1,0);
+		fputs(xline,stdout);
+	}
+	return 0;
+}
+int urlunescape_main(int ac,const char *av[]){
+	CStr(line,1024);
+	CStr(xline,1024);
+	while( fgets(line,sizeof(line),stdin) != NULL ){
+		URL_unescape(line,AVStr(xline),1,0);
+		fputs(xline,stdout);
+	}
+	return 0;
+}
