@@ -1,11 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1999 Electrotechnical Laboratry (ETL), AIST, MITI
-Copyright (c) 1999 Yutaka Sato
+Copyright (c) 1999-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2008 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, and distribute this material for any purpose
-and without fee is hereby granted, provided that the above copyright
-notice and this permission notice appear in all copies.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -20,6 +23,8 @@ History:
 #include <stdio.h>
 #include "ystring.h"
 #include "vsocket.h"
+#include "log.h"
+int daemonlog(const char *what,const char *fmt,...);
 
 typedef struct sockaddr_in SIN;
 typedef struct sockaddr_in6 SIN6;
@@ -32,22 +37,48 @@ int VSA_port(VSAddr *sap);
 int VSA_decomp(VSAddr *sap,const char **baddr,int *btype,const char **bport);
 void path2hostlocal(PCStr(path),PVStr(host),int size);
 
+int sock_isconnectedX(int sock,int sinonly);
 int sock_isconnected(int sock){
+	return sock_isconnectedX(sock,1);
+}
+int sock_isconnectedX(int sock,int sinonly){
 	int len;
 	VSAddr sin;
 
+	((SUN*)&sin)->sun_family = 0;
 	len = sizeof(sin);
 	if( getpeername(sock,(SAP)&sin,&len) == 0 )
-/*
-	if( len == sizeof(SIN) )
-*/
 	if( len == sizeof(SIN) || len == sizeof(SIN6) )
 	{
 		if( VSA_port(&sin) != 0 )
 			return 1;
 	}
+	else
+	if( !sinonly ){
+		if( 0 < len && ((SUN*)&sin)->sun_family == AF_UNIX ){
+			return 1;
+		}else
+		if( len == 0 && ((SUN*)&sin)->sun_family == 0 ){
+			/* maybe socketpair() */
+			/* on MacOSX and Solaris10 */
+			return 1;
+		}
+	}
+
 	return 0;
 }
+int sock_peerfamlen(int sock){
+	int len;
+	VSAddr sin;
+
+	((SUN*)&sin)->sun_family = 0;
+	len = sizeof(sin);
+	if( getpeername(sock,(SAP)&sin,&len) == 0 ){
+		return ((SUN*)&sin)->sun_family * 1000 + len;
+	}else{
+		return -1;
+	}
+}
 int sock_isv6(int sock){
 	int len;
 	VSAddr sin;
@@ -58,6 +89,50 @@ int sock_isv6(int sock){
 			return 1;
 	return 0;
 }
+int sock_isAFUNIX(int sock){
+	int len;
+	SUN Sun;
+
+	len = sizeof(Sun);
+	if( getsockname(sock,(SAP)&Sun,&len) == 0 )
+		if( Sun.sun_family == AF_UNIX )
+			return 1;
+	return 0;
+}
+extern int isUDPsock(int sock);
+int strfSocket(PVStr(desc),int size,PCStr(fmt),int sock){
+	CStr(tmp,1024);
+	VSAddr ssa;
+	VSAddr psa;
+	SIN *sip = (SIN*)&ssa;
+	int port;
+	int len;
+
+	if( isUDPsock(sock) )
+		strcpy(desc,"UDP ");
+	else	strcpy(desc,"TCP ");
+
+	len = sizeof(ssa);
+	bzero(&ssa,sizeof(ssa));
+	if( getsockname(sock,(SAP)&ssa,&len) == 0 ){
+		switch( sip->sin_family ){
+			case AF_UNIX: Xsprintf(TVStr(desc),"AF_UNIX "); break;
+			case AF_INET: Xsprintf(TVStr(desc),"AF_INET "); break;
+			default:
+			Xsprintf(TVStr(desc),"AF_%d ",sip->sin_family); break;
+			break;
+		}
+		port = VSA_port(&ssa);
+		Xsprintf(TVStr(desc),":%d ",port);
+	}
+	len = sizeof(psa);
+	bzero(&psa,sizeof(psa));
+	if( getpeername(sock,(SAP)&psa,&len) == 0 ){
+		port = VSA_port(&psa);
+		Xsprintf(TVStr(desc),"<< :%d ",port);
+	}
+	return 1;
+}
 
 static const char VSA_afunixdom[] = ".af-local";
 static const char VSA_afunixaddr[] = "127.0.0.127";
@@ -151,7 +226,10 @@ int VSA_addrisANY(VSAddr *sap)
 	int *ip;
 
 	sin6 = (SIN6*)sap;
+	/*
 	if( sin6->sin6_family == AF_INET6 ){
+	*/
+	if( ((SIN*)sin6)->sin_family == AF_INET6 ){
 		ip = (int*)&sin6->sin6_addr;
 		return ip[0]==0 && ip[1]==0 && ip[2]==0 && ip[3]==0;
 	}
@@ -196,7 +274,7 @@ if(0)
 			if( ip[3] == 1 ){
 				ip[3] = 0x7F000001; /* 127.0.0.1 */
 			}
-fprintf(stderr,"#### >>>>> IPv6 %X %X %X %X\n",ip[0],ip[1],ip[2],ip[3]);
+ fprintf(stderr,"#### >>>>> IPv6 %X %X %X %X\n",ip[0],ip[1],ip[2],ip[3]);
 			return AF_INET;
 		}
 		/*
@@ -212,24 +290,58 @@ fprintf(stderr,"#### >>>>> IPv6 %X %X %X %X\n",ip[0],ip[1],ip[2],ip[3]);
 
 #ifdef _MSC_VER
 #define NO_INET_NTOP 1
+#undef inet_ntop
 #define inet_ntop
 #else
 #define NO_INET_NTOP 0
 #endif
 
+/*
 static char ntoa_buf[64];
+*/
+typedef struct {
+	MStr(a_buf6,64);
+	MStr(a_buf4,32);
+} NtoaBuf;
+static NtoaBuf ntoaBufs[32];
+#define ntoa_buf4 ntoaBufs[getthreadgix(0)%elnumof(ntoaBufs)].a_buf4
+#define ntoa_buf6 ntoaBufs[getthreadgix(0)%elnumof(ntoaBufs)].a_buf6
+
+char *inet_ntoaX(struct in_addr in){
+	char *aa;
+	aa = inet_ntoa(in);
+	if( aa ){
+		strcpy(ntoa_buf4,aa);
+		return ntoa_buf4;
+	}
+	return aa;
+}
+#define inet_ntoa(in) inet_ntoaX(in)
+
+const char *VSA_ntoaX(VSAddr *sap);
 const char *VSA_ntoa(VSAddr *sap)
+{
+	const char *aa;
+	aa = VSA_ntoaX(sap);
+	return aa;
+}
+const char *VSA_ntoaX(VSAddr *sap)
 {	SIN *sip;
 	const char *addr;
 	SUN *sup;
 
 	sup = (SUN*)sap;
 	if( sup->sun_family == AF_UNIX )
+	if( sup->sun_path[0] ){
+		/* should convert to "xxx.af-loadl" format ? */
+		return sup->sun_path;
+	}else{
 		return VSA_afunixaddr;
+	}
 
 	if( sup->sun_family == AF_INET6 ){
-		char hbuf[128];
-		char *ap;
+		CStr(hbuf,128);
+		char *ap; /**/
 		int id = ((SIN6*)sap)->sin6_scope_id;
 		int ok;
 		if( IPV6_unify4mapped ){
@@ -247,6 +359,40 @@ const char *VSA_ntoa(VSAddr *sap)
 				return addr;
 			}
 		}
+
+		ok = 0;
+		if( ok == 0 ){
+			int ix;
+			const unsigned char *bp;
+			bp = (unsigned char*)&((SIN6*)sap)->sin6_addr;
+			for( ix = 0; ix < 15; ix++ ){
+				if( bp[ix] != 0 )
+					break;
+			}
+			if( ix == 15 )
+			if( bp[ix] == 0 || bp[ix] == 1 ) 
+			{
+				if( bp[ix] != 0 )
+					sprintf(hbuf,"::%d",bp[ix]);
+				else	strcpy(hbuf,"::");
+				ok = 1;
+			}
+
+			if( ok == 0 ){
+				for( ix = 0; ix < 14; ix++ ){
+					if( bp[ix] != 0xFF )
+						break;
+				}
+				if( ix == 14 ){
+					sprintf(hbuf,
+					"FFFF:FFFF:FFFF:FFFF:FFFF:FFFF:FFFF:%04X",
+						(bp[14]<<8)|bp[15]);
+					ok = 1;
+				}
+			}
+		}
+
+		if( ok == 0 )
 		if( NO_INET_NTOP )
 			ok = getnameinfo((SA*)sap,sizeof(SIN6),hbuf,sizeof(hbuf),NULL,0,NI_NUMERICHOST) == 0;
 		else	ok = inet_ntop(AF_INET6,&((SIN6*)sap)->sin6_addr,hbuf,sizeof(hbuf)) != NULL;
@@ -260,8 +406,9 @@ const char *VSA_ntoa(VSAddr *sap)
 				Xsprintf(ZVStr(ap,sizeof(hbuf)-(ap-hbuf)),
 					"%%%d",id);
 			}
-			Xstrcpy(FVStr(ntoa_buf),hbuf);
-			return ntoa_buf;
+
+			strcpy(ntoa_buf6,hbuf);
+			return ntoa_buf6;
 		}
 		return ":::::::";
 	}
@@ -275,26 +422,47 @@ const char *VSA_htoa(struct hostent *hp)
 	SIN6 sin6;
 
 	if( hp->h_addrtype == AF_INET6 ){
+		bzero(&sin6,sizeof(SIN6));
+		/*
 		sin6.sin6_family = hp->h_addrtype;
-		bcopy(hp->h_addr,&sin6.sin6_addr,hp->h_length);
+		*/
+		((SIN*)&sin6)->sin_family = hp->h_addrtype;
+/* length=20 expecting sin6_scope_id[4] is immediately after sin6_addr */
+		if( hp->h_length < 0 || IPV6_ADDRLENG < hp->h_length ){
+			daemonlog("F","#### VSA_htoa() bad leng: %d\n",
+				hp->h_length);
+		}else
+		bcopy(hp->h_addr,&sin6.sin6_addr,hp->h_length);/**/
 		return VSA_ntoa((VSAddr*)&sin6);
 	}
 	sin.sin_family = hp->h_addrtype;
-	bcopy(hp->h_addr,&sin.sin_addr,hp->h_length);
+	Xbcopy(hp->h_addr,GVStr(&sin.sin_addr),hp->h_length); /**/
 	return VSA_ntoa((VSAddr*)&sin);
 }
 const char *VSA_ltoa(const unsigned char *baddr,int len,int type)
 {	SIN sin;
 
+	if( baddr == 0 ){
+		porting_dbg("#### VSA_ltoa(%X,%d,%X) NULL",baddr,len,type);
+		return "0.0.0.255";
+	}
 	if( type == AF_INET6 ){
 		SIN6 sin6;
 		bzero(&sin6,sizeof(SIN6));
+		/*
 		sin6.sin6_family = type;
-		bcopy(baddr,&sin6.sin6_addr,len);
+		*/
+		((SIN*)&sin6)->sin_family = type;
+/* length=20 expecting sin6_scope_id[4] is immediately after sin6_addr */
+		if( len < 0 || IPV6_ADDRLENG < len ){
+			daemonlog("F","#### VSA_htoa() bad leng: %d\n",len);
+		}else
+		bcopy(baddr,&sin6.sin6_addr,len); /**/
 		return VSA_ntoa((VSAddr*)&sin6);
 	}
+	bzero(&sin,sizeof(SIN)); /* to set sin_len=0 if exists */
 	sin.sin_family = type;
-	bcopy(baddr,&sin.sin_addr,len);
+	Xbcopy(baddr,GVStr(&sin.sin_addr),len); /**/
 	return VSA_ntoa((VSAddr*)&sin);
 }
 int VSA_af(VSAddr *sap){
@@ -309,7 +477,10 @@ int VSA_port(VSAddr *sap)
 		return VSA_afunixport;
 
 	if( sup->sun_family == AF_INET6 ){
+		/*
 		return ((SIN6*)sup)->sin6_port;
+		*/
+		return ntohs(((SIN6*)sup)->sin6_port);
 	}
 	sip = (SIN*)sap;
 	return ntohs(sip->sin_port);
@@ -321,7 +492,7 @@ char *VSA_xtoap(VSAddr *sa,PVStr(buf),int siz)
 }
 int VSA_cto_(char addr[]){
 	if( VSA_strisaddr(addr) == AF_INET6 ){
-		char *cp;
+		char *cp; /**/
 		for( cp = addr; *cp; cp++ ){
 			if( *cp == ':' ){
 				*cp = '_';
@@ -352,7 +523,7 @@ static int _toc(PCStr(src),PVStr(dst)){
 	return 0;
 }
 int xgetaddrinfo(PCStr(addr),PCStr(serv),const struct addrinfo *hi,struct addrinfo **res){
-	CStr(xaddr,128);
+	CStr(xaddr,MaxHostNameLen);
 	_toc(addr,AVStr(xaddr));
 	addr = xaddr;
 	return getaddrinfo(addr,serv,hi,res);
@@ -383,22 +554,109 @@ int Inet_pton(int af,const char *src,void *dst){
 	return 0;
 }
 
+#define isNum(c)   ('0' <= c && c <= '9')
+#define isHex(c)   ('A' <= c && c <= 'F' || 'a' <= c && c <= 'f')
+#define isAlpha(c) ('A' <= c && c <= 'X' || 'a' <= c && c <= 'z')
+#define isAlnum(c) (isNum(c) || isAlpha(c))
+int strcanbeHostname(int af,PCStr(name)){
+	const char *np;
+	char nc;
+
+	for( np = name; nc = *np; np++ ){
+		if( isAlnum(nc) )
+			continue;
+		if( nc == '-' )
+			continue;
+		if( nc == '_' ) /* might be _service or _protocol */
+			continue;
+		if( nc == '.' ){
+			if( isAlnum(np[1]) || np[1] == 0 ) 
+				continue;
+		}
+		return 0;
+	}
+	return 1;
+}
+int strcanbeHostaddr(int af,PCStr(addr)){
+	const char *ap;
+	char ac;
+	char nac;
+	int nd = 0;
+	int nc = 0;
+
+	for( ap = addr; ac = *ap; ap++ ){
+		if( isNum(ac) )
+			continue;
+		nac = ap[1];
+		if( ac == '.' ){
+			if( isNum(nac) ){
+				nd++;
+				continue;
+			}
+		}
+		if( af == AF_INET6 ){
+			if( isHex(ac) )
+				continue;
+			if( ac == '_' || ac == ':' ){
+				if( nac == '_' || nac == ':' || nac == 0
+				 || isHex(nac) || isNum(nac)
+				){
+					nc++;
+					continue;
+				}
+			}
+			if( 2 <= nc && ac == '%' ){
+				continue;
+			}
+		}
+		return 0;
+	}
+	if( nd == 0 && nc == 0 || 0 < nd && nd != 3 || 0 < nc && nc < 2 ){
+		return 0;
+	}
+	return 1;
+}
+
 int xinet_pton(int af,const char *src,void *dst){
-	CStr(ab,128);
+	CStr(ab,MaxHostNameLen);
+
+	if( !strcanbeHostaddr(af,src) ){
+		P_LV("---- xinet_pton(%d,%s) not an address",af,src);
+		return 0;
+	}
+
 	_toc(src,AVStr(ab));
+	if( strchr(src,'_') || strchr(src,':') ){
+		int rcode;
+		P_LV("---- xinet_pton(%d,%s)...",af,src);
+		rcode = inet_pton(af,ab,dst);
+		P_LV("---- xinet_pton(%d,%s)=%d",af,src,rcode);
+		return rcode;
+	}
 	return inet_pton(af,ab,dst);
 }
 int VSA_strisaddr(PCStr(addr))
 {
-	/*
-	if( inet_addr(addr) != -1 )
-	*/
 	struct in_addr ina;
 	char i6[16];
+	CStr(iaddr,MaxHostNameLen);
 
+	if( addr == 0 || *addr == 0 ){
+		if( addr == 0 )
+			syslog_ERROR("#### VSA_strisaddr() NULL ADDR\n");
+		return 0;
+	}
+
+	if( 128 <= strlen(addr) ){
+		return 0;
+	}
 	if( inet_aton(addr,&ina) )
 		return 1;
 
+	if( strchr(addr,'%') ){
+		wordscanY(addr,AVStr(iaddr),sizeof(iaddr),"^%");
+		addr = iaddr;
+	}
 	if( xinet_pton(AF_INET6,addr,i6) == 1 ){
 		return AF_INET6;
 	}
@@ -422,15 +680,15 @@ int VSA_stosa(VSAddr *sap,int atype,PCStr(socks))
 		SIN6 *sip6;
 		sip6 = (SIN6*)sap;
 		bzero(sip6,sizeof(SIN6));
-		sip6->sin6_family = atype;
-		bcopy(socks,&sip6->sin6_addr,16);
-		bcopy(socks+16,&sip6->sin6_port,2);
+		((SIN*)sip6)->sin_family = atype;
+		Xbcopy(socks,GVStr(&sip6->sin6_addr),16);
+		Xbcopy(socks+16,GVStr(&sip6->sin6_port),2);
 	}else{
-	sip = (SIN*)sap;
-	bzero(sip,sizeof(SIN));
-	sip->sin_family = atype;
-	bcopy(socks,&sip->sin_addr,4);
-	bcopy(socks+4,&sip->sin_port,2);
+		sip = (SIN*)sap;
+		bzero(sip,sizeof(SIN));
+		sip->sin_family = atype;
+		Xbcopy(socks,GVStr(&sip->sin_addr),4);
+		Xbcopy(socks+4,GVStr(&sip->sin_port),2);
 	}
 	return sizeof(SIN);
 }
@@ -441,12 +699,15 @@ int VSA_btosa(VSAddr *sap,int atype,unsigned char *baddr,int port)
 	bzero(sip,sizeof(SIN));
 	if( atype == AF_INET6 ){
 		SIN6 *sip6 = (SIN6*)sip;
+		/*
 		sip6->sin6_family = atype;
-		bcopy(baddr,&sip6->sin6_addr,16);
+		*/
+		((SIN*)sip6)->sin_family = atype;
+		Xbcopy(baddr,GVStr(&sip6->sin6_addr),16);
 		sip6->sin6_port = htons(port);
 	}else{
 	sip->sin_family = atype;
-	bcopy(baddr,&sip->sin_addr,4);
+		Xbcopy(baddr,GVStr(&sip->sin_addr),4);
 	sip->sin_port = htons(port);
 	}
 	return sizeof(SIN);
@@ -471,10 +732,50 @@ int VSA_atosa(VSAddr *sa,int port,PCStr(addr))
 	struct addrinfo *ip6;
 	const char *sidp;
 	int sid = 0;
+	CStr(path,1024);
+
+	if( addr == NULL ){
+		syslog_ERROR("#### VSA_atosa() NULL ADDR\n");
+		addr = "";
+	}
+
+	if( hostlocal2path(addr,AVStr(path),sizeof(path)) ){
+		addr = path;
+	}
+
+	if( addr[0] == '/' || addr[0] == '\\'
+	 || addr[1] == ':' &&(addr[2] == '\\' || addr[2] == '/')
+	){
+		SUN *sup;
+		sup = (SUN*)sa;
+		bzero(sup,sizeof(SUN));
+		sup->sun_family = AF_INET;
+		Xstrcpy(FVStr(sup->sun_path),addr);
+		return sizeof(SUN);
+	}
 
 	if( (sidp = strchr(addr,'%')) )
 		sid = atoi(sidp+1);
 
+	if( strneq(addr,"__",2) ){
+		const char *d1 = addr+2;
+		if( streq(d1,"") || streq(d1,"0") || streq(d1,"1") ){
+			SIN6 *sip = (SIN6*)sa;
+			int len = sizeof(sip->sin6_addr);
+
+			bzero(sip,sizeof(SIN6));
+			/*
+			sip->sin6_family = AF_INET6;
+			*/
+			((SIN*)sip)->sin_family = AF_INET6;
+			sip->sin6_port = htons(port);
+			sip->sin6_flowinfo = 0;
+			((char*)&sip->sin6_addr)[15] = atoi(d1);
+			sip->sin6_scope_id = sid;
+			return sizeof(SIN6);
+		}
+	}
+
 	if( sidp == 0 || 0 < sid/*numeric scope id*/ )
 	if( VSA_strisaddr(addr) == AF_INET6 ){
 		int i6[16];
@@ -485,10 +786,13 @@ int VSA_atosa(VSAddr *sa,int port,PCStr(addr))
 			/*
 			sip->sin6_len = sizeof(SIN6);
 			*/
+			/*
 			sip->sin6_family = AF_INET6;
+			*/
+			((SIN*)sip)->sin_family = AF_INET6;
 			sip->sin6_port = htons(port);
 			sip->sin6_flowinfo = 0;
-			bcopy(i6,(char*)&sip->sin6_addr,16);
+			Xbcopy(i6,GVStr(&sip->sin6_addr),16); /*QA*/
 			sip->sin6_scope_id = sid;
 			return sizeof(SIN6);
 		}
@@ -507,12 +811,15 @@ int VSA_atosa(VSAddr *sa,int port,PCStr(addr))
 		if( xgetaddrinfo(addr,NULL,&hints,&ai) == 0 ){
 			sip = (SIN6*)sa;
 			bzero(sip,sizeof(SIN6));
-			bcopy(ai->ai_addr,sip,ai->ai_addrlen);
+			Xbcopy(ai->ai_addr,GVStr(sip),ai->ai_addrlen); /*QA*/
 			sip->sin6_port = htons(port);
+			/*
 			sip->sin6_family = ai->ai_family;
+			*/
+			((SIN*)sip)->sin_family = ai->ai_family;
 
 /*
-fprintf(stderr,"### atsa %X IS ADDR %s fa=%d len=%d (%d) FLOW=%X, SCOPE=%d\n",
+ fprintf(stderr,"### atsa %X IS ADDR %s fa=%d len=%d (%d) FLOW=%X, SCOPE=%d\n",
 sa,addr,ai->ai_family,ai->ai_addrlen,sizeof(SIN6),
 sip->sin6_flowinfo,sip->sin6_scope_id);
 */
@@ -533,6 +840,34 @@ sip->sin6_flowinfo,sip->sin6_scope_id);
 	sip->sin_port = htons(port);
 	return sizeof(SIN);
 }
+int VSA_aptosa(VSAddr *sa,PCStr(addrport)){
+	CStr(addr,128);
+	refQStr(pp,addr);
+	int port;
+
+	bzero(sa,sizeof(VSAddr));
+	strcpy(addr,"");
+	port = 0;
+	if( strrchr(addrport,':') ){
+		strcpy(addr,addrport);
+		if( pp = strchr(addr,':') ){
+			truncVStr(pp);
+			port = atoi(pp+1);
+		}
+	}
+	return VSA_atosa(sa,port,addr);
+}
+int VSA_satoap(VSAddr *sa,PVStr(addrport)){
+	const char *addr;
+	if( addr = VSA_ntoa(sa) ){
+		sprintf(addrport,"%s:%d",addr,VSA_port(sa));
+		return 0;
+	}else{
+		sprintf(addrport,"255.255.255.255:0");
+		return -1;
+	}
+}
+
 int VSA_htosa(VSAddr *sap,int port,struct hostent *hp,int hi)
 {	SIN *sip;
 	const char *baddr;
@@ -542,14 +877,19 @@ int VSA_htosa(VSAddr *sap,int port,struct hostent *hp,int hi)
 	baddr = hp->h_addr_list[hi];
 	sip->sin_family = hp->h_addrtype;
 	if( sip->sin_family == AF_INET6 ){
-		bcopy(baddr,&((SIN6*)sip)->sin6_addr,hp->h_length);
+/* length=20 expecting sin6_scope_id[4] is immediately after sin6_addr */
+		if( hp->h_length < 0 || IPV6_ADDRLENG < hp->h_length ){
+			daemonlog("F","#### VSA_htoa() bad leng: %d\n",
+				hp->h_length);
+		}else
+		bcopy(baddr,&((SIN6*)sip)->sin6_addr,hp->h_length);/**/
 		((SIN6*)sip)->sin6_port = htons(port);
 		((SIN6*)sip)->sin6_scope_id = 0;
 		if( hp->h_length == IPV6_ADDRLENG ){
 			bcopy(baddr+16,&((SIN6*)sip)->sin6_scope_id,4);
 		}
 	}else{
-	bcopy(baddr,&sip->sin_addr,hp->h_length);
+		Xbcopy(baddr,GVStr(&sip->sin_addr),hp->h_length); /**/
 	sip->sin_port = htons(port);
 	}
 	return sizeof(SIN);
@@ -563,7 +903,7 @@ int VSA_size(VSAddr *sap)
 	}
 	return sizeof(SIN);
 }
-int VSA_atob(PCStr(aaddr),char baddrb[],int *btypep)
+int VSA_atob(PCStr(aaddr),PVStr(baddrb),int *btypep)
 {	VSAddr Addr;
 	const char *baddr;
 	int bleng;
@@ -575,7 +915,7 @@ int VSA_atob(PCStr(aaddr),char baddrb[],int *btypep)
 		bleng = IPV6_ADDRLENG;
 		/* expecting sin6_scope_id is just after sin6_addr X-) */
 	}
-	bcopy(baddr,baddrb,bleng);
+	Bcopy(baddr,baddrb,bleng);
 	return bleng;
 }
 int VSA_decomp(VSAddr *sap,const char **baddr,int *btype,const char **bport)
@@ -605,7 +945,10 @@ void VSA_prftp(VSAddr *sap,PVStr(mport))
 	}
 	if( sup->sun_family == AF_INET6 )
 	{
+		/*
 		sprintf(mport,"|||%d|",((SIN6*)sap)->sin6_port);
+		*/
+		sprintf(mport,"|||%d|",ntohs(((SIN6*)sap)->sin6_port));
 		return;
 	}
 
@@ -627,16 +970,28 @@ void VSA_ftptosa(void *sap,PCStr(port))
 
 		if( strncmp(port,"|||",3) == 0 ){
 			if( Xsscanf(port,"|||%d",&pn) == 1 ){
+				/*
 				((SIN6*)sap)->sin6_port = pn;
+				*/
+				((SIN6*)sap)->sin6_port = htons(pn);
 				return;
 			}
 		}
+
+		/* IPv6 */
 		if( strncmp(port,"|2|",3) == 0 ){
 			if( Xsscanf(port,"|2|%[^|]|%d",AVStr(addr),&pn)==2 ){
 				VSA_atosa((VSAddr*)sap,pn,addr);
 				return;
 			}
 		}
+		/* IPv4 */
+		if( strncmp(port,"|1|",3) == 0 ){
+			if( Xsscanf(port,"|1|%[^|]|%d",AVStr(addr),&pn)==2 ){
+				VSA_atosa((VSAddr*)sap,pn,addr);
+				return;
+			}
+		}
 	}
 
 	sip = (SIN*)sap;
@@ -655,7 +1010,10 @@ int VSA_dnstosa(void *sap,int port,PCStr(revaddr))
 	char *bp; /**/
 
 	if( domain_ipv6(revaddr,(SIN6*)sap) ){
+		/*
 		((SIN6*)sap)->sin6_family = AF_INET6;
+		*/
+		((SIN*)sap)->sin_family = AF_INET6;
 		((SIN6*)sap)->sin6_port = port;
 		return sizeof(SIN6);
 	}
@@ -743,7 +1101,10 @@ int VSA_islocal(VSAddr *vsa){
 	}
 }
 int VSA_6to4(VSAddr *vsa){
+	/*
 	if( ((SIN6*)vsa)->sin6_family == AF_INET6 ){
+	*/
+	if( ((SIN*)vsa)->sin_family == AF_INET6 ){
 		int *ip = (int*)&((SIN6*)vsa)->sin6_addr;
 		int port = ((SIN6*)vsa)->sin6_port;
 		if( ip[0]==0 && ip[1]==0 && ip[2]==0xFFFF ){
@@ -764,6 +1125,24 @@ void inetNtoa(int addr,PVStr(saddr))
 	strcpy(saddr,inet_ntoa(sin.sin_addr));
 }
 
+int getsocktype(int sock);
+int dupSocket(int sock){
+	int len;
+	VSAddr sin;
+	SIN *sinp;
+	int nsock;
+	int d,type,protocol;
+
+	len = sizeof(sin);
+	if( getsockname(sock,(SAP)&sin,&len) != 0 )
+		return -1;
+	sinp = (SIN*)&sin;
+	d = sinp->sin_family;
+	type = getsocktype(sock);
+	nsock = socket(d,type,0);
+	/* must bind the host-address if bound */
+	return nsock;
+}
 int SocketPair(int d,int type,int protocol,int sv[2])
 {	SIN ina;
 	int asock,len;
@@ -859,12 +1238,12 @@ int Inet_aton(PCStr(addr),struct in_addr *inap)
 }
 
 int domain_ipv6(PCStr(revaddr),SIN6 *sap){
-	char *dp;
+	const char *dp;
 	int nc;
 	int i;
 	int sx;
 	int v[32];
-	char *ap;
+	char *ap; /**/
 
 	if( (dp = strcasestr(revaddr,".IP6.INT")) )
 	if( dp[8] == 0 ){
