@@ -45,6 +45,7 @@ typedef unsigned short int octet2;
 #define TY_TXT	   16	/* text strings */
 #define TY_XPTR	   20	/* a pointer for an alias */
 #define TY_AAAA	   28	/* IPv6 */
+#define TY_SRV     33	/* a service */
 #define TY_A6	   38	/* IPv6 */
 /*
  *	QTYPE values
@@ -74,6 +75,7 @@ typedef unsigned short int octet2;
 #define O_STATUS    2
 
 #define uc(p,i,s)	(((octet*)p)[i]<<s)
+#define getLeng(p)	(uc(p,0,8)|uc(p,1,0))
 #define getShort(p,v)	{v = uc(p,0,8)|uc(p,1,0); p += 2;}
 #define getLong(p,v)	{v = uc(p,0,24)|uc(p,1,16)|uc(p,2,8)|uc(p,3,0); p += 4;}
 
@@ -143,12 +145,15 @@ typedef struct rr {
 #define DBG_ALL		0xFF
 #define DBG_ANY		0xFE
 #define DBG_FORCE	-1
+#define DBG_TRACE	0x100
+#define DBG_DUMP	0x200
 
 #define RT_CACHE 'C'
 #define RT_FILE	'F'
 #define	RT_NIS	'N'
 #define RT_DNS	'D'
 #define RT_SYS	'S'
+#define RT_UNKNOWN 'U'
 extern const char *_RSLV_CONF;
 extern const char *_HOSTSFILE;
 extern const char *_NISMAP_NAME;
@@ -159,6 +164,8 @@ extern const char *_NISMAP_ADDR;
 extern int RSLV_TIMEOUT1;
 extern int RSLV_TIMEOUT;
 extern int RSLV_INV_TIMEOUT;
+extern char *RES_AF; /* "4", "6", "46", or "64" */
+extern int RES_ASIS; /* don't try search with DNSRCH, DEFDNAME extensions */
 
 extern const char *RES_HC_DIR;
 extern int   RES_HC_EXPIRE;
@@ -175,8 +182,12 @@ int DNS_putbyname(PCStr(name));
 int DNS_getbyname(PCStr(name));
 int DNS_parent(int nid);
 int DNS_nodename(int nid,PVStr(name));
-void DNS_putattr(int nid,int flag,int ttl,PCStr(data),int leng);
+int DNS_putattr(int nid,int flag,int ttl,PCStr(data),int leng);
 int DNS_getattr(int nid,int flag,int ttl,int ac,char *av[]);
 int DNS_nodephase(int nid,int phase);
 void DNS_nodedump(int nid);
 void DNS_dump();
+
+#define RESOLVERS_SIZ 512
+char *RES_resolvers(PVStr(resolvers));
+
