@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1995 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1995-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -55,6 +56,57 @@ static int NodesA;
 static int NodesX = 2;
 static int NID = 1;
 
+#define NBSIZE 128
+static Node *Nodebank;
+static int Nodebanki;
+static Node *NewNode(){
+	Node *Np;
+	if( Nodebank == 0 || Nodebanki == NBSIZE ){
+		Nodebank = (Node*)malloc(NBSIZE*sizeof(Node));
+		Nodebanki = 0;
+	}
+	Np = &Nodebank[Nodebanki++];
+	bzero(Np,sizeof(Node));
+	return Np;
+}
+#define SBSIZE 512
+static char *Stringb;
+static int Stringi;
+static char *NewString(int len){
+	char *sp;
+	if( Stringi == 0 || SBSIZE-Stringi < len ){
+		Stringb = (char*)malloc(SBSIZE);
+		Stringi = 0;
+	}
+	sp = Stringb+Stringi;
+	Stringi += len;
+	return sp;
+}
+static const char *NewLabel(PCStr(label)){
+	char *lp;
+	int len;
+	len = strlen(label) + 1;
+	lp = NewString(len);
+	Xstrcpy(ZVStr(lp,len),label);
+	return lp;
+}
+#define RBSIZE 512
+static int *Recordb;
+static int Recordi;
+static int *NewRecord(int siz){
+	int isiz;
+	int *rp;
+
+	isiz = (siz+sizeof(int)-1)/sizeof(int) +1/*for safety?*/;
+	if( Recordi == 0 || RBSIZE-Recordi < isiz ){
+		Recordb = (int*)malloc(RBSIZE*sizeof(int));
+		Recordi = 0;
+	}
+	rp = &Recordb[Recordi];
+	Recordi += isiz;
+	return rp;
+}
+
 #define getnode(nid)	Nodes[nid]
 
 static Node *newnode()
@@ -66,6 +118,10 @@ static Node *newnode()
 	if( NodesX <= NID ){
 		NodesX = ++NodesA * MAXLEV;
 		nsize = NodesX * sizeof(Node*);
+		if( nsize < 1024 ){
+			/* 9.7.0 to reduce realloc() */
+			nsize = 1024;
+		}
 		if( Nodes == Nodes0 ){
 			Nodes = (Node**)malloc(nsize);
 			Nodes[0] = Nodes0[0];
@@ -74,7 +130,10 @@ static Node *newnode()
 		debug(DBG_ANY,"newnode() id=%d %d %X\n",NID,NodesX,Nodes);
 	}
 
+	/*
 	Np = NewStruct(Node);
+	*/
+	Np = NewNode();
 	Nodes[NID] = Np;
 	Np->n_nid = NID;
 	return Np;
@@ -124,7 +183,10 @@ static Node *nsearch(PCStr(name),int create)
 				return &NoNode;
 
 			Lp = newnode();
+			/*
 			Lp->n_label = stralloc(label);
+			*/
+			Lp->n_label = NewLabel(label);
 			Lp->n_parent = Np;
 			Tp = Np->n_child;
 			Np->n_child = Lp;
@@ -166,17 +228,25 @@ void addr2dom(PCStr(addr),PVStr(inaddr),int isize)
 	sprintf(inaddr,"%d.%d.%d.%d.%s",a4,a3,a2,a1,REVERSE_DOM);
 }
 
+int ipv6_domain(PCStr(addr),PVStr(dom));
 int DNS_putbyaddr(PCStr(addr))
 {	CStr(name,256);
 	int a1,a2,a3,a4;
+	int ns;
 
+	ns =
 	sscanf(addr,"%d.%d.%d.%d",&a1,&a2,&a3,&a4);
+	if( ns == 4 )
 	sprintf(name,"%d.%d.%d.%d.%s",a4,a3,a2,a1,REVERSE_DOM);
+	else{
+		ipv6_domain(addr,AVStr(name));
+	}
 	return DNS_putbyname(name);
 }
 int DNS_getbyaddr(PCStr(addr))
 {	CStr(name,256);
 
+	/* should care IPv6 address here ...*/
 	sprintf(name,"%s.%s",addr,REVERSE_DOM);
 	return DNS_getbyname(name);
 }
@@ -205,7 +275,7 @@ int DNS_nodename(int nid,PVStr(name))
 {
 	return getnodename(getnode(nid),AVStr(name));
 }
-void DNS_putattr(int nid,int flag,int ttl,PCStr(data),int leng)
+int DNS_putattr(int nid,int flag,int ttl,PCStr(data),int leng)
 {	Node *Np;
 	Attr *Sp,*Ap;
 
@@ -214,18 +284,22 @@ void DNS_putattr(int nid,int flag,int ttl,PCStr(data),int leng)
 	for( Ap = Np->n_attr; Ap; Ap = Ap->a_peer ){
 		if( flag == Ap->a_flag && 
 		    leng == Ap->a_leng && bcmp(Ap->a_data,data,leng) == 0 )
-			return;
+			return 0;
 	}
 
+	/*
 	Ap = (Attr*)malloc(sizeof(Attr)+leng);
+	*/
+	Ap = (Attr*)NewRecord(sizeof(Attr)+leng);
 	Ap->a_flag = flag;
 	Ap->a_leng = leng;
 	Ap->a_ttl  = ttl;
-	bcopy(data,Ap->a_data,leng);
+	bcopy(data,Ap->a_data,leng); /**/
 
 	Sp = Np->n_attr;
 	Np->n_attr = Ap;
 	Ap->a_peer = Sp;
+	return 1;
 }
 int DNS_getattr(int nid,int flag,int ttl,int ac,char *av[])
 {	Node *Np;
