@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1995 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1995-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -19,7 +20,7 @@ History:
 	950822	created
 //////////////////////////////////////////////////////////////////////#*/
 
-#include "ysocket.h"
+#include "vsocket.h"
 static int dummy; /* to avoid empty symbol table */
 
 #ifndef STD_HOSTENT
@@ -42,8 +43,7 @@ void *gethostent()
 
 #ifndef gethostbyname
 extern void *RES_gethostbyname();
-void *gethostbyname(name)
-	char *name;
+void *gethostbyname(char *name)
 {
 	return RES_gethostbyname(name);
 }
@@ -51,8 +51,7 @@ void *gethostbyname(name)
 
 #ifndef gethostbyaddr
 extern void *RES_gethostbyaddr();
-void *gethostbyaddr(addr,len,type)
-	char *addr;
+void *gethostbyaddr(char *addr,int len,int type)
 {
 	return RES_gethostbyaddr(addr,len,type);
 }
