@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1994 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1994-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -25,6 +26,7 @@ ToDo:
 #include <stdlib.h>
 #include <ctype.h>
 #include "ystring.h"
+#include "vaddr.h"
 #include "file.h"
 void minit_timer();
 void minit_resconf();
@@ -89,21 +91,54 @@ static int getMASK(unsigned int iaddr)
 }
 
 typedef struct {
-	int	full;
+	char	full;
+	int	serno;
+/*
 	int	maskedL;
 	int	maskedH;
+*/
+	unsigned int maskedL;
+	unsigned int maskedH;
   const	char	*name;
 } DName;
 
-static DName DnTab[0x10000];
+static DName DnTab[0x20000];
 static int DnTabX;
 
+/* decending order, narrower range first */
+static int dacmp(DName *a,DName *b){
+	if( a->maskedL != b->maskedL ){
+		if( b->maskedL > a->maskedL )
+			return 1;
+		else	return -1;
+	}else{
+		if( a->maskedH == b->maskedH ){
+			/* in the original order for netdom.hosts
+			 * (older upper entry first)
+			 */
+			if( b->serno < a->serno )
+				return 1;
+			else	return -1;
+		}else
+		if( a->maskedH > b->maskedH )
+			return 1;
+		else	return -1;
+	}
+}
+static void sort_dntab(){
+qsort(DnTab,DnTabX,sizeof(DName),(int(*)(const void*,const void*))dacmp);
+}
 static void load_dn1(PCStr(addrL),PCStr(addrH),PCStr(name))
 {	int addri;
 	const char *np;
 	DName *dnp;
 
+	if( elnumof(DnTab) <= DnTabX ){
+		fprintf(stderr,"---- ntod: DnTab[%d] Overflow\n",DnTabX);
+		return;
+	}
 	dnp = &DnTab[DnTabX++];
+	dnp->serno = DnTabX-1;
 	if( inet_addr(addrL) != -1 ){
 		dnp->full = 1;
 		dnp->maskedL = inet_addr(addrL);
@@ -212,6 +247,10 @@ static void load_dntab(PCStr(dbfile))
 			fprintf(stderr,"? %s",line);
 			continue;
 		}
+		if( name[strlen(name)-1] == '.' ){
+			fprintf(stderr,"? %s",line);
+			setVStrEnd(name,strlen(name)-1);
+		}
 
 		if( dp = strchr(addr,'/') )
 			inet_range(AVStr(addr));
@@ -237,23 +276,123 @@ static void load_dntab(PCStr(dbfile))
 		}
 	}
 	fclose(dntab);
-	fprintf(stderr,"LOADED %s [%d]\n",dbfile,DnTabX-tabx);
+	fprintf(stderr,"LOADED %s [%d] / %d / %d\n",dbfile,
+		DnTabX-tabx,DnTabX,elnumof(DnTab));
 }
 
+static int addrHit;
+static int addrMiss;
+static int addrTries;
+/*
 static const char *getdomainbyaddr(int iaddr)
+*/
+static const char *getdomainbyaddrX(int iaddr,int ai,int inc,int maxs);
+static const char *getdomainbyaddr(unsigned int iaddr)
 {	int ma,ai;
 	DName *dnp;
+	const char *ap;
+	int inc;
+	int base;
+	int dir = 1;
+	int sbase;
 
+	inc = DnTabX / 2;
+	dir = 1;
+	base = inc;
+	while( 0 < inc ){
+		dnp = &DnTab[base];
+		if( iaddr == dnp->maskedL ){
+			dir = 0;
+			break;
+		}
+		if( iaddr < dnp->maskedL ){
+			base += inc;
+			dir = 1;
+		}else{
+			base -= inc;
+			dir = -1;
+		}
+		inc /= 2;
+	}
+
+	if( dir == 0 ){
+		int i;
+		for( i = base-1; 0 <= i; i-- ){
+			dnp = &DnTab[i];
+			if( iaddr != dnp->maskedL )
+				break;
+/*
+fprintf(stderr,"--- . %d %d %d %s\n",dir,inc,base,DnTab[base].name);
+fprintf(stderr,"--- p %d %d %d %s\n",dir,inc,i,DnTab[i].name);
+*/
+			base = i;
+		}
+		if( ap = getdomainbyaddrX(iaddr,base,1,1) ){
+			addrHit++;
+			return ap;
+		}
+	}else{
+		sbase = base;
+		if( dir < 0 ){
+			int i;
+			for( i = base; 0 < i; i-- ){
+				dnp = &DnTab[i];
+				if( iaddr<dnp->maskedL && iaddr<dnp->maskedH ){
+					sbase = i;
+					break;
+				}
+			}
+		}
+		if( ap = getdomainbyaddrX(iaddr,sbase,1,1024) ){
+			addrHit++;
+			return ap;
+		}
+	}
+
+	addrMiss++;
+	return getdomainbyaddrX(iaddr,0,1,0);
+}
+static const char *getdomainbyaddrX(int iaddr,int ai,int inc,int maxs)
+/*
+{	int si,ma;
+*/
+{	int si;
+	unsigned int ma;
+	DName *dnp;
+
+/*
 	ma = iaddr & getMASK(iaddr);
 	for( ai = 0; ai < DnTabX; ai++ ){
+*/
+	ma = iaddr;
+	for( si = 0; 0 <= ai && ai < DnTabX; ai += inc ){
+		if( maxs && maxs <= si )
+			break;
+		si++;
+		addrTries++;
 		dnp = &DnTab[ai];
 		if( dnp->full ){
 			if( dnp->maskedL <= iaddr && iaddr <= dnp->maskedH )
+			{
 				return dnp->name;
+			}
 		}else{
 			if( dnp->maskedL <= ma && ma <= dnp->maskedH )
+			{
 				return dnp->name;
+			}
 		}
+		if( maxs ){
+			if( inc < 0 ){
+				if( ma < dnp->maskedL && ma < dnp->maskedH ){
+					return 0;
+				}
+			}
+		}
+/*
+if( si % 1000 == 0 )
+fprintf(stderr,"--- %2d %2d %4d %8X [%8X - %8X]\n",si,inc,ai,ma,dnp->maskedL,dnp->maskedH);
+*/
 	}
 	return 0;
 }
@@ -272,7 +411,10 @@ static int is_inetaddr(PCStr(addr))
 }
 static void strip_inaddr(PVStr(host))
 {	const char *sp;
+	/*
 	const char *psp;
+	*/
+	refQStr(psp,host);
 	int a1,a2,a3,a4;
 
 	if( (sp = strstr(host,".IN-ADDR.ARPA"))
@@ -281,7 +423,21 @@ static void strip_inaddr(PVStr(host))
 		for( sp--; host < sp; sp-- ){
 			if( *sp == '.' ){
 				if( sp[1] < '0' || '9' < sp[1] ){
+					/*
 					truncVStr(psp);
+					*/
+		if( psp ){
+			switch( sscanf(psp+1,"%d.%d.%d.%d",&a1,&a2,&a3,&a4) ){
+			case 4:
+				sprintf(host,"%d.%d.%d.%d",a4,a3,a2,a1);
+				return;
+			case 3:
+				sprintf(host,"%d.%d.%d.%d",a3,a2,a1,0);
+				return;
+			}
+		}
+					strsubst(BVStr(host),".","-");
+					strcat(host,".BROKEN");
 					break;
 				}
 				psp = sp;
@@ -312,6 +468,10 @@ int ntod_main(int ac,char *av[])
 	CStr(aliasb,1024);
 	CStr(hostsdb,1024);
 	int ai,add_host;
+	int dump_tab = 0;
+	int li;
+	double Start,Prev,Now;
+	int fskip = 0;
 
 	if( strstr(av[0],"uinc") )
 		return uinc_main(ac,av);
@@ -320,6 +480,11 @@ int ntod_main(int ac,char *av[])
 	for( ai = 1; ai < ac; ai++ ){
 		if( av[ai][0] == '+' )
 			add_host = 1;
+		if( strcmp(av[ai],"-d") == 0 ){
+			dump_tab = 1;
+		}
+		if( av[ai][0] == '-' && isdigit(av[ai][1]) )
+			fskip = atoi(&av[ai][1]);
 	}
 	if( add_host )
 		RES_verify("");
@@ -344,21 +509,66 @@ int ntod_main(int ac,char *av[])
 
 	load_dntab(hostsfile);
 	load_dntab(dbfile);
+	sort_dntab();
+	if( dump_tab ){
+		int i;
+		for( i = 0; i < DnTabX; i++ )
+		printf("%8X - %8X %5d %5d %s\n",DnTab[i].maskedL,
+			DnTab[i].maskedH,i,DnTab[i].serno,DnTab[i].name);
+		// printf("%8X - %8X\n",DnTab[i].maskedL,DnTab[i].maskedH);
+		exit(0);
+	}
 
 	RES_order("D",VStrNULL);
 
+	Start = Prev = Time();
+	li = 0;
 	while( Fgets(AVStr(iline),sizeof(iline),stdin) != NULL ){
+		li++;
+		if( li % 50000 == 0 ){
+			Now = Time();
+			fprintf(stderr,"%6.1f %4.1f %6d ... %5d : %5d (%d)\n",
+				Now-Start,Now-Prev,li,addrHit,addrMiss,addrTries);
+			Prev = Now;
+		}
 		hp = hostaddr;
 		prefix[0] = 0;
+		sp = iline;
+		if( fskip ){
+			const char *dp,*np;
+			int fi;
+			dp = iline;
+			for( fi = 0; fi < fskip; fi++ ){
+				np = strchr(dp,' ');
+				if( np == 0 )
+					break;
+				dp = np;
+				while( *dp == ' ' )
+					dp++;
+			}
+			if( iline < dp ){
+				QStrncpy(prefix,iline,dp-iline+1);
+			}
+			sp = dp;
+		}
+		/*
 		for( sp = iline; sc = *sp; sp++ ){
+		*/
+		for( ; sc = *sp; sp++ ){
 			if( sc == '@' ){
 				truncVStr(sp);
 				strcpy(prefix,iline);
+				strcat(prefix,"@");
 				hp = hostaddr;
 				continue;
 			}
 			if( sc==' '||sc=='\t'||sc=='\r'||sc=='\n' )
 				break;
+			if( sc==',' )
+				break;
+
+			if( sc == '.' && (sp[1] == 0 || isspace(sp[1])) ){
+			}else
 			if( isupper(sc) )
 				setVStrPtrInc(hp,tolower(sc));
 			else	setVStrPtrInc(hp,sc);
@@ -426,14 +636,24 @@ int ntod_main(int ac,char *av[])
 			generic_domain(AVStr(hostaddr));
 		}
 	PUT:
+/*
 		fseek(stdout,0,2);
+*/
 		if( prefix[0] != 0 )
+			fprintf(stdout,"%s",prefix);
+			/*
 			fprintf(stdout,"%s@",prefix);
+			*/
 		fputs(hostaddr,stdout);
 		fputs(remain,stdout);
 		fputs("\n",stdout);
+/*
 		fflush(stdout);
+*/
 	}
+	Now = Time();
+	fprintf(stderr,"%6.1f %4.1f %6d ... %5d : %5d (%d) Finished\n",
+		Now-Start,Now-Prev,li,addrHit,addrMiss,addrTries);
 	return 0;
 }
 
@@ -466,7 +686,7 @@ static int uinc_main(int ac,char *av[])
 			REV = 1;
 	}
 
-	hostid = strid_create(0x40000);
+	hostid = strid_create(0x80000);
 
 	while( fgets(line,sizeof(line),in) != NULL ){
 		if( Xsscanf(line,"%s %*s %*s %[^,]",AVStr(date),AVStr(host)) == 2 ){
@@ -533,3 +753,7 @@ int acceptViaSocks(int sock,PVStr(rhost),int *rport){ return -1; }
 int bindViaSocks(DGC*Conn,PCStr(dsthost),int dstport,PVStr(rhost),int *rport){ return -1; }
 int GetViaSocks(DGC*Conn,PCStr(host),int port){ return 0; }
 int CTX_auth(DGC*ctx,PCStr(user),PCStr(pass)){ return 0; }
+int VSA_getViaSocksX(DGC*ctx,PCStr(h),int p,VSAddr *sv,AuthInfo *au,VSAddr *lo){
+        return 0;
+}
+int serverPid(){ return getpid(); }
