@@ -1,11 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1997-1999 Yutaka Sato
-Copyright (c) 1997-1999 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1997-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, and distribute this material for any purpose
-and without fee is hereby granted, provided that the above copyright
-notice and this permission notice appear in all copies.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -64,6 +67,11 @@ static FILE *hosts_cache(int byname,PCStr(nameaddr),PCStr(mode),PVStr(cpath))
 	else	sprintf(cdirs,"%s/%s",tmpdir,RES_HC_DIR);
 	sprintf(cdirg,"%s/%s",cdirs,resid);
 	sprintf(cdir1,"%s/%s",cdirg,byname?"byname":"byaddr");
+
+	if( byname != 1 ){
+		strcat(cdir1,"6");
+	}
+
 	sprintf(cpath,"%s/%02x",cdir1,hidx%32);
 	if( *mode == 'r' )
 		debug(DBG_ANY,"lookup cache: %s\n",cpath);
@@ -201,7 +209,11 @@ int gethostbynameaddr_cacheX(PCStr(dir),PCStr(name),int rrc,char *rv[],xPVStr(rb
 	return ac;
 }
 
+const char *VSA_ltoa(const unsigned char *baddr,int len,int type);
+/*
 void puthost_cache(PCStr(nameaddr),char *rv[],int byname,PVStr(cname))
+*/
+void puthost_cache(PCStr(nameaddr),char *rv[],int byname,PVStr(cname),int len,int type)
 {	int ai;
 	const char *n1;
 	const unsigned char *a1;
@@ -240,7 +252,10 @@ void puthost_cache(PCStr(nameaddr),char *rv[],int byname,PVStr(cname))
 	    }else
 	    for( ai = 0; a1 = (unsigned char *)rv[ai]; ai++ ){
 		fprintf(cache,"%d ",now);
+/*
 		fprintf(cache,"%d.%d.%d.%d\t",a1[0],a1[1],a1[2],a1[3]);
+*/
+		fprintf(cache,"%s\t",VSA_ltoa(a1,len,type));
 		if( cname && cname[0] && strcmp(cname,nameaddr) != 0 )
 			fprintf(cache,"%s ",cname);
 		fprintf(cache,"%s\n",nameaddr);
