@@ -1,11 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1995-1999 Yutaka Sato
-Copyright (c) 1995-1999 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1995-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, and distribute this material for any purpose
-and without fee is hereby granted, provided that the above copyright
-notice and this permission notice appear in all copies.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -24,6 +27,7 @@ History:
 #include "ystring.h"
 #include "vsocket.h"
 #include "file.h"
+#include "log.h"
 #include "dns.h"
 void SOCKS_addserv(PCStr(dhost),int dport,PCStr(shost),int sport);
 int RES_next_res(PCStr(where),int ri,PVStr(res),PVStr(arg));
@@ -71,10 +75,11 @@ typedef struct {
 	char	am_addr[ADDRLEN];
 	char	am_mask[ADDRLEN];
 } AddrMask;
-#define RESOLVERS_SIZ 512
 
 typedef struct {
 	MStr(	re_Resolvers,RESOLVERS_SIZ);
+  const char   *re_ResolversPushed[64];
+	int	re_ResolversPushedFlags[64];
 	AddrMask re_sort_list[MAX_SORTLIST]; /**/
 	int	re_Nsortlist;
 	MStr(	re__confid,64);
@@ -92,6 +97,9 @@ static ResolvyEnv *resolvyEnv;
 #define Resolvers	RE.re_Resolvers
 /* MStr(Resolvers,re_Resolvers) */
 /**/
+#define PushedResolvers	RE.re_ResolversPushed
+#define PushedResolversFlags	RE.re_ResolversPushedFlags
+#define RES_ADDLAST 1
 #define sort_list	RE.re_sort_list
 #define Nsortlist	RE.re_Nsortlist
 #define _confid		RE.re__confid
@@ -133,10 +141,52 @@ void minit_resconf()
 	}
 	minit_reshost();
 }
-char *RES_resolvers()
+int getthreadix();
+char *RES_resolvers(PVStr(resolvers))
 {
+	const char *res;
+	const char *pushed;
+	int tix = getthreadix();
+	if( 0 <= tix && tix < elnumof(PushedResolvers) ){
+		if( pushed = PushedResolvers[tix] ){
+			if( PushedResolversFlags[tix] & RES_ADDLAST ){
+				strcpy(resolvers,Resolvers);
+				if( strstr(resolvers,pushed) == 0 )
+					strcat(resolvers,pushed);
+			}else{
+				strcpy(resolvers,pushed);
+			}
+			return (char*)resolvers;
+		}
+	}
+	strcpy(resolvers,Resolvers);
 	return Resolvers;
 }
+int RES_orderSet_FL(FL_PAR,PCStr(resolvers)){
+	strcpy(Resolvers,resolvers);
+	return 0;
+}
+int RES_orderGet_FL(FL_PAR,PVStr(resolvers)){
+	strcpy(resolvers,Resolvers);
+	return 0;
+}
+int RES_orderPush_FL(FL_PAR,PCStr(tmpres),int flags){
+	int tix = getthreadix();
+	if( 0 <= tix && tix < elnumof(PushedResolvers) ){
+		PushedResolvers[tix] = tmpres;
+		PushedResolversFlags[tix] = flags;
+	}else{
+	}
+	return 0;
+}
+int RES_orderPop_FL(FL_PAR){
+	int tix = getthreadix();
+	if( 0 <= tix && tix < elnumof(PushedResolvers) ){
+		PushedResolvers[tix] = 0;
+	}else{
+	}
+	return 0;
+}
 
 const char *_RSLV_CONF = RSLV_CONF;
 const char *_HOSTSFILE = HOSTSFILE;
@@ -184,6 +234,18 @@ char *RES_confid(PVStr(id))
 	return (char*)id;
 }
 #define clear_confid()	_confid[0] = 0;
+char *RES_confidSet_FL(FL_PAR,PCStr(wh),PVStr(prev)){
+	IStr(oldid,128);
+	IStr(newid,128);
+	if( prev ){
+		strcpy(prev,_confid);
+	}
+	strcpy(oldid,_confid);
+	clear_confid();
+	RES_confid(AVStr(newid));
+	debug(DBG_FORCE,"confid(%s)[%s]<-[%s]\n",wh,newid,oldid);
+	return _confid;
+}
 
 static int bindINS(VSAddr *sin,int nlisten)
 {	int sock,rcode;
@@ -213,10 +275,21 @@ static int connINS(VSAddr *sin)
 	}
 }
 
+int RES_SYS_TIMEOUT = 3;
 static struct hostent *_GethostByname(PCStr(name))
 {	struct hostent *hp;
+	double St = Time();
+	double Et;
 
+	/*
 	hp = (struct hostent *)callFuncTimeout(3,NULL,(void*(*)(void*,...))EX_GETHOSTBYNAME,name);
+	*/
+	hp = (struct hostent *)callFuncTimeout(RES_SYS_TIMEOUT,NULL,(void*(*)(void*,...))EX_GETHOSTBYNAME,name);
+	Et = Time() - St;
+	if( 3 <= Et ){
+		fprintf(stderr,"[%d] slow gethostbyname(%s)=%X [%.2f]\n",getpid(),name,hp,Et);
+		debug(DBG_FORCE,"slow gethostbyname(%s)=%X [%.2f/%d]\n",name,hp,Et,RES_SYS_TIMEOUT);
+	}
 	return hp;
 }
 
@@ -308,7 +381,28 @@ void RES_nsloopcheck(int mysock)
 	}
 }
 
+const char *scanHostport(PCStr(ns),PVStr(nsb),int *portp){
+	strcpy(nsb,ns);
+	if( strchr(ns,':') ){
+		Xsscanf(ns,"%[^:]:%d",BVStr(nsb),portp);
+		return nsb;
+	}else
+	if( strstr(ns,"..") ){
+		refQStr(np,nsb);
+		strcpy(nsb,ns);
+		if( np = strstr(nsb,"..") ){
+			setVStrEnd(np,0);
+			*portp = atoi(np+2);
+			return nsb;
+		}
+	}
+	return ns;
+}
+
+/*
 void RES_socks(PCStr(ns),PCStr(socks));
+*/
+void RES_socks(VSAddr *nsa,PCStr(socks));
 void RES_ns1(State *res,PCStr(ns),PCStr(domain))
 {
 	VSAddr sin;
@@ -324,6 +418,13 @@ void RES_ns1(State *res,PCStr(ns),PCStr(domain))
 
 	if( ns_stop )
 		return;
+	if( *ns == '$' ){
+		if( ServerMain )
+		debug(DBG_FORCE,"---- RES_ns1(%s) ignored\n",ns);
+		/* RES_ORDER=D:$N as an internal format */
+		return;
+	}
+	putResTrace("NS(%s)",ns);
 
 	clear_confid();
 	nsx = -1;
@@ -350,10 +451,13 @@ void RES_ns1(State *res,PCStr(ns),PCStr(domain))
 	}
 
 	port = PORT_DNS;
+	ns = scanHostport(ns,AVStr(nsbp),&port);
+	/*
 	if( strchr(ns,':') ){
 		Xsscanf(ns,"%[^:]:%d",AVStr(nsbp),&port);
 		ns = nsbp;
 	}
+	*/
 
 	if( VSA_strisaddr(ns) )
 		VSA_atosa(&sin,port,ns);
@@ -382,7 +486,10 @@ void RES_ns1(State *res,PCStr(ns),PCStr(domain))
 		return;
 	}
 	saddr = VSA_ntoa(&sin);
+	RES_socks(&sin,ssb);
+	/*
 	RES_socks(saddr,ssb);
+	*/
 
 	if( strcmp(domain,RES_NSDOM0) == 0 ){
 	  int nsi;
@@ -424,6 +531,22 @@ int RES_getns1(int nsi,VSAddr *sin)
 	}
 	return 0;
 }
+const char *VSA_ntoa(VSAddr *sap);
+int RES_getnslist(PVStr(list)){
+	refQStr(lp,list);
+	int ni;
+	VSAddr sin;
+	const char *addr;
+
+	for( ni = 0; ni < _RES.nscount; ni++ ){
+		if( RES_getns1(ni,&sin) ){
+			if( addr = VSA_ntoa(&sin) ){
+				Rsprintf(lp,"%s%s",0<ni?",":"",addr);
+			}
+		}
+	}
+	return ni;
+}
 
 /* acting as a proxy */
 int RES_proxy(){
@@ -592,6 +715,11 @@ void RES_verifyFaiure(PCStr(host),PVStr(badhost))
 
 int RES_debug(PCStr(debug))
 {
+	if( streq(debug,"0") ){
+		_RES.options = 0;
+		DNS_debug = 0;
+		return 0;
+	}
 	_RES.options |= RES_DEBUG;
 	if( strncmp(debug,"0x",2) == 0 )
 	sscanf(debug+2,"%x",&DNS_debug);
@@ -813,6 +941,7 @@ void RES_init()
 	CStr(res1,RESOLVERS_SIZ);
 	CStr(arg,RESOLVERS_SIZ);
 
+	putResTrace("Init");
 	minit_resconf();
 
 	if( _RES.options & RES_INIT )
@@ -1016,20 +1145,34 @@ int RES_timeout(int timeout)
 	return 1;
 }
 
+/*
 void RES_socks(PCStr(ns),PCStr(socks))
+*/
+void RES_socks(VSAddr *nsa,PCStr(socks))
 {	CStr(host,256);
 	int port;
+	const char *ns;
+	int nsport;
 
 	if( *socks == 0 )
 		return;
 
 	port = 1080;
+	socks = scanHostport(socks,AVStr(host),&port);
+	/*
 	Xsscanf(socks,"%[^:]:%d",AVStr(host),&port);
+	*/
 	if( !VSA_strisaddr(host) )
 		return;
 
+	ns = VSA_ntoa(nsa);
+	nsport = VSA_port(nsa);
+	debug(DBG_ANY,"SOCKS=%s:%d:%s..%d\n",host,port,ns,nsport);
+	SOCKS_addserv(ns,nsport,host,port);
+	/*
 	debug(DBG_ANY,"SOCKS=[%s][%s:%d]\n",ns,host,port);
 	SOCKS_addserv(ns,PORT_DNS,host,port);
+	*/
 }
 
 char **res_DNSRCH(){
