@@ -1,11 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1995-1999 Yutaka Sato
-Copyright (c) 1995-1999 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1995-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, and distribute this material for any purpose
-and without fee is hereby granted, provided that the above copyright
-notice and this permission notice appear in all copies.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -25,15 +28,12 @@ History:
 #include "dns.h"
 void RES_init();
 void res_log(int which,int byname,PCStr(name),char *rv[],PCStr(cname));
-
 void sort_ipaddrs(const char *addrs[]);
-char *RES_resolvers();
-#define resolvers RES_resolvers()
 
 int ipv6_domain(PCStr(caddr),PVStr(name));
-int gethostbynameaddr_dns(PCStr(name),int qtype,int rrc,char *rrv[],PVStr(rrb),PVStr(cname));
+int gethostbynameaddr_dns(PCStr(ns),PCStr(name),int qtype,int rrc,char *rrv[],PVStr(rrb),PVStr(cname));
 int gethostbynameaddr_sys(PCStr(name),int qtype,int rrc,char *rv[],PVStr(rb),PVStr(cname));
-int gethostbynameaddr_dnsrch(PCStr(name),int qtype,int rrc,char *rv[],PVStr(rb),PVStr(cname));
+int gethostbynameaddr_dnsrch(PCStr(where),PCStr(ns),PCStr(name),int qtype,int rrc,char *rv[],PVStr(rb),PVStr(cname));
 int gethostbynameaddr_cache(PCStr(dir),PCStr(name),int rrc,char *rv[],PVStr(rb),int byname,PVStr(cname),int noexpire);
 void puthost_cache(PCStr(nameaddr),char *rv[],int byname,PVStr(cname),int len,int type);
 int rem_unknown(char *rv[],PCStr(unknown),int leng);
@@ -83,6 +83,16 @@ void minit_reshost()
 	}
 }
 
+#if !defined(_MSC_VER)
+struct hostent *gethostbyname2X(const char *name,int af);
+#define gethostbyname2(n,a) gethostbyname2X(n,a)
+#endif
+int with_gethostbyname2(){
+	struct hostent *ht;
+	ht = gethostbyname2("::",AF_INET6);
+	return ht != NULL;
+}
+
 int RES_NOSYS;
 int gethostbynameaddr_sys(PCStr(name),int qtype,int rrc,char *rv[],xPVStr(rb),PVStr(cname))
 {	struct hostent *ht;
@@ -153,9 +163,50 @@ int gethostbynameaddr_sys(PCStr(name),int qtype,int rrc,char *rv[],xPVStr(rb),PV
 	return ac;
 }
 
+int RES_next_res(PCStr(where),int ri,PVStr(res),PVStr(arg));
+int (*RES_hlmatch)(PCStr(hlist),PCStr(host));
+const char *RES_client;
+int RES_client_dependent;
+static int clientmatch(PCStr(host),PCStr(where),PCStr(res1),PVStr(arg)){
+	int match = 1;
+	refQStr(client,arg);
+
+	client = strchr(arg,'<');
+	if( client == 0 ){
+		return -1;
+	}
+	setVStrPtrInc(client,0);
+	if( RES_client == 0 ){
+		return 0;
+	}
+	RES_client_dependent = 1;
+	match = (*RES_hlmatch)(client,RES_client);
+	return match;
+}
+static int dommatch(PCStr(host),PCStr(where),PCStr(res1),PVStr(arg)){
+	int match = 1;
+	refQStr(dom,arg);
+
+	if( RES_hlmatch == 0 )
+		return 1;
+
+	dom = strchr(arg,'#');
+	if( dom == 0 ){
+		return 1;
+	}
+	setVStrPtrInc(dom,0);
+	match = (*RES_hlmatch)(dom,host);
+	return match;
+}
+/*
 static int dnsrch1(PCStr(dname),PCStr(name),int qtype,int rrc,char *rv[],PVStr(rb),PVStr(cname))
+*/
+static int dnsrch1(PCStr(where),PCStr(dname),PCStr(name),int qtype,int rrc,char *rv[],PVStr(rb),PVStr(cname))
 {	CStr(fqdn,512);
 	int ac;
+	int ri;
+	CStr(res1,512);
+	CStr(arg,512);
 
 	if( dname == NULL )
 		strcpy(fqdn,name);
@@ -164,7 +215,23 @@ static int dnsrch1(PCStr(dname),PCStr(name),int qtype,int rrc,char *rv[],PVStr(r
 		sprintf(fqdn,"%s.",name);
 	else	sprintf(fqdn,"%s.%s",name,dname);
 
+	/*
 	ac = gethostbynameaddr_dns(fqdn,qtype,rrc,rv,AVStr(rb),AVStr(cname));
+		should call gethosbyname_all() to match xxx.domain not
+		in DNS resolver ?
+	*/
+	ac = 0;
+	for( ri = 0; ri = RES_next_res(where,ri,AVStr(res1),AVStr(arg)); ){
+		if( res1[0] != RT_DNS )
+			continue;
+		if( dommatch(fqdn,where,res1,AVStr(arg)) == 0 ){
+			continue;
+		}
+		ac = gethostbynameaddr_dns(arg,fqdn,qtype,rrc,rv,AVStr(rb),AVStr(cname));
+		if( 0 < ac )
+			break;
+	}
+
 	if( 0 < ac ){
 		if( cname != NULL && *cname == 0 )
 		if( strcmp(name,fqdn) != 0 )
@@ -176,14 +243,26 @@ static int dnsrch1(PCStr(dname),PCStr(name),int qtype,int rrc,char *rv[],PVStr(r
 extern int MIN_ABSNDOTS;
 extern char **res_DNSRCH();
 extern char *res_DEFDNAME();
+int RES_ASIS;
+int RES_CACHED_UNKNOWN;
 
+/*
 int gethostbynameaddr_dnsrch(PCStr(name),int qtype,int rrc,char *rv[],PVStr(rb),PVStr(cname))
+*/
+int gethostbynameaddr_dnsrch(PCStr(where),PCStr(ns),PCStr(name),int qtype,int rrc,char *rv[],PVStr(rb),PVStr(cname))
 {	int ndots,tryabs,abstrial;
 	int ac,si;
 	const char *np;
 	const char *dname;
 	char **dnsrch;
 
+	if( RES_ASIS ){
+		ac = gethostbynameaddr_dns(ns,name,qtype,rrc,rv,AVStr(rb),AVStr(cname));
+		if( 0 < ac )
+			return ac;
+		else	return -1;
+	}
+
 	ndots = 0;
 	for( np = name; *np; np++ )
 		if( *np == '.' )
@@ -194,7 +273,7 @@ int gethostbynameaddr_dnsrch(PCStr(name),int qtype,int rrc,char *rv[],PVStr(rb),
 
 	if( tryabs ){
 		abstrial++;
-		if( 0 < (ac = dnsrch1(NULL, name,qtype,rrc,rv,AVStr(rb),AVStr(cname))) )
+		if( 0 < (ac = dnsrch1(where,NULL, name,qtype,rrc,rv,AVStr(rb),AVStr(cname))) )
 			return ac;
 	}
 	if( dnsrch = res_DNSRCH() ){
@@ -202,17 +281,17 @@ int gethostbynameaddr_dnsrch(PCStr(name),int qtype,int rrc,char *rv[],PVStr(rb),
 		if( strcmp(dname,".") == 0 )
 			abstrial++;
 		debug(DBG_ANY,"DNSRCH[%d] = %s\n",si,dname);
-		if( 0 < (ac = dnsrch1(dname,name,qtype,rrc,rv,AVStr(rb),AVStr(cname))) )
+		if( 0 < (ac = dnsrch1(where,dname,name,qtype,rrc,rv,AVStr(rb),AVStr(cname))) )
 			return ac;
 	    }
 	}else
 	if( dname = res_DEFDNAME() ){
 		debug(DBG_ANY,"DEFDNAME = %s\n",dname);
-		if( 0 < (ac = dnsrch1(dname,name,qtype,rrc,rv,AVStr(rb),AVStr(cname))) )
+		if( 0 < (ac = dnsrch1(where,dname,name,qtype,rrc,rv,AVStr(rb),AVStr(cname))) )
 			return ac;
 	}
 	if( abstrial == 0 ){
-		if( 0 < (ac = dnsrch1(NULL, name,qtype,rrc,rv,AVStr(rb),AVStr(cname))) )
+		if( 0 < (ac = dnsrch1(where,NULL, name,qtype,rrc,rv,AVStr(rb),AVStr(cname))) )
 			return ac;
 	}
 	return -1;
@@ -268,7 +347,12 @@ static int caching(int lastres,int ac,PCStr(nameaddr),char *rv[],int byname,PVSt
 			byname = AF_INET6;
 		puthost_cache(nameaddr,rv,byname,AVStr(cname),len,type);
 	}else{
+		int iac = ac;
 		ac = rem_unknown(rv,unknown_mark,marklen);
+		if( 0 < iac && ac == 0 ){
+			/* should return 2 for byname & IPv6 */
+			RES_CACHED_UNKNOWN = 1;
+		}
 	}
 	return ac;
 }
@@ -280,6 +364,8 @@ int gethostbyname_all(PCStr(where),char rwhere[],PCStr(name),int rrc,char *rv[],
 	int lastres;
 	int with_cache = 0;
 	int leng;
+	int byname = (RES_QTYPE == AF_INET6) ? AF_INET6 : 1;
+	int clmatch = 0;
 
 	res_log(0,0,0,0,0);
 
@@ -298,32 +384,49 @@ int gethostbyname_all(PCStr(where),char rwhere[],PCStr(name),int rrc,char *rv[],
 	for( ri = 0; ri = RES_next_res(where,ri,AVStr(res1),AVStr(arg)); ){
 	  debug(DBG_ANY,"        RES[%s] %s\n",res1,where);
 	  res = res1[0];
+
+	  if( (clmatch = clientmatch(name,where,res1,AVStr(arg))) == 0 ){
+		continue;
+	  }
+	  if( !RES_ASIS && res == RT_DNS ){
+		/* will be filtered in dnsrch() after extented with DEFDNAME */
+	  }else
+	  if( dommatch(name,where,res1,AVStr(arg)) == 0 ){
+		continue;
+	  }
 	  lastres = res;
 	  switch(res){
+	    case RT_UNKNOWN: /* emulate cached unknown */
+		rv[0] = 0;
+		RES_CACHED_UNKNOWN = 3 /* UNKNOWN_V4|UNKNOWN_V6 */;
+		debug(DBG_NS,"Hit: 0 (forced Unknown for '%s')\n",name);
+		return 0;
+
 	    case RT_CACHE:
-	    {	int byname;
 		with_cache = 1;
-		if( RES_QTYPE == AF_INET6 )
-			byname = AF_INET6;
-		else	byname = 1;
 		ac = gethostbynameaddr_cache(arg,name,rrc,rv,AVStr(rb),byname,AVStr(cname),0);
 
 /*
 		ac = gethostbynameaddr_cache(arg,name,rrc,rv,AVStr(rb),1,AVStr(cname),0);
 */
 		break;
-	    }
 	    case RT_FILE:
+		ac = gethostbynameaddr_file(arg,name,rrc,rv,AVStr(rb),byname,AVStr(cname));
+		/*
 		ac = gethostbynameaddr_file(arg,name,rrc,rv,AVStr(rb),1,AVStr(cname));
+		*/
 		break;
 	    case RT_NIS:
+		ac = gethostbynameaddr_nis(_NISMAP_NAME,arg,name,rrc,rv,AVStr(rb),byname,AVStr(cname));
+		/*
 		ac = gethostbynameaddr_nis(_NISMAP_NAME,arg,name,rrc,rv,AVStr(rb),1,AVStr(cname));
+		*/
 		break;
 	    case RT_DNS:
 		if( RES_QTYPE == AF_INET6 ){
-ac = gethostbynameaddr_dnsrch(name,TY_AAAA,rrc,rv,AVStr(rb),AVStr(cname));
+		ac = gethostbynameaddr_dnsrch(where,arg,name,TY_AAAA,rrc,rv,AVStr(rb),AVStr(cname));
 		}else
-		ac = gethostbynameaddr_dnsrch(name,TY_A,rrc,rv,AVStr(rb),AVStr(cname));
+		ac = gethostbynameaddr_dnsrch(where,arg,name,TY_A,rrc,rv,AVStr(rb),AVStr(cname));
 		break;
 	    case RT_SYS:
 		if( RES_QTYPE == AF_INET6 ){
@@ -332,6 +435,7 @@ ac = gethostbynameaddr_sys(name,TY_AAAA,rrc,rv,AVStr(rb),AVStr(cname));
 		ac = gethostbynameaddr_sys(name,TY_A,rrc,rv,AVStr(rb),AVStr(cname));
 		break;
 	  }
+	  putResTrace("N{%c%d}",res,ac);
 	  if( 0 < ac ){
 		rwhere[0] = res;
 		break;
@@ -342,7 +446,12 @@ ac = gethostbynameaddr_sys(name,TY_AAAA,rrc,rv,AVStr(rb),AVStr(cname));
 		rv[ac] = 0;
 	else	rv[0] = 0;
 
+	RES_CACHED_UNKNOWN = 0;
 	if( with_cache && lastres )
+	if( RES_client_dependent ){
+		/* don't cache client dependent result */
+	}
+	else
 		ac = caching(lastres,ac,name,rv,1,AVStr(cname),UNKNOWN_HOSTADDR,4);
 
 	debug(DBG_NS,"Hit: %d\n",ac);
@@ -369,7 +478,7 @@ static int isHostnameOf(int namec,char *namev[],PCStr(nameb),PCStr(caddr))
 
 	ok = 0;
 	for( ai = 0; ai < namec; ai++ ){
-		na = gethostbynameaddr_dns(namev[ai],TY_A,elnumof(av),av,AVStr(ab),AVStr(cn));
+		na = gethostbynameaddr_dns("",namev[ai],TY_A,elnumof(av),av,AVStr(ab),AVStr(cn));
 		if( na == 0 ){
 			debug(DBG_FORCE,"DNS INCONSISTENT: %s -> %s -> ?\n",
 				caddr,namev[ai]);
@@ -394,6 +503,7 @@ int gethostbyaddr_all(PCStr(where),char rwhere[],PCStr(caddr),int rrc,char *rv[]
 	CStr(res1,512);
 	CStr(arg,512);
 	int with_cache = 0;
+	int clmatch = 0;
 
 	res_log(0,0,0,0,0);
 
@@ -402,11 +512,31 @@ int gethostbyaddr_all(PCStr(where),char rwhere[],PCStr(caddr),int rrc,char *rv[]
 	lastres = 0;
 	rwhere[0] = 0;
 
+	if( isWindowsCE() ){
+		if( strheadstrX(caddr,"192.168.",0)
+		 || strheadstrX(caddr,"127.0.0.",0)
+		){
+			putResTrace("IGN{%s/%s}",caddr,where);
+			return 0;
+		}
+	}
 	for( ri = 0; ri = RES_next_res(where,ri,AVStr(res1),AVStr(arg)); ){
 	  debug(DBG_ANY,"        RES[%s] %s\n",res1,where);
+	  if( (clmatch = clientmatch(caddr,where,res1,AVStr(arg))) == 0 ){
+		continue;
+	  }
+	  if( dommatch(caddr,where,res1,AVStr(arg)) == 0 ){
+		continue;
+	  }
 	  res = res1[0];
 	  lastres = res;
 	  switch(res){
+	    case RT_UNKNOWN: /* emulate cached unknown */
+		rv[0] = 0;
+		RES_CACHED_UNKNOWN = 3;
+		debug(DBG_NS,"Hit: 0 (forced Unknown for '%s')\n",caddr);
+		return 0;
+
 	    case RT_CACHE:
 		with_cache = 1;
 		ac = gethostbynameaddr_cache(arg,caddr,rrc,rv,AVStr(rb),0,VStrNULL,0);
@@ -429,7 +559,7 @@ int gethostbyaddr_all(PCStr(where),char rwhere[],PCStr(caddr),int rrc,char *rv[]
 		sprintf(name,"%d.%d.%d.%d.%s.",a4,a3,a2,a1,REVERSE_DOM);
 		}
 		if( res == RT_DNS )
-			ac = gethostbynameaddr_dns(name,TY_PTR,rrc,rv,AVStr(rb),VStrNULL);
+			ac = gethostbynameaddr_dns(arg,name,TY_PTR,rrc,rv,AVStr(rb),VStrNULL);
 		else	ac = gethostbynameaddr_sys(caddr,TY_PTR,rrc,rv,AVStr(rb),VStrNULL);
 		if( ac == 1 && rv[0] != NULL && rv[0][0] == 0 ){
 			debug(DBG_FORCE,"### ignored empty for byaddr()\n");
@@ -440,6 +570,7 @@ int gethostbyaddr_all(PCStr(where),char rwhere[],PCStr(caddr),int rrc,char *rv[]
 		}
 		break;
 	  }
+	  putResTrace("A{%c%d}",res,ac);
 	  if( 0 < ac ){
 		rwhere[0] = res;
 		break;
@@ -450,7 +581,12 @@ int gethostbyaddr_all(PCStr(where),char rwhere[],PCStr(caddr),int rrc,char *rv[]
 		rv[ac] = 0;
 	else	rv[0] = 0;
 
+	RES_CACHED_UNKNOWN = 0;
 	if( with_cache && lastres )
+	if( RES_client_dependent ){
+		/* don't cache client dependent result */
+	}
+	else
 		ac = caching(lastres,ac,caddr,rv,0,VStrNULL,UNKNOWN_HOSTNAME,0);
 
 	debug(DBG_NS,"Hit: %d\n",ac);
@@ -459,6 +595,7 @@ int gethostbyaddr_all(PCStr(where),char rwhere[],PCStr(caddr),int rrc,char *rv[]
 }
 
 char *RES_AF = "46";
+int strcanbeHostname(int af,PCStr(name));
 struct hostent *RES_gethostbyaddr(PCStr(baddr), int len, int type);
 struct hostent *RES_gethostbyname(PCStr(name))
 {	int ac;
@@ -468,14 +605,18 @@ struct hostent *RES_gethostbyname(PCStr(name))
 	const char *types;
 	const char *tp;
 	int qtype;
+	IStr(resolvers,RESOLVERS_SIZ);
 
 	RES_init();
 	debug(DBG_ANY,"gethostbyname(%s)\n",name);
 
 	if( isinetAddr(name) ){
 		int bleng,btype;
+		CStr(baddr,IPV6_ADDRLENG);
+		/*
 		CStr(baddr,16);
-		bleng = VSA_atob(name,baddr,&btype);
+		*/
+		bleng = VSA_atob(name,AVStr(baddr),&btype);
 		return RES_gethostbyaddr(baddr,bleng,btype);
 	}
 
@@ -491,6 +632,11 @@ struct hostent *RES_gethostbyname(PCStr(name))
 	if( strncmp(name,"_64.",4) == 0 ){ types = "64"; name += 4; }else
 	if( strncmp(name,"_46.",4) == 0 ){ types = "46"; name += 4; }
 
+	if( !strcanbeHostname(0,name) ){
+		debug(DBG_ANY,"---- RES_gethostbyname(%s) not a name\n",name);
+		return NULL;
+	}
+	RES_resolvers(AVStr(resolvers));
 	ac = 0;
 	for( tp = types; *tp; tp++ ){
 		if( *tp == '6' )
@@ -498,6 +644,10 @@ struct hostent *RES_gethostbyname(PCStr(name))
 		else	RES_QTYPE = 0;
 		ac = gethostbyname_all(resolvers,where,name,elnumof(Raddrv),Raddrv,AVStr(Raddrb),AVStr(cname));
 
+		if( RES_CACHED_UNKNOWN == 3 ){
+			/* don't try anymore for forced unknown */
+			break;
+		}
 		if( 0 < ac )
 			break;
 		if( *name == '_' ) /* retrieved _service not A/AAAA */
@@ -543,6 +693,7 @@ struct hostent *RES_gethostbyaddr(PCStr(baddr), int len, int type)
 	CStr(where2,8);
 	int ac;
 	const unsigned char *ba = (const unsigned char *)baddr;
+	IStr(resolvers,RESOLVERS_SIZ);
 
 	RES_init();
 	if( type == AF_INET6 ){
@@ -551,6 +702,7 @@ struct hostent *RES_gethostbyaddr(PCStr(baddr), int len, int type)
 	sprintf(caddr,"%d.%d.%d.%d",ba[0],ba[1],ba[2],ba[3]);
 	debug(DBG_ANY,"gethostbyaddr(%s)\n",caddr);
 
+	RES_resolvers(AVStr(resolvers));
 	ac = gethostbyaddr_all(resolvers,where,caddr,elnumof(Rnamev),Rnamev,AVStr(Rnameb));
 	if( ac <= 0 )
 		return NULL;
@@ -623,12 +775,29 @@ int RES_matchLine(PCStr(what),int byname,PCStr(name),PCStr(line),int rrc,char *r
 			if( host[0] == 0 )
 				break;
 					
+			/*
 			if( strcasecmp(host,name) == 0 ){
+			*/
+			if( strcasecmp(host,name) == 0
+			 || rexpmatchX(host,name,"c")
+			){
+				/*
 				CStr(baddr,16);
+				*/
+				CStr(baddr,IPV6_ADDRLENG);
 				int bleng;
 				debug(DBG_ANY,"RES: <%s> %s\n",what,line);
+				bleng = VSA_atob(addr,AVStr(baddr),NULL);
+				if( byname != AF_INET6 && 16 <= bleng ){
+					continue;
+				}else
+				if( bleng == 4 &&
+				    bcmp(baddr,UNKNOWN_HOSTADDR,bleng) == 0 ){
+				}else
+				if( byname == AF_INET6 && bleng < 16 ){
+					continue;
+				}
 				rv[ac++] = (char*)rb;
-				bleng = VSA_atob(addr,baddr,NULL);
 				Bcopy(baddr,rb,bleng);
 				rb += bleng;
 				strcpy(cname,host1);
@@ -710,8 +879,17 @@ int gethostbynameaddr_file(PCStr(path),PCStr(name),int rrc,char *rv[],xPVStr(rb)
 	HostFile *hp;
 	const char *lp;
 	int leng;
+	int hi;
 
 	hp = &Hosts[0];
+	for( hi = 0; hi < elnumof(Hosts)-1; hi++ ){
+		hp = &Hosts[hi];
+		if( hp->h_path == 0 )
+			break;
+		if( streq(hp->h_path,path) )
+			break;
+	}
+
 	now = time(0);
 	if( hp->h_buff == NULL || 60 < (now-hp->h_date) ){
 		if( path == NULL || path[0] == 0 )
@@ -726,7 +904,10 @@ int gethostbynameaddr_file(PCStr(path),PCStr(name),int rrc,char *rv[],xPVStr(rb)
 			return -1;
 		}
 
+		/*
 		readhosts1(path,fp,&Hosts[0]);
+		*/
+		readhosts1(path,fp,hp);
 
 		if( strncmp(path,"sh:",3) == 0 )
 			pclose(fp);
