@@ -18,18 +18,25 @@ Description:
 History:
 	950817	created
 //////////////////////////////////////////////////////////////////////#*/
-
-Finish(code)
+#include "ystring.h"
+#include "vaddr.h"
+void FinishX(PCStr(F),int L,int code)
 {
 	exit(code);
 }
 
-setBinaryIO(){ }
-start_service(){}
-WINthread(){}
-int WAIT_WNOHANG = -1;
+void setBinaryIO(){ }
+void WINthread(){}
 
-main(ac,av) char *av[]; { return resolvy_main(ac,av); }
+int resolvy_main(int ac,const char *av[]);
+int main(int ac,char *av[]){ return resolvy_main(ac,(const char**)av); }
 
-VSocket(){ return -1; }
-GetViaSocks(){ return 0; }
+typedef struct DGCtx DGC;
+int acceptViaSocks(int sock,PVStr(rhost),int *rport){ return -1; }
+int bindViaSocks(DGC*Conn,PCStr(dsthost),int dstport,PVStr(rhost),int *rport){ return -1; }
+int GetViaSocks(DGC*Conn,PCStr(host),int port){ return 0; }
+int CTX_auth(DGC*ctx,PCStr(user),PCStr(pass)){ return 0; }
+int VSA_getViaSocksX(DGC*ctx,PCStr(h),int p,VSAddr *sv,AuthInfo *au,VSAddr *lo){
+	return 0;
+}
+int serverPid(){ return getpid(); }
