@@ -1,11 +1,14 @@
 /*///////////////////////////////////////////////////////////////////////
-Copyright (c) 1995-1999 Electrotechnical Laboratry (ETL), AIST, MITI
-Copyright (c) 1995-1999 Yutaka Sato
+Copyright (c) 1995-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, and distribute this material for any purpose
-and without fee is hereby granted, provided that the above copyright
-notice and this permission notice appear in all copies.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -24,6 +27,54 @@ History:
 #include "vsocket.h"
 #include "fpoll.h"
 #include "dns.h"
+#include "log.h"
+
+#define MAXRSIZ	2*1024
+
+int DNS_dbgvul = 0;
+int DNS_svtcp = 0;
+int DNS_cltcp = 0;
+
+static int Start;
+static int Nsearch;
+static int Nrecv;
+static int Qlen;
+static int Rlen;
+static char ErrStat[128];
+extern int DNS_debug;
+
+static void errlog(PCStr(fmt),...){
+	CStr(msg,128);
+	int rem;
+	VARGS(8,fmt);
+
+	sprintf(msg,fmt,VA8);
+	rem = sizeof(ErrStat) - strlen(ErrStat) - 1;
+	if( rem <= strlen(msg) )
+		setVStrEnd(msg,rem);
+	Xstrcat(FVStr(ErrStat),msg);
+}
+static void dump(PCStr(what),int inTCP,const void *str,int len){
+	unsigned char *up = (unsigned char*)str;
+	int i;
+
+	if( (DNS_debug & DBG_DUMP) == 0 ){
+		return;
+	}
+	if( inTCP )
+		fprintf(stderr,"-- leng=%d/%d id=%d\n",len,(up[0]<<8)|up[1],
+			(up[2]<<8)|up[3]);
+	else	fprintf(stderr,"-- leng=%d id=%d\n",len,(up[0]<<8)|up[1]);
+	for( i = 0; i < len; i++){
+		if( i % 16 == 0 ){
+			if( i != 0 )
+				fprintf(stderr,"\n");
+			fprintf(stderr,"%X %s: ",&up[i],what);
+		}
+		fprintf(stderr,"%02X ",up[i]);
+	}
+	fprintf(stderr,"\n");
+}
 
 int setCloseOnExec(int fd);
 int file_ISSOCK(int fd);
@@ -151,8 +202,14 @@ static char *putName(xPVStr(qp),PCStr(name))
 	for( np = name; *np; np++ ){
 		lp = label;
 		if( *np == '.' ){
+			/*
 			debug(DBG_ANY,"FATAL ERROR: empty label [%s]\n",name);
 			return NULL;
+				... 9.0.3 NULL result from putName() is
+				... not cared at several callers
+			*/
+			debug(DBG_FORCE,"FATAL ERROR: empty label [%s]\n",name);
+			break;
 		}
 		lx = label + sizeof(label) - 1;
 		while( *np && *np != '.' )
@@ -182,6 +239,8 @@ static char *makeQuestion(PVStr(question),PCStr(name),int type,int xclass)
 	qp = putName(QVStr(qp,question),name);
 	if( qp == NULL )
 		return NULL;
+	if( question[0] == 0 )
+		return NULL;
 
 	setVStrPtrInc(qp,0xFF & (type >> 8));
 	setVStrPtrInc(qp,0xFF & type);
@@ -195,6 +254,13 @@ static const char *scanName(PCStr(msg),PCStr(ssp),PVStr(name),int siz,int lev)
 	int len,low,off,here;
 
 	cpyQStr(np,name);
+
+	if( siz <= 1 || 32 <= lev ){
+		errlog("InfiniteRecursion(lev=%d,rem=%d).",lev,siz);
+debug(DBG_FORCE,"### pointer loop? (%d): %x %x\n",lev,msg,ssp);
+		setVStrEnd(np,0);
+		return (const char*)sp;
+	}
 	while( len = *sp++ ){
 		if( len & 0xC0 ){
 			/* This is a pointer (2 bytes).  First 2 bits should
@@ -214,10 +280,20 @@ debug(DBG_ANY,"### bad pointer(%d): %x: %x\n",lev,here,len);
 			here = (char*)sp - msg;
 			if( here <= off ){
 debug(DBG_ANY,"### bad pointer(%d): %d <= %d: %x %x\n",lev,here,off,len,low);
+				errlog("ForwardPointer(%d/%d).",here,off);
 				sprintf(np,"BAD-POINTER-%x-%x",len,low);
 				np += strlen(np);
 				break;
 			}
+			if( off < here && here-2 <= off
+			 || ssp <= &msg[off]
+			){
+debug(DBG_FORCE,"### loop pointer: %d->%d: %X->%X\n",here,off,ssp,msg+off);
+				errlog("LoopPointer(%d/%d).",here,off);
+				sprintf(np,"LOOP-POINTER-%x",off);
+				np += strlen(np);
+				break;
+			}
 			scanName(msg,&msg[off],QVStr(np,name),siz-(np-name),lev+1);
 			return (char*)sp;
 		}else{
@@ -244,12 +320,17 @@ static const char *scanQuery(PCStr(msg),PCStr(question),PVStr(name),int nsiz,int
 	getShort(qp,*classp);
 	return qp;
 }
-static const char *dumpQuestion(int qdi,PCStr(msg),PCStr(question))
+static const char *dumpQuestion(int qdi,PCStr(msg),int size,PCStr(question))
 {	const char *qp;
 	CStr(name,512);
 	int type,xclass;
 	int nid;
 
+	if( msg+size <= question ){
+		debug(DBG_FORCE,"ERROR: dumpQ Overrun [%2d]: %X <= %X\n",
+			qdi,msg+size,question);
+		return NULL;
+	}
 	qp = scanQuery(msg,question,AVStr(name),sizeof(name),&type,&xclass);
 
 	nid = DNS_putbyname(name);
@@ -291,16 +372,16 @@ char *dumpResourceRecord(int whatx,int rri,PCStr(msg),int size,PCStr(rr))
 	const char *stype;
 	const char *sclass;
 	int nid,rnid;
-	/*
-	CStr(addr,32);
-	*/
 	CStr(addr,64);
 	CStr(rdata,512);
 	refQStr(rp,rdata);
 	int pref;
+	int newput;
 
 	what = SYM_RR[whatx];
 	if( msg+size <= rr ){
+		debug(DBG_FORCE,"ERROR: dumpR Overrun %s[%2d]: %X <= %X\n",
+			what,rri,msg+size,rr);
 		debug(DBG_ANY,"### %s[%2d]: %d >= size %d\n",
 			what,rri,rr-msg,size);
 		return 0;
@@ -313,6 +394,10 @@ char *dumpResourceRecord(int whatx,int rri,PCStr(msg),int size,PCStr(rr))
 	getShort(mp,xclass);
 	getLong(mp,ttl);
 	getShort(mp,rdlength);
+	if( size-(mp-msg) < rdlength ){
+		debug(DBG_FORCE,"ERROR: bad RRlength %d > %d\n",
+			rdlength,size-(mp-msg));
+	}
 
 	if( type != TY_AAAA )
 	if( type <= 0 || 16 < type || xclass <= 0 ||  4 < xclass ){
@@ -341,16 +426,13 @@ debug(DBG_ANY,"### %s[%2d]%8d: <%d>%d class=%x type=%x\n",
 	    case TY_AAAA:
 		strcpy(addr,VSA_ltoa((unsigned char*)mp,16,AF_INET6));
 		rnid = DNS_putbyaddr(addr);
-		bcopy(mp,(char*)rp,16);
+		Bcopy(mp,rp,16);
 		rp += 16;
 		DNS_putattr(nid,typemask(type),ttl,rdata,rp-rdata);
 		debug(DBG_RR,"<%d>%s\n",rnid,addr);
 		break;
 
 	    case TY_A:
-		/*
-		sprintf(addr,"%d.%d.%d.%d",mp[0],mp[1],mp[2],mp[3]);
-		*/
 		strcpy(addr,VSA_ltoa((unsigned char*)mp,4,AF_INET));
 		rnid = DNS_putbyaddr(addr);
 		setVStrPtrInc(rp,mp[0]);
@@ -359,10 +441,6 @@ debug(DBG_ANY,"### %s[%2d]%8d: <%d>%d class=%x type=%x\n",
 		setVStrPtrInc(rp,mp[3]);
 		DNS_putattr(nid,typemask(type),ttl,rdata,6);
 		debug(DBG_RR,"<%d>%s\n",rnid,addr);
-		/*
-		debug(DBG_RR,"<%d>%d.%d.%d.%d\n",
-			rnid,mp[0],mp[1],mp[2],mp[3]);
-		*/
 
 /*
 may be useful for inverse query, but could be bad for consistency...
@@ -410,6 +488,7 @@ debug(DBG_FORCE,"%s <%d>%d.%d.%d.%d\n",name,rnid,mp[0],mp[1],mp[2],mp[3]);
 CStr(mname,512);
 CStr(rname,512);
 int serial,refresh,retry,expire,minimum;
+		const char *xmp = mp;
 		int mnid,rnid;
 		mp =
 		scanName(msg,mp,AVStr(mname),sizeof(mname),0);
@@ -422,6 +501,10 @@ int serial,refresh,retry,expire,minimum;
 		getLong(mp,minimum);
 		debug(DBG_RR,"%s %s %d %d %d %d %d\n",mname,rname,
 			serial,refresh,retry,expire,minimum);
+		if( mname[0] == 0 || rname[0] == 0 ){
+			debug(DBG_FORCE,"### empty SOA name for %s %s\n",
+				stype,name);
+		}
 
 		mnid = DNS_putbyname(mname);
 		rnid = DNS_putbyname(rname);
@@ -432,8 +515,14 @@ int serial,refresh,retry,expire,minimum;
 		putLongVStr(rp,retry);
 		putLongVStr(rp,expire);
 		putLongVStr(rp,minimum);
+		newput =
 		DNS_putattr(nid,typemask(type),ttl,rdata,rp-rdata);
+		if( newput ){
+			debug(DBG_FORCE,"SOA got [%s][%s][%s] %d %d %d %d %d\n",
+			name,mname,rname,serial,refresh,retry,expire,minimum);
+		}
 		Xstrcpy(FVStr(LastSOAdom),name);
+		mp = xmp;
 }
 		break;
 	    default:
@@ -457,17 +546,16 @@ static int makeMysock()
 	int mysock;
 	int af;
 	const char *me;
+	VSAddr Ns;
 
 	if( Me == 0 )
 		Me = NewStruct(VSAddr);
 
-	/*
-	mysock = socket(AF_INET,SOCK_DGRAM,0);
-	*/
+	RES_getns1(0,&Ns);
+	if( VSA_af(&Ns) == AF_INET6 ){
+		af = AF_INET6;
+	}else
 	af = AF_INET;
-/*
-af = AF_INET6;
-*/
 	mysock = socket(af,SOCK_DGRAM,0);
 	if( mysock < 0 ){
 		debug(DBG_FORCE,"RESOLVY cannot get socket() [errno=%d]\n",
@@ -477,10 +565,10 @@ af = AF_INET6;
 	/*
 	len = VSA_atosa(Me,0,"0.0.0.0");
 	*/
+	if( VSA_af(&Ns) == AF_INET6 ){
+		me = "::";
+	}else
 	me = "0.0.0.0";
-/*
-me = "::";
-*/
 	len = VSA_atosa(Me,0,me);
 	rcode = bind(mysock,(SAP)Me,len);
 	if( rcode < 0 ){
@@ -698,24 +786,57 @@ static int dumpResponse(int qidbase,PVStr(resp),int rc,Header *hp,PCStr(addr))
 {	const char *mp;
 	CStr(qname,1024);
 	int nqd,qdi,rri;
+	const char *mx = &resp[rc-1];
 
 	mp = dumpHeader(resp,hp);
 	if( hp->id < qidbase )
 		debug(DBG_NS,"OBSOLETE[%d<%d]from[%s]\n",hp->id,qidbase,addr);
 
+	if( hp->id != qidbase ){
+		debug(DBG_FORCE,"%s(%d) ID=Q%d:R%d QD=%d AN=%d NS=%d AR=%d\n",
+			addr,rc,qidbase,hp->id,
+			hp->qdcount,hp->ancount,hp->nscount,hp->arcount);
+	}
+
 	qname[0] = 0;
 	if( nqd = hp->qdcount ){
-		for( qdi = 0; qdi < nqd; qdi++ ){
+		for( qdi = 0; mp && qdi < nqd; qdi++ ){
+			if( !DNS_dbgvul && mx <= mp ){
+			debug(DBG_FORCE,"### QD-Overrun %X %X %d\n",mx,mp,qdi);
+			errlog("QD-Overrun(%d/%d/%d).",qdi,hp->qdcount,mx-mp);
+			return hp->ancount + hp->nscount + hp->arcount;
+			}
 			scanName(resp,mp,AVStr(qname),sizeof(qname),0);
-			mp = dumpQuestion(qdi,resp,mp);
+			mp = dumpQuestion(qdi,resp,rc,mp);
 		}
 	}
 	for( rri = 0; mp && rri < hp->ancount; rri++ )
+	{
+		if( !DNS_dbgvul && mx <= mp ){
+			debug(DBG_FORCE,"### AN-Overrun %X %X %d\n",mx,mp,rri);
+			errlog("AN-Overrun(%d/%d/%d).",rri,hp->ancount,mx-mp);
+			return hp->ancount + hp->nscount + hp->arcount;
+		}
 		mp = dumpResourceRecord(RR_ANSWER,rri,resp,rc,mp);
+	}
 	for( rri = 0; mp && rri < hp->nscount; rri++ )
+	{
+		if( !DNS_dbgvul && mx <= mp ){
+			debug(DBG_FORCE,"### NS-Overrun %X %X %d\n",mx,mp,rri);
+			errlog("NS-Overrun(%d/%d/%d).",rri,hp->nscount,mx-mp);
+			return hp->ancount + hp->nscount + hp->arcount;
+		}
 		mp = dumpResourceRecord(RR_SERVER,rri,resp,rc,mp);
+	}
 	for( rri = 0; mp && rri < hp->arcount; rri++ )
+	{
+		if( !DNS_dbgvul && mx <= mp ){
+			debug(DBG_FORCE,"### AR-Overrun %X %X %d\n",mx,mp,rri);
+			errlog("AR-Overrun(%d/%d/%d).",rri,hp->arcount,mx-mp);
+			return hp->ancount + hp->nscount + hp->arcount;
+		}
 		mp = dumpResourceRecord(RR_ADDITIONAL,rri,resp,rc,mp);
+	}
 
 setVStrEnd(resolv_errmsg.resolv_errmsg,0);
 if( addr[0] )
@@ -733,16 +854,40 @@ static int recvResponse(int mysock,int qidbase,VSAddr *from,Header *hp)
 {	int fromlen;
 	int rc;
 	const char *mp;
-	CStr(resp,2048);
+	CStr(resp,MAXRSIZ);
 	CStr(addr,64);
 
 	fromlen = sizeof(*from);
-rc = SOCKS_recvfrom(mysock,(char*)resp,sizeof(resp),0,(SAP)from,&fromlen);
+	if( DNS_svtcp ){
+		unsigned const char *up = (unsigned const char*)resp;
+		int leng;
+		getpeername(mysock,(SAP)from,&fromlen);
+		rc = recv(mysock,(char*)resp,sizeof(resp),0);
+		Rlen = rc;
+		if( rc < 2 ){
+			debug(DBG_FORCE,"failed TCP recv()=%d\n",rc);
+			return -1;
+		}
+		if( 2048 < rc ){
+			errlog("LargeResp(%d).",rc);
+		}
+		leng = (up[0] << 8) | up[1];
+		bcopy(resp+2,(char*)resp,rc-2);
+		if( leng+2 != rc ){
+			errlog("InconsistentRespLen(rcc=%d/len=%d).",rc,leng);
+		}
+	}else{
+rc = SOCKS_recvfrom(mysock,AVStr(resp),sizeof(resp),0,(SAP)from,&fromlen);
+		dump("RECV",0,resp,rc);
+	}
 	if( rc <= 0 )
 	{
 		debug(DBG_FORCE,"recvfrom(%d)=%d [errno=%d]\n",mysock,rc,errno);
 		return -1;
 	}
+	if( Nsearch ){
+		Nrecv++;
+	}
 	if( fromlen <= 0 ){
 		debug(DBG_FORCE,"recvResponse(%d): rc=%d fromlen=%d\n",
 			mysock,rc,fromlen);
@@ -805,7 +950,20 @@ int initDNSconn(){
 
 int RES_BACKGROUND = 1;
 
-int getRRbynameaddr(int timeout,PCStr(name),int qtype,int rrc,char *rrv[])
+int (*RES_hltovsa)(PCStr(hlist),int ac,VSAddr av[]);
+static int scanns(PCStr(ns),int mns,VSAddr servers[]){
+	int nsx;
+
+	if( *ns == '$' && RES_hltovsa ){
+		nsx = (*RES_hltovsa)(ns+1,mns,servers);
+		return nsx;
+	}
+	return 0;
+}
+
+int dialupTOX(PCStr(wh),int sock,void *addr,int leng,int timeout,PVStr(cstat));
+
+int getRRbynameaddr(int timeout,PCStr(ns0),PCStr(name),int qtype,int rrc,char *rrv[])
 {	int wc;
 	int nid;
 	int nhit;
@@ -817,7 +975,7 @@ int getRRbynameaddr(int timeout,PCStr(name),int qtype,int rrc,char *rrv[])
 	int nns,nsi,sns,nque,nrns,rnsi;
 	int nsent,nrecv;
 	int qidbase;
-	CStr(qbuf,1024);
+	CStr(qbuf,MAXRSIZ);
 	refQStr(qp,qbuf);
 	int qlen;
 	int start;
@@ -831,17 +989,26 @@ int getRRbynameaddr(int timeout,PCStr(name),int qtype,int rrc,char *rrv[])
 	int M,gotAA;
 	int gotRecovErr; /* got recoverable errors */
 	int nturn;
+	int ndialup = 0;
+	int serrno = 0;
 
+	if( *name == 0 ){
+		/* should ignore any invalid name ? */
+		debug(DBG_FORCE,"getRRbynameaddr(NULL)\n");
+		return 0;
+	}
 	nid = DNS_putbyname(name);
 	if( nhit = lookupCache(nid,qtype,0,rrc,rrv) )
+	{
+		if( lSINGLEP() ){
+			putResTrace("{ignRR%d}",nhit);
+		}else
 		goto EXIT;
+	}
 
 	if( initDNSconn() < 0 )
 		goto EXIT;
 
-	/*
-	qidbase = ++QID;
-	*/
 	QID = (QID + 1) & 0xFFFF;
 	qidbase = QID;
 	qp = makeHeader(qbuf,QID, 0,O_QUERY,0,0,1,0,0,0, 1,0,0,0);
@@ -852,7 +1019,55 @@ int getRRbynameaddr(int timeout,PCStr(name),int qtype,int rrc,char *rrv[])
 	qlen = (char*)qp - qbuf;
 
 dumpHeader(qbuf,&Head);
-dumpQuestion(0,qbuf,qbuf+12);
+dumpQuestion(0,qbuf,qlen,qbuf+12);
+dump("SEND",0,qbuf,qlen);
+
+	if( DNS_svtcp ){
+		int connectTO(int sock,SAP addr,int leng,int timeout);
+		int conx;
+		int conok;
+		int rsock;
+
+		bcopy(qbuf,qbuf+2,qlen);
+		setVStrElem(qbuf,0,qlen>>8);
+		setVStrElem(qbuf,1,qlen);
+		qlen += 2;
+
+		if( ns0 && *ns0 && (nns=scanns(ns0,elnumof(servers),servers)) ){
+			debug(DBG_FORCE,"SV[%s:%d] <- %s [%s]\n",
+				VSA_ntoa(&servers[0]),VSA_port(&servers[0]),
+				name,symTYPE(qtype));
+		}else{
+			nns = getns(nid,128,servers);
+		}
+		ns = &servers[0];
+		nhit = 0;
+		rsock = socket(VSA_af(ns),SOCK_STREAM,0);
+		conok = 0;
+		for( conx = 0; conx < 5; conx++ ){
+			conok = connectTO(rsock,(SAP)ns,VSA_size(ns),1000)==0;
+			if( conok ){
+				break;
+			}
+			msleep(200);
+			close(rsock);
+			rsock = socket(VSA_af(ns),SOCK_STREAM,0);
+		}
+		if( conok ){
+			int wcc;
+			int nans;
+			wcc = send(rsock,qbuf,qlen,0);
+			Qlen = qlen;
+			nans = recvResponse(rsock,qidbase,&rns,&Head);
+			nhit = lookupCache(nid,qtype,0,rrc,rrv);
+		}else{
+			debug(DBG_FORCE,"cannot connect to server (%d)\n",
+				rsock);
+			if( Nsearch ) sleep(10);
+		}
+		close(rsock);
+		return nhit;
+	}
 
 	gotAA = 0;
 	gotRecovErr = 0;
@@ -889,6 +1104,11 @@ dumpQuestion(0,qbuf,qbuf+12);
 			break;
 		}
 
+		if( ns0 && *ns0 && (nns=scanns(ns0,elnumof(servers),servers)) ){
+			debug(DBG_FORCE,"SV[%s:%d] <- %s [%s]\n",
+				VSA_ntoa(&servers[0]),VSA_port(&servers[0]),
+				name,symTYPE(qtype));
+		}else
 		nns = getns(nid,128,servers);
 		if( nns <= 0){
 			debug(DBG_NS,"no server\n");
@@ -896,19 +1116,42 @@ dumpQuestion(0,qbuf,qbuf+12);
 		}
 
 		if( 0 < nturn ){ /* to escape possible tight loop ... */
-			debug(DBG_FORCE,"*%d q=%d,a=%d, s=%d,r=%d (%ds)\n",
+			debug(DBG_FORCE,"[%s]*%d q=%d,a=%d, s=%d,r=%d (%ds)\n",
+				name,
 				nturn,nque,nrns,nsent,nrecv,time(0)-start);
 			msleep(50);
 		}
 
 		fds[0] = mysock0;
 		sns = 0;
+		salen = 0;
 		for( nsi = 0; nsi<nns && sns<MAX_NS_PARA; nsi++ ){
 			xns = ns = &servers[nsi];
 			for( rnsi = 0; rnsi < nrns; rnsi++ ){
 				if( VSA_comp(xns,&rservers[rnsi]) == 0 )
 					goto NEXT;
 			}
+			if( isWindowsCE() ){
+				porting_dbg("DNS q=%d r=%d n=%X/%d l=%d e=%d",
+					nque,nrns,ns,nns,salen,serrno);
+				if( salen == 0 && ns != 0 ){
+					salen = VSA_size(ns);
+				} 
+			}
+			if( 0 < nque )
+			if( nrns == 0 )
+			if( 0 < salen )
+			if( isWindowsCE() ) /* with dialup network */
+			/*
+			if( serrno == EHOSTUNREACH || serrno == ENETUNREACH )
+			*/
+			{
+				IStr(cstat,256);
+				errno = serrno;
+				if( -1 <= dialupTOX("DNS",mysock1,ns,salen,30,
+				  AVStr(cstat)) )
+				  putResTrace("(DNSdialup%d,%d)",++ndialup,nsi);
+			}
 			for( pnsi = 0; pnsi < npns; pnsi++ ){
 			    if( VSA_comp(xns,&qservers[pnsi]) == 0 ){
 				ntry = qserverx[pnsi];
@@ -919,6 +1162,7 @@ if( ntry == MAX_RETRY ){
 ns = &qservers[pnsi];
 salen = VSA_size(ns);
 SOCKS_sendto(mysock0,qbuf,qlen,0,(SAP)ns,salen);
+serrno = errno;
 debug(DBG_NS|DBG_QANDR,"sent BACKGROUND, ID=%d, sent=%d, recv=%d, time=%d, %s\n",
 QID,nsent,nrecv,time(0)-start,VSA_ntoa(ns));
 }
@@ -939,6 +1183,7 @@ QID,nsent,nrecv,time(0)-start,VSA_ntoa(ns));
 			qns = *ns;
 salen = VSA_size(ns);
 wc = SOCKS_sendto(mysock1,qbuf,qlen,0,(SAP)ns,salen);
+serrno = errno;
 			sns++;
 			nsent++;
 
@@ -1059,7 +1304,7 @@ static int mxsort(char **rr1,char **rr2)
 	rp2 = *rr2+6; getShort(rp2,pri2);
 	return  pri1 - pri2;
 }
-int gethostbynameaddr_dns(PCStr(name),int qtype,int rrc,char *rrv[],PVStr(rrb),PVStr(cname))
+int gethostbynameaddr_dns(PCStr(ns),PCStr(name),int qtype,int rrc,char *rrv[],PVStr(rrb),PVStr(cname))
 {	const char *rp;
 	int rri,rro;
 	int ival;
@@ -1072,6 +1317,7 @@ int gethostbynameaddr_dns(PCStr(name),int qtype,int rrc,char *rrv[],PVStr(rrb),P
 	int mx,nmx,mxi,rmx;
 	char *mxv[64]; /**/
 
+	putResTrace("DNS{%s}",name);
 	if( strncasecmp(name,"_srv.",5) == 0 ){
 		name += 5;
 		qtype = TY_SRV;
@@ -1080,7 +1326,7 @@ int gethostbynameaddr_dns(PCStr(name),int qtype,int rrc,char *rrv[],PVStr(rrb),P
 	mx = qtype == TY_A && strncmp(name,"-MX.",4) == 0;
 	if( mx ){
 		name += 4;
-		nmx = gethostbynameaddr_dns(name,TY_MX,rrc,rrv,AVStr(rrb),AVStr(cname));
+		nmx = gethostbynameaddr_dns(ns,name,TY_MX,rrc,rrv,AVStr(rrb),AVStr(cname));
 		/* multiple MXs should be sorted by their preference...
 		 * - returned in -PREFERENCE-VALUE.mx-host.domain
 		 * - sort 
@@ -1091,10 +1337,10 @@ int gethostbynameaddr_dns(PCStr(name),int qtype,int rrc,char *rrv[],PVStr(rrb),P
 			CStr(cn,512);
 			strcpy(cn,cname);
 			setVStrEnd(cname,0);
-			nmx = gethostbynameaddr_dns(cn,TY_MX,rrc,rrv,AVStr(rrb),AVStr(cname));
+			nmx = gethostbynameaddr_dns(ns,cn,TY_MX,rrc,rrv,AVStr(rrb),AVStr(cname));
 		}
 		if( nmx <= 0 )
-			return gethostbynameaddr_dns(name,qtype,rrc,rrv,AVStr(rrb),AVStr(cname));
+			return gethostbynameaddr_dns(ns,name,qtype,rrc,rrv,AVStr(rrb),AVStr(cname));
 
 		if( elnumof(mxv) <= nmx )
 			nmx = elnumof(mxv);
@@ -1106,7 +1352,7 @@ int gethostbynameaddr_dns(PCStr(name),int qtype,int rrc,char *rrv[],PVStr(rrb),P
 		for( mxi = 0; mxi < nmx; mxi++ ){
 			if( strncasecmp(mxv[mxi],"-MX",3) == 0 )
 				continue;
-			rmx += gethostbynameaddr_dns(mxv[mxi],TY_A,
+			rmx += gethostbynameaddr_dns(ns,mxv[mxi],TY_A,
 				rrc-rmx,rrv+rmx,AVStr(rrp),VStrNULL);
 			rrp = rrv[rmx+1];
 		}
@@ -1114,14 +1360,11 @@ int gethostbynameaddr_dns(PCStr(name),int qtype,int rrc,char *rrv[],PVStr(rrb),P
 	}
 
 	start = time(0);
-	/*
-	if( qtype == TY_A )
-	*/
 	if( qtype == TY_A || qtype == TY_AAAA )
 		timeout = RSLV_TIMEOUT;
 	else	timeout = RSLV_INV_TIMEOUT;
 
-	rrc = getRRbynameaddr(timeout,name,qtype,rrc,rrv);
+	rrc = getRRbynameaddr(timeout,ns,name,qtype,rrc,rrv);
 	rrp = (char*)rrb;
 
 	if( qtype == TY_MX && 0 < rrc ){
@@ -1185,7 +1428,7 @@ int gethostbynameaddr_dns(PCStr(name),int qtype,int rrc,char *rrv[],PVStr(rrb),P
 	{
 		debug(DBG_ANY,"search A/AAAA of '%s' for '%s'\n",CNAME,name);
 		rro =
-		gethostbynameaddr_dns(CNAME,qtype,rrc,rrv,AVStr(rrb),VStrNULL);
+		gethostbynameaddr_dns(ns,CNAME,qtype,rrc,rrv,AVStr(rrb),VStrNULL);
 	}
 	return rro;
 }
@@ -1206,23 +1449,50 @@ static struct hostent *gethostbyN(PCStr(name))
 {
 	return gethostbyNX(name,0,0);
 }
+extern int RES_client_dependent;
+static char NXDOM[128];
+static int nNXDOM;
+static int NXDOM_RCU; /* cached RES_CACHED_UNKNOWN for NXDOM */
+extern int RES_CACHED_UNKNOWN;
+
 static struct hostent *gethostbyNX(PCStr(name),PCStr(af),PCStr(domain))
 {	struct hostent *ht;
 	CStr(xname,512);
 	const char *dp;
 	const char *saf;
-	const char *sdomain;
+	int asis;
 
+	if( *name == 0 ){
+		debug(DBG_FORCE,"Skip gethostbyNX(NULL)\n");
+		return NULL;
+	}
+
+	if( streq(NXDOM,name) ){
+		if( nNXDOM++ == 0 )
+		debug(DBG_FORCE,"Skip retrying non-existent name [%s]\n",name);
+		RES_CACHED_UNKNOWN = NXDOM_RCU;
+		return NULL;
+	}
 	if( af ){
 		saf = RES_AF;
 		RES_AF = (char*)af;
 	}
-	if( domain ){
-		sdomain = DNS_DOMAIN;
-		DNS_DOMAIN = domain;
-	}
 
 	ht = NULL;
+	asis = RES_ASIS;
+	RES_ASIS = 1;
+
+	/*
+	 * 1) xxx.Dom in file,NIS
+	 * 2) xxx     in file,NIS,DNS if with .Dom in the query
+	 * 3) xxx.Dom in DNS
+	 * 4) xxx.Dom in file,NIS,DNS if without .Dom in the query
+	 */
+	if( !RES_proxy() ){ /* without DNS */
+		ht = _GETHOSTBYNAME(name);
+	}
+
+	if( ht == NULL )
 	if( DNS_DOMAIN[0] )
 	if( dp = strcasestr(name,DNS_DOMAIN) )
 	if( dp[strlen(DNS_DOMAIN)] == 0 ){
@@ -1234,9 +1504,12 @@ static struct hostent *gethostbyNX(PCStr(name),PCStr(af),PCStr(domain))
 		ht = _GETHOSTBYNAME(xname);
 	}
 	if( ht == NULL )
+	if( RES_proxy() ){ /* with DNS */
 		ht = _GETHOSTBYNAME(name);
+	}
 
 	if( DNS_DOMAIN[0] )
+	if( strtailstr(name,DNS_DOMAIN) == 0 )
 	if( ht == NULL ){
 		sprintf(xname,"%s.%s",name,DNS_DOMAIN);
 		ht = _GETHOSTBYNAME(xname);
@@ -1245,9 +1518,22 @@ static struct hostent *gethostbyNX(PCStr(name),PCStr(af),PCStr(domain))
 	if( af ){
 		RES_AF = (char*)saf;
 	}
-	if( domain ){
-		DNS_DOMAIN = (char*)sdomain;
+
+	if( RES_client_dependent ){
+		/* don't cache client dependent result */
+	}else
+	if( ht == NULL ){
+		if( strlen(name) < sizeof(NXDOM) ){
+		Xstrcpy(FVStr(NXDOM),name);
+		NXDOM_RCU = RES_CACHED_UNKNOWN;
+		}
+	}else{
+		Xstrcpy(FVStr(NXDOM),"");
+		NXDOM_RCU = 0;
 	}
+	nNXDOM = 0;
+	RES_ASIS = asis;
+
 	return ht;
 }
 static
@@ -1271,10 +1557,6 @@ static int putRR_A(PVStr(rrb),char **rrpp,PCStr(name))
 	const char *baddr;
 
 	rrp = *rrpp;
-
-/*
-	ht = gethostbyN(name);
-*/
 	ht = gethostbyNX(name,"4","");
 	if( ht == NULL )
 		return 0;
@@ -1300,14 +1582,11 @@ static int putRR_AAAA(PVStr(rrb),char **rrpp,PCStr(name))
 	int anc,rttl,rrdlength;
 	int ai;
 	const char *baddr;
-	CStr(xname,256);
+	CStr(xname,512);
 
 	rrp = *rrpp;
 
 	sprintf(xname,"-AAAA.%s",name);
-/*
-	ht = gethostbyN(xname);
-*/
 	ht = gethostbyNX(xname,"6","");
 	if( ht == NULL )
 		return 0;
@@ -1385,19 +1664,20 @@ static int putRR_SOA(PVStr(rrb),char **rrpp,PCStr(name))
 
 	rrp = *rrpp;
 
+debug(DBG_FORCE,"---- RR_SOA[%s][%s]\n",name,DNS_DOMAIN);
 	if( strtailstr(name,DNS_DOMAIN) == NULL ){
 		if( !RES_proxy() ){
 			return 0;
 		}
-		rrc = getRRbynameaddr(RSLV_TIMEOUT,name,TY_SOA,4,rrv);
+		rrc = getRRbynameaddr(RSLV_TIMEOUT,"",name,TY_SOA,4,rrv);
 		anc = relay_cachedRRs(AVStr(rrb),rrpp,name,TY_SOA);
 		if( 0 < anc )
+		{
+debug(DBG_FORCE,"---- RR_SOA[%s][%s] = %d\n",name,DNS_DOMAIN,anc);
 			return anc;
+		}
 	}
 
-/*
-	ht = gethostbyN(name);
-*/
 	ht = gethostbyNX(name,"4","");
 	if( ht == NULL )
 		return 0;
@@ -1515,8 +1795,16 @@ static int putRR_SRV(PVStr(rrb),char **rrpp,PCStr(name))
 	int rrc;
 	char *rrv[4];
 
+	if( strtailstr(name,DNS_DOMAIN) != NULL ){
+		anc = 0;
+		return anc;
+	}
+
+	if( !RES_proxy() ){
+		return 0;
+	}
 	rrp = *rrpp;
-	rrc = getRRbynameaddr(RSLV_TIMEOUT,name,TY_SRV,elnumof(rrv),rrv);
+	rrc = getRRbynameaddr(RSLV_TIMEOUT,"",name,TY_SRV,elnumof(rrv),rrv);
 	anc = relay_cachedRRs(AVStr(rrb),rrpp,name,TY_SRV);
 	return anc;
 }
@@ -1590,13 +1878,47 @@ int dns_search(PVStr(reply),PCStr(query),int qcc,PCStr(froma),int fromp)
 	int unknown;
 	int isauth;
 	refQStr(xrrp,RRbuf);
+	int unsupported = 0;
+	const char *rp0;
+
+	if( Nsearch == 0 ){
+		Start = time(0);
+	}
+	Nsearch++;
+	ErrStat[0] = 0;
+	dump("Q",DNS_cltcp,query,qcc);
+	if( DNS_cltcp ){
+		unsigned char *up = (unsigned char*)query;
+		int len;
+		len = (up[0] << 8) | up[1];
+		if( len != qcc-2 ){
+			debug(DBG_FORCE,"BAD LENGTH %d/%d\n",len,qcc);
+		}
+		bcopy(query+2,(char*)query,qcc-2);
+		qcc -= 2;
+	}
+
+	/*
+	 * should not do retry when acting as a recursive server
+	 * because the retrial will be scheduled by the client
+	 */
+	RSLV_TIMEOUT = 10;
 
 	qp = query;
 	qp = scanHeader(qp,&Head);
 	qp = scanQuery(query,qp,AVStr(name),sizeof(name),&qtype,&qclass);
 
+	/*
 	debug(DBG_FORCE,"QUERY %s %s %s ID=%d\n",
 		name,symCLASS(qclass),symTYPE(qtype),Head.id);
+	*/
+	/*
+	debug(DBG_FORCE,"QUERY (%d) %s %s %s ID=%d\n",
+		H_OPCODE(Head.M),name,symCLASS(qclass),symTYPE(qtype),Head.id);
+	*/
+	debug(DBG_FORCE,"QUERY (%d) %s %s %s ID=%d <<%s:%d\n",
+		H_OPCODE(Head.M),name,symCLASS(qclass),symTYPE(qtype),Head.id,
+		froma,fromp);
 	setVStrEnd(resolv_errmsg.resolv_errmsg,0);
 
 	if( H_QR(Head.M) != 0 ){
@@ -1608,6 +1930,15 @@ int dns_search(PVStr(reply),PCStr(query),int qcc,PCStr(froma),int fromp)
 
 	anc = 0;
 	nauth = 0;
+	RES_CACHED_UNKNOWN = 0;
+
+	if( 2 <= H_OPCODE(Head.M) ){
+		errlog("NotSupported(op=%d).",H_OPCODE(Head.M));
+		debug(DBG_FORCE,"Not supported, opcode=%d\n",H_OPCODE(Head.M));
+		unsupported = 1;
+		goto PUTRESP;
+	}
+
 	switch( qtype ){
 	    case TY_SRV:anc += putRR_SRV(AVStr(RRbuf),(char**)&rrp,name); break;
 /*
@@ -1632,8 +1963,10 @@ int dns_search(PVStr(reply),PCStr(query),int qcc,PCStr(froma),int fromp)
 			anc += putRR_PTR(AVStr(RRbuf),(char**)&rrp,name);
 			break;
 	}
+	if( RES_CACHED_UNKNOWN == 0 )
 	if( anc == 0 || qtype == TY_SOA || qtype == TY_SRV ){
 		nauth = putRR_SOA(AVStr(RRbuf),(char**)&rrp,name);
+debug(DBG_FORCE,"---- SOA nauth=%d\n",nauth);
 	}
 
 	xrrp = rrp;
@@ -1644,6 +1977,7 @@ int dns_search(PVStr(reply),PCStr(query),int qcc,PCStr(froma),int fromp)
 	}
 	isauth = strtailstr(name,DNS_DOMAIN) != NULL;
 
+PUTRESP:
 	QM = Head.M;
 	RM = 0;
 	SET_QR(RM,1);
@@ -1662,11 +1996,6 @@ int dns_search(PVStr(reply),PCStr(query),int qcc,PCStr(froma),int fromp)
 	SET_RA(RM,0);
 	if( anc == 0 )
 	{
-		/*
-		if( gethostbyN(name) == NULL ){
-		SET_RCODE(RM,3);
-		}
-		*/
 		if( unknown ){
 		    if( isauth )
 			SET_RCODE(RM,3); /* UNKNOWN */
@@ -1680,6 +2009,9 @@ int dns_search(PVStr(reply),PCStr(query),int qcc,PCStr(froma),int fromp)
 			SET_RCODE(RM,5); /* REFUSED */
 		    }
 		}
+		if( unsupported ){
+			SET_RCODE(RM,4); /* UNSUPPORTED */
+		}
 	}
 	qdc = 1;
 
@@ -1688,16 +2020,19 @@ int dns_search(PVStr(reply),PCStr(query),int qcc,PCStr(froma),int fromp)
 	putShort((char*)rp,RM);
 	putShort((char*)rp,qdc);
 	putShort((char*)rp,anc);
-/*
-	putShort((char*)rp,0);
-*/
 	putShort((char*)rp,nauth);
 	putShort((char*)rp,0);
+	rp0 = rp;
 	rp = makeQuestion(QVStr(rp,reply),name,qtype,qclass);
+	if( rp == NULL ){
+		rp = (char*)rp0;
+		rp = makeQuestion(QVStr(rp,reply),"-",qtype,qclass);
+		debug(DBG_FORCE,"makeQuesion(%s) = NULL -> %X\n",name,rp);
+		if( rp == NULL ){
+			return -1;
+		}
+	}
 
-/*
-	if( anc ){
-*/
 	if( anc || nauth ){
 		Bcopy(RRbuf,rp,rrp-RRbuf);
 		rp += (rrp-RRbuf);
@@ -1710,6 +2045,25 @@ int dns_search(PVStr(reply),PCStr(query),int qcc,PCStr(froma),int fromp)
 		debug(DBG_FORCE,"ANSWER %s [ID=%d]\n",
 			resolv_errmsg.resolv_errmsg,QID);
 	}
+
+	if( DNS_cltcp ){
+		Xbcopy(reply,DVStr(reply,2),rleng);
+		setVStrElem(reply,0,rleng>>8);
+		setVStrElem(reply,1,rleng);
+		rleng += 2;
+	}
+	dump("R",DNS_cltcp,reply,rleng);
+	if( (DNS_debug & DBG_TRACE) != 0 ){
+		fprintf(stderr,"-- %4d %4dQ %4dR %20s %dC %5d [%s]\n",
+		time(0)-Start,Nsearch-1,Nrecv-1,name,H_RCODE(RM),Rlen,ErrStat);
+	}
+	if( ErrStat[0] ){
+		debug(DBG_FORCE,"ERROR [qid=%d][RES#%d] %s Code=%d %d [%s]\n",
+			Nsearch-1,Nrecv-1,name,H_RCODE(RM),Rlen,ErrStat);
+	}
+	if( DNS_svtcp ){
+		msleep(100); /* wait the test-case to finish */
+	}
 	return rleng;
 }
 
