@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1994 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1994-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2005 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -59,7 +60,7 @@ static int GetInt4(FILE *fc,FILE *ts)
 	return iv;
 }
 
-static int CS(Connection *Conn,int fcbsize,int peer,int omask)
+static int XCS(Connection *Conn,int fcbsize,int peer,int omask)
 {	CStr(buf,0x4000);
 	int gotsig;
 	int rcc;
@@ -152,7 +153,11 @@ if( op == 74 || op == 76 || op == 77 ){
 
 	if( op == 77 ){
 		for( i = 0; i < n; i++ ){
+			/*
 			och = getc(fc)<<8 | getc(fc);
+			*/
+			och = getc(fc) << 8;
+			och = och | getc(fc);
 DBGMSG("OP ImageTEXT16[%3d / %3d/%3d]: outch=%4x x=%4d, y=%4d\n",i,n,l,och,x,y);
 			putc(och >> 8,ts);
 			putc(och&0xFF,ts);
@@ -225,7 +230,7 @@ DBGMSG("OP %4d / %4d / %4d\n",op,n,l);
 
 	return total;
 }
-static int SC(Connection *Conn,int fsbsize,int peer,int omask)
+static int XSC(Connection *Conn,int fsbsize,int peer,int omask)
 {	CStr(buf,0x4000);
 	int gotsig;
 	int rcc,wcc;
@@ -239,7 +244,7 @@ static int SC(Connection *Conn,int fsbsize,int peer,int omask)
 
 	if( setjmp(tel_env) == 0 ){
 		sigsetmask(omask);
-		while( 0 < (rcc = read(FromS,buf,fsbsize)) ){
+		while( 0 < (rcc = read(FromS,buf,QVSSize(buf,fsbsize))) ){
 			count += 1;
 			total += rcc;
 			if( (wcc = write(ToC,buf,rcc)) <= 0 )
@@ -268,10 +273,10 @@ static int bidirectional_relay(Connection *Conn,int fcbsize,int fsbsize)
 	signal(SIGTERM,sigTERM);
 
 	if( (cpid = Fork("bidirectional_relay")) == 0 ){
-		total = SC(Conn,fsbsize,ppid,omask);
+		total = XSC(Conn,fsbsize,ppid,omask);
 		Finish(0);
 	}else{
-		total = CS(Conn,fcbsize,cpid,omask);
+		total = XCS(Conn,fcbsize,cpid,omask);
 Kill(cpid,SIGTERM);
 		wait(0);
 		sigsetmask(omask);
