@@ -16,8 +16,11 @@
 #define RELAY_JAVA	0x0300 /* relay <APPLET> <OBJECT> and <EMBED> */
 #define RELAY_ORIGIN	0x0400 /* origin server */
 #define RELAY_VHOST	0x0800 /* relay to host in Host: field */
+#define RELAY_ORIGDST	0x1000 /* relay to host in SO_ORIGINAL_DST */
 
+#define MaxAuthServLen	2048
 
+int   CTX_withAuth(DGCTX);
 int   CTX_auth(DGCTX,PCStr(user),PCStr(pass));
 int   CTX_auth_cache(DGCTX,int store,int expire,PCStr(proto),PCStr(user),PCStr(pass),PCStr(host),int port);
 int   CTX_with_auth_admin(DGCTX);
@@ -36,9 +39,11 @@ const char *getClientUser(DGCTX);
 const char *getClientUserX(DGCTX);
 const char *getClientUserC(DGCTX);
 
+const char *getAdminAuthorizer(DGCTX,PVStr(authsv),int asiz,int any);
 int   DGAuth_port(int create,PVStr(host),int *portp);
-int   getDigestPass(PCStr(host),PCStr(user),PVStr(spass));
-int   genPass(PCStr(host),PCStr(user),PVStr(pass));
+int   getDigestPassX(DGCTX,PCStr(host),PCStr(user),PVStr(spass));
+#define getDigestPass(host,user,spass) getDigestPassX(MainConn(),host,user,spass)
+int   genPass(DGCTX,PCStr(host),PCStr(user),PVStr(pass));
 void  dumpCKey(int force);
 void  NonceKey(PVStr(key));
 int   get_MYAUTH(DGCTX,PVStr(myauth),PCStr(proto),PCStr(dhost),int dport);
@@ -51,6 +56,7 @@ typedef struct MD5_CTX MD5;
 MD5  *newMD5();
 void  addMD5(MD5*ctx,PCStr(str), int len);
 void  endMD5(MD5*ctx,char digest[]);
+int   fMD5(FILE *fp,char digest[]);
 void  toMD5dots(PCStr(pfx),PCStr(str),PVStr(dots),int len);
 void  MD5toa(PCStr(digest),char md5a[]);
 int   msgMD5(FILE *fs,FILE *tc,char md5a[]);
@@ -58,4 +64,10 @@ int   putDigestPass(FILE *fp,PCStr(fmt),PCStr(host),PCStr(user));
 int   authAPOP(DGCTX,PCStr(domain),PCStr(user),PCStr(seed),PVStr(mpass));
 int   withAuthDigest(DGCTX,PVStr(authserv));
 
+int AuthTimeout(DGCTX);
+int AuthThru(DGCTX,PCStr(user));
+
+int SignRSA(PCStr(privkey),PCStr(data),PCStr(pass),PCStr(md5),int mlen,PVStr(sig),unsigned int *slen);
+int VerifyRSA(PCStr(pubkey),PCStr(data),PCStr(md5),int mlen,PCStr(sig),unsigned int slen);
+
 #endif /* _AUTH_H */
