@@ -1,11 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1994-1999 Yutaka Sato
-Copyright (c) 1994-1999 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1994-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, and distribute this material for any purpose
-and without fee is hereby granted, provided that the above copyright
-notice and this permission notice appear in all copies.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -48,6 +51,16 @@ static int renameR(PCStr(old),PCStr(xnew),int mode);
 int mkdirRX(PCStr(dir))
 {
 	NumSymLink = 0;
+
+	if( isWindows() && strpbrk(dir,"?*<>|") ){
+		/* this will not occur, rewritten by others already */
+		IStr(dirx,4*1024);
+		int rcode;
+		strcpy(dirx,dir);
+		path_escchar(AVStr(dirx));
+		rcode = mkdirR(dirx,MKDIR_MODE);
+		return rcode;
+	}
 	return mkdirR(dir,MKDIR_MODE);
 }
 int linkRX(PCStr(to),PCStr(from))
@@ -152,9 +165,19 @@ static int mkdirR(PCStr(dir),int mode)
 		}
 	}
 
+	if( dp == dir && *dp == '/' ){
+		dp++;
+		/* don't try mkdir1("",mode) */
+	}
 	while( dp = strchr(dp,'/') ){
+		int rcode;
 		*(char*)dp = 0;
+		rcode =
 		mkdir1(dir,mode);
+		if( rcode != 0 && !fileIsdir(dir) ){
+			*(char*)dp = '/';
+			return -1;
+		}
 		*(char*)dp++ = '/';
 	}
 	return mkdir1(dir,mode);
@@ -163,6 +186,9 @@ static int linkR(PCStr(to),PCStr(from),int mode)
 {	CStr(dir,1024);
 	const char *dp;
 
+	/*
+	 * should do path_escchar() ...
+	 */
 	if( link(to,from) == 0 )
 		return 0;
 
@@ -183,7 +209,10 @@ static int renameR(PCStr(old),PCStr(xnew),int mode)
 	path_escchar(AVStr(path));
 	xnew = path;
 
+	/*
 	if( File_size(old) < 0 )
+	*/
+	if( !File_is(old) )
 		return -1;
 
 	if( rename(old,xnew) == 0 )
@@ -239,6 +268,10 @@ FILE *dirfopen(PCStr(what),PVStr(file),PCStr(mode))
 	if( file[strlen(file)-1] == '/' )
 		strcat(file,DIR_DESCRIBER);
 
+	if( streq(mode,"r-") ){
+		/* don't create the upper directories in read mode */
+		mode = "r";
+	}else
 	if( dp = strrchr(file,'/') ){
 		*(char*)dp = 0;
 		mkdirRX(file);
@@ -305,10 +338,18 @@ FILE *expfopen(PCStr(what),int expire,PVStr(file),PCStr(mode),int *datep)
 	}
 
 	stat = file_stat(fileno(fp),&size,&mtime,&atime);
+	/*
 	if( stat != 0 || size <= 0 ){
+	*/
+	if( stat != 0
+	 || size == 0 /* 9.2.2 can be < 0 (can be zero for FTP-cache ...) */
+	){
 		expfopen_log(what,Start,file,atime,"cache-EMPTY: %d",size,0);
 		fclose(fp);
 		fp = NULL;
+	}else
+	if( expire == CACHE_DONTEXP ){
+		return fp;
 	}else{
 		if( datep )
 			*datep = mtime;
@@ -328,29 +369,124 @@ FILE *expfopen(PCStr(what),int expire,PVStr(file),PCStr(mode),int *datep)
 	return fp;
 }
 
+/*
 const char *cachefmt();
+*/
+const char *cachefmt(PCStr(base));
 void evalPATHexp(PVStr(xpath),PCStr(pathfmt),PCStr(proto),PCStr(server),int port,PCStr(path));
+
+int CTX_cache_pathX(DGC*ctx,PCStr(base),PCStr(proto),PCStr(server),int iport,PCStr(path1),PVStr(cachepath));
 int CTX_cache_path(DGC*ctx,PCStr(proto),PCStr(server),int iport,PCStr(path1),PVStr(cachepath))
+{
+	setVStrEnd(cachepath,0);
+	if( cachedir() == 0 )
+		return 0;
+	return CTX_cache_pathX(ctx,cachedir(),proto,server,iport,path1,BVStr(cachepath));
+}
+
+/*
+ * Unify Escaped URLs into an Upper Case Format
+ */
+#define UUNIF_UCASE	1
+int CACHE_URLUNIFY = UUNIF_UCASE;
+
+#define isDec(ch)	('0'<=ch && ch<='9')
+#define isLHex(ch)	('a'<=ch && ch<='f')
+#define isUHex(ch)	('A'<=ch && ch<='F')
+#define isHex(ch)	(isLHex(ch) || isUHex(ch))
+#define isDecHex(ch)	(isDec(ch) || isHex(ch))
+#define toBin(c)	(isDec(c) ? (c-'0') : \
+			(isLHex(c)? (c-'a'+10) : (isUHex(c) ? c-'A'+10 : 0)))
+
+int url_escunify(PVStr(eurl),PCStr(surl)){
+	refQStr(ep,eurl);
+	const char *sp;
+	int sc,nsc1,nsc2;
+	int ne = 0;
+
+	for( sp = surl; sc = *sp; sp++ ){
+		if( sc == '%' )
+		if( (nsc1 = sp[1]) && isDecHex(nsc1) )
+		if( (nsc2 = sp[2]) && isDecHex(nsc2) ){
+			sprintf(ep,"%%%c%c",toupper(nsc1),toupper(nsc2));
+			ep += 3;
+			sp += 2;
+			ne++;
+			continue;
+		}
+
+		if( isalnum(sc) ){
+			setVStrPtrInc(ep,sc);
+		}else
+		switch( sc ){
+			case '!': case '#': case '$': case '&':
+			case '(': case ')': case '*':
+			case '+': case ',': case '-':
+			case '.': case '/':
+			case ':': case ';': case '=': case '?':
+			case '@':
+			case '[': case ']': case '^': case '_':
+			case '`':
+			case '{': case '|': case '}': case '~':
+				setVStrPtrInc(ep,sc);
+				break;
+			default:
+				/*
+				sprintf(ep,"%%%02X",sc);
+				*/
+				sprintf(ep,"%%%02X",0xFF&sc);
+				ep += strlen(ep);
+				ne++;
+				break;
+		}
+	}
+	setVStrPtrInc(ep,0);
+	if( ne && strcmp(surl,eurl) != 0 ){
+		syslog_ERROR("url_escunify: %s -> %s\n",surl,eurl);
+	}
+	return ne;
+}
+
+int getNvserv(DGC*Conn,PVStr(nvserv));
+int CTX_cache_pathX(DGC*ctx,PCStr(base),PCStr(proto),PCStr(server),int iport,PCStr(path1),PVStr(cachepath))
 {	const char *rp;
 	CStr(pathb,4096);
+	CStr(pathe,4096);
 	CStr(xpath,4096);
 	int plen;
 	char sc;
 	CStr(lproto,256);
-	CStr(aserver,256);
-	CStr(lserver,256);
+	CStr(aserver,MaxHostNameLen);
+	CStr(lserver,MaxHostNameLen);
 	int si;
+	IStr(nvserv,MaxHostNameLen);
+	refQStr(nvport,nvserv);
 
 	setVStrEnd(cachepath,0);
-	if( cachedir() == 0 )
-		return 0;
 
 	strtolower(proto,lproto);
 
+	if( getNvserv(ctx,AVStr(nvserv)) ){
+		int port = 0;
+		if( nvport = strchr(nvserv,':') ){
+			setVStrPtrInc(nvport,0);
+			port = atoi(nvport);
+		}
+		if( port == 0 )
+			port = serviceport(proto);
+		if( !strcaseeq(server,nvserv) || iport != port ){
+			sv1log("CACHE name based vhost: %s:%d [%s:%d]\n",
+				nvserv,port,server,iport);
+			server = nvserv;
+			iport = port;
+		}
+	}
 	if( VSA_strisaddr(server) ){
 		gethostbyAddr(server,AVStr(aserver));
+		if( !streq(server,aserver) ){
 		sv1log("CACHE hostname: %s -> %s\n",server,aserver);
 		server = aserver;
+		}
 	}
 
 	strtolower(server,lserver);
@@ -364,11 +500,19 @@ int CTX_cache_path(DGC*ctx,PCStr(proto),PCStr(server),int iport,PCStr(path1),PVS
 		strncpy(pathb,path1,plen); setVStrEnd(pathb,plen);
 		path1 = pathb;
 	}
+	if( CACHE_URLUNIFY ){
+		url_escunify(AVStr(pathe),path1);
+		path1 = pathe;
+	}
+
 	xpath[0] = 0;
 	chdir_cwd(AVStr(xpath), path1, 0);
 	if( strtailchr(path1) == '/' && strtailchr(xpath) != '/' ) 
 		strcat(xpath,"/");
+	/*
 	evalPATHexp(AVStr(cachepath),cachefmt(),lproto,lserver,iport,xpath);
+	*/
+	evalPATHexp(AVStr(cachepath),cachefmt(base),lproto,lserver,iport,xpath);
 	return 1;
 }
 int cache_path(PCStr(proto),PCStr(server),int iport,PCStr(path1),PVStr(cachepath))
@@ -428,7 +572,10 @@ void cache_delete(PCStr(cpath))
 	unlink(cpath);
 }
 
+/*
 int CTX_cache_remove(DGC*ctx,PCStr(proto),PCStr(host),int port,PCStr(path))
+*/
+int CTX_cache_remove(DGC*ctx,PCStr(proto),PCStr(host),int port,PCStr(path),int dir)
 {	CStr(base,1024);
 	CStr(cpath,1024);
 	const char *dp;
@@ -440,13 +587,24 @@ int CTX_cache_remove(DGC*ctx,PCStr(proto),PCStr(host),int port,PCStr(path))
 	CTX_cache_path(ctx,proto,host,port,"",AVStr(base));
 	len = strlen(base);
 	if( base[len-1] == '/' )
-		base[len-1] = 0;
+		setVStrEnd(base,len-1);
 
 	if( strstr(path,base) == NULL )
+	{
+		path_escchar(AVStr(base));
+		if( strstr(path,base) == NULL )
 		return -1;
+	}
 
 	if( unlink(path) != 0 )
+	{
+		sv1log("## cache_remove ERROR: failed unlink(%s)\n",path);
 		return -1;
+	}
+	Verbose("## cache_remove DONE: unlink(%s)\n",path);
+
+	if( dir == 0 )
+		return 0;
 
 	strcpy(cpath,path);
 	while( dp = strrchr(cpath,'/') ){
@@ -582,7 +740,7 @@ static int evalServer(PVStr(xpath),PCStr(fmt),PCStr(proto),PCStr(server),int por
 {	const char *fp;
 	const char *p1;
 	CStr(tmp,256);
-	CStr(fqdn,256);
+	CStr(fqdn,MaxHostNameLen);
 	int neval = 0;
 	int tomd5;
 	refQStr(xp,xpath); /**/
@@ -590,9 +748,38 @@ static int evalServer(PVStr(xpath),PCStr(fmt),PCStr(proto),PCStr(server),int por
 	setVStrEnd(xpath,0);
 	for( fp = fmt; *fp; fp++ )
 	if( *fp == '%' && fp[1] ){
+		int width;
 		fp++;
+		width = 0;
+		if( *fp == '-' ){
+			fp++;
+			while( isdigit(*fp) ){
+				width = width*10 + *fp-'0';
+				fp++;
+			}
+		}
 		if( tomd5 = (fp[0] == 'm') && (fp[1] != 0) )
 			fp++;
+		if( *fp == '(' && (p1 = strchr(fp+1,')')) ){
+			IStr(a1,256); IStr(v1,256); IStr(op,256);
+			IStr(a2,256); IStr(v2,256); IStr(f1,256); IStr(f2,256);
+			const char *fo;
+			Xsscanf(fp+1,"%[^!=<>]%[!=<>]%[^?]?%[^:)]:%[^)]",
+			    AVStr(a1),AVStr(op),AVStr(a2),AVStr(f1),AVStr(f2));
+			Verbose("--{%s %s %s? %s : %s}\n",a1,op,a2,f1,f2);
+			evalServer(AVStr(v1),a1,proto,server,port,path);
+			evalServer(AVStr(v2),a2,proto,server,port,path);
+			if( streq(op,"==") &&  streq(v1,v2)
+			 || streq(op,"!=") && !streq(v1,v2)
+			 || streq(op,">=") && strstr(v1,v2)
+			 || streq(op,"<=") && strstr(v2,v1)
+			){
+				fo = f1;
+			}else	fo = f2;
+			evalServer(AVStr(xp),fo,proto,server,port,path);
+			Verbose("--{%s %s %s? => %s => %s}\n",v1,op,v2,fo,xp);
+			fp = p1;
+		}else
 		switch( *fp ){
 		    default:  return -1;
 		    case '%': strcpy(xp,"%"); break;
@@ -624,12 +811,18 @@ static int evalServer(PVStr(xpath),PCStr(fmt),PCStr(proto),PCStr(server),int por
 		    case 'p':
 			strcpy(xp,path);
 			break;
+		    case 'u':
+			sprintf(xp,"%s://%s:%d/%s",proto,server,port,path);
+			break;
 		}
 		neval++;
 		if( tomd5 ){
 			toMD5(xp,tmp);
 			strcpy(xp,tmp);
 		}
+		if( 0 < width && width < strlen(xp) ){
+			setVStrEnd(xp,width);
+		}
 		xp += strlen(xp);
 	}else{
 		assertVStr(xpath,xp);
@@ -704,6 +897,16 @@ FILE *new_shared()
 {
 	return sharedfp = TMPFILE("new_shared");
 }
+int close_shared(){
+	int fd;
+	if( sharedfp ){
+		fd = fileno(sharedfp);
+		fclose(sharedfp);
+		sharedfp = 0;
+		return fd;
+	}
+	return -1;
+}
 int get_shared(PVStr(buf),int size,FILE *fp)
 {	int rc;
 
@@ -742,7 +945,7 @@ int get_equiv_user(PCStr(clhost),int clport,PVStr(eqhost),PVStr(equser))
 	const char *bp;
 	const char *np;
 	CStr(equserhost,1024);
-	CStr(chost1,256);
+	CStr(chost1,MaxHostNameLen);
 
 	if( get_shared(AVStr(buff),sizeof(buff),NULL) <= 0 )
 		return 0;
@@ -799,21 +1002,228 @@ whom has different EXPIRE period.
 }
 
 void cache_done(int gotok,FILE *cachefp,PCStr(cpath),PCStr(xcpath))
-{	int size;
+{	FileSize size;
 
 	if( cachefp == NULL ){
 		sv1log("CACHE: no cache opened.\n");
 		return;
 	}
+	/*
 	size = ftell(cachefp);
+	*/
+	size = Lseek(fileno(cachefp),0,2);
 	fclose(cachefp);
 
 	if( gotok ){
 		if( renameRX(xcpath,cpath) == 0 )
-			sv1log("CACHE: got = [%d] %s\n",size,cpath);
+			sv1log("CACHE: got = [%lld] %s\n",size,cpath);
 		else	sv1log("CACHE: can't link %s => %s\n",cpath,xcpath);
 	}else{
-		sv1log("CACHE: err = [%d] %s\n",size,xcpath);
+		sv1log("CACHE: err = [%lld] %s\n",size,xcpath);
 		unlink(xcpath);
 	}
 }
+
+const char *DELEGATE_ACLFILE = "$[server:%P/%L/%p]";
+FILE *ACL_fopen(PCStr(proto),PCStr(host),int port,PCStr(upath),int wr,PVStr(cpath)){
+	CStr(fmt,1024);
+	CStr(lproto,32);
+	CStr(lhost,128);
+	CStr(aclpath,1024);
+	FILE *fp;
+
+	strcpy(fmt,"${ADMDIR}");
+	Substfile(fmt);
+	Xsprintf(TVStr(fmt),"/attrs/%s",DELEGATE_ACLFILE);
+	strtolower(proto,lproto);
+	strtolower(host,lhost);
+	evalPATHexp(AVStr(aclpath),fmt,lproto,lhost,port,upath);
+	strcat(aclpath,"#attr");
+
+	if( wr ){
+		fp = dirfopen("ACL",AVStr(aclpath),"r+");
+		if( fp == NULL )
+			fp = dirfopen("ACL",AVStr(aclpath),"w+");
+	}else{
+		fp = dirfopen("ACL",AVStr(aclpath),"r-");
+	}
+
+	if( lGATEWAY() )
+	fprintf(stderr,"ACL_fopen(wr=%d) = %X [%s]\n",wr,fp,aclpath);
+
+	if( cpath ) strcpy(cpath,aclpath);
+	return fp;
+}
+
+#define A_GET	0
+#define A_ADD	1
+#define A_DEL	2
+#define A_RPL	3
+#define A_SET	3
+
+int ACL_edit(FILE *fp,int op,PCStr(name),PVStr(value)){
+	CStr(nam,1024);
+	CStr(val,1024);
+	int ndel = 0;
+	CStr(data,64*1024);
+	refQStr(dp,data);
+
+	if( fp == NULL )
+		return -1;
+
+	for(;;){
+		if( fgets(dp,sizeof(data)-(dp-data),fp) == NULL )
+			break;
+		truncVStr(nam);
+		truncVStr(val);
+		Xsscanf(dp," %[^:] : %[^\r\n]",AVStr(nam),AVStr(val));
+		switch( op ){
+		case A_GET:
+			if( strcaseeq(name,nam) ){
+				strcpy(value,val);
+				return 1;
+			}
+			break;
+		case A_ADD:
+			if( strcaseeq(name,nam) ){
+				if( strcaseeq(value,val) ){
+					return 0;
+				}
+			}
+			break;
+		case A_DEL:
+			if( strcaseeq(name,nam) ){
+				if( *value == 0 || streq(value,val) ){
+					ndel++;
+					continue;
+				}
+			}
+			dp += strlen(dp);
+			break;
+		case A_RPL:
+			if( strcaseeq(name,nam) ){
+				ndel++;
+				continue;
+			}
+			dp += strlen(dp);
+			break;
+		}
+	}
+	switch( op ){
+	case A_GET:
+		truncVStr(value);
+		return 0;
+	case A_ADD:
+		fprintf(fp,"%s:%s\n",name,value);
+		fflush(fp);
+		return 1;
+	case A_DEL:
+		if( ndel ){
+			fseek(fp,0,0);
+			fputs(data,fp);
+			Ftruncate(fp,0,1);
+		}
+		return ndel;
+	case A_RPL:
+		if( ndel ){
+			fseek(fp,0,0);
+			fputs(data,fp);
+			Ftruncate(fp,0,1);
+		}
+		fprintf(fp,"%s:%s\n",name,value);
+		fflush(fp);
+		return ndel;
+	}
+	return -1;
+}
+
+int ACL_get(PCStr(proto),PCStr(host),int port,PCStr(upath),PCStr(nam),PVStr(val)){
+	FILE *afp;
+	int code;
+
+	truncVStr(val);
+	afp = ACL_fopen(proto,host,port,upath,0,VStrNULL);
+	if( afp == NULL )
+		return 0;
+	code = ACL_edit(afp,A_GET,nam,BVStr(val));
+	fclose(afp);
+	if( code ){
+		return code;
+	}else{
+		return 0;
+	}
+}
+
+int acledit_main(int ac,const char *av[]){
+	int ai;
+	CStr(line,1024);
+	CStr(com,128);
+	CStr(arg1,1024);
+	CStr(arg2,128);
+	CStr(proto,128);
+	CStr(host,128);
+	CStr(path,128);
+	CStr(xpath,128);
+	int port = 0;
+	const char *ap;
+	FILE *fp = NULL;
+	int op;
+	int code;
+
+	truncVStr(proto);
+	truncVStr(host);
+	truncVStr(path);
+	strcpy(proto,"nntp");
+	strcpy(host,"localhost");
+	strcpy(path,"mail-lists.delegate/1");
+
+	for(;;){
+		fprintf(stderr,"[%s://%s:%d/%s] ",proto,host,port,path);
+		fflush(stderr);
+		if( fgets(line,sizeof(line),stdin) == NULL )
+			break;
+		ap = wordScan(line,com);
+		if( streq(com,"quit") ){ break; }
+		ap = wordScan(ap,arg1);
+		lineScan(ap,arg2);
+
+		if( streq(com,"proto") ){ strcpy(proto,arg1); }else
+		if( streq(com,"host") ){  strcpy(host,arg1); }else
+		if( streq(com,"port") ){  port = atoi(arg1); }else
+		if( streq(com,"path") ){  strcpy(path,arg1); }else
+		if( streq(com,"get")
+		 || streq(com,"add")
+		 || streq(com,"del")
+		 || streq(com,"rpl")
+		){
+			if( streq(com,"get") ) op = A_GET; else
+			if( streq(com,"add") ) op = A_ADD; else
+			if( streq(com,"del") ) op = A_DEL; else
+			if( streq(com,"set") ) op = A_RPL; else
+			if( streq(com,"rpl") ) op = A_RPL; else op = A_GET;
+
+			if( port == 0 ){
+				port = serviceport(proto);
+			}
+			strcpy(xpath,path);
+			if( streq(proto,"nntp") ){
+				strsubst(AVStr(xpath),".","/");
+			}
+			fp = ACL_fopen(proto,host,port,xpath,1,VStrNULL);
+			if( fp == NULL ){
+				fprintf(stderr,"cannot open ACL\n");
+				continue;
+			}
+			code = ACL_edit(fp,op,arg1,AVStr(arg2));
+			fclose(fp);
+			fprintf(stderr,"=%d",code);
+			if( op == A_GET ){
+				fprintf(stderr,": %s",arg2);
+			}
+			fprintf(stderr,"\n");
+		}else{
+			fprintf(stderr,"Unkown: %s",line);
+		}
+	}
+	return 0;
+}
