@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1997 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1997-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -18,31 +19,64 @@ Description:
 History:
 	970125	created
 //////////////////////////////////////////////////////////////////////#*/
+#include <stdlib.h>
 #include <stdio.h>
+#include <errno.h>
+#include "ystring.h"
 #include "vsignal.h"
 #include "delegate.h"
-#include <errno.h>
+#include "fpoll.h"
+#include "proc.h"
+#include "file.h"
 int MAX_CC = 16;
 
-extern FILE *fopenCC();
-extern FILE *fcloseCC();
+void clear_DGclnt(Connection *Conn);
+FILE *fopenCC(PCStr(server),PCStr(mode),PVStr(path));
+FILE *fcloseCC();
+void del_CC(int pid,PCStr(how));
+void sayStartCC();
+void sayFinishCC();
+
+typedef struct {
+	MStr(	ce_Mypath,256);
+	int	ce_CC_master;
+	int	ce_CC_Report[2];
+	int	ce_CC_Active;
+	int	ce_CC_Procs[256];
+} CCacheEnv;
+static CCacheEnv *ccacheEnv;
+#define Mypath		ccacheEnv[0].ce_Mypath
+/**/
+#define CC_master	ccacheEnv[0].ce_CC_master
+#define CC_Report	ccacheEnv[0].ce_CC_Report
+#define CC_Active	ccacheEnv[0].ce_CC_Active
+#define CC_Procs	ccacheEnv[0].ce_CC_Procs
+void minit_ccache()
+{
+	if( ccacheEnv == 0 ){
+		ccacheEnv = NewStruct(CCacheEnv);
+		CC_Report[0] = -1;
+		CC_Report[1] = -1;
+	}
+}
+
 
 #define LOCALHOST	"127.0.0.1"
 
-static tryOpen(Conn,user,new,path,portp)
-	Connection *Conn;
-	char *user;
-	char *path;
-	int *portp;
+extern int CON_RETRY;
+static int tryOpen(Connection *Conn,PCStr(user),int xnew,PVStr(path),int *portp)
 {	FILE *ccfp;
-	char url[1024],server[1024],buff[128];
+	CStr(url,1024);
+	CStr(server,1024);
+	CStr(buff,128);
 	int cci,svport,svpid,sock;
 	int rcode;
+	int conretry;
 
 	/* ServerSiceCC should not connect to another ServerSideCC
 	 * of the same protocol with itself...
 	 */
-	if( !new && ImCC /*&& strcmp(DST_PROTO,ImCC_PROTO) == 0*/ ){
+	if( !xnew && ImCC /*&& strcmp(DST_PROTO,ImCC_PROTO) == 0*/ ){
 		sv1log("serverCC: don't connect svCC from svCC\n");
 		return -1;
 	}
@@ -56,8 +90,8 @@ static tryOpen(Conn,user,new,path,portp)
 
 	for( cci = 0; cci < MAX_CC; cci++ ){
 		sprintf(server,"%s-%d",url,cci);
-		if( ccfp = fopenCC(server,"r+",path) ){
-			if( new ){
+		if( ccfp = fopenCC(server,"r+",AVStr(path)) ){
+			if( xnew ){
 				fcloseCC();
 				continue;
 			}
@@ -75,7 +109,10 @@ static tryOpen(Conn,user,new,path,portp)
 				continue;
 			}
 
+			conretry = CON_RETRY;
+			CON_RETRY = 1;
 			sock = client_open("serverCC",DST_PROTO,LOCALHOST,svport);
+			CON_RETRY = conretry;
 			fcloseCC();
 
 			if( 0 <= sock ){
@@ -86,14 +123,14 @@ static tryOpen(Conn,user,new,path,portp)
 			unlink(path);
 
 		}else{
-			if( !new )
+			if( !xnew )
 				break;
 
-			if( (ccfp = fopenCC(server,"w+",path)) == NULL )
+			if( (ccfp = fopenCC(server,"w+",AVStr(path))) == NULL )
 				continue;
 
 			if( sock == -1 )
-				sock = server_open("serverCC",LOCALHOST,0,1);
+				sock = server_open("serverCC",CVStr(LOCALHOST),0,1);
 			svport = sockPort(sock);
 			sprintf(buff,"%d %d\n",svport,getpid());
 			fputs(buff,ccfp);
@@ -104,34 +141,111 @@ static tryOpen(Conn,user,new,path,portp)
 		}
 	}
 	if( 0 < sock )
-		sv1log("serverCC: %s [%s] %s",new?"wrote":"read",path,buff);
-	else if( new )
+		sv1log("serverCC: %s [%s] %s",xnew?"wrote":"read",path,buff);
+	else if( xnew )
 		sv1log("serverCC: cannot open Connection Cache file\n");
 	return sock;
 }
+int CC_open(PCStr(proto),PCStr(host),int port,PCStr(user),int xnew){
+	Connection ConnBuf, *Conn = &ConnBuf;
+	CStr(path,1024);
+	int sock;
+	int xport;
 
-extern char *TIMEFORM_ANSI_C;
+	bzero(Conn,sizeof(Connection));
+	strcpy(REAL_PROTO,proto);
+	strcpy(REAL_HOST,host);
+	REAL_PORT = port;
+	sock = tryOpen(Conn,user,xnew,AVStr(path),&xport);
+	return sock;
+}
+int CC_connect(PCStr(proto),PCStr(host),int port,PCStr(user)){
+	int svsock;
+	svsock = CC_open(proto,host,port,user,0);
+	sv1log("---- CC connect got %d\n",svsock);
+	return svsock;
+}
+int serverPid();
+static int CCready(int timeout,int svacc,int svcon){
+	int fdv[2],rdv[2],nready;
+	double Start,Elp;
 
-static char mypath[256];
-static void sigTERM(sig)
+	fdv[0] = svacc;
+	fdv[1] = svcon;
+	sv1log("---- CC watching acc[%d]con[%d]pid[%d]\n",
+		svacc,svcon,serverPid());
+
+	Start = Time();
+	for(;;){
+		Elp = Time() - Start;
+		Verbose("---- CC watching acc[%d]con[%d]pid[%d]%.1fs\n",
+			svacc,svcon,serverPid(),Elp);
+		if( timeout < Elp ){
+			sv1log("---- CC timeout[%d]\n",svcon);
+			break;
+		}
+		nready = PollIns(5*1000,2,fdv,rdv);
+		if( 0 < nready ){
+			if( 0 < rdv[1] ){
+				sv1log("---- CC eof from the server[%d]\n",
+					svcon);
+				break;
+			}
+			if( 0 < rdv[0] ){
+				return 1;
+			}
+		}
+		if( !procIsAlive(serverPid()) ){
+			sv1log("---- CC service done [%d]\n",serverPid());
+			break;
+		}
+	}
+	return -1;
+}
+int CC_TIMEOUT = 180;
+int CC_accept(PCStr(proto),PCStr(host),int port,PCStr(user),int fromS){
+	int mysock;
+	int clsock;
+
+	mysock = CC_open(proto,host,port,user,1);
+	sv1log("---- CC accept got mysock=%d\n",mysock);
+	if( 0 <= mysock ){
+		Connection ConnBuf, *Conn = &ConnBuf;
+		bzero(Conn,sizeof(Connection));
+		ProcTitle(Conn,"SFTPCC://%s@%s:%d",user,host,port);
+
+		/*
+		if( 0 < PollIn(mysock,0) ){
+		*/
+		if( 0 < CCready(CC_TIMEOUT,mysock,fromS) ){
+			clsock = ACCEPT(mysock,1,-1,10);
+		}else	clsock = -1;
+		sv1log("---- CC accept got clsock=%d\n",clsock);
+		close(mysock);
+		return clsock;
+	}
+	sv1log("---- CC accept can't get mysock\n");
+	return -1;
+}
+
+static void sigTERM(int sig)
 {
-	sv1log("SIGTERM for BoundProxy: %s\n",mypath);
-	unlink(mypath);
+	sv1log("SIGTERM for BoundProxy: %s\n",Mypath);
+	unlink(Mypath);
 	Finish(0);
 }
 
-beBoundProxy(Conn,user,timeout,func,a1,a2,a3,a4)
-	Connection *Conn;
-	char *user;
-	int (*func)();
-	void *a1,*a2,*a3,*a4;
+void beBoundProxy(Connection *Conn,PCStr(user),int timeout,iFUNCP func,...)
 {	int svsock,svport,clsock,clport;
-	char path[1024],stime[128],clhost[256];
+	CStr(path,1024);
+	CStr(stime,128);
+	CStr(clhost,MaxHostNameLen);
 	int rcode;
 	FILE *ccfp;
 	int ccc,start,done;
 	int startSerno;
 	int fdv[3],rfdv[3];
+	VARGS(8,func);
 
 	if( ServViaCc )
 		return;
@@ -145,13 +259,13 @@ beBoundProxy(Conn,user,timeout,func,a1,a2,a3,a4)
 		return;
 	}
 
-	svsock = tryOpen(Conn,user,1,path,&svport);
+	svsock = tryOpen(Conn,user,1,AVStr(path),&svport);
 	if( svsock < 0 )
 		return;
 
 	sayStartCC();
 
-	strcpy(mypath,path);
+	strcpy(Mypath,path);
 	Vsignal(SIGINT,sigTERM);
 	Vsignal(SIGTERM,sigTERM);
 	close_FSERVER(Conn,0);
@@ -175,7 +289,14 @@ beBoundProxy(Conn,user,timeout,func,a1,a2,a3,a4)
 			break;
 		}
 		if( PollIn(FromS,1) != 0 ){
-			sv1log("serverCC: exit by something from server.\n");
+			CStr(buf,128);
+			int cc;
+			cc = readTIMEOUT(FromS,AVStr(buf),sizeof(buf)-1);
+			if( 0 < cc )
+				setVStrEnd(buf,cc);
+			else	setVStrEnd(buf,0);
+			sv1log("serverCC: exit by something from server: %s\n",
+				buf);
 			break;
 		}
 		clsock = ACCEPT(svsock,1,-1,10);
@@ -198,7 +319,7 @@ BIND/CONNECT AT LOCLHOST PORT ONLY
 		rcode = lock_exclusiveTO(fileno(ccfp),1000,NULL);
 		start = time(0L);
 		startSerno = RequestSerno;
-		StrftimeLocal(stime,sizeof(stime),TIMEFORM_ANSI_C,start,0);
+		StrftimeLocal(AVStr(stime),sizeof(stime),TIMEFORM_ANSI_C,start,0);
 		fprintf(ccfp,"[%s] start\n",stime);
 		fflush(ccfp);
 		sv1log("%sCC [%d] restart\n",DST_PROTO,ccc);
@@ -208,12 +329,12 @@ BIND/CONNECT AT LOCLHOST PORT ONLY
 		ImCC = 1;
 
 		setsockbuf(clsock,32*1024,32*1024);
-		rcode = (*func)(Conn,a1,a2,a3,a4);
+		rcode = (*func)(Conn,VA8);
 		ImCC = 0;
 		close(clsock);
 
 		done = time(0L);
-		StrftimeLocal(stime,sizeof(stime),TIMEFORM_ANSI_C,done,0);
+		StrftimeLocal(AVStr(stime),sizeof(stime),TIMEFORM_ANSI_C,done,0);
 		fprintf(ccfp,"[%s] done, %d seconds\n",stime,done-start);
 		fflush(ccfp);
 		lock_unlock(fileno(ccfp));
@@ -226,19 +347,17 @@ BIND/CONNECT AT LOCLHOST PORT ONLY
 		}
 	}
 	close(svsock);
+	if( ccfp != NULL )
 	fclose(ccfp);
 	unlink(path);
 	sayFinishCC();
 }
 
-connectToCache(Conn,user,svsockp)
-	Connection *Conn;
-	char *user;
-	int *svsockp;
-{	char path[1024];
+int connectToCache(Connection *Conn,PCStr(user),int *svsockp)
+{	CStr(path,1024);
 	int clsock,svport;
 
-	clsock = tryOpen(Conn,user,0,path,&svport);
+	clsock = tryOpen(Conn,user,0,AVStr(path),&svport);
 	if( 0 <= clsock ){
 		*svsockp = clsock;
 		ServViaCc = 1;
@@ -246,13 +365,7 @@ connectToCache(Conn,user,svsockp)
 	}else	return 0;
 }
 
-
-static int CC_master;
-static int CC_Report[2] = {-1,-1};
-static int CC_Active = 0;
-static int CC_Procs[256];
-
-set_CC()
+void set_CC()
 {
 	if( CC_Report[0] < 0 ){
 		CC_master = getpid();
@@ -262,14 +375,14 @@ set_CC()
 		setCloseOnExec(CC_Report[1]);
 	}
 }
-putCCinfo(pid)
+void putCCinfo(int pid)
 {	int fd;
 
 	if( (fd = CC_Report[1]) < 0 )
 		return;
 	write(fd,&pid,sizeof(pid));
 }
-sayStartCC(){
+void sayStartCC(){
 	/* When the CC_master is not the parent of this process,
 	 * it will not able to count down this CC by wait().
 	 * Such CC will be fork()ed as a NNTPCC on timeout...
@@ -278,13 +391,12 @@ sayStartCC(){
 		return;
 	putCCinfo(getpid());
 }
-sayFinishCC(){
+void sayFinishCC(){
 	if( getppid() != CC_master )
 		return;
 	putCCinfo(-getpid());
-
 }
-num_CC()
+int num_CC()
 {	int fd,pid;
 
 	if( (fd = CC_Report[0]) < 0 )
@@ -293,7 +405,7 @@ num_CC()
 	if( getpid() != CC_master )
 		return 0;
 
-	while( readTimeoutBlocked(fd,&pid,sizeof(pid),1) == sizeof(pid) ){
+	while( readTimeoutBlocked(fd,GVStr(&pid),sizeof(pid),1)==sizeof(pid) ){
 		sv1log("## got_CCinfo: %d\n",pid);
 		if( 0 <= pid ){
 			sv1log("## add_CC [%d] %d\n",CC_Active,pid);
@@ -304,8 +416,7 @@ num_CC()
 	}
 	return CC_Active;
 }
-del_CC(pid,how)
-	char *how;
+void del_CC(int pid,PCStr(how))
 {	int ci;
 
 	for( ci = 0; ci < CC_Active; ci++ ){
@@ -319,7 +430,7 @@ del_CC(pid,how)
 		}
 	}
 }
-kill_CC()
+int kill_CC()
 {	int ci,pid,nkill;
 
 	nkill = 0;
