@@ -18,15 +18,18 @@ Description:
 History:
 	950527	created
 //////////////////////////////////////////////////////////////////////#*/
+#include "vsocket.h"
 #include "delegate.h"
-extern char *_inet_ntoaV4I();
+#include "fpoll.h"
+int logVERBOSE();
+
 
 typedef unsigned long U_long;
 typedef unsigned short U_short;
 typedef unsigned char U_char;
 
-static char serverhost[17];
-static int serverport;
+#define SendToB(s,b,l,h,p)	SendTo(s,(char*)b,l,h,p)
+#define RecvFromB(s,b,l,h,p)	RecvFrom(s,(char*)b,l,h,p)
 
 #define kGroup			htonS(0)
 #define kClient			htonS(1)
@@ -46,7 +49,7 @@ static int serverport;
 typedef struct addr {
 	short	family;
 	U_short	port;
-	U_long	addr;
+	U_long	addrI;
 } Addr;
 typedef struct Routing {
 	Addr	dest;
@@ -63,7 +66,7 @@ typedef struct OpenContinuePacket {
 	VideoPacketHeader	header;
 	short	clientCount;
 	U_long	seqNum;
-	char	name[20];
+	char	name[20]; /**/
 	U_char	sendMode;
 	U_char	recvMode;
 	U_char	flags;
@@ -78,9 +81,9 @@ typedef struct OpenContinuePacket {
 #define	CLIENT_TIMEOUT	10
 
 typedef struct {
-	Addr	addr;
+	Addr	addrA;
 	int	routerx;
-	char	Rhost[17];
+	MStr(	e_Rhost,17);
 	int	Rport;
 	int	clientID;
 	int	confID;
@@ -89,60 +92,83 @@ typedef struct {
 	int	disable;
 } Client;
 
-static Client *_clients;
-static Client *_routers;
-static int  clientx;
-static int  clientID;
-static int  routerx;
+typedef struct {
+	MStr(	ce_Serverhost,17);
+	int	ce_Serverport;
+	Client *ce__clients;
+	Client *ce__routers;
+	int	ce_Clientx;
+	int	ce_ClientID;
+	int	ce_Routerx;
+	FILE   *ce_Save;
+	OpenContinuePacket ce_Vctl;
+	int	ce_SVsock;
+	int	ce_openedReflector;
+} CUSeeMeEnv;
+static CUSeeMeEnv *cUSeeMeEnv;
+#define Serverhost	cUSeeMeEnv->ce_Serverhost
+/**/
+#define Serverport	cUSeeMeEnv->ce_Serverport
+#define _clients	cUSeeMeEnv->ce__clients
+#define _routers	cUSeeMeEnv->ce__routers
+#define Clientx		cUSeeMeEnv->ce_Clientx
+#define ClientID	cUSeeMeEnv->ce_ClientID
+#define Routerx		cUSeeMeEnv->ce_Routerx
+#define Save		cUSeeMeEnv->ce_Save
+#define Vctl		cUSeeMeEnv->ce_Vctl
+#define SVsock		cUSeeMeEnv->ce_SVsock
+#define openedReflector	cUSeeMeEnv->ce_openedReflector
+void minit_cuseeme()
+{
+	if( cUSeeMeEnv == 0 )
+		cUSeeMeEnv = NewStruct(CUSeeMeEnv);
+}
 
-static Client *routerp(rx)
+static Client *routerp(int rx)
 {
 	if( _routers == NULL )
 		_routers = (Client*)calloc(NUM_ROUTER,sizeof(Client));
 	return &_routers[rx];
 }
-static Client *clientp(cx)
+static Client *clientp(int cx)
 {
 	if( _clients == NULL )
 		_clients = (Client*)calloc(NUM_CLIENT,sizeof(Client));
 	return &_routers[cx];
 }
 
-static getRouter(host,port)
-	char *host;
+static int getRouter(PCStr(host),int port)
 {	int ri;
 	Client *rp;
 
-	for( ri = 1; ri <= routerx; ri++ ){
+	for( ri = 1; ri <= Routerx; ri++ ){
 		rp = routerp(ri);
-		if( strcmp(rp->Rhost,host) == 0 && rp->Rport == port )
+		if( strcmp(rp->e_Rhost,host) == 0 && rp->Rport == port )
 			return ri;
 	}
-	rp = routerp(++routerx);
+	rp = routerp(++Routerx);
 	sv1log("added route[%d] %s:%d\n",ri,host,port);
-	strcpy(rp->Rhost,host);
+	strcpy(rp->e_Rhost,host);
 	rp->Rport = port;
-	return routerx;
+	return Routerx;
 }
-static addClient(curtime,host,port,caddr)
-	char *host;
-	Addr *caddr;
+static void addClient(int curtime,PCStr(host),int port,Addr *caddr)
 {	int ci;
 	Client *ca;
 	int routerx;
 
 	routerx = getRouter(host,port);
-	for( ci = 1; ci <= clientx; ci++ ){
+	for( ci = 1; ci <= Clientx; ci++ ){
 		ca = clientp(ci);
-		if( ca->addr.addr==caddr->addr && ca->addr.port==caddr->port ){
+		if( ca->addrA.addrI==caddr->addrI && ca->addrA.port==caddr->port ){
 			if( routerx != ca->routerx )
 				sv1log("moved client#%d [%d/%d]%x:%d via [%d]%s:%d\n",
-					ca->clientID,ci,clientx,
-					caddr->addr,ntohS(caddr->port),
+					ca->clientID,ci,Clientx,
+					caddr->addrI,ntohS(caddr->port),
 					routerx,host,port);
 			if( ca->disable )
 				sv1log("enabled client#%d [%d/%d]\n",
-					ca->clientID,ci,clientx);
+					ca->clientID,ci,Clientx);
 
 			ca->routerx = routerx;
 			ca->failed = 0;
@@ -151,62 +177,57 @@ static addClient(curtime,host,port,caddr)
 			return;
 		}
 	}
-	ca = clientp(++clientx);
-	ca->clientID = ++clientID;
+	ca = clientp(++Clientx);
+	ca->clientID = ++ClientID;
 
-	ca->addr = *caddr;
+	ca->addrA = *caddr;
 	ca->routerx = routerx;
 	ca->failed = 0;
 	ca->lasttime = curtime;
 	ca->disable = 0;
 
 	sv1log("added client#%d [%d/%d]%s:%d via [%d]%s:%d\n",
-		ca->clientID,ci,clientx,
-		_inet_ntoaV4I(caddr->addr),ntohS(caddr->port),routerx,host,port);
+		ca->clientID,ci,Clientx,
+		_inet_ntoaV4I(caddr->addrI),ntohS(caddr->port),routerx,host,port);
 }
-static delClient(why,ci)
-	char *why;
+static void delClient(PCStr(why),int ci)
 {	Client *ca;
 	int cj;
 
 	ca = clientp(ci);
 	sv1log("removed client#%d [%d/%d]: %x (%s)\n",
-		ca->clientID,ci,clientx,ca->addr.addr,why);
-	for( cj = ci; cj <= clientx; cj++ )
+		ca->clientID,ci,Clientx,ca->addrA.addrI,why);
+	for( cj = ci; cj <= Clientx; cj++ )
 		*clientp(cj) = *clientp(cj+1);
-	clientx--;
+	Clientx--;
 }
-static isClient(caddr)
-	Addr *caddr;
+static int isClient(Addr *caddr)
 {	int ci;
 	Client *ca;
 
-	for( ci = 1; ci <= clientx; ci++ ){
+	for( ci = 1; ci <= Clientx; ci++ ){
 		ca = clientp(ci);
-		if( caddr->addr==ca->addr.addr && caddr->port==ca->addr.port )
+		if( caddr->addrI==ca->addrA.addrI && caddr->port==ca->addrA.port )
 			return ci;
 	}
 	return 0;
 }
-static activeClients()
+static int activeClients()
 {	int ci,nact;
 	nact = 0;
-	for( ci = 1; ci <= clientx; ci++ )
+	for( ci = 1; ci <= Clientx; ci++ )
 		if( !clientp(ci)->disable )
 			nact++;
 	return nact;
 }
-static closeClient(caddr)
-	Addr *caddr;
+static void closeClient(Addr *caddr)
 {	int ci;
 
 	if( ci = isClient(caddr) )
 		clientp(ci)->disable = 1;
 }
 
-static FILE *Save;
-static toClients(curtime,SVsock,vp,len)
-	VideoPacketHeader *vp;
+static void toClients(int curtime,int svsock,VideoPacketHeader *vp,int len)
 {	int ci;
 	int wcc;
 	Client *ca;
@@ -218,12 +239,12 @@ static toClients(curtime,SVsock,vp,len)
 	if( Save != NULL )
 		fwrite(vp,len,1,Save);
 
-	for( ci = 1; ci <= clientx; ci++ ){
+	for( ci = 1; ci <= Clientx; ci++ ){
 		ca = clientp(ci);
 		rp = routerp(ca->routerx);
 
 		if( !ca->disable ){
-			wcc = SendTo(SVsock,vp,len,rp->Rhost,rp->Rport);
+			wcc = SendToB(svsock,vp,len,rp->e_Rhost,rp->Rport);
 			if( wcc <= 0 ){
 				if( 10 < ca->failed++ ){
 					delClient("CANTSEND",ci);
@@ -237,12 +258,12 @@ static toClients(curtime,SVsock,vp,len)
 		}
 	}
 }
-static dumpPacket(vp)
-	VideoPacketHeader *vp;
-{	char dhost[32],shost[32];
+static void dumpPacket(VideoPacketHeader *vp)
+{	CStr(dhost,32);
+	CStr(shost,32);
 
-	strcpy(dhost,_inet_ntoaV4I(vp->routing.dest.addr)),
-	strcpy(shost,_inet_ntoaV4I(vp->routing.src.addr)),
+	strcpy(dhost,_inet_ntoaV4I(vp->routing.dest.addrI)),
+	strcpy(shost,_inet_ntoaV4I(vp->routing.src.addrI)),
 fprintf(stderr,
 	"PACKET dst=[%d]%s:%d  src=[%d]%s:%d seq=%d msg=%d type=%d len=%d\n",
 	ntohS(vp->routing.dest.family),dhost,ntohS(vp->routing.dest.port),
@@ -251,16 +272,15 @@ fprintf(stderr,
 	ntohS(vp->message), ntohS(vp->dataType), ntohS(vp->len)
 	);
 }
-static dumpPakect2(rcc,rhost,rport,vp)
-	VideoPacketHeader *vp;
+static void dumpPakect2(int rcc,int rhost,int rport,VideoPacketHeader *vp)
 {
 fprintf(stderr,"%d %x:%d  %d.%x:%d >> %d.%x:%d msg=%3d type=%3d len=%3d #%d\n",
 	rcc,rhost,rport,
 	vp->routing.src.family,
-	vp->routing.src.addr,
+	vp->routing.src.addrI,
 	vp->routing.src.port,
 	vp->routing.dest.family,
-	vp->routing.dest.addr,
+	vp->routing.dest.addrI,
 	vp->routing.dest.port,
 	vp->message,
 	vp->dataType,
@@ -269,14 +289,9 @@ fprintf(stderr,"%d %x:%d  %d.%x:%d >> %d.%x:%d msg=%3d type=%3d len=%3d #%d\n",
 	);
 }
 
-static OpenContinuePacket Vctl;
-static int SVsock;
-static int openedReflector;
-
-static dumpVctl(sh,sp)
-	char *sh;
+static void dumpVctl(PCStr(sh),int sp)
 {	int i;
-	unsigned char *vp;
+	const unsigned char *vp;
 
 	fprintf(stderr,"[%s:%d]\n",sh,sp);
 	vp = (unsigned char*)&Vctl;
@@ -288,8 +303,7 @@ static dumpVctl(sh,sp)
 	fprintf(stderr,"\n");
 }
 
-static closeServer(serverhost,serverport)
-	char *serverhost;
+static void closeServer(PCStr(serverhost),int serverport)
 {
 	sv1log("CloseConnection: no client.\n");
 	openedReflector = 0;
@@ -297,11 +311,9 @@ static closeServer(serverhost,serverport)
 	Vctl.header.dataType = kConfigVideoType;
 	Vctl.header.message = kCloseConnection;
 	/*dumpVctl(serverhost,serverport);*/
-	SendTo(SVsock,&Vctl,sizeof(Vctl),serverhost,serverport);
+	SendToB(SVsock,&Vctl,sizeof(Vctl),serverhost,serverport);
 }
-static openServer(serverhost,serverport,vp)
-	char *serverhost;
-	VideoPacketHeader *vp;
+static void openServer(PCStr(serverhost),int serverport,VideoPacketHeader *vp)
 {
 	for(;;){
 		sv1log("OpenConnection: confid=%d\n",
@@ -310,7 +322,7 @@ static openServer(serverhost,serverport,vp)
 		Vctl.header.dataType = kConfigVideoType;
 		Vctl.header.message = kOpenConnection;
 		Vctl.header.routing.dest.port = vp->routing.dest.port;
-		SendTo(SVsock,&Vctl,sizeof(Vctl),serverhost,serverport);
+		SendToB(SVsock,&Vctl,sizeof(Vctl),serverhost,serverport);
 		if( 0 < PollIn(SVsock,1000) ){
 			sv1log("Opened.\n");
 			openedReflector = 1;
@@ -320,29 +332,28 @@ static openServer(serverhost,serverport,vp)
 	}
 }
 
-static getServerAddr(myhost,myport)
-	char *myhost;
+static void getServerAddr(PCStr(myhost),int myport)
 {	int retry;
 	int wcc,rcc;
-	char rhost[17];
+	CStr(rhost,17);
 	int rport;
 	int bgntime;
 	VideoPacketHeader vpk;
 
 	Vctl.header.routing.src.family = kClient;
-	Vctl.header.routing.src.addr = _inet_addrV4(myhost);
+	Vctl.header.routing.src.addrI = _inet_addrV4(myhost);
 	Vctl.header.routing.src.port = htonS(myport);
 
 	Vctl.header.routing.dest.family = kReflector;
-	Vctl.header.routing.dest.addr = _inet_addrV4(serverhost);
-	Vctl.header.routing.dest.port = htonS(serverport);
+	Vctl.header.routing.dest.addrI = _inet_addrV4(Serverhost);
+	Vctl.header.routing.dest.port = htonS(Serverport);
 
 	Vctl.header.seqNum = htonL(255);
 	Vctl.header.message = kOpenConnection;
 	Vctl.header.dataType = kConfigVideoType;
 	Vctl.header.len = 0;
 
-	if( logVERBOSE() ) dumpPacket(&Vctl);
+	if( logVERBOSE() ) dumpPacket((VideoPacketHeader*)&Vctl);
 	Vctl.flags = WANT_VERSION;
 
 	bgntime = time(0);
@@ -351,82 +362,83 @@ static getServerAddr(myhost,myport)
 		Vctl.seqNum = Vctl.header.seqNum;
 
 		/*dumpVctl(serverhost,serverport);*/
-		wcc = SendTo(SVsock,&Vctl,sizeof(Vctl),serverhost,serverport);
+		wcc = SendToB(SVsock,&Vctl,sizeof(Vctl),Serverhost,Serverport);
 		if( PollIn(SVsock,1000) <= 0 ){
 			if( 30 < time(0)-bgntime ){
 				sv1log("give up ;-<\n");
 				break;
 			}
 			sv1log("NO RESPONSE FROM %s:%d, retry...\n",
-				serverhost,serverport);
+				Serverhost,Serverport);
 			continue;
 		}
-		rcc = RecvFrom(SVsock,&vpk,sizeof(vpk),rhost,&rport);
+		rcc = RecvFromB(SVsock,&vpk,sizeof(vpk),AVStr(rhost),&rport);
 		if( vpk.routing.src.family == kReflector )
 		{
 			if( logVERBOSE() ) dumpPacket(&vpk);
 			sv1log("REFLECTOR: %s:%d -> %s:%d\n",
-				serverhost,serverport,rhost,rport);
-			strcpy(serverhost,rhost);
-			serverport = rport;
+				Serverhost,Serverport,AVStr(rhost),rport);
+			strcpy(Serverhost,rhost);
+			Serverport = rport;
 			break;
 		}
 	}
 	INC_NL(Vctl.header.seqNum,1);
 	Vctl.header.message = kCloseConnection;
-	SendTo(SVsock,&Vctl,sizeof(Vctl),serverhost,serverport);
-	if( logVERBOSE() ) dumpPacket(&Vctl);
+	SendToB(SVsock,&Vctl,sizeof(Vctl),Serverhost,Serverport);
+	if( logVERBOSE() ) dumpPacket((VideoPacketHeader*)&Vctl);
 }
 
-extern char *gethostaddr();
-
-service_cuseeme(Conn,svsock,myport)
-	Connection *Conn;
-{	char buff[0x8000];
+int service_cuseeme(Connection *Conn,int svsock,int myport)
+{	CStr(buff,0x8000);
 	int rcc;
 	VideoPacketHeader *vp;
-	char host[128];
-	char myhost[17];
-	char rhost[17];
+	CStr(host,128);
+	CStr(myhost,17);
+	CStr(rhost,17);
 	int rport;
 	Addr client;
 	int ucount,ubytes,dcount,dbytes;
 	int bgntime,curtime,nexttime;
-	char shost[128]; /* socket host if using */
+	CStr(shost,128); /* socket host if using */
 	int sport;
-	char *aaddr;
+	const char *aaddr;
+
+	minit_cuseeme();
 
 	SVsock = svsock;
 	if( (aaddr = gethostaddr(DST_HOST)) == NULL ){
 		sv1log("#### ERROR: bad destination host [%s]\n",DST_HOST);
-		return;
+		return -1;
 	}
-	strcpy(serverhost,aaddr);
-	serverport = DST_PORT;
-	if( hostIFfor1(host,1,DST_PROTO,DST_HOST,DST_PORT) == 0 )
-	if( hostIFfor(DST_HOST,host) == 0 )
+	strcpy(Serverhost,aaddr);
+	Serverport = DST_PORT;
+	if( hostIFfor1(AVStr(host),1,DST_PROTO,DST_HOST,DST_PORT) == 0 )
+	if( hostIFfor(DST_HOST,AVStr(host)) == 0 )
 		gethostname(host,sizeof(host));
+	/*
 	strcpy(myhost,gethostaddr(host));
+	*/
+	strcpy(myhost,gethostaddrX(host));
 	sv1log("myname=%s[%s]\n",host,myhost);
 
-{
-extern char *getenv();
-char *file;
+ {
+const char *file;
 
 	if( file = getenv("CUIN") ){
 		Save = fopen(file,"w");
 		fprintf(stderr,"CUIN: %s : %x\n",file,Save);
 	}
-}
+ }
 
 	if( 0 < socks_addservers() )
-	if( SOCKS_udpassocsock(SVsock,myhost,myport,shost,&sport) == 0 ){
+	if( SOCKS_udpassocsock(SVsock,myhost,myport,AVStr(shost),&sport) == 0 ){
 		sv1log("via SOCKS: %s:%d -> %s:%d\n",myhost,myport,shost,sport);
 		strcpy(myhost,shost);
 		myport = sport;
 	}
 
-	closeServer(serverhost,serverport);
+	closeServer(Serverhost,Serverport);
 	getServerAddr(myhost,myport);
 
 	ucount = ubytes = 0;
@@ -435,24 +447,24 @@ char *file;
 	nexttime = (bgntime/10+1) * 10;
 	vp = (VideoPacketHeader *)buff;
 
-	sv1log("REFLECTOR: %s:%d\n",serverhost,serverport);
+	sv1log("REFLECTOR: %s:%d\n",Serverhost,Serverport);
 	for(;;){
-		rcc = RecvFrom(SVsock,buff,sizeof(buff),rhost,&rport);
+		rcc = RecvFromB(SVsock,buff,sizeof(buff),AVStr(rhost),&rport);
 		if( rcc < 0 )
 			continue;
 		/* dumpPacket2(vp); */
 
 		if( vp->routing.src.family == kReflector )
-		if( strcmp(serverhost,rhost) != 0 || serverport != rport )
+		if( strcmp(Serverhost,rhost) != 0 || Serverport != rport )
 		{
 			sv1log("REFLECTOR: %s:%d -> %s:%d\n",
-				serverhost,serverport,rhost,rport);
-			strcpy(serverhost,rhost);
-			serverport = rport;
+				Serverhost,Serverport,rhost,rport);
+			strcpy(Serverhost,rhost);
+			Serverport = rport;
 		}
 
 		curtime = time(0);
-		if( strcmp(rhost,serverhost) == 0 && rport == serverport ){
+		if( strcmp(rhost,Serverhost) == 0 && rport == Serverport ){
 			dcount++;
 			dbytes += rcc;
 			toClients(curtime,SVsock,vp,rcc);
@@ -462,15 +474,15 @@ char *file;
 
 			if( vp->message == kCloseConnection ){
 				closeClient(&vp->routing.src);
-				SendTo(SVsock,vp,rcc,serverhost,serverport);
+				SendToB(SVsock,vp,rcc,Serverhost,Serverport);
 
 				if( openedReflector && activeClients() == 0 )
-					closeServer(serverhost,serverport);
+					closeServer(Serverhost,Serverport);
 			}else{
 				if( openedReflector == 0 )
-					openServer(serverhost,serverport,vp);
+					openServer(Serverhost,Serverport,vp);
 				addClient(curtime,rhost,rport,&vp->routing.src);
-				SendTo(SVsock,vp,rcc,serverhost,serverport);
+				SendToB(SVsock,vp,rcc,Serverhost,Serverport);
 			}
 		}
 		if( nexttime <= curtime ){
@@ -478,8 +490,9 @@ char *file;
 
 			Bps = ((ubytes+dbytes)*8.0)/(curtime-bgntime);
 			daemonlog("I","%d clnts, %d/%dup+%d/%ddown, %5.2fKbps\n",
-				clientx,ubytes,ucount,dbytes,dcount,Bps/1000);
+				Clientx,ubytes,ucount,dbytes,dcount,Bps/1000);
 			nexttime = (curtime/10+1) * 10;
 		}
 	}
+	return 0;
 }
