@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1996 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1996-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -46,6 +47,10 @@ int makeDistribution(Connection *Conn,FILE *cachefp,PCStr(cpath))
 
 	if( !INHERENT_fork() )
 		return -1;
+	if( actthreads() ){
+		/* 9.6.1 fork() after thread_fork() seems to cause jamming */
+		return -1;
+	}
 
 	sprintf(lpath,"%s#rdist",cpath);
 	sock = bind_un("RESPDIST",AVStr(lpath),10,AVStr(spath),sizeof(spath));
@@ -68,6 +73,7 @@ FILE *recvDistribution(Connection *Conn,PCStr(cpath),int *updated)
 	const char *dp;
 	int timeout;
 
+	*updated = 0;
 	if( !INHERENT_fork() )
 		return NULL;
 
@@ -106,7 +112,10 @@ FILE *recvDistribution(Connection *Conn,PCStr(cpath),int *updated)
 	errlog("#### got recvDistribution(%d): %s %s\n",sock,stat,lpath);
 
 	if( *stat == '2' )
+	{
+		ServerFlags |= PF_IS_DISTRIB;
 		return fs;
+	}
 	else{
 		*updated = 1;
 		fclose(fs);
@@ -147,8 +156,12 @@ void stopDistribution(Connection *Conn,FILE *cachefp,PCStr(cpath))
 	*/
 	while( 0 < PollIn(Conn->ca_distsock,1) ){
 		rsock = ACCEPT(Conn->ca_distsock,1,-1,10);
+		/*
 		errlog("#### discard accepted Distribution [%d]->[%d]\n",
 			Conn->ca_distsock,rsock);
+		*/
+		errlog("#### discard accepted Distribution [%d]->[%d] err=%d\n",
+			Conn->ca_distsock,rsock,errno);
 		if( rsock < 0 )
 			break;
 		close(rsock);
@@ -189,11 +202,13 @@ static int copy1(FILE *cachefp,int toC,int from,int doflush)
 			wcc += wcc1;
 
 		if( wcc1 != rcc1 ){
+if( 2 <= LOGLEVEL )
 Verbose("#### [%d] write incomplete Distribution %d / %d\n",toC,wcc1,rcc1);
 			break;
 		}
 	}
 
+if( 2 <= LOGLEVEL )
 Verbose("#### [%d] Distribution %d / %d [%d-%d]\n",toC,wcc,leng,from,savoff);
 	fseek(cachefp,savoff,0);
 	return wcc;
@@ -203,7 +218,10 @@ static void flushDistribution(Connection *Conn,FILE *cachefp,int eof)
 {	int rx;
 	int wcc;
 	int nput;
+	/*
 	int *receptors;
+	*/
+	short *receptors;
 	int *putoff;
 	int remain;
 	int lastput;
@@ -218,18 +236,32 @@ static void flushDistribution(Connection *Conn,FILE *cachefp,int eof)
 		for( rx = 0; rx < Conn->ca_receptorN; rx++ ){
 			if( putoff[rx] == Conn->ca_curoff )
 				continue;
+			if( putoff[rx] < 0 ){
+				continue;
+			}
 			nsigPIPE = 0;
 			wcc = copy1(cachefp,receptors[rx],putoff[rx],1);
 			if( 0 < wcc ){
 				nput += wcc;
 				putoff[rx] += wcc;
+if( 2 <= LOGLEVEL )
 Verbose("#### (%d)[%d] sendDistribution: caught up %d (%d/%d)\n",
 	rx,receptors[rx], wcc,putoff[rx],Conn->ca_curoff);
 			}else
 			if( nsigPIPE ){
-				putoff[rx] = -1;
-errlog("#### (%d)[%d] flushDistribution: caught SIGPIPE\n",
-	rx,receptors[rx]);
+errlog("#### (%d)[%d] flushDistribution: caught SIGPIPE (%d)\n",
+	rx,receptors[rx], putoff[rx]);
+				if( 0 < putoff[rx] )
+					putoff[rx] = -putoff[rx];
+				else	putoff[rx] = -1;
+			}
+			else
+			if( wcc <=0 && IsConnected(receptors[rx],NULL) == 0 ){
+errlog("---- (%d)[%d] flushDistribution: disconnected (%d/%d)\n",
+	rx,receptors[rx],putoff[rx],Conn->ca_curoff);
+				if( 0 < putoff[rx] )
+					putoff[rx] = -putoff[rx];
+				else	putoff[rx] = -2;
 			}
 		}
 		if( !eof )
@@ -285,6 +317,7 @@ static void closeServer(Connection *Conn,FILE *fs,FILE *cachefp,int infd)
 static int accept2(Connection *Conn,int rsock)
 {	int rx;
 
+	expsockbuf(rsock,0,0x20000);
 	rx = Conn->ca_receptorN++;
 	Conn->ca_receptors[rx] = rsock;
 	errlog("#### (%d)[%d] accept cache Distribution request\n",rx,rsock);
@@ -311,7 +344,11 @@ static void accept1(Connection *Conn,int svsock,FILE *cachefp)
 }
 
 static void distrib(Connection *Conn,PCStr(buff),int leng)
+{
+/*
 {	int *receptors;
+*/
+	short *receptors;
 	int *putoff;
 	int rx,wcc;
 
@@ -319,11 +356,15 @@ static void distrib(Connection *Conn,PCStr(buff),int leng)
 	putoff = Conn->ca_putoff;
 
 	for( rx = 0; rx < Conn->ca_receptorN; rx++ ){
+		/* send to syncronized receivers only. other receivers will be
+		 * syncronized (catch up) by flushDistirbution()
+		 */
 		if( putoff[rx] != Conn->ca_curoff )
 			continue;
 
 		wcc = write(receptors[rx],buff,leng);
 		if( wcc != leng ){
+if( 2 <= LOGLEVEL )
 Verbose("#### (%d)[%d] sendDistribution incomplete: %d/%d/%d\n",
 	rx,receptors[rx], wcc,leng,Conn->ca_curoff);
 		}
@@ -335,6 +376,8 @@ Verbose("#### (%d)[%d] sendDistribution incomplete: %d/%d/%d\n",
 int sendDistribution(Connection *Conn,FILE *cachefp,FILE *fs,FILE *tc,PCStr(buff),int leng)
 {	int timeout,fdv[2],rfdv[2];
 	int off;
+	double Start;
+	int nready;
 
 	if( cachefp == NULL || !Conn->ca_distsock_open )
 		return 0;
@@ -342,6 +385,37 @@ int sendDistribution(Connection *Conn,FILE *cachefp,FILE *fs,FILE *tc,PCStr(buff
 	if( !INHERENT_fork() )
 		return 0;
 
+	if( actthreads() ){
+		/* 9.6.3 response is in some encoding */
+		sv1log("## STOP sendDistribution: ath=%d [%d/%d] %d\n",
+			actthreads(),
+			Conn->ca_distsock,Conn->ca_distsock_open,
+			Conn->ca_receptorN
+		);
+		/* should unlink the socket */
+		close(Conn->ca_distsock);
+		Conn->ca_distsock = -1;
+		Conn->ca_distsock_open = 0;
+		return 0;
+	}
+
+	ClientEOF &= ~CLEOF_NOACTCL;
+	if( 0 < Conn->ca_receptorN ){
+		int rx;
+		int act = 0;
+		for( rx = 0; rx < Conn->ca_receptorN; rx++ ){
+			if( 0 <= Conn->ca_putoff[rx] ){
+				act = 1;
+				break;
+			}
+		}
+		if( act == 0 ){
+			sv1log("---- no active Distribution recveiver/%d\n",
+				Conn->ca_receptorN);
+			ClientEOF |= CLEOF_NOACTCL;
+		}
+	}
+
 	distrib(Conn,buff,leng);
 
 	off = Conn->ca_curoff;
@@ -368,7 +442,14 @@ int sendDistribution(Connection *Conn,FILE *cachefp,FILE *fs,FILE *tc,PCStr(buff
 		timeout = 10;
 	else	timeout = 1000;
 
+	Start = Time();
+	nready =
 	PollIns(timeout,2,fdv,rfdv);
+	if( 2 <= LOGLEVEL && nready == 0
+	 || 3 <= LOGLEVEL && (1 < Time()-Start) ){
+		sv1log("---- sendDistribution: ready=%d [%d][%d] %f/%d\n",
+			nready,fileno(fs),fdv[1],Time()-Start,timeout);
+	}
 	if( rfdv[1] <= 0 )
 		return 0;
 
@@ -383,17 +464,22 @@ int sendDistribution(Connection *Conn,FILE *cachefp,FILE *fs,FILE *tc,PCStr(buff
 			checkCloseOnTimeout(0);
 			closeServer(Conn,fs,cachefp,sio[0]);
 			close(sio[1]);
+			ServerFlags |= PF_IS_DISTRIB;
 			return 1;
 		}else{
 			int rx;
 			detachFile(tc);
 			ClientEOF = 1;
+			ClientFlags |= PF_IS_DISTRIB;
+			Vsignal(SIGPIPE,sigPIPE);
 			close(sio[0]);
 			rx = accept2(Conn,sio[1]);
 			Conn->ca_putoff[rx] = ftell(cachefp);
 
 HTTP_clntClose(Conn,"-:distributor");
 /* close clsock ? */
+			/* 9.6.3 to disable threads for gzip (wont happen) */
+			LOG_type2 |= L_NOTHREAD;
 		}
 	}
 	accept1(Conn,fdv[1],cachefp);
@@ -403,6 +489,11 @@ HTTP_clntClose(Conn,"-:distributor");
 void detachFile(FILE *fp)
 {	int fd,null;
 
+	if( lSINGLEP() ){
+		fprintf(stderr,"---ignored detachFile %X/%d\n",fp,fileno(fp));
+		return;
+	}
+
 	fd = fileno(fp);
 	close(fd);
 	null = open("/dev/null",2);
