@@ -1,11 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1997-1999 Yutaka Sato
-Copyright (c) 1997-1999 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1997-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, and distribute this material for any purpose
-and without fee is hereby granted, provided that the above copyright
-notice and this permission notice appear in all copies.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -18,17 +21,16 @@ History:
 //////////////////////////////////////////////////////////////////////#*/
 #include <stdio.h>
 #include "ystring.h"
-extern char *getADMIN();
-#define LNSIZE	1024
-
+#include "file.h"
+#include "proc.h"
 #include "delegate.h"
 #include "param.h"
+#define LNSIZE	1024
+const char *get_builtin_ADMINPASS();
 
-static getparam(line,param,in,out)
-	char *line,*param;
-	FILE *in,*out;
+static int getparam(PVStr(line),PCStr(param),FILE *in,FILE *out)
 {	int len;
-	char *dp;
+	const char *dp;
 
 	if( *param == '-' )
 		strcpy(line,param);
@@ -37,24 +39,26 @@ static getparam(line,param,in,out)
 
 	fprintf(out,"%s",line);
 	fflush(out);
-	fgets(line+len,LNSIZE-len,in);
+	Xfgets(DVStr(line,len),LNSIZE-len,in);
 	if( dp = strpbrk(line,"\r\n") )
-		*dp = 0;
+		truncVStr(dp);
 	if( line[len] )
 		return 1;
 	else	return 0;
 }
 
-#define GETPARAM(param)	getparam(line,param,in,out)
+#define GETPARAM(param)	getparam(AVStr(line),param,in,out)
 
-editconf(ac,avp,in,out)
-	char ***avp;
-	FILE *in,*out;
-{	char line[LNSIZE];
-	static char *nav[128];
-	char **av,arg1[128];
+int editconf(int ac,const char ***avp,FILE *in,FILE *out)
+{	CStr(line,LNSIZE);
+	static const char **nav;
+	const char **av;
+	CStr(arg1,128);
 	int ac0,ai;
 
+	if( nav == 0 )
+		nav = (const char**)StructAlloc(128*sizeof(char*));
+
 	ac0 = ac;
 	av = *avp;
 	for( ai = 0; ai < ac; ai++ )
@@ -63,28 +67,28 @@ editconf(ac,avp,in,out)
 
 	fprintf(out,"Administrator's E-mail ## ADMIN=user@domain [%s]\n",
 		getADMIN());
-	if( GETPARAM("ADMIN") ) nav[ac++] = strdup(line);
+	if( GETPARAM("ADMIN") ) nav[ac++] = stralloc(line);
 
 	fprintf(out,"Port of the DeleGate ## -P[hostName:]portNumber\n");
-	if( GETPARAM("-P") ) nav[ac++] = strdup(line);
+	if( GETPARAM("-P") ) nav[ac++] = stralloc(line);
 	else{
 #ifndef MAIN
 		int sock,port;
-		sock = server_open("-",NULL,0,1);
+		sock = server_open("-",VStrNULL,0,1);
 		if( 1 < (port = sockPort(sock)) ){
 			fprintf(out,"-P%d\n",port);
 			sprintf(arg1,"-P%d/%d",port,sock);
-			nav[ac++] = strdup(arg1);
+			nav[ac++] = stralloc(arg1);
 		}
 #endif
 	}
 
 	fprintf(out,"Client's Protocol ## SERVER=protocol[://defaultServer/]\n");
-	if( GETPARAM("SERVER") ) nav[ac++] = strdup(line);
+	if( GETPARAM("SERVER") ) nav[ac++] = stralloc(line);
 
 /*
 	fprintf(out,"Upper DeleGate's port ## MASTER=hostName:portNumber\n");
-	if( GETPARAM("MASTER") ) nav[ac++] = strdup(line);
+	if( GETPARAM("MASTER") ) nav[ac++] = stralloc(line);
 */
 
 	nav[ac++] = "-v";
@@ -101,8 +105,8 @@ editconf(ac,avp,in,out)
 #define getEnv(name) DELEGATE_getEnv(name)
 #define getEnvADMIN()	(getEnv(P_ADMIN)?getEnv(P_ADMIN):getEnv(P_MANAGER))
 
-char *getADMIN1()
-{	char *env;
+const char *getADMIN1()
+{	const char *env;
 
 	if( env = getEnvADMIN() )
 		return env;
@@ -110,8 +114,8 @@ char *getADMIN1()
 		return DELEGATE_ADMIN_DFLT;
 	return NULL;
 }
-char *getADMIN()
-{	char *admin;
+const char *getADMIN()
+{	const char *admin;
 
 	if( admin = getADMIN1() )
 		return admin;
@@ -128,10 +132,10 @@ char *getADMIN()
 #define ACAPS	4
 #define AVERS	5
 
-checkADMINPASS(Conn,passspec,admin,pass)
-	Connection *Conn;
-	char *passspec,*admin,*pass;
-{	char passb[1024],*passv[8],md5[64];
+int checkADMINPASS(DGC*Conn,PCStr(passspec),PCStr(admin),PCStr(pass))
+{	CStr(passb,1024);
+	const char *passv[8]; /**/
+	CStr(md5,64);
 	int pi,pc;
 
 	if( *passspec == 0 )
@@ -153,7 +157,7 @@ ERRMSG("ERROR! bad ADMINPASS\n");
 		}
 	}
 	if( *passv[AHOST] ){
-		char host[256];
+		CStr(host,MaxHostNameLen);
 		gethostname(host,sizeof(host));
 		toMD5(host,md5);
 		if( strcmp(md5,passv[AHOST]) != 0 ){
@@ -164,14 +168,14 @@ ERRMSG("ERROR! not allowed to run on this host.\n");
 
 	return 0;
 }
-checkADMIN(Conn,proto)
-	Connection *Conn;
-	char *proto;
-{	char *admin,owner[256];
-	int explicit;
+void checkADMIN(DGC*Conn,PCStr(proto))
+{	const char *admin;
+	CStr(owner,256);
+	int xexplicit;
+	const char *pass;
 
-	explicit = getEnvADMIN() != 0;
-	getUsername(getuid(),owner);
+	xexplicit = getEnvADMIN() != 0;
+	getUsername(getuid(),AVStr(owner));
 
 	if( admin = getADMIN() )
 		DELEGATE_ADMIN = admin;
@@ -184,7 +188,7 @@ ERRMSG("ERROR! ADMIN=\"your_mail_address\" must be specified.\n");
 		sleep(3);
 		Finish(-1);
 	}else
-	if( !explicit ){
+	if( !xexplicit ){
 		if( isatty(2) )
 		if( strncmp(DELEGATE_ADMIN,owner,strlen(owner)) != 0 ){
 ERRMSG("WARNING! ADMIN=\"your_mail_address\" should be specified.\n");
@@ -192,8 +196,8 @@ ERRMSG("INFO: using ADMIN=%s given at compile time.\n",DELEGATE_ADMIN);
 		}
 	}
 
-	if( builtin_ADMINPASS[0] != 0 ){
-		if( checkADMINPASS(Conn,builtin_ADMINPASS,DELEGATE_ADMIN,
+	if( pass = get_builtin_ADMINPASS() ){
+		if( checkADMINPASS(Conn,pass,DELEGATE_ADMIN,
 		getEnv(P_ADMINPASS)) != 0 ){
 			sleep(3);
 			Finish(-1);
@@ -201,15 +205,9 @@ ERRMSG("INFO: using ADMIN=%s given at compile time.\n",DELEGATE_ADMIN);
 	}
 }
 
-
-extern int  main_argc;
-extern char **main_argv;
-editconf1(acp,avp,in,out)
-	int *acp;
-	char **avp[];
-	FILE *in,*out;
+void editconf1(int *acp,const char **avp[],FILE *in,FILE *out)
 {	int ac;
-	char **av;
+	const char **av;
 
 	ac = *acp;
 	av = *avp;
@@ -223,12 +221,12 @@ editconf1(acp,avp,in,out)
 	main_argc = ac;
 	main_argv = av;
 }
-extern char EXEC_PATH[];
-service_admin(Conn)
-	Connection *Conn;
+int service_admin(DGC*Conn)
 {	FILE *fc,*tc;
 	int ai,nac;
-	char *arg,*nav[128],**navp;
+	const char *arg;
+	const char *nav[128]; /**/
+	const char **navp;
 
 	tc = fdopen(ToC,"w");
 	fc = fdopen(FromC,"r");
@@ -236,18 +234,21 @@ service_admin(Conn)
 	if( !service_permitted(Conn,"admin") ){
 		fprintf(tc,"you have no permission to admin.\n");
 		fflush(tc);
-		return;
+		return -1;
 	}
 
 	nac = 0;
 	navp = nav;
 	navp[nac++] = EXEC_PATH;
 	for( ai = 1; ai < main_argc; ai++ ){
+		if( elnumof(nav)-1 <= nac ){
+			break;
+		}
 		arg = main_argv[ai];
 		if( strncmp(arg,"SERVER=",7) == 0 ) continue;
 		if( strncmp(arg,"-P",2) == 0 ) continue;
 		if( strncmp(arg,"--",2) == 0 ) break;
-		nav[nac++] = arg;
+		nav[nac++] = (char*)arg;
 	}
 	navp[nac] = 0;
 	editconf1(&nac,&navp,fc,tc);
@@ -260,34 +261,40 @@ service_admin(Conn)
 	fclose(fc);
 
 	Execvp("ADMIN",EXEC_PATH,navp);
+	return 0;
 }
 
 #ifdef MAIN
-main(ac,av)
-	char *av[];
+main(int ac,char *av[]){
 {
-	editconf(ac,&av);
+	editconf(ac,(const char***)&av);
 }
 #endif
 
 
-askADMIN(out,in,admin,size)
-	FILE *out,*in;
-	char *admin;
-{	char *dp;
-	char yn[128];
+int askADMIN(FILE *out,FILE *in,PVStr(admin),int size)
+{	const char *dp;
+	CStr(yn,128);
+
+	if( !isatty(fileno(in)) ){
+		return -1;
+	}
 
 	for(;;){
 		fprintf(out,"ADMIN=");
 		fflush(out);
-		fgets(admin,128,in);
+		if( fgets(admin,128,in) == NULL ){
+			return -1;
+		}
 		if( dp = strpbrk(admin,"\r\n") )
-			*dp = 0;
+			truncVStr(dp);
 		for(;;){
 			fprintf(out,"ADMIN=%s ... OK ?  [y] / n / x(abort): ",
 				admin);
 			fflush(out);
-			fgets(yn,sizeof(yn),in);
+			if( fgets(yn,sizeof(yn),in) == NULL ){
+				return -1;
+			}
 			switch( yn[0] ){
 				case 'x':
 					return -1;
@@ -301,21 +308,26 @@ askADMIN(out,in,admin,size)
 	return 0;
 }
 
-static char *cached_cachedir;
-static char *cachedir_name(dir,sdir)
-	char *dir,*sdir;
+static const char *cached_cachedir;
+static void cachedir_name(PVStr(dir),PVStr(sdir))
 {
 	strcpy(dir,DELEGATE_CACHEDIR);
-	strsubstDirEnv(dir,DELEGATE_DGROOT,DELEGATE_VARDIR);
+	strsubstDirEnv(AVStr(dir),DELEGATE_DGROOT,DELEGATE_VARDIR);
 	if( !isBoundpath(dir) ){
 		strcpy(sdir,dir);
 		sprintf(dir,"%s/%s",DELEGATE_VARDIR,sdir);
-		strsubstDirEnv(dir,DELEGATE_DGROOT,DELEGATE_VARDIR);
+		strsubstDirEnv(AVStr(dir),DELEGATE_DGROOT,DELEGATE_VARDIR);
 	}
-	stripPATHexp(dir,sdir);
+	stripPATHexp(dir,AVStr(sdir));
 }
-char *cachedir()
-{	char dir[1024],sdir[1024];
+const char *cachedirX(Connection *Conn);
+const char *cachedir()
+{
+	return cachedirX(MainConn());
+}
+const char *cachedirX(Connection *Conn)
+{	CStr(dir,1024);
+	CStr(sdir,1024);
 
 	if( cached_cachedir )
 		return cached_cachedir;
@@ -323,46 +335,52 @@ char *cachedir()
 	if( without_cache() )
 		return 0;
 
-	cachedir_name(dir,sdir);
+	cachedir_name(AVStr(dir),AVStr(sdir));
 
 	if( access_RWX(sdir) != 0 )
 		return 0;
 
-	Strdup(&cached_cachedir,dir);
+	Strdup((char**)&cached_cachedir,dir);
 	return cached_cachedir;
 }
+char *getcachedir(PVStr(path),int size)
+{	const char *cdir;
 
-static isexpire(Conn,cron1,isp)
-	Connection *Conn;
-	char *cron1;
-	int *isp;
+	if( (cdir = cachedir()) == NULL )
+		return NULL;
+	if( 0 <= Readlink(cdir,AVStr(path),size) )
+		return (char*)path;
+	else	return (char*)cdir;
+}
+
+static int isexpire(DGC*Conn,PCStr(cron1),int *isp)
 {
 	if( strstr(cron1,"expire") ){
 		*isp = 1;
 		return 1;
 	}else	return 0;
 }
-static withCRONexpire(Conn)
-	Connection *Conn;
+static int withCRONexpire(DGC*Conn)
 {	int is;
 
 	if( getEnv(P_CRON) == NULL )
 		return 0;
 	is = 0;
-	DELEGATE_scanEnv(Conn,P_CRON,isexpire,&is);
+	DELEGATE_scanEnv(Conn,P_CRON,(scanPFUNCP)isexpire,&is);
 	return is;
 }
 
-YesOrNo(out,in,msg,yn)
-	FILE *out,*in;
-	char *msg,*yn;
-{	int try;
+void YesOrNo(FILE *out,FILE *in,PCStr(msg),PVStr(yn))
+{	int xtry;
 
-	yn[0] = 0;
-	for( try = 0; try < 3; try++ ){
+	setVStrEnd(yn,0);
+	for( xtry = 0; xtry < 3; xtry++ ){
 		fprintf(stderr,"%s ? [y] / n : ",msg);
 		fflush(stderr);
+		/*
 		fgets(yn,128,stdin);
+		*/
+		tty_fgets(yn,128,stdin);
 		switch( yn[0] ){
 			case '\r':
 			case '\n':
@@ -374,17 +392,19 @@ YesOrNo(out,in,msg,yn)
 	} GOT:;
 }
 
-checkCACHEDIR(Conn)
-	Connection *Conn;
-{	char dir[1024],sdir[1024],test[1024];
-	char msg[1024],*mp;
+int checkCACHEDIR(DGC*Conn)
+{	CStr(dir,1024);
+	CStr(sdir,1024);
+	CStr(test,1024);
+	CStr(msg,1024);
+	refQStr(mp,msg); /**/
 	FILE *fp;
-	char yn[128];
+	CStr(yn,128);
 
 	if( without_cache() )
 		return 0;
 
-	cachedir_name(dir,sdir);
+	cachedir_name(AVStr(dir),AVStr(sdir));
 
 	if( fileIsdir(sdir) && (access_RWX(sdir) == 0) ){
 		sprintf(test,"%s/test-%d-%d",sdir,time(0),getpid());
@@ -400,7 +420,6 @@ checkCACHEDIR(Conn)
 		return 0;
 	}
 
-	mp = msg;
 	sprintf(mp,"Cache directory ");
 	mp += strlen(mp);
 	if( fileIsdir(sdir) )
@@ -427,7 +446,7 @@ checkCACHEDIR(Conn)
 	if( !File_is(sdir) ){
 		if( isatty(fileno(stderr)) && isatty(fileno(stdin)) ){
 			fprintf(stderr,"#### %s\n",msg);
-			YesOrNo(stderr,stdin,"#### Create Cachedir",yn);
+			YesOrNo(stderr,stdin,"#### Create Cachedir",AVStr(yn));
 			if( yn[0] == 'y' )
 			if( mkdirRX(sdir) == 0 ){
 				fprintf(stderr,"#### Created: %s\n",sdir);
