@@ -1,11 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1996-1999 Yutaka Sato
-Copyright (c) 1996-1999 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1996-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, and distribute this material for any purpose
-and without fee is hereby granted, provided that the above copyright
-notice and this permission notice appear in all copies.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -34,6 +37,7 @@ FILE *linkArticle(FILE *afp,PCStr(group),int anum,PCStr(apath),PCStr(msgid),PCSt
 #define GPREFIX	""
 #define ACTIVE_DIR	"groups"
 #define HISTORY_DIR	"history"
+#define ResentMid	"X-Resent-Message-ID"
 
 #define EOM(line)	(line[0]=='.' && (line[1]=='\r'||line[1]=='\n'))
 #define EOH(line)	(line[0] == '\r' || line[0] == '\n')
@@ -59,6 +63,18 @@ static void file2group(PCStr(gname),PVStr(gpath))
 	XsetVStrEnd(AVStr(gp),0);
 }
 
+FILE *news_fopen(const char *F,int L,const char *file,const char *mode){
+	FILE *fp;
+	CStr(xfile,1024);
+
+	strcpy(xfile,file);
+	path_escchar(AVStr(xfile));
+	fp = fopen(xfile,mode);
+	return fp;
+}
+#undef fopen
+#define fopen(file,mode) news_fopen(__FILE__,__LINE__,file,mode)
+
 static FILE *fopen_active(PCStr(group),PCStr(mode1),PCStr(mode2))
 {	CStr(actpath1,LINESIZE);
 	CStr(apath,LINESIZE);
@@ -145,6 +161,75 @@ static void put_history(PCStr(msgid),PCStr(group),int anum,PCStr(artpath))
 	}
 	fclose(hfp);
 }
+/* history of only the first article */
+static void put_history1st(PCStr(msgid),PCStr(group),int anum,PCStr(artpath))
+{	FILE *hfp;
+	CStr(idline,256);
+	CStr(line,256);
+	const char *sp;
+	char sc;
+	int found;
+	refQStr(dp,idline); /**/
+	int mt0,mt1;
+
+	if( (hfp = open_history(msgid,1)) == NULL )
+		return;
+
+	found = 0;
+	for( sp = group; sc = *sp; sp++ ){
+		assertVStr(idline,dp+1);
+		if( sc == '.' )
+			setVStrPtrInc(dp,'/');
+		else	setVStrPtrInc(dp,sc);
+	}
+	sprintf(dp,"/%d %s",anum,artpath);
+	truncVStr(line);
+	if( fgets(line,sizeof(line),hfp) != NULL ){
+		if( dp = strpbrk(line,"\r\n") )
+			truncVStr(dp);
+
+		mt0 = File_mtime(artpath);
+		if( dp = strchr(line,' ') )
+			mt1 = File_mtime(dp+1);
+		else	mt1 = 0;
+		if( 0 < mt0 && 0 < mt1 ){
+			if( mt1 <= mt0 ){
+				found = 1;
+			}
+		}else
+		if( strlen(line) < strlen(idline) ){
+			found = 1;
+		}else
+		if( strcmp(line,idline) <= 0 ){
+			found = 1;
+		}
+	}
+	if( found == 0 ){
+		if( line[0] )
+			sv1log("#### replaced HISTORY [%s]<[%s]\n",idline,line);
+		else	sv1log("#### added HISTORY [%s]\n",idline);
+		fseek(hfp,0,0);
+		fprintf(hfp,"%s\r\n",idline);
+		Ftruncate(hfp,0,1);
+	}
+	fclose(hfp);
+}
+
+static FILE *fopen_article(PCStr(msgid),PCStr(group),int anum,PCStr(mode),PVStr(artpath));
+int anomidToGroup(PCStr(msgid),PVStr(rgroup));
+static FILE *open_anomid(PCStr(msgid),PCStr(mode)){
+	CStr(apath,1024);
+	CStr(group,256);
+	int anum;
+	FILE *afp;
+
+	if( anum = anomidToGroup(msgid,AVStr(group)) ){
+		afp = fopen_article(NULL,group,anum,"r",AVStr(apath));
+		return afp;
+	}
+	return NULL;
+}
+
 static int get_history(PCStr(msgid),PVStr(artpath),int all,int abspath)
 {	CStr(histpath,256);
 	FILE *hfp;
@@ -153,6 +238,21 @@ static int get_history(PCStr(msgid),PVStr(artpath),int all,int abspath)
 	CStr(vpath,256);
 	CStr(apath,256);
 	refQStr(lp,artpath); /**/
+	CStr(xmsgid,256);
+	FILE *afp;
+
+	if( anomidToGroup(msgid,VStrNULL) ){
+	  if( afp = open_anomid(msgid,"r") ){
+		fgetsHeaderField(afp,"Message-ID",AVStr(xmsgid),sizeof(xmsgid));
+		fclose(afp);
+		if( *xmsgid == 0 ){
+			return 0;
+		}
+		msgid = xmsgid;
+	  }else{
+		return 0;
+	  }
+	}
 
 	if( (hfp = open_history(msgid,0)) == NULL )
 		return 0;
@@ -185,7 +285,9 @@ int ENEWS_path(PCStr(msgid),PVStr(artpath),int is_filegroup)
 	else	return 0;
 }
 
+/*
 #define fgetsHF(f,m,b) fgetsHeaderField(f,m,AVStr(b),sizeof(b))
+*/
 
 static const char *getartfmt(PCStr(grppath),PVStr(path),PVStr(fmt)){
 	const char *dp;
@@ -206,6 +308,7 @@ static const char *getartfmt(PCStr(grppath),PVStr(path),PVStr(fmt)){
 	return NULL;
 }
 
+void putPosterHistory(PCStr(from),PCStr(group),int anum,PCStr(artpath));
 static FILE *fopen_article(PCStr(msgid),PCStr(group),int anum,PCStr(mode),PVStr(artpath))
 {	FILE *actfp,*artfp;
 	CStr(grppath,LINESIZE);
@@ -213,10 +316,13 @@ static FILE *fopen_article(PCStr(msgid),PCStr(group),int anum,PCStr(mode),PVStr(
 	int min,max;
 	CStr(msgidbuf,256);
 	CStr(actline,1024);
+	CStr(from,256);
 
 	if( msgid ){
 		if( get_history(msgid,AVStr(artpath),0,1) )
 			return fopen(artpath,"r");
+		if( artfp = open_anomid(msgid,"r") )
+			return artfp;
 		return NULL;
 	}
 
@@ -257,9 +363,17 @@ static FILE *fopen_article(PCStr(msgid),PCStr(group),int anum,PCStr(mode),PVStr(
 
 	fgetsHF(artfp,"Message-ID",msgidbuf);
 	put_history(msgidbuf,group,anum,artpath);
+	if( fgetsHF(artfp,ResentMid,msgidbuf) ){
+		put_history(msgidbuf,group,anum,artpath);
+	}
+
+	if( fgetsHF(artfp,"From",from) ){
+		putPosterHistory(from,group,anum,artpath);
+	}
 
 	return artfp;
 }
+
 FILE *ENEWS_article(PCStr(msgid),PCStr(group),int anum)
 {	CStr(artpath,LINESIZE);
 	FILE *afp;
@@ -295,6 +409,88 @@ int ENEWS_getSeqno(PCStr(head))
 	)	return atoi(seqno);
 	return 0;
 }
+int RFC822_getSeqno(FILE *afp){
+	return getSeqno(afp);
+}
+
+static void addHeader(FILE *nfp,FILE *sfp,PCStr(newhead)){
+	CStr(aline,LINESIZE);
+	int inHead = 1;
+	while( fgets(aline,sizeof(aline),sfp) ){
+		if( inHead ){
+			if( EOH(aline) ){
+				fputs(newhead,nfp);
+				inHead = 0;
+			}
+		}
+		fputs(aline,nfp);
+	}
+}
+/* Control: add-resent-message-id group num <message-id> */
+static int controlMessage(PVStr(stat),FILE *afp){
+	CStr(line,LINESIZE);
+	CStr(com,LINESIZE);
+	CStr(grp,LINESIZE);
+	int num;
+	CStr(mid,LINESIZE);
+	CStr(rmid,LINESIZE);
+	int rcode = 0;
+	int off = ftell(afp);
+	CStr(artpath,LINESIZE);
+	CStr(nartpath,LINESIZE);
+	CStr(oartpath,LINESIZE);
+	FILE *sfp;
+	FILE *nfp;
+	int mtime;
+	int an;
+
+	while( fgets(line,sizeof(line),afp) != NULL ){
+		if( strncmp(line,"Control:",8) != 0 ){
+			continue;
+		}
+		an = Xsscanf(line+8,"%s %s %d <%[^>]>",AVStr(com),
+			AVStr(grp),&num,AVStr(mid));
+		if( an != 4 ){
+			sv1log("##Control: Ignored: %s",line);
+			continue;
+		}
+		truncVStr(artpath);
+		sfp = fopen_article(NULL,grp,num,"r",AVStr(artpath));
+		sv1log("Control: %s %s %d %s [%s]%X\n",com,grp,num,mid,
+			artpath,sfp);
+		if( sfp == NULL ){
+			sv1log("##Control: not spooled: %s:%d\n",grp,num);
+			strcpy(stat,"441 control message: bad article.\r\n");
+			rcode = -1;
+			continue;
+		}
+		if( fgetsHF(sfp,ResentMid,rmid) && strstr(rmid,mid) ){
+			sv1log("##Control: already has %s: %s\n",ResentMid,mid);
+		}else{
+			CStr(newhead,LINESIZE);
+			sprintf(newhead,"%s: <%s>\r\n",ResentMid,mid);
+			mtime = file_mtime(fileno(sfp));
+			sprintf(nartpath,"%s#",artpath);
+			sprintf(oartpath,"%s~",artpath);
+			if( nfp = fopen(nartpath,"w") ){
+				addHeader(nfp,sfp,newhead);
+				fclose(nfp);
+				if( set_utimes(nartpath,0,mtime) == 0 ){
+					rename(artpath,oartpath);
+					if( rename(nartpath,artpath) == 0 )
+						unlink(oartpath);
+					else	rename(oartpath,artpath);
+				}
+			}
+		}
+		fclose(sfp);
+		put_history(mid,grp,num,artpath);
+		strcpy(stat,"240 processed control message.\r\n");
+		rcode = 1;
+	}
+	fseek(afp,off,0);
+	return rcode;
+}
 
 void copyheader(FILE *in,FILE *out,PVStr(eoh));
 int ENEWS_post(PVStr(stat),FILE *afp,PCStr(agroup),PCStr(asubj),PCStr(afrom))
@@ -319,6 +515,21 @@ int ENEWS_post(PVStr(stat),FILE *afp,PCStr(agroup),PCStr(asubj),PCStr(afrom))
 CStr(lkpath,LINESIZE);
 int lkfd;
 
+/*
+fprintf(stderr,"-------- Posting ...\n");
+int off;
+off = ftell(afp);
+RFC821_skipheader(afp,stderr,NULL);
+fseek(afp,off,0);
+*/
+
+	CStr(control,1024);
+	if( fgetsHF(afp,"Control",control) ){
+		int rcode;
+		if( rcode = controlMessage(AVStr(stat),afp) )
+			return rcode;
+	}
+
 	anum = ctime = 0;
 	UnixFrom[0] = 0;
 	fgetsHF(afp,"UNIX-From",UnixFrom);
@@ -538,12 +749,12 @@ static scanDirFunc minmax(PCStr(file),PCStr(dir),int *min,int *max)
 	}
 	return 0;
 }
+
+static int find_minmax(PCStr(dirpath),PCStr(artfmt),int *max,int *min);
 static int make_active(PCStr(group),int *max,int *min,int *cdate,int *mdate,PVStr(dir))
 {	CStr(artpath1,1024);
 	CStr(dirpath,1024);
 	CStr(artfmt,32);
-	CStr(sdir,1024);
-	int xmin,xmax;
 
 	*max = *min = -1;
 	*mdate = 0;
@@ -562,6 +773,12 @@ static int make_active(PCStr(group),int *max,int *min,int *cdate,int *mdate,PVSt
 	if( *cdate <= 0 )
 		*cdate = *mdate;
 
+	return find_minmax(dirpath,artfmt,max,min);
+}
+static int find_minmax(PCStr(dirpath),PCStr(artfmt),int *max,int *min){
+	int xmin,xmax;
+	CStr(sdir,1024);
+
 	Scandir(dirpath,scanDirCall minmax,dirpath,min,max);
 	if( artfmt[0] && 0 <= *min && 0 <= *max ){
 		xmin = xmax = -1;
@@ -580,6 +797,12 @@ static int make_active(PCStr(group),int *max,int *min,int *cdate,int *mdate,PVSt
 	if( *min < 0 ) *min = 0;
 	return 1;
 }
+int ENEWS_active(PCStr(spool),PCStr(artfmt),int *max,int *min){
+	*max = 0;
+	*min = 0;
+	return find_minmax(spool,artfmt,max,min);
+}
+
 static int addgroup(PCStr(actdir),PCStr(group),int *max,int *min,int *cdate,int *mdate,PVStr(dir))
 {	int ok;
 	FILE *afp;
@@ -756,3 +979,150 @@ void ENEWS_newnews(FILE *out,PCStr(group),int date)
 		}
 	}
 }
+
+
+/*
+ * mapping a Message-ID to the server local one
+ *   mail-lists.gggggg:nnnnn <-> <_Annnnn@gggggg.ML_>
+ */
+typedef struct {
+	const char *m_lg;
+	const char *m_sg;
+} MidMap;
+static MidMap midMap[] = {
+	{"mail-lists.", ".ML_"}, /* should be like "mail-lists.* <-> *.ML_" */
+	0
+};
+#define mmLg(x)		midMap[x].m_lg
+#define mmLgLen(x)	strlen(mmLg(x))
+#define mmSg(x)		midMap[x].m_sg
+
+extern const char *MIME_mapPosterBase;
+int strCRC32(PCStr(str),int len);
+int enBase32(PCStr(src),int sbits,PVStr(dst),int dsiz);
+char mailClientId[16];
+char mailClientAddr[32];
+char mailPosterId[16];
+
+int mappedMailAddr(PVStr(sgr),int siz,PCStr(group),int anum){
+	CStr(bgr,5);
+	const char *gp;
+	int gnum;
+
+	if( strncmp(group,mmLg(0),mmLgLen(0)) == 0 )
+		gp = group + mmLgLen(0);
+	else	gp = group;
+
+	gnum = 0xFFFF & (strCRC32(gp,strlen(gp))>>16);
+	setVStrElem(bgr,4,0xFF&(gnum>>8));
+	setVStrElem(bgr,3,(0xFF&(gnum))^(0xFF&(anum>>24)) );
+	setVStrElem(bgr,2,0xFF&(anum>>16));
+	setVStrElem(bgr,1,0xFF&(anum>>8));
+	setVStrElem(bgr,0,0xFF&(anum));
+	enBase32(bgr,5*8,AVStr(sgr),siz);
+	strtolowerX(sgr,BVStr(sgr),siz);
+	return 0;
+}
+void putPosterHistory(PCStr(from),PCStr(group),int anum,PCStr(artpath)){
+	CStr(maddr,256);
+	CStr(msgidbuf,256);
+
+	RFC822_addresspartX(from,AVStr(maddr),sizeof(maddr));
+	sprintf(msgidbuf,"%s.poster",maddr);
+	put_history1st(msgidbuf,group,anum,artpath);
+}
+int _mapPosterAddr(PCStr(addr),PVStr(xaddr)){
+	CStr(msgid,256);
+	CStr(apath,1024);
+	refQStr(ap,apath);
+	int anum;
+	CStr(sgr,16);
+	const char *ma = MIME_mapPosterBase;
+
+	sprintf(msgid,"<%s.poster>",addr);
+	if( ENEWS_path(msgid,AVStr(apath),0) ){
+		if( ap = strchr(apath,' ') ){
+			setVStrEnd(ap,0);
+		}
+		if( ap = strrchr(apath,'/') ){
+			setVStrEnd(ap,0);
+			anum = atoi(ap+1);
+			strsubst(AVStr(apath),"/",".");
+			mappedMailAddr(AVStr(sgr),sizeof(sgr),apath,anum);
+			if( streq(mailClientAddr,"127.0.0.1")
+			 || strneq(mailClientAddr,"192.168.1",9) )
+				sprintf(xaddr,"p%s.%s",sgr,ma);
+			else	sprintf(xaddr,"p%s-%s.%s",sgr,mailClientId,ma);
+			Xstrcpy(FVStr(mailPosterId),sgr);
+			return 1;
+		}
+	}
+	return 0;
+}
+int (*MIME_mapPosterAddr)(PCStr(maddr),PVStr(xmaddr)) = _mapPosterAddr;
+
+int _mapMessageId(PCStr(xref),PCStr(msgid),PVStr(xmsgid)){
+	CStr(apath,1024);
+	refQStr(ap,apath);
+	int anum;
+
+	sprintf(xmsgid,"<%s>",msgid);
+	if( strncmp(xref,mmLg(0),mmLgLen(0)) == 0 ){
+		strcpy(apath,xref);
+		if( ap = strchr(apath,':') ){
+			setVStrPtrInc(ap,0);
+			anum = atoi(ap);
+			sprintf(xmsgid,"_A%d@%s%s",anum,apath+mmLgLen(0),mmSg(0));
+			return 1;
+		}
+	}
+	if( ENEWS_path(msgid,AVStr(apath),0) ){
+		if( ap = strchr(apath,' ') ){
+			setVStrEnd(ap,0);
+		}
+		if( ap = strrchr(apath,'/') ){
+			setVStrEnd(ap,0);
+			anum = atoi(ap+1);
+			strsubst(AVStr(apath),"/",".");
+			if( strncmp(apath,mmLg(0),mmLgLen(0)) == 0 )
+				ap = apath + mmLgLen(0);
+			else	ap = apath;
+			sprintf(xmsgid,"_A%d@%s%s",anum,ap,mmSg(0));
+			return 1;
+		}
+	}
+	return 0;
+}
+int (*MIME_mapMessageId)(PCStr(xref),PCStr(msgid),PVStr(xmsgid)) = _mapMessageId;
+
+int anomidToGroup(PCStr(msgid),PVStr(rgroup)){
+	const char *mid;
+	const char *dp;
+	int anum;
+
+	mid = msgid;
+	if( *mid == '<' )
+		mid++;
+	if( dp = strstr(mid,"@-group.") ){
+		if( rgroup ){
+			truncVStr(rgroup);
+			Xsscanf(dp+8,"%[^>]",BVStr(rgroup));
+		}
+		return atoi(mid);
+	}
+	if( strneq(mid,"_A",2) && (anum = atoi(mid+2)) )
+	if( strstr(mid,mmSg(0)) ){
+		if( dp = strchr(mid,'@') ){
+			if( rgroup ){
+				strcpy(rgroup,dp+1);
+				if( dp = strstr(rgroup,mmSg(0)) )
+				if( dp[4] == 0 || dp[4] == '>' )
+					truncVStr(dp);
+				Strins(BVStr(rgroup),mmLg(0));
+			}
+			return anum;
+		}
+	}
+	return 0;
+}
+
