@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1995 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1995-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -56,6 +57,7 @@ static const char *dflt_filetypes[] = {
 	".h      :0 :TXT :text       :text/plain",
 	".doc    :0 :TXT :text       :application/octet-stream",
 	".html   :0 :HTM :text       :text/html",
+	".shtml  :0 :HTM :text       :text/html",
 	".dhtml  :0 :HTM :text       :text/html",
 	".htm    :0 :HTM :text       :text/html",
 	".xml    :0 :HTM :text       :text/xml",
@@ -63,6 +65,7 @@ static const char *dflt_filetypes[] = {
 	".ps     :0 :PS  :document   :application/postscript",
 	".ppt    :I :PPT :document   :application/vnd.ms-powerpoint",
 	".ico    :I :ICO :image      :image/x-icon",
+	".bmp    :I :BMP :image      :image/bmp",
 	".xbm    :I :XBM :image      :image/x-xbitmap",
 	".gif    :g :GIF :image      :image/gif", 
 	".png    :I :PNG :image      :image/png",
@@ -86,6 +89,8 @@ static const char *dflt_filetypes[] = {
 	"INDEX   :0 :TXT :text       :text/plain",
 	"HELP    :0 :TXT :text       :text/plain",
 	"/       :1 :DIR :directory  :text/html",
+	".conf   :0 :TXT :text       :text/plain",
+	".cnf    :0 :TXT :text       :text/plain",
 	0
 };
 
@@ -127,11 +132,12 @@ static void init_types()
 	for( fx = 0; filetype = dflt_filetypes[fx]; fx++ )
 		scan_FILETYPE(0L,filetype);
 }
+char *strrcasestr(const char*,const char*);
 static int suffix(PCStr(path),PCStr(sfx))
 {	const char *sp;
 	int len;
 
-	if( sp = strcasestr(path,sfx) ){
+	if( sp = strrcasestr(path,sfx) ){
 		len = strlen(sfx);
 		if( sp[len] == 0 ) return 1;
 		if( sp[len] == '~' && sp[len+1] == 0 ) return 1;
@@ -149,7 +155,10 @@ const char *filename2ctype(PCStr(path))
 	}
 	return 0;
 }
+/*
 int filename2gtype(PCStr(name))
+*/
+int filename2gtypeX(PCStr(name))
 {	int ci;
 	Filetype *ft;
 
@@ -158,6 +167,12 @@ int filename2gtype(PCStr(name))
 		if( suffix(name,ft->f_suffix) )
 			return ft->f_gtype;
 	}
+	return 0;
+}
+int filename2gtype(PCStr(name)){
+	int gt;
+	if( gt = filename2gtypeX(name) )
+		return gt;
 	return '9';
 }
 char *filename2icon(PCStr(path),const char **ialt)
@@ -196,3 +211,13 @@ int fileMaybeText(PCStr(path))
 			return 0;
 	return 1;
 }
+int fileSeemsBinary(PCStr(path))
+{	char gtype;
+
+	if( gtype = filename2gtypeX(path) ){
+		if( strchr("9Ig",gtype) )
+			return 1;
+		else	return -1;
+	}
+	return 0;
+}
