@@ -22,14 +22,8 @@ History:
 	940303	created
 //////////////////////////////////////////////////////////////////////#*/
 #include "delegate.h"
-#include "http.h" /* putHttpHeader */
-
-extern char *fgetsTIMEOUT();
-extern char *DDI_fgetsFromC();
-extern char *Sprintf();
-
-extern FILE *cache_fopen_rd();
-extern FILE *cache_fopen_rw();
+#include "proc.h"
+#include "file.h"
 
 #define SELSZ	2048
 #define OBUFSIZ	0x2000	/* should be smaller than 0x10000 for NEWS-OS */
@@ -38,23 +32,28 @@ extern FILE *cache_fopen_rw();
 #define IS_DIR(gtype)		(gtype=='1')
 #define IS_SEARCH(gtype)	(gtype=='7')
 
-proxyable_gtype(gtype)
+int is_redirected_selector(PCStr(sel));
+void del_DGlocalheader(Connection *Conn);
+int CTX_get_clientgtype(Connection *Conn);
+int gopher_EXPIRE(Connection *Conn,PCStr(url));
+void delegate_selector(Connection *Conn,PVStr(xselector),PCStr(host),int iport,int gtype);
+void putAncestorAnchor(Connection *Conn,FILE *dst,PCStr(proto),PCStr(host),int port,PCStr(path),int hidepass);
+
+int proxyable_gtype(int gtype)
 {
 	return !strchr("38T",gtype);
 }
 
-static is_INFO(req)
-	char *req;
-{	char *tab;
+static int is_INFO(PCStr(req))
+{	const char *tab;
 
 	if( strlen(req) )
 	if( tab = strrchr(req,'\t') )
 		return tab[1] == '!';
 	return 0;
 }
-static is_DIR(req)
-	char *req;
-{	char *dp;
+static int is_DIR(PCStr(req))
+{	const char *dp;
 
 	if( dp = strpbrk(req,"\t\r\n") )
 		if( dp[-1] == '/' )
@@ -62,20 +61,18 @@ static is_DIR(req)
 	return 0;
 }
 
+int decomp_dirent(PVStr(line),PVStr(attr),const char **uname,const char **selector,const char **host,int *iport,const char **ext);
+
 /*
  *	Put Directory Entity redirecting it to me.
  */
-static gopher_fputs(Conn,rgtype,request,line,fp)
-	Connection *Conn;
-	char *request;
-	char *line;
-	FILE *fp;
-{	char bline[2048];
-	char iline[2048];
+static int gopher_fputs(Connection *Conn,int rgtype,PCStr(request),xPVStr(line),FILE *fp)
+{	CStr(bline,2048);
+	CStr(iline,2048);
 	char egtype;
 
 	strcpy(bline,line);
-	line = bline;
+	setPStr(line,bline,sizeof(bline));
 	egtype = *line;
 
 	if( BORN_SPECIALIST || ACT_SPECIALIST )
@@ -83,82 +80,72 @@ static gopher_fputs(Conn,rgtype,request,line,fp)
 	if( egtype != '.' && egtype != '-' && egtype != ' '
 	 && egtype != '8' && egtype != 'T'
 	){
-		char xline[2048];
-		char attr[256];
-		char *Uname;
-		char *selector;
-		char *host;
+		CStr(xline,2048);
+		CStr(attr,256);
+		const char *Uname;
+		const char *selector;
+		const char *host;
 		int   iport;
-		char *ext;
-		char xselector[SELSZ];
-		char myhost[SELSZ];
+		const char *ext;
+		CStr(xselector,SELSZ);
+		CStr(myhost,SELSZ);
 		int  myport;
 
 		strcpy(xline,line);
-		egtype = decomp_dirent(xline,attr,
+		egtype = decomp_dirent(AVStr(xline),AVStr(attr),
 			&Uname,&selector,&host,&iport,&ext);
 		if( egtype == 0 )
 			goto PUT;
 
 		strcpy(xselector,selector);
 		if( proxyable_gtype(egtype) )
-			delegate_selector(xselector,host,iport,egtype);
+			delegate_selector(Conn,AVStr(xselector),host,iport,egtype);
 
 		if( ACT_SPECIALIST && !META_SPECIALIST ){
 			strcpy(myhost,DELEGATE_LHOST);
 			myport = DELEGATE_LPORT;
-		}else	myport = ClientIF_H(Conn,myhost);
+		}else	myport = ClientIF_H(Conn,AVStr(myhost));
 
 		sprintf(line,"%s%c%s\t%s\t%s\t%d%s",
 			attr,egtype,Uname,xselector,myhost,myport,ext);
 	}
 PUT:
 	if( BORN_SPECIALIST || ACT_SPECIALIST || ACT_TRANSLATOR )
-	if( cur_codeconvCL(NULL) ){
-		line_codeconv(line,iline,"x-gopher/dirent");
-		line = iline;
+	if( CTX_cur_codeconvCL(Conn,VStrNULL) ){
+		CTX_line_codeconv(Conn,line,AVStr(iline),"x-gopher/dirent");
+		setPStr(line,iline,sizeof(iline));
 	}
 	return fputs(line,fp);
 }
 
-gopher_filter(ac,av,Conn)
-	char *av[];
-	Connection *Conn;
-{	char line[2048];
-
-	BORN_SPECIALIST = 1;
-	while( fgetsTIMEOUT(line,sizeof(line),stdin) )
-		gopher_fputs(Conn,'1',"1",line,stdout);
-}
-
-static Modified(Conn,cdate,req)
-	Connection *Conn;
-	char *req;
+static int Modified(Connection *Conn,int cdate,PCStr(req))
 {	static Connection *MConn;
-	char sel[SELSZ];
+	CStr(sel,SELSZ);
 	FILE *fs;
-	char line[1024],mdate[256],scdate[256];
+	CStr(line,1024);
+	CStr(mdate,256);
+	CStr(scdate,256);
 	int modified;
 
 	if( MConn == 0 )
-		MConn = (Connection*)calloc(sizeof(Connection),1);
+		MConn = NewStruct(Connection);
 	*MConn = *Conn;
 	Conn = MConn;
 	if( connect_to_serv(Conn,FromC,ToC,0) < 0 )
 		return 0;
 
-	sscanf(req,"%[^\t\r\n]",sel);
-	sprintf(sel+strlen(sel),"\t!\r\n\r\n");
+	Xsscanf(req,"%[^\t\r\n]",AVStr(sel));
+	Xsprintf(TVStr(sel),"\t!\r\n\r\n");
 	write(ToS,sel,strlen(sel));
 
 	fs = fdopen(FromS,"r");
 
-	StrftimeGMT(scdate,sizeof(scdate),TIMEFORM_GOPHER,cdate);
+	StrftimeGMT(AVStr(scdate),sizeof(scdate),TIMEFORM_GOPHER,cdate,0);
 	modified = 1;
-	while( fgetsTIMEOUT(line,sizeof(line),fs) != NULL ){
+	while( fgetsTIMEOUT(AVStr(line),sizeof(line),fs) != NULL ){
 		if( strncasecmp(" Mod-Date:",line,9) == 0 ){
 			sv1log("Mod-Date: %s\n",line);
-			if( sscanf(line,"%*[^<]<%[^>]",mdate) ){
+			if( Xsscanf(line,"%*[^<]<%[^>]",AVStr(mdate)) ){
 				modified = (0 < strcmp(mdate,scdate));
 				sv1log("Mod-Date: %s > %s = %d\n",
 					mdate,scdate,modified);
@@ -176,19 +163,16 @@ static Modified(Conn,cdate,req)
 	return modified;
 }
 
-static readout_cache(Conn,req,gtype,cachefp,tc)
-	Connection *Conn;
-	char *req;
-	FILE *cachefp,*tc;
-{	char line[2048];
+static int readout_cache(Connection *Conn,PCStr(req),int gtype,FILE *cachefp,FILE *tc)
+{	CStr(line,2048);
 	int totalc = 0;
 
 	rewind(cachefp);
 	if( IS_DIR(gtype) || is_DIR(req) || IS_SEARCH(gtype) || is_INFO(req) ){
-		check_codeconvSP(Conn,1);
-		while( fgetsTIMEOUT(line,sizeof(line),cachefp) ){
+		CTX_check_codeconvSP(Conn,1);
+		while( fgetsTIMEOUT(AVStr(line),sizeof(line),cachefp) ){
 			totalc += strlen(line);
-			gopher_fputs(Conn,gtype,req,line,tc);
+			gopher_fputs(Conn,gtype,req,AVStr(line),tc);
 		}
 	}else
 	if( IS_TEXT(gtype) )
@@ -198,23 +182,21 @@ static readout_cache(Conn,req,gtype,cachefp,tc)
 	return totalc;
 }
 
-static fgets_selector(Conn,str,size,fp)
-	Connection *Conn;
-	char *str;
-	FILE *fp;
-{	char gtype,cgtype;
-	char rproto[64];
-	char rserver[256];
+static int fgets_selector(Connection *Conn,PVStr(str),int size,FILE *fp)
+{	char gtype;
+	char cgtype;
+	CStr(rproto,64);
+	CStr(rserver,MaxHostNameLen);
 	int riport;
 
-	if( DDI_fgetsFromC(Conn,str,size,fp) == NULL )
+	if( DDI_fgetsFromC(Conn,AVStr(str),size,fp) == NULL )
 		return 0;
 
 	sv1log("GOPHER SELECTOR: %s",str);
 
 	if( *str == 0 || strchr(" \t\r\n",*str) != 0 )
 	if( D_SELECTOR[0] ){
-		char tmp[128];
+		CStr(tmp,128);
 
 		strcpy(tmp,str);
 		sprintf(str,"%s%s",D_SELECTOR,tmp);
@@ -223,7 +205,7 @@ static fgets_selector(Conn,str,size,fp)
 
 /*	if( BORN_SPECIALIST ) */
 	if( BORN_SPECIALIST || ACT_SPECIALIST )
-	if( CTX_url_derefer(Conn,"gopher",str,NULL,DELEGATE_FLAGS,rproto,rserver,&riport) ){
+	if( CTX_url_derefer(Conn,"gopher",AVStr(str),VStrNULL,AVStr(DELEGATE_FLAGS),AVStr(rproto),AVStr(rserver),&riport) ){
 
 /* if rproto != "gopher" then it's ERROR because gopher client cannot
  * receive response from non-gopher server */
@@ -241,7 +223,7 @@ DFLT_PORT = riport;
 			DELEGATE_FLAGS,rproto,rserver,riport,str);
 	}
 
-	gtype = get_gtype(str,str);
+	gtype = get_gtype(str,AVStr(str));
 	if( cgtype = CTX_get_clientgtype(Conn) ) /* sent from client */
 		gtype = cgtype;
 	else	CTX_set_clientgtype(Conn,gtype);
@@ -249,28 +231,27 @@ DFLT_PORT = riport;
 	return gtype;
 }
 
-static cache_Path(host,iport,gtype,req,path)
-	char *host,*req,*path;
-{	char selector[2048];
-	char *cache_dir;
-	char cachefile[2048];
-	char ext[256];
-	char *rp;
-	char *dp;
-	char *cp;
+static int cache_Path(PCStr(host),int iport,int gtype,PCStr(req),PVStr(path))
+{	CStr(selector,2048);
+	const char *cache_dir;
+	CStr(cachefile,2048);
+	CStr(ext,256);
+	const char *rp;
+	const char *dp;
+	refQStr(cp,cachefile); /**/
 
-	*path = 0;
+	setVStrEnd(path,0);
 	strcpy(selector,"?");
 
 	if( (cache_dir = cachedir()) == 0 )
 		return 0;
 
-	cp = Sprintf(cachefile,"%s/%s/%s",cache_dir,"gopher",host);
+	cp = Sprintf(AVStr(cachefile),"%s/%s/%s",cache_dir,"gopher",host);
 	if( iport != serviceport("gopher") )
-		cp = Sprintf(cp,":%d",iport);
+		cp = Sprintf(AVStr(cp),":%d",iport);
 
 	selector[0] = ext[0] = 0;
-	sscanf(req,"%*c/%[^\r\n\t]\t%[^\r\n\t]",selector,ext);
+	Xsscanf(req,"%*c/%[^\r\n\t]\t%[^\r\n\t]",AVStr(selector),AVStr(ext));
 
 	if( selector[0] == 0 ){
 		sv1log("empty selector, request = %s",req);
@@ -285,12 +266,12 @@ static cache_Path(host,iport,gtype,req,path)
 
 	if( IS_DIR(gtype) ){
 		if( selector[0] )
-			cp = Sprintf(cp,"/%s/",selector);
-		else	cp = Sprintf(cp,"/");
+			cp = Sprintf(AVStr(cp),"/%s/",selector);
+		else	cp = Sprintf(AVStr(cp),"/");
 
 		if( dp = strchr(req,'\t') ){
 			strcpy(ext,"?");
-			sscanf(dp+1,"%[^\r\n\t ]",ext);
+			Xsscanf(dp+1,"%[^\r\n\t ]",AVStr(ext));
 			strcat(cp,ext);
 		}else	strcat(cp,"=");
 
@@ -298,26 +279,27 @@ static cache_Path(host,iport,gtype,req,path)
 		return 1;
 	}
 	if( gtype && strchr(GOPHER_CACHE_ITEM,gtype) ){
-		Sprintf(cp,"/%s%s",selector,ext);
+		Sprintf(AVStr(cp),"/%s%s",selector,ext);
 		strcpy(path,cachefile);
 		return 1;
 	}
 	return 0;
 }
 
-static relay_response();
-service_gopher(Conn)
-	Connection *Conn;
+static int relay_response(Connection *Conn,PCStr(req),int gtype,FILE *fs,FILE *tc,FILE *cachefp);
+
+int service_gopher(Connection *Conn)
 {	FILE *fc,*tc,*fs;
 	FILE *cachefp;
-	char gtype,req[2048];
-	char line[0x10000];
-	char o_buff[OBUFSIZ];
-	char c_buff[OBUFSIZ];
-	char *server = DFLT_HOST;
+	char gtype;
+	CStr(req,2048);
+	CStr(line,0x10000);
+	CStr(o_buff,OBUFSIZ);
+	CStr(c_buff,OBUFSIZ);
+	const char *server = DFLT_HOST;
 	int iport = DFLT_PORT;
 	int useCache;
-	char cpath[2048];
+	CStr(cpath,2048);
 	int cdate;
 	int totalc;
 
@@ -331,29 +313,29 @@ service_gopher(Conn)
 		Exit(-1,"fdopen(ToC=%d) failed\n",ToC);
 	setbuffer(tc,o_buff,sizeof(o_buff));
 
-	if( (gtype = fgets_selector(Conn,req,sizeof(req),fc)) == 0 ){
+	if( (gtype = fgets_selector(Conn,AVStr(req),sizeof(req),fc)) == 0 ){
 		sv1log("GOPHER null REQUEST ?\n");
-		return;
+		return -1;
 	}
 
-	if( !service_permitted(Conn,"gopher",FromC) ){
-		char myhp[256];
+	if( !service_permitted(Conn,"gopher" /*,FromC*/) ){
+		CStr(myhp,MaxHostNameLen);
 
-		ClientIF_HP(Conn,myhp);
+		ClientIF_HP(Conn,AVStr(myhp));
 		fprintf(tc,"--1\r\n");
 		fprintf(tc,"1 Forbidden by DeleGate on `%s'.\r\n",myhp);
 		fprintf(tc,"0 Proxy Gopher Server %s\r\n",DELEGATE_version());
 		fprintf(tc,".\r\n");
 		fcloseLinger(tc);
 		fclose(fc);
-		return;
+		return -1;
 	}
 
 cpath[0] = 0;
 	setupConnect(Conn);
 	if( without_cache() )
 		useCache = 0;
-	else	useCache = cache_Path(server,iport,gtype,req,cpath);
+	else	useCache = cache_Path(server,iport,gtype,req,AVStr(cpath));
 	cachefp = NULL;
 	cdate = -1;
 
@@ -374,14 +356,14 @@ cpath[0] = 0;
 			goto give_up;
 
 		if( !DontReadCache )
-		if( cachefp = cache_fopen_rd("GOPHER",cpath,expire,&cdate) ){
+		if( cachefp = cache_fopen_rd("GOPHER",AVStr(cpath),expire,&cdate) ){
 			if( lock_for_rd("GOPHER",cretry,cpath,cachefp) != 0 ){
 				sleep(CACHE_RDRETRY_INTERVAL);
 				goto retry_read;
 			}
 
 			sv1log("GOPHER <= [%s] %s %d %s",cpath, server,iport,req);
-			fgetsTIMEOUT(line,sizeof(line),cachefp);
+			fgetsTIMEOUT(AVStr(line),sizeof(line),cachefp);
 			if(line[0]=='-' && line[1]=='-'){
 				sv1log("GOPHER <= ??? %s",line);
 				cache_delete(cpath);
@@ -392,9 +374,9 @@ cpath[0] = 0;
 
 			fflush(tc); set_linger(fileno(tc),DELEGATE_LINGER);
 			sv1log("GOPHER %d bytes of cached data transfered\n",totalc);
-			return;
+			return 0;
 		}
-		if( cachefp = cache_fopen_rw("GOPHER",cpath) ){
+		if( cachefp = cache_fopen_rw("GOPHER",AVStr(cpath)) ){
 			if( file_lock_wr("GOPHER",cachefp) != 0 ){
 				if( !DontReadCache && !dontWaitCache ){
 					sleep(CACHE_WRRETRY_INTERVAL);
@@ -433,7 +415,7 @@ cpath[0] = 0;
 			fprintf(tc,"DeleGate: connection to %s:%d failed.\n",
 				DFLT_HOST,DFLT_PORT);
 			fflush(tc);
-			return;
+			return -1;
 		}
 	}
 
@@ -447,7 +429,7 @@ cpath[0] = 0;
 	fs = fdopen(FromS,"r");
 	if( fs == NULL ){
 		sv1log("Error fdopen(%d) failed\n",FromS);
-		return;
+		return -1;
 	}
 
 /* Send selector message if the server is a real server, otherwise
@@ -470,29 +452,27 @@ cpath[0] = 0;
 		}
 		fclose(cachefp); /* unlock Exclusive lock */
 	}
+	return 0;
 }
 
-static relay_response(Conn,req,gtype,fs,tc,cachefp)
-	Connection *Conn;
-	char *req;
-	FILE *fs,*tc,*cachefp;
+static int relay_response(Connection *Conn,PCStr(req),int gtype,FILE *fs,FILE *tc,FILE *cachefp)
 {	int error,totalc,rcc;
-	char head[32];
-	char line[2048];
+	CStr(head,32);
+	CStr(line,2048);
 
 	error = 0;
 	totalc = 0;
 	if( IS_DIR(gtype) || is_DIR(req) || IS_SEARCH(gtype) || is_INFO(req) ){
-		check_codeconvSP(Conn,1);
+		CTX_check_codeconvSP(Conn,1);
 		for(;;){
 			line[0] = 0;
-			if( fgetsTIMEOUT(line,sizeof(line),fs) == NULL ){
+			if( fgetsTIMEOUT(AVStr(line),sizeof(line),fs) == NULL ){
 				error = 1;
 				sv1log("GOPHER << premature EOF\n");
 
 				/* Gopher+ */
 				if( cachefp )
-					gopher_fputs(Conn,gtype,req,"--2\r\n",tc);
+					gopher_fputs(Conn,gtype,req,CVStr("--2\r\n"),tc);
 				break;
 			}
 			if( line[0] == '-' ){
@@ -502,7 +482,7 @@ static relay_response(Conn,req,gtype,fs,tc,cachefp)
 			}
 
 			totalc += strlen(line);
-			gopher_fputs(Conn,gtype,req,line,tc);
+			gopher_fputs(Conn,gtype,req,AVStr(line),tc);
 			fflush(tc);
 			if( cachefp )
 				fputs(line,cachefp);
@@ -517,7 +497,7 @@ static relay_response(Conn,req,gtype,fs,tc,cachefp)
 		if( (totalc = CCV_relay_text(Conn,fs,tc,cachefp)) < 0 )
 			error = 1;
 	}else{
-		rcc = freadTIMEOUT(head,1,sizeof(head),fs);
+		rcc = freadTIMEOUT(AVStr(head),1,sizeof(head),fs);
 		if( rcc == 0 )
 			error = 1;
 		else{
@@ -540,18 +520,15 @@ static relay_response(Conn,req,gtype,fs,tc,cachefp)
 	return totalc;
 }
 
-decomp_dirent(line,attr,uname,selector,host,iport,ext)
-	char *line,*attr;
-	int *iport;
-	char **uname,**selector,**host,**ext;
-{	char *dp;
+int decomp_dirent(PVStr(line),PVStr(attr),const char **uname,const char **selector,const char **host,int *iport,const char **ext)
+{	const char *dp;
 	int egtype;
 
-	attr[0] = 0;
+	setVStrEnd(attr,0);
 	egtype = *line;
 	if( egtype == '+' ){
 		if( dp = strchr(line,':') ){
-			*dp++ = 0;
+			truncVStr(dp); dp++;
 			strcpy(attr,line);
 			strcat(attr,":");
 			if( *dp == ' ' ){
@@ -564,19 +541,18 @@ decomp_dirent(line,attr,uname,selector,host,iport,ext)
 	}
 	dp = line + 1;
 	*uname = dp;
-	if(dp=strchr(dp,'\t')){ *dp++ = 0; *selector = dp; }else return 0;
-	if(dp=strchr(dp,'\t')){ *dp++ = 0; *host = dp;     }else return 0;
-	if(dp=strchr(dp,'\t')){ *dp++ = 0; *iport=atoi(dp);}else return 0;
+	if(dp=strchr(dp,'\t')){ truncVStr(dp);dp++; *selector = dp; }else return 0;
+	if(dp=strchr(dp,'\t')){ truncVStr(dp);dp++; *host = dp;     }else return 0;
+	if(dp=strchr(dp,'\t')){ truncVStr(dp);dp++; *iport=atoi(dp);}else return 0;
 	if(dp=strchr(dp,'\t')){ *ext = dp;               }
 			else  { *ext = "\r\n";	         }
 
 	return egtype;
 }
 
-static isGopherPlus(host)
-	char *host;
-{	char path[1024];
-	char *cache_dir;
+static int isGopherPlus(PCStr(host))
+{	CStr(path,1024);
+	const char *cache_dir;
 
 	cache_dir = cachedir();
 	sprintf(path,"%s/%s/+",cache_dir,"gopher",host);
@@ -588,18 +564,24 @@ static isGopherPlus(host)
 	return 0;
 }
 
-static dirent2html(Conn,fs,tc,iconbase)
-	Connection *Conn;
-	FILE *fs,*tc;
-	char *iconbase;
-{	char dirent[2048],cdirent[2048];
-	char html[2048];
-	char attr[256],gtype,*uname,*selector,*host,*ext;
-	char hostport[256],xsel[SELSZ];
-	char *space;
+static int dirent2html(Connection *Conn,FILE *fs,FILE *tc,PCStr(iconbase))
+{	CStr(dirent,2048);
+	CStr(cdirent,2048);
+	CStr(html,2048);
+	CStr(attr,256);
+	char gtype;
+	const char *uname;
+	const char *selector;
+	const char *host;
+	const char *ext;
+	CStr(hostport,MaxHostNameLen);
+	CStr(xsel,SELSZ);
+	const char *space;
 	int iport;
-	char *hp;
-	char url[SELSZ],*img,*alt;
+	refQStr(hp,html); /**/
+	CStr(url,SELSZ);
+	const char *img;
+	const char *alt;
 	int lines,gplus,error;
 	int do_conv;
 	int totalc = 0;
@@ -607,14 +589,14 @@ static dirent2html(Conn,fs,tc,iconbase)
 	gplus = 0;
 	lines = 0;
 	error = 0;
-	do_conv = check_codeconvSP(Conn,1);
+	do_conv = CTX_check_codeconvSP(Conn,1);
 
 	/* Avoid X-Mosaic bug (?) which specially treat '?' character
 	 * in within first 2 bytes (?)
 	 */
 	space = "- ";
 
-	while( fgetsTIMEOUT(dirent,sizeof(dirent),fs) != NULL ){
+	while( fgetsTIMEOUT(AVStr(dirent),sizeof(dirent),fs) != NULL ){
 		totalc += strlen(dirent);
 		if( streq(dirent,".\r\n") )
 			break;
@@ -623,11 +605,11 @@ static dirent2html(Conn,fs,tc,iconbase)
 		lines++;
 
 		if( do_conv ){
-			line_codeconv(dirent,cdirent,"text/html");
+			CTX_line_codeconv(Conn,dirent,AVStr(cdirent),"text/html");
 			strcpy(dirent,cdirent);
 		}
 
-		gtype = decomp_dirent(dirent,attr,
+		gtype = decomp_dirent(AVStr(dirent),AVStr(attr),
 			&uname,&selector,&host,&iport,&ext);
 
 		if( gtype == 0 ){
@@ -656,7 +638,7 @@ static dirent2html(Conn,fs,tc,iconbase)
 		    default:  alt="[???]"; img="unknown.gif";	break;
 		}
 
-		nonxalpha_escape(selector,xsel);
+		nonxalpha_escapeX(selector,AVStr(xsel),sizeof(xsel));
 		if( iport == 0 || iport == serviceport("gopher") )
 			sprintf(hostport,"%s",host);
 		else	sprintf(hostport,"%s:%d",host,iport);
@@ -668,15 +650,15 @@ static dirent2html(Conn,fs,tc,iconbase)
 				sprintf(url,"http://%s/%s",hostport,xsel);
 			else{
 			sprintf(url,"gopher://%s/%c%s",hostport,gtype,xsel);
-			redirect_url(Conn,url,url);
+			redirect_url(Conn,url,AVStr(url));
 			}
 		}
 
 		hp = html;
-		hp = Sprintf(hp,"<IMG ALT=\"%s\" SRC=\"%s%s\" ALIGN=TOP>",alt,
+		hp = Sprintf(AVStr(hp),"<IMG ALT=\"%s\" SRC=\"%s%s\" ALIGN=TOP>",alt,
 			iconbase,img);
-		hp = Sprintf(hp,"<A HREF=\"%s\">",url);
-		hp = Sprintf(hp,"%s%s</A><BR>\n",space,uname);
+		hp = Sprintf(AVStr(hp),"<A HREF=\"%s\">",url);
+		hp = Sprintf(AVStr(hp),"%s%s</A><BR>\n",space,uname);
 		fputs(html,tc);
 	}
 	fprintf(tc,"<HR>\n");
@@ -689,21 +671,19 @@ static dirent2html(Conn,fs,tc,iconbase)
 	return totalc;
 }
 
-HttpGopher(Conn,vno,svsock,server,iport,gtype,path)
-	Connection *Conn;
-	char *server,*path;
+int HttpGopher(Connection *Conn,int vno,int svsock,PCStr(server),int iport,int gtype,PCStr(path))
 {	int msock;
 	FILE *tc,*fs;
-	char *ctype;
-	char xpath[2048];
-	char iconbase[1024];
+	const char *ctype;
+	CStr(xpath,2048);
+	CStr(iconbase,1024);
 	int totalc;
-	char request[2048];
-	char dserv[256];
+	CStr(request,2048);
+	CStr(dserv,256);
 	int toHTML;
 
 	del_DGlocalheader(Conn); /* expects raw/transparent gopher server */
-	nonxalpha_unescape(path,xpath,1);
+	nonxalpha_unescape(path,AVStr(xpath),1);
 	path = xpath;
 	sprintf(request,"%s\r\n",path);
 
@@ -711,7 +691,8 @@ HttpGopher(Conn,vno,svsock,server,iport,gtype,path)
 
 	sprintf(dserv,"Gopher/HTTP gateway ETL-DeleGate/%s",DELEGATE_ver());
 	if( gtype == '7' ){
-		char req[2048],*dp;
+		CStr(req,2048);
+		const char *dp; /* not "const" but fixed length */
 
 		if( strchr(path,'?') == 0 ){
 			sv1log("GENERATED Searchable Gopher Index\n");
@@ -724,7 +705,7 @@ HttpGopher(Conn,vno,svsock,server,iport,gtype,path)
 		}
 		strcpy(req,path);
 		if( dp = strchr(req,'?') )
-			*dp = '\t';
+			*(char*)dp = '\t';
 		sprintf(request,"%s\r\n",req);
 	}
 	if( streq(request,"/\r\n") )
@@ -768,16 +749,16 @@ HttpGopher(Conn,vno,svsock,server,iport,gtype,path)
 
 	if( DELEGATE_IMAGEDIR )
 		strcpy(iconbase,DELEGATE_IMAGEDIR);
-	else	getCERNiconBase(Conn,iconbase);
+	else	getCERNiconBase(Conn,AVStr(iconbase));
 
 	if( gtype == '0' ){
 		totalc += CCV_relay_text(Conn,fs,tc,NULL);
 	}else
 	if( gtype == '1' || gtype == '7' ){
-		char hostport[256];
-		char gpath[2048];
+		CStr(hostport,MaxHostNameLen);
+		CStr(gpath,2048);
 
-		HostPort(hostport,"gopher",server,iport);
+		HostPort(AVStr(hostport),"gopher",server,iport);
 		fprintf(tc,"<I> GopherMenu </I> ");
 		fprintf(tc,"<B>\n");
 /*
@@ -801,7 +782,7 @@ fprintf(tc,"gopher://%s/%c%s\n",hostport,gtype,path);
 	if( toHTML )
 		fputs("</PRE>",tc);
 	fcloseLinger(tc);
-	tc == NULL;
+	tc = NULL;
 	Verbose("GOPHER: %d bytes put.\n",totalc);
 
 	fclose(fs);
