@@ -1,11 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1998 Electrotechnical Laboratry (ETL), AIST, MITI
-Copyright (c) 1998 Yutaka Sato
-
-Permission to use, copy, and distribute this material for any purpose
-and without fee is hereby granted, provided that the above copyright
-notice and this permission notice appear in all copies.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Copyright (c) 1998-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
+        
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 //////////////////////////////////////////////////////////////////////////
@@ -287,7 +290,7 @@ void scan_ICP(DGC*Conn,PCStr(conf));
 
 void icp_server(int sock)
 {	ICPpack qpack,rpack;
-	CStr(froma,32);
+	CStr(froma,64);
 	int fromp;
 	int rcc;
 	const char *url;
@@ -455,7 +458,7 @@ static void icp_reply(int sock,ICPpack *qpack,ICPpack *rpack,PCStr(froma),int fr
 		int pxtypes[32];
 		double timeout;
 		int sx;
-		CStr(sxaddr,32);
+		CStr(sxaddr,64);
 		int sxport;
 		int *hitobjp;
 
@@ -575,7 +578,7 @@ int icp_client(int ac,const char *av[])
 	float fv;
 	FILE *log;
 	int sx;
-	CStr(sxhost,32);
+	CStr(sxhost,64);
 	int sxport;
 	int hitobj;
 	CStr(objbuf,MAX_PACKSIZE);
@@ -704,7 +707,7 @@ int icp_select(PCStr(url),int icpopts[],const char *svaddrs[],int svports[],doub
 	CStr(msg,128+MAX_PACKSIZE);
 	int qoptions;
 	int opcode,options,optdata,sender,hsize,rsize;
-	CStr(froma,32);
+	CStr(froma,64);
 	int fromp;
 	int hit = -1;
 	int miss = -1;
@@ -839,7 +842,7 @@ int icp_select(PCStr(url),int icpopts[],const char *svaddrs[],int svports[],doub
 			hop = (unsigned char *)(url + strlen(url) + 1);
 			rsize = (hop[0]<<24)|(hop[1]<<16)|(hop[2]<<8)|(hop[3]);
 			hop += 4;
-			hsize = (int)hop - (int)&rpack;
+			hsize = (char*)hop - (char*)&rpack;
 
 			Lprintf(log,"(ICP)HIT_OBJ %d + %d = %d / %d\n",
 				hsize,rsize,hsize+rsize,rcc);
