@@ -16,13 +16,13 @@ Description:
 History:
 	000301	extracted from access.c
 //////////////////////////////////////////////////////////////////////#*/
+#include "ystring.h"
 #include "hostlist.h"
 #include "delegate.h"
-#include "ystring.h"
+#include "fpoll.h"
 #include <ctype.h>
 #include <errno.h>
 
-extern HostList *IdentHosts();
 extern double IDENT_TIMEOUT;
 
 #define IDENT_NOTYET	 0
@@ -33,10 +33,12 @@ extern double IDENT_TIMEOUT;
 #define IDENT_EOF	-4
 #define IDENT_BAD	-5
 
-static getIdent(timeout,idsv,lport,rport,ident)
-	AuthInfo *ident;
+static int getIdent(int timeout,int idsv,int lport,int rport,AuthInfo *ident)
 {	FILE *ti,*fi;
-	char resp[512],iresp[512],syst[512],user[512],*dp;
+	CStr(resp,512);
+	CStr(iresp,512);
+	CStr(syst,512);
+	CStr(user,512);
 	int rcode = -1;
 
 	ident->i_user[0] = 0;
@@ -53,19 +55,18 @@ static getIdent(timeout,idsv,lport,rport,ident)
 		Verbose("-- ident: server response TIMEOUT (%ds).\n",	
 			timeout);
 	}else
-	if( fgetsTimeout(resp,sizeof(resp),fi,timeout) == NULL ){
+	if( fgetsTimeout(AVStr(resp),sizeof(resp),fi,timeout) == NULL ){
 		ident->i_stat = IDENT_EOF;
 		Verbose("-- ident: EOF from the server.\n");
 	}else{
 		lineScan(resp,iresp);
 		user[0] = 0;
-		sscanf(iresp,"%*d%*[ ,]%*d%*[ :]USERID%*[ :]%[^ :]%*[ :]%s\n",
-			syst,user);
+		Xsscanf(iresp,"%*d%*[ ,]%*d%*[ :]USERID%*[ :]%[^ :]%*[ :]%s\n",AVStr(syst),AVStr(user));
 		if( user[0] ){
 			Verbose("-- ident: %s : %s\n",syst,user);
 			ident->i_stat = IDENT_GOT;
-			lineScan(syst,ident->i_syst);
-			lineScan(user,ident->i_user);
+			linescanX(syst,AVStr(ident->i_syst),sizeof(ident->i_syst));
+			linescanX(user,AVStr(ident->i_user),sizeof(ident->i_user));
 			rcode = 0;
 		}else{
 			ident->i_stat = IDENT_BAD;
@@ -76,19 +77,16 @@ static getIdent(timeout,idsv,lport,rport,ident)
 	fclose(fi);
 	return rcode;
 }
-static char *getClientIdent0(Conn,host,port,ident)
-	Connection *Conn;
-	char *host;
-	AuthInfo *ident;
+static const char *getClientIdent0(Connection *Conn,PCStr(host),int port,AuthInfo *ident)
 {	int idsv;
 	int timeout;
 	int myport;
-	char myaddr[64];
+	CStr(myaddr,64);
 
-	VA_inetNtoah(&Conn->cl_sockHOST,myaddr);
+	VA_inetNtoah(&Conn->cl_sockHOST,AVStr(myaddr));
 	myport = Conn->cl_sockHOST.a_port;
 	Verbose("-- ident: %s [%s:%d]\n",host,myaddr,myport);
-	idsv = Socket1("IDENT",-1,NULL,NULL,NULL, myaddr,0,NULL,0, 0,NULL,0);
+	idsv = Socket1("IDENT",-1,NULL,NULL,NULL, AVStr(myaddr),0,NULL,0, 0,NULL,0);
 
 	Verbose("-- ident: %d\n",idsv);
 	if( idsv < 0 ){
@@ -96,7 +94,7 @@ static char *getClientIdent0(Conn,host,port,ident)
 		return NULL;
 	}
 
-	timeout = IDENT_TIMEOUT * 1000;
+	timeout = (int)(IDENT_TIMEOUT * 1000);
 	if( IDENT_TIMEOUT != 0 && timeout == 0 )
 		timeout = 1;
 
@@ -112,10 +110,8 @@ static char *getClientIdent0(Conn,host,port,ident)
 	return NULL;
 }
 
-static char *getClientIdentH0(Conn,host,port,force,ident)
-	Connection *Conn;
-	char *host;
-	AuthInfo *ident;
+HostList *IdentHosts();
+static const char *getClientIdentH0(Connection *Conn,PCStr(host),int port,int force,AuthInfo *ident)
 {	int isin;
 
 	if( ident == NULL )
@@ -138,40 +134,39 @@ EXIT:
 		return ident->i_user;
 	else	return NULL;
 }
-char *getClientUser(Conn)
-	Connection *Conn;
+const char *getClientUser(Connection *Conn)
 {
 	if( VA_getClientAddr(Conn) )
 		return getClientIdentH0(Conn,Client_Host,Client_Port,1,NULL);
 	return NULL;
 }
-char *getClientUserC(Conn)
-	Connection *Conn;
+const char *getClientUserC(Connection *Conn)
 {
 	if( VA_getClientAddr(Conn) )
 		return getClientIdentH0(Conn,Client_Host,Client_Port,0,NULL);
 	return NULL;
 }
-char *getClientUserMbox(Conn,mbox)
-	Connection *Conn;
-	char *mbox;
+const AuthInfo *getClientIdentX(Connection *Conn)
+{
+	if( getClientUserC(Conn) )
+		return &Client_Ident;
+	return NULL;
+}
+char *getClientUserMbox(Connection *Conn,PVStr(mbox))
 {	AuthInfo ident;
-	char host[256];
+	CStr(host,MaxHostNameLen);
 
 	strcpy(mbox,"?");
 	if( VA_getClientAddr(Conn) ){
 		if( getClientIdent0(Conn,Client_Host,Client_Port,&ident) ){
-			getFQDN(Client_Host,host);
+			getFQDN(Client_Host,AVStr(host));
 			sprintf(mbox,"%s@%s",ident.i_user,host);
-			return mbox;
+			return (char*)mbox;
 		}
 	}
 	return NULL;
 }
-char *getClientHostPortUser(Conn,host,portp)
-	Connection *Conn;
-	char *host;
-	int *portp;
+const char *getClientHostPortUser(Connection *Conn,PVStr(host),int *portp)
 {
 	VA_getClientAddr(Conn);
 	if( host ) strcpy(host,Client_Host);
@@ -181,9 +176,8 @@ char *getClientHostPortUser(Conn,host,portp)
 	else	return NULL;
 }
 
-char *VA_getOriginatorIdent(Conn,ident)
-	Connection *Conn;
-	AuthInfo *ident;
+int CTX_VA_getOriginatorAddr(Connection *Conn,VAddr *Vaddr);
+const char *VA_getOriginatorIdent(Connection *Conn,AuthInfo *ident)
 {	VAddr *vaddr;
 
 	vaddr = &ident->i_addr;
