@@ -19,14 +19,21 @@ History:
 #include <stdio.h>
 #include <ctype.h>
 #include "delegate.h"
-extern char *malloc();
+#include "fpoll.h"
+static int int1st;
+#define LDAP_SNOOP	0
+
+extern int IO_TIMEOUT;
+static int timeoutms(){
+	return IO_TIMEOUT * 1000;
+}
 
 #define CLASS_OF(type)	((type >> 6) & 3)
 #define CLASS_UNV	0
 #define CLASS_APP	1
 #define CLASS_CTX	2
 #define CLASS_PRI	3
-static char *classname[4] = {"UNV","APP","CTX","PRI"};
+static const char *classname[4] = {"UNV","APP","CTX","PRI"};
 #define classof(type)	classname[CLASS_OF(type)]
 
 #define TYPE_STRUCT	0x20
@@ -52,17 +59,12 @@ typedef unsigned char *Message;
 #define FGETTYPE(in)	getc(in)
 #define MGETTYPE(mp)	(*mp++)
 
-static scan_type(octets,typep)
-	Message octets;
-	int *typep;
+static int scan_type(Message octets,int *typep)
 {
 	if( typep ) *typep = octets[0];
 	return 1;
 }
-static int scan_leng(in,octets,lengp)
-	FILE *in;
-	Message octets;
-	int *lengp;
+static int scan_leng(FILE *in,Message octets,int *lengp)
 {	int bytes,bi,noct,leng,ch;
 
 	bytes = 0;
@@ -87,8 +89,7 @@ static int scan_leng(in,octets,lengp)
 	if( lengp ) *lengp = leng;
 	return bytes;
 }
-static add_leng(octets,disp,inc)
-	Message octets;
+static int add_leng(Message octets,int disp,int inc)
 {	int leng,bytes,bi;
 
 	bytes = scan_leng(NULL,octets+disp,&leng);
@@ -106,8 +107,7 @@ static add_leng(octets,disp,inc)
 		return 0;
 	}
 }
-static put_leng(octets,leng)
-	Message octets;
+static int put_leng(Message octets,int leng)
 {	int bytes,bi;
 
 	if( leng < 128 ){
@@ -125,9 +125,16 @@ static put_leng(octets,leng)
 	octets[0] = 0x80 | bytes;
 	return 1 + bytes;
 }
+static int scan_int(Message octets,int leng)
+{	int lx,ival;
 
-static mssglen(mssg)
-	Message mssg;
+	ival = 0;
+	for( lx = 0; lx < leng; lx++ )
+		ival = (ival << 8) | octets[lx];
+	return ival;
+}
+
+static int mssglen(Message mssg)
 {	int off,type,leng;
 
 	off = 0;
@@ -136,8 +143,8 @@ static mssglen(mssg)
 	return off + leng;
 }
 
-static Message read_mssg(in)
-	FILE *in;
+#define MAXMSGSZ 5*1024*1024
+static Message read_mssg(FILE *in)
 {	int type,leng;
 	Message mssg;
 	unsigned char tbuf[128];
@@ -145,6 +152,11 @@ static Message read_mssg(in)
 
 	tlen = 0;
 
+	if( fPollIn(in,timeoutms()) <= 0 ){
+		sv1log("#### LDAP read_mssg timeout\n");
+		return NULL;
+	}
+
 	type = getc(in);
 	if( type == EOF ) return NULL;
 	tbuf[tlen++] = type;
@@ -152,11 +164,15 @@ static Message read_mssg(in)
 	tlen += len1 = scan_leng(in,&tbuf[tlen],&leng);
 	if( len1 < 0 )
 		return NULL;
+	if( MAXMSGSZ < leng ){
+		syslog_ERROR("ERROR: too large message (%d)\n",leng);
+		return NULL;
+	}
 
 	mssg = (Message)malloc(tlen+leng);
 	bcopy(tbuf,mssg,tlen);
 
-	rcc = fread(mssg+tlen,1,leng,in);
+	rcc = fread(mssg+tlen,1,leng,in); /**/
 	if( rcc != leng ){
 		free(mssg);
 		syslog_ERROR("ERROR: premature EOF %x/%x\n",rcc,leng);
@@ -169,6 +185,8 @@ static Message read_mssg(in)
 #define EDIT_NOOP	0
 #define EDIT_ERASE	1
 #define EDIT_APPEND	2
+#define EDIT_GETSEQ	0x10
+#define EDIT_GETAPP	0x20
 
 typedef struct {
 	int	e_edit;
@@ -176,7 +194,10 @@ typedef struct {
 	short	e_eno[16];
 } MssgEdit;
 static MssgEdit mssgEdit[] = {
+	{ EDIT_GETSEQ, ident(CLASS_UNV,0,TYPE_INT),     {1,1,0,0} },
+	{ EDIT_GETAPP, ident(CLASS_APP,0,TYPE_INT),     {1,2,0,0} },
 	{ EDIT_ERASE,  ident(CLASS_UNV,0,TYPE_CSTRING), {1,2,1,0} },
+	{ EDIT_ERASE,  ident(CLASS_UNV,0,TYPE_CSTRING), {1,2,7,2} },
 	{ EDIT_APPEND, ident(CLASS_UNV,0,TYPE_CSTRING), {1,2,1,0} },
 	{ 0 },
 };
@@ -189,16 +210,18 @@ typedef struct {
 	Message	m_work;
 	Message	m_push;
 	int	m_modified;
-	char   *m_arg;
+	defQStr(m_arg);
+	char	m_apptype;
+	int	m_seq;
+	MStr(	m_base,1024);
 unsigned char	m_xmssg[0x10000];
 } MssgNest;
 
-static dump_octets(data,leng)
-	Message data;
-{	char line[1024],*lp;
-	int ci,ch;
+static void dump_octets(Message data,int leng)
+{	int ci,ch;
+	CStr(line,1024);
+	refQStr(lp,line); /**/
 
-	lp = line;
 	line[0] = 0;
 	for( ci = 0; ci < leng; ci++ ){
 		if( ci % 8 == 0 ){
@@ -214,38 +237,57 @@ static dump_octets(data,leng)
 	}
 	syslog_DEBUG("%s\n",line);
 }
-static sprint_unit(ptype,type,leng,data,buff)
-	Message data;
-	char *buff;
-{	char *typename = NULL;
-	char *bp;
-	int off,ch;
+static void scan_cstring(Message data,int leng,Message buff,int max)
+{	int off,ch;
+	char *bp = (char*)buff; /**/
+
+	*bp++ = '"';
+	for( off = 0; off < leng && off < max; off++ ){
+		ch = data[off];
+		if( 0x20 <= ch && ch <= 0x7F )
+			*bp++ = ch;
+		else	*bp++ = '?';
+	}
+	*bp++ = '"';
+	*bp = 0;
+}
+static void sprint_unit(int ptype,int type,int leng,Message data,PVStr(buff))
+{	const char *xtypename = NULL;
 
 	if( type & TYPE_STRUCT ){
 	  switch( CLASS_OF(type) ){
 	  case CLASS_APP:
 	    switch( TYPE_OF(type) ){
-		case LDAP_BIND: typename = "BIND"; break;
-		case LDAP_BIND_RESULT: typename = "BIND-RESULT"; break;
-		case LDAP_SEARCH: typename = "SEARCH"; break;
-		case LDAP_SEARCH_RESULT: typename="SEARCH-RESULT"; break;
-		case LDAP_SEARCH_DONE: typename = "SEARCH-DONE"; break;
+		case LDAP_BIND: xtypename = "BIND"; break;
+		case LDAP_BIND_RESULT: xtypename = "BIND-RESULT"; break;
+		case LDAP_SEARCH: xtypename = "SEARCH"; break;
+		case LDAP_SEARCH_RESULT: xtypename="SEARCH-RESULT"; break;
+		case LDAP_SEARCH_DONE: xtypename = "SEARCH-DONE"; break;
 	    }
 	  break;
 
 	  case CLASS_CTX:
 	    if( ptype == ident(CLASS_APP,1,LDAP_SEARCH) )
 	    switch( TYPE_OF(type) ){
-		case 0: typename = "AND"; break;
-		case 1: typename = "OR"; break;
-		case 2: typename = "NOT"; break;
-		case 3: typename = "EQ"; break;
+		case 0: xtypename = "AND"; break;
+		case 1: xtypename = "OR"; break;
+		case 2: xtypename = "NOT"; break;
+		case 3: xtypename = "EQ"; break;
 	    }
 	  break;
 	  }
-	  if( typename != NULL )
-		sprintf(buff,"<%s>",typename);
+	  if( xtypename != NULL )
+		sprintf(buff,"<%s>",xtypename);
 	}else{
+		if( CLASS_OF(type) == CLASS_CTX ){
+			switch( TYPE_OF(type) ){
+				case 7: xtypename = "PRESENT"; break;
+			}
+			if( xtypename != NULL ){
+				sprintf(buff,"<%s>",xtypename);
+				scan_cstring(data,leng,(unsigned char*)buff+strlen(buff),80);
+			}
+		}else
 		if( CLASS_OF(type) == CLASS_APP
 		 && TYPE_OF(type) == LDAP_UNBIND
 		){
@@ -259,26 +301,15 @@ static sprint_unit(ptype,type,leng,data,buff)
 			sprintf(buff,"0x%x",data[0]);
 			break;
 		case TYPE_INT:
-			 sprintf(buff,"%d",data[0]);
+			sprintf(buff,"%d",scan_int(data,leng));
 			break;
 		case TYPE_CSTRING:
-			bp = buff;
-			*bp++ = '"';
-			for( off = 0; off < leng && off < 160; off++ ){
-				ch = data[off];
-				if( 0x20 <= ch && ch <= 0x7F )
-					*bp++ = ch;
-				else	*bp++ = '?';
-			}
-			*bp++ = '"';
-			*bp = 0;
+			scan_cstring(data,leng,(unsigned char*)buff,160);
 			break;
 		}
 	}
 }
-static push1(Mp,xtype,xclen,xdata,toff)
-	MssgNest *Mp;
-	Message xdata;
+static int push1(MssgNest *Mp,int xtype,int xclen,Message xdata,int toff)
 {	Message work;
 	int nput;
 
@@ -311,11 +342,8 @@ Mp->m_work[2],
 	return nput;
 }
 
-static scan_mssg1();
-static scan_next(Mp,lev,celp,ptype,pleng,data,toff)
-	MssgNest *Mp;
-	int *celp;
-	Message data;
+static int scan_mssg1(MssgNest *Mp,int ptype,int lev,int nel,int off0,Message data);
+static int scan_next(MssgNest *Mp,int lev,int *celp,int ptype,int pleng,Message data,int toff)
 {	int len;
 	int type,tlen,clen,ileng1,xleng1;
 	Message mp;
@@ -338,18 +366,23 @@ static scan_next(Mp,lev,celp,ptype,pleng,data,toff)
 
 	edit = EDIT_NOOP;
 	Mp->m_eno[lev] = *celp;
+	Mp->m_eno[lev+1] = 0;
 
 	if( Mp->m_edit != EDIT_NOOP ){
 		for( ei = 0; edit = mssgEdit[ei].e_edit; ei++ ){ 
 			if( edit != Mp->m_edit )
+			if( edit != EDIT_GETSEQ )
+			if( edit != EDIT_GETAPP )
 				continue;
 			Ep = &mssgEdit[ei];
 			for( li = 0; li <= lev; li++ ){
 				if( Mp->m_eno[li] != Ep->e_eno[li] )
 					break;
 			}
+			if( Mp->m_eno[li] == 0 )
 			if( Ep->e_eno[li] == 0 ){
 				/* apply "edit" command to the current element */
+				/* should check e_type */
 				break;
 			}
 		}
@@ -376,15 +409,31 @@ static scan_next(Mp,lev,celp,ptype,pleng,data,toff)
 	if( Mp->m_edit != EDIT_NOOP ){
 		Message dp;
 		int ylen;
-		bcopy(xdata,Mp->m_work,clen);
-		Mp->m_work[clen] = 0;
+		int tlen;
+
+		tlen = 1024 < clen ? 1024:clen;
+		bcopy(xdata,Mp->m_work,tlen);
+		Mp->m_work[tlen] = 0;
 
 		switch( edit ){
+		  case EDIT_GETSEQ:
+			if( TYPE_OF(type) == TYPE_INT ){
+				Mp->m_seq = scan_int(mp,clen);
+			}
+			break;
+		  case EDIT_GETAPP:
+			Mp->m_apptype = type;
+			syslog_DEBUG("#### apptype=%X [%s %d]\n",type,
+				classof(type),TYPE_OF(type));
+			break;
 		  case EDIT_ERASE:
-			if( dp = (Message)strrchr(Mp->m_work,'@') ){
-				ylen = strlen(dp);
-				strcpy(Mp->m_arg,dp);
-				Mp->m_arg[ylen] = 0;
+			if( TYPE_OF(type) != TYPE_CSTRING )
+				break;
+			wordscanX((char*)Mp->m_work,AVStr(Mp->m_base),sizeof(Mp->m_base));
+			if( dp = (Message)strrchr((char*)Mp->m_work,'@') ){
+				ylen = strlen((char*)dp);
+				strcpy(Mp->m_arg,(char*)dp);
+				setVStrEnd(Mp->m_arg,ylen);
 /*
 sv1log("******* ERASE: %d -> %d [%s]\n",clen,clen-ylen,Mp->m_arg);
 */
@@ -420,17 +469,15 @@ sv1log("******* APPEND: %d -> %d [%s]\n",clen,clen+ylen,Mp->m_arg);
 			push1(Mp,ptype,total,data,0);
 		}
 	}
+	return 0;
 }
 
 
-static scan_mssg1(Mp,ptype,lev,nel,off0,data)
-	MssgNest *Mp;
-	Message data;
+static int scan_mssg1(MssgNest *Mp,int ptype,int lev,int nel,int off0,Message data)
 {	int li;
 	int tlen,len1,type,leng,stype,sleng;
 	int off,cel;
 	int ch;
-	char line[1024],*lp;
 	Message push,work,temp;
 
 	tlen = 0;
@@ -438,13 +485,18 @@ static scan_mssg1(Mp,ptype,lev,nel,off0,data)
 	tlen += len1 = scan_leng(NULL,data,&leng); data += len1;
 	Mp->m_eno[lev] = nel;
 
+	if( type == TYPE_INT && int1st < 0 ){
+		int1st = scan_int(data,leng);
+	}
+
 	if( Mp->m_dump ){
+		CStr(line,1024);
+		refQStr(lp,line); /**/
 		if( lev == 0 ){
 			syslog_ERROR("-- leng=%x(%d) type=%x --\n",
 				tlen+leng,tlen+leng,type);
 			dump_octets(data-tlen,tlen+leng);
 		}
-		lp = line;
 		sprintf(lp,"%03x %2d %2d %03x ", Mp->m_off,lev,nel,off0);
 		lp += strlen(lp);
 
@@ -452,13 +504,13 @@ static scan_mssg1(Mp,ptype,lev,nel,off0,data)
 			sprintf(lp,"%d.",Mp->m_eno[li]);
 			lp += strlen(lp);
 		}
-		*lp++ = ' ';
+		setVStrPtrInc(lp,' ');
 
 		sprintf(lp,"[%s %2d](%2d) ",classof(type),type&0x1F,leng);
 		lp += strlen(lp);
-		*lp = 0;
+		setVStrEnd(lp,0);
 
-		sprint_unit(ptype,type,leng,data,lp);
+		sprint_unit(ptype,type,leng,data,QVStr(lp,line));
 		syslog_ERROR("%s\n",line);
 	}
 
@@ -471,30 +523,80 @@ static scan_mssg1(Mp,ptype,lev,nel,off0,data)
 	Mp->m_off += leng;
 	return tlen + leng;
 }
-static scan_mssg(mssg,dump,edit,arg)
-	Message mssg;
-	char *arg;
-{	MssgNest Ma,*Mp = &Ma;
+static int scan_mssgX(Message mssg,int dump,int edit,PVStr(arg),MssgNest *Mp)
+{
 	int leng;
+	int bsiz;
+	const unsigned char *buff;
 
 	Mp->m_off = 0;
 	Mp->m_dump = dump;
 	Mp->m_edit = edit;
+
+	/*
+	 * one for working input, one for temporary for editing,
+	 * and one for output
+	 */
+	bsiz = mssglen(mssg)*3 + 1024;
+	buff = 0;
+	if( edit != EDIT_NOOP && sizeof(Mp->m_xmssg) < bsiz ){
+		syslog_ERROR("scan_mssg: editing large data: %d / %d\n",
+			mssglen(mssg),bsiz);
+		Mp->m_work = (unsigned char*)(buff = (const unsigned char*)malloc(bsiz));
+		Mp->m_btm  = Mp->m_work + bsiz;
+	}else{
 	Mp->m_work = Mp->m_xmssg;
 	Mp->m_btm  = Mp->m_xmssg + sizeof(Mp->m_xmssg);
+	}
+
 	Mp->m_push = Mp->m_btm;
-	Mp->m_arg = arg;
+	Mp->m_seq = 0;
+	Mp->m_apptype = 0;
+	setQStr(Mp->m_arg,arg,UTail(arg)-arg+1);
 	Mp->m_modified = 0;
-	scan_mssg1(Mp,0,0,1,0,mssg);
+	Mp->m_base[0] = 0;
+	leng = scan_mssg1(Mp,0,0,1,0,mssg);
 
 	if( Mp->m_modified )
 		bcopy(Mp->m_push,mssg,mssglen(Mp->m_push));
+
+
+	if( buff ){
+		free((char*)buff);
+	}
 	return leng;
 }
+static int scan_mssg(Message mssg,int dump,int edit,PVStr(arg))
+{	MssgNest Ma,*Mp = &Ma;
 
-static Message make_LDAPResult(msgid,type,code,comment)
-	char *comment;
-{	unsigned char resp[1024];
+	return scan_mssgX(mssg,dump,edit,AVStr(arg),Mp);
+}
+
+static Message make_LDAPResult2(int msgid,int type)
+{	unsigned char resp[1024]; /**/
+	Message mresp;
+	int ri;
+
+	ri = 0;
+	resp[ri++] = ident(CLASS_UNV,1,16);
+	resp[ri++] = 9;
+	resp[ri++] = ident(CLASS_UNV,0,TYPE_INT);
+	resp[ri++] = 1;
+	resp[ri++] = msgid;
+	resp[ri++] = ident(CLASS_APP,1,type);
+	resp[ri++] = 4;
+	resp[ri++] = ident(CLASS_UNV,0,TYPE_CSTRING);
+	resp[ri++] = 0;
+	resp[ri++] = ident(CLASS_UNV,1,16);
+	resp[ri++] = 0;
+
+	mresp = (Message)malloc(ri);
+	bcopy(resp,mresp,ri);
+	return mresp;
+}
+
+static Message make_LDAPResult(int msgid,int type,int code,PCStr(comment))
+{	unsigned char resp[1024]; /**/
 	Message mresp,tail;
 	int ri,comlen,comlenlen;
 	unsigned char comlenbuf[8];
@@ -506,9 +608,11 @@ static Message make_LDAPResult(msgid,type,code,comment)
 
 	ri = 0;
 	resp[ri++] = ident(CLASS_UNV,1,16);
-	resp[ri++] = 12+comlen; 
+	resp[ri++] = 14+comlen; 
 	resp[ri++] = ident(CLASS_UNV,0,TYPE_INT);
-	resp[ri++] = 1;
+	resp[ri++] = 3;
+	resp[ri++] = msgid >> 16;
+	resp[ri++] = msgid >> 8;
 	resp[ri++] = msgid;
 	resp[ri++] = ident(CLASS_APP,1,type);
 	resp[ri++] = 7+comlen;
@@ -535,62 +639,179 @@ ri++; tail[-ri] = comlen;
 	bcopy(resp,mresp,ri);
 	return mresp;
 }
+static void putEmptyResult(Connection *Conn,int msgid)
+{	Message resp;
+	int mlen;
+
+	resp = make_LDAPResult2(msgid,LDAP_SEARCH_RESULT);
+	mlen = mssglen(resp);
+	scan_mssg(resp,1,EDIT_NOOP,VStrNULL);
+	write(ToC,resp,mlen);
+
+	resp = make_LDAPResult(msgid, LDAP_SEARCH_DONE,RESULT_OK,"");
+	mlen = mssglen(resp);
+	scan_mssg(resp,1,EDIT_NOOP,VStrNULL);
+	write(ToC,resp,mlen);
+}
+
+/*
+ * MOUNT should be achieved in scan_mssg() to generate baseObject
+ */
+/*
+static void mount(Connection *Conn,MssgNest *Mp,PVStr(srchroot))
+*/
+static void mount(Connection *Conn,MssgNest *Mp,PVStr(srchroot),PVStr(proto))
+{	const char *opts;
+	CStr(server,1024);
+	const char *dp;
+
+	if( strchr(srchroot,'@') )
+		return;
+
+	wordscanX(Mp->m_base,AVStr(server),sizeof(server));
+	if( opts = CTX_mount_url_to(Conn,NULL,"GET",AVStr(server)) ){
+		sv1log("MOUNTed to %s <= %s\n",server,Mp->m_base);
+		if( strneq(server,"ldaps://",8) ){
+			strcpy(proto,"ldaps");
+			strsubst(AVStr(server),"ldaps","ldap");
+		}
+		if( strneq(server,"ldap://",7) ){
+			setVStrElem(srchroot,0,'@');
+			dp = wordscanY(server+7,QVStr(srchroot+1,srchroot),256,"^/");
+			if( *dp == '/' ){
+				linescanX(dp+1,AVStr(Mp->m_base),sizeof(Mp->m_base));
+				/* baseObject in the message should be
+				 * replaced with this
+				 */
+			}
+		}
+	}
+}
 
 /* REMOVE "@real-server-name" in baseObject's LDAPDN in SEARCH requests
  * APPEND "@real-server-name" to baseObject's LDAPDN in SEARCH responses
  * This rewriting should be controled with MOUNT parameter.
  */
-static proxy_ldap(Conn)
-	Connection *Conn;
+static void proxy_ldap(Connection *Conn)
 {	FILE *fc,*tc,*fs,*ts;
+	int nmsg[2];
 	Message bindmssg,srchmssg,mssg,nmssg,resp;
-	char srchroot[512],ldapserv[512],host[512];
+	CStr(srchroot,512);
+	CStr(ldapserv,512);
+	CStr(proto,128);
+	CStr(host,512);
 	int mlen;
 	int port;
-	FILE *fpv[2],*ifp;
-	int rds[2],tov[2],nmsg[2],fi;
 	MssgEdit Me,*Ep = &Me;
+	MssgNest Ma,*Mp = &Ma;
+	int type;
 
 	fc = fdopen(FromC,"r");
 	nmsg[0] = nmsg[1] = 0;
 
+	if( !isMYSELF(DFLT_HOST) ){
+		fs = fdopen(FromS,"r");
+		srchroot[0] = 0;
+		goto RELAY;
+	}
+
 	nmsg[0]++;
 	bindmssg = read_mssg(fc);
-	scan_mssg(bindmssg,1,EDIT_NOOP,NULL);
+	if( bindmssg == NULL )
+		return;
+	int1st = -1;
 
-	resp = make_LDAPResult(bindmssg[4],
+	srchroot[0] = 0;
+	scan_mssgX(bindmssg,1,EDIT_ERASE,AVStr(srchroot),Mp);
+
+	if( TYPE_OF(Mp->m_apptype) == LDAP_SEARCH )
+	if( srchroot[0] == 0 )
+	{
+		syslog_ERROR("#### SearchReuest for RootDSE\n");
+		putEmptyResult(Conn,Mp->m_seq);
+
+		nmsg[0]++;
+		bindmssg = read_mssg(fc);
+		if( bindmssg == NULL ){
+			return;
+		}
+		srchroot[0] = 0;
+		scan_mssgX(bindmssg,1,EDIT_ERASE,AVStr(srchroot),Mp);
+	}
+
+	resp = make_LDAPResult(int1st,
 		LDAP_BIND_RESULT,RESULT_OK,
 		"Bound by proxy (DeleGate)");
 
 	mlen = mssglen(resp);
 	syslog_ERROR("#### proxy BIND response (%d)\n",mlen);
-	scan_mssg(resp,1,EDIT_NOOP,NULL);
+	scan_mssg(resp,1,EDIT_NOOP,VStrNULL);
 	write(ToC,resp,mlen);
 
 	nmsg[0]++;
 	srchmssg = read_mssg(fc);
+	if( srchmssg == NULL ){
+		syslog_ERROR("#### disconnected by client\n");
+		return;
+	}
 	srchroot[0] = 0;
-	scan_mssg(srchmssg,1,EDIT_ERASE,srchroot);
+	scan_mssgX(srchmssg,1,EDIT_ERASE,AVStr(srchroot),Mp);
+
+	if( TYPE_OF(Mp->m_apptype) == LDAP_SEARCH )
+	if( srchroot[0] == 0 )
+	if( Mp->m_base[0] == 0 )
+	{
+		syslog_ERROR("#### SearchReuest for RootDSE\n");
+		putEmptyResult(Conn,Mp->m_seq);
+
+		nmsg[0]++;
+		srchmssg = read_mssg(fc);
+		if( srchmssg == NULL ){
+			return;
+		}
+		srchroot[0] = 0;
+		scan_mssgX(srchmssg,1,EDIT_ERASE,AVStr(srchroot),Mp);
+	}
+
+	type = Mp->m_apptype;
+	if( TYPE_OF(type) == LDAP_UNBIND ){
+		syslog_ERROR("#### not a SEARCH message: %X = %s %d\n",
+			type,classof(type),TYPE_OF(type));
+		return;
+	}
+
+	strcpy(proto,"ldap");
+	mount(Conn,Mp,AVStr(srchroot),AVStr(proto));
+	/*
+	mount(Conn,Mp,AVStr(srchroot));
+	*/
+
 	if( srchroot[0] == '@' )
 		strcpy(ldapserv,srchroot+1);
 	else	ldapserv[0] = 0;
 
-	port = scan_hostport("ldap",ldapserv,host);
+	/*
+	port = scan_hostport("ldap",ldapserv,AVStr(host));
+	*/
+	port = scan_hostport(proto,ldapserv,AVStr(host));
 	syslog_ERROR("LDAP-SERVER=[%s]=[%s:%d]\n",srchroot,host,port);
+	set_realserver(Conn,proto,host,port);
+	/*
 	set_realserver(Conn,"ldap",host,port);
+	*/
 	if( connect_to_serv(Conn,FromC,ToC,0) < 0 ){
 		syslog_ERROR("#### proxy connection error response\n");
-		resp = make_LDAPResult(srchmssg[4],
+		resp = make_LDAPResult(Mp->m_seq,
 			LDAP_SEARCH_DONE,RESULT_UNAVAIL,
 			"Can't connect to LDAP server by proxy (DeleGate)");
-		goto ERROR;
+		goto xERROR;
 	}
 
 	fs = fdopen(FromS,"r");
 
 	mlen = mssglen(bindmssg);
 	syslog_ERROR("#### relay client's BIND (%d)\n",mlen);
-	scan_mssg(bindmssg,1,EDIT_NOOP,NULL);
+	scan_mssg(bindmssg,1,EDIT_NOOP,VStrNULL);
 	write(ToS,bindmssg,mlen);
 
 	syslog_ERROR("#### wait BIND response from the server\n");
@@ -598,22 +819,29 @@ static proxy_ldap(Conn)
 	resp = read_mssg(fs);
 	if( resp == NULL ){
 		syslog_ERROR("#### EOF from server\n");
-		resp = make_LDAPResult(srchmssg[4],
+		resp = make_LDAPResult(Mp->m_seq,
 			LDAP_SEARCH_DONE,RESULT_UNAVAIL,
 			"EOF from LDAP server to proxy (DeleGate)");
-		goto ERROR;
+		goto xERROR;
 	}
-	scan_mssg(resp,1,EDIT_NOOP,NULL);
+	scan_mssg(resp,1,EDIT_NOOP,VStrNULL);
 
 	mlen = mssglen(srchmssg);
 	syslog_ERROR("#### forward search request (%d)\n",mlen);
-	scan_mssg(srchmssg,1,EDIT_NOOP,NULL);
+	scan_mssg(srchmssg,1,EDIT_NOOP,VStrNULL);
 	write(ToS,srchmssg,mlen);
 
+ RELAY:{
+	FILE *fpv[2],*ifp;
+	int rds[2],tov[2],fi;
+
 	syslog_ERROR("#### start bidirectional relay\n");
 	fpv[0] = fc; tov[0] = ToS;
 	fpv[1] = fs; tov[1] = ToC;
+	/*
 	while( !feof(fc) && !feof(fs) && 0 < fPollIns(0,2,fpv,rds) )
+	*/
+	while( !feof(fc) && !feof(fs) && 0 < fPollIns(timeoutms(),2,fpv,rds) )
 	for( fi = 0; fi < 2; fi++ )
 	if( rds[fi] ){
 		int type,bytes;
@@ -631,9 +859,12 @@ static proxy_ldap(Conn)
 		type = mssg[1+bytes+3];
 		syslog_ERROR("#### %s[%d] (%d) %x\n",fi==0?"C-S":"S-C",nmsg[fi],mlen,type);
 
+		if( LDAP_SNOOP )
+			scan_mssg(mssg,1,EDIT_NOOP,VStrNULL);
+
 		if( fi == 0 ){
 			if( type == ident(CLASS_APP,1,LDAP_SEARCH ) ){
-				scan_mssg(mssg,0,EDIT_ERASE,srchroot);
+				scan_mssg(mssg,0,EDIT_ERASE,AVStr(srchroot));
 				mlen = mssglen(mssg);
 			}
 		}else{
@@ -646,24 +877,30 @@ static proxy_ldap(Conn)
 				bcopy(mssg,nmssg,mssglen(mssg));
 				free(mssg);
 				mssg = nmssg;
-				scan_mssg(mssg,0,EDIT_APPEND,srchroot);
+				scan_mssg(mssg,0,EDIT_APPEND,AVStr(srchroot));
 				mlen = mssglen(mssg);
 			}
 		}
 		write(tov[fi],mssg,mlen);
 	}
+ }
 	return;
 
-ERROR:
+xERROR:
 	mlen = mssglen(resp);
-	scan_mssg(resp,1,EDIT_NOOP,NULL);
+	scan_mssg(resp,1,EDIT_NOOP,VStrNULL);
 	write(ToC,resp,mlen);
 }
 
-service_ldap(Conn)
-	Connection *Conn;
+int service_tcprelay(Connection *Conn);
+int service_ldap(Connection *Conn)
 {
 	if( isMYSELF(DFLT_HOST) )
                 proxy_ldap(Conn);
+	else
+	if( LDAP_SNOOP ){
+                proxy_ldap(Conn);
+	}
 	else	service_tcprelay(Conn);
+	return 0;
 }
