@@ -17,9 +17,12 @@ History:
 	981028	created
 //////////////////////////////////////////////////////////////////////#*/
 #include <stdio.h>
-#include <string.h>
-extern char *getenv();
-extern char *freadfile();
+#include <stdlib.h>
+#include "ystring.h"
+#include "time.h"
+#include "fpoll.h"
+#include "file.h"
+#include "dglib.h"
 
 #define JC_PRINT	1
 #define JC_RECEIVE	2
@@ -58,10 +61,11 @@ typedef struct {
 	int	l_ctrl_siz;
 } LprJob;
 
-static relay_RECEIVE(lpj,ts,fs,tc,fc)
-	LprJob *lpj;
-	FILE *ts,*fs,*tc,*fc;
-{	char req[1024],resp[1],op,*buff;
+static void relay_RECEIVE(LprJob *lpj,FILE *ts,FILE *fs,FILE *tc,FILE *fc)
+{	CStr(req,1024);
+	CStr(resp,1);
+	char op;
+	const char *buff;
 	int size,rcc;
 	FILE *ts_sav,*fs_sav;
 
@@ -113,6 +117,7 @@ static relay_RECEIVE(lpj,ts,fs,tc,fc)
 				lpj->l_data_siz = rcc;
 			}
 			fwrite(buff,1,size,ts);
+			free((char*)buff);
 			fflush(ts);
 			lpj->l_off += rcc;
 
@@ -133,10 +138,11 @@ static relay_RECEIVE(lpj,ts,fs,tc,fc)
 		}
 	}
 }
-static relay_commands(lpj,ts,fs,tc,fc)
-	LprJob *lpj;
-	FILE *ts,*fs,*tc,*fc;
-{	char req[1024],resp[0x8000],op,rcc;
+static int relay_commands(LprJob *lpj,FILE *ts,FILE *fs,FILE *tc,FILE *fc)
+{	CStr(req,1024);
+	CStr(resp,0x8000);
+	char op;
+	char rcc;
 
 	while( fgets(req,sizeof(req),fc) ){
 		op = req[0];
@@ -175,14 +181,17 @@ static relay_commands(lpj,ts,fs,tc,fc)
 	return 0;
 }
 
-service_lpr(Conn,_1,_2,fromC,toC,svproto,svhost,svport,svpath)
-	void *Conn;
-	char *svproto,*svhost,*svpath;
+int open_lpr(PCStr(host),int port,FILE **tsp,FILE **fsp);
+int stat_lpr(PCStr(lphost),int lpport,PCStr(queue),PCStr(opt),PCStr(fname),PVStr(stat));
+
+int service_lpr(DGC*Conn,int _1,int _2,int fromC,int toC,PCStr(svproto),PCStr(svhost),int svport,PCStr(svpath))
 {	FILE *ts,*fs,*fc,*tc;
-	char req[1024],resp[0x8000],op;
+	CStr(req,1024);
+	CStr(resp,0x8000);
+	char op;
 	int nowait;
-	FILE *tmp,*TMPFILE();
-	char *host = svhost;
+	FILE *tmp;
+	const char *host = svhost;
 	int port = svport;
 	LprJob lpj_buf, *lpj = &lpj_buf;
 
@@ -217,14 +226,14 @@ if( withMount || filter_withCFI(Conn,X_FTOSV) )
 /*
 {
 int off,siz,rcc;
-char *buff;
+const char *buff;
 sv1log("#### DATA:%d+%d CTRL:%d+%d SIZE=%d\n",
 lpj->l_data_off, lpj->l_data_siz,
 lpj->l_ctrl_off, lpj->l_ctrl_siz, lpj->l_off);
 if( siz = lpj->l_data_siz ){
 fseek(tmp,lpj->l_data_off,0);
 buff = (char*)malloc(lpj->l_data_siz+1);
-rcc = fread(buff,1,lpj->l_data_siz,tmp);
+rcc = fread (buff,1,lpj->l_data_siz,tmp);
 buff[rcc] = 0;
 fprintf(stderr,"---\n%s---\n",buff);
 free(buff);
@@ -235,9 +244,9 @@ fseek(tmp,0,0);
 	}
 	if( open_lpr(host,port,&ts,&fs) != 0 ){
 		syslog_ERROR("cannot connect to LPR %s:%d\n",host,port);
-		resp[0] = -1;
+		resp[0] = (char)-1;
 		write(toC,resp,1);
-		return;
+		return -1;
 	}
 	relay_commands(lpj,ts,fs,tc,fc);
 
@@ -255,12 +264,11 @@ fseek(tmp,0,0);
 	set_linger(fileno(ts),30);
 	fclose(ts);
 	fclose(fs);
+	return 0;
 }
 
-open_lpr(host,port,tsp,fsp)
-	char *host;
-	FILE **tsp,**fsp;
-{	char *lpr_host;
+int open_lpr(PCStr(host),int port,FILE **tsp,FILE **fsp)
+{	const char *lpr_host;
 	int lpr_port;
 	int sock;
 
@@ -287,13 +295,11 @@ open_lpr(host,port,tsp,fsp)
 	return 0;
 }
 
-static getAgentInfo(Conn,user,orig_host,orig_user)
-	void *Conn;
-	char *user,*orig_host,*orig_user;
+static void getAgentInfo(DGC*Conn,PCStr(user),PVStr(orig_host),PVStr(orig_user))
 {
 	if( Conn ){
-		strfConn(Conn,"%h",orig_host);
-		strfConn(Conn,"%u",orig_user);
+		strfConnX(Conn,"%h",AVStr(orig_host),64);
+		strfConnX(Conn,"%u",AVStr(orig_user),64);
 		if( streq(orig_user,"-") ){
 			if( user && *user )
 				strcpy(orig_user,user);
@@ -304,42 +310,39 @@ static getAgentInfo(Conn,user,orig_host,orig_user)
 		strcpy(orig_user,"unknown-user");
 	}
 }
-send_lpr(Conn,lphost,lpport,queue,dfp,dlen,user,fname,stat)
-	void *Conn;
-	char *lphost,*queue;
-	FILE *dfp;
-	char *user,*fname;
-	char *stat;
+int send_lpr(DGC*Conn,PCStr(lphost),int lpport,PCStr(queue),FILE *dfp,int dlen,PCStr(user),PCStr(fname),PVStr(stat))
 {	FILE *ts,*fs;
-	char job[64];
-	char resp[1024];
+	CStr(job,64);
+	CStr(resp,1024);
 	int rcc;
 	int jnum;
-	char src_files[64];
+	CStr(src_files,64);
 	int data_size,ctrl_size;
-	char data_name[64],ctrl_name[64];
-	char orig_host[64],orig_user[64];
-	char *data;
-	char ctrl_data[1024],*cfp;
-	char *fmt;
+	CStr(data_name,64);
+	CStr(ctrl_name,64);
+	CStr(orig_host,64);
+	CStr(orig_user,64);
+	const char *pdata;
+	CStr(ctrl_data,1024);
+	refQStr(cfp,ctrl_data); /**/
+	const char *fmt;
 
 	if( queue == NULL || *queue == 0 )
 		queue = DEFAULT_QUEUE;
 	jnum = time(0) % 1000;
 
 	data_size = dlen;
-	data = freadfile(dfp,&data_size);
+	pdata = freadfile(dfp,&data_size);
 	if( fname )
 		strcpy(src_files,fname);
 	else	strcpy(src_files,"standard input");
 	syslog_ERROR("Data Size: %d\n",data_size);
 
 	fmt = "f";
-	getAgentInfo(Conn,user,orig_host,orig_user);
+	getAgentInfo(Conn,user,AVStr(orig_host),AVStr(orig_user));
 	sprintf(data_name,"dfA%03d%s",jnum,orig_host);
 	sprintf(ctrl_name,"cfA%03d%s",jnum,orig_host);
 
-	cfp = ctrl_data;
 	sprintf(cfp,"%c%s\n",CF_HOSTNAME,orig_host); cfp += strlen(cfp);
 	sprintf(cfp,"%c%s\n",CF_USERID,  orig_user); cfp += strlen(cfp);
 	sprintf(cfp,"%c%s\n",CF_JOBNAME, src_files); cfp += strlen(cfp);
@@ -366,7 +369,9 @@ send_lpr(Conn,lphost,lpport,queue,dfp,dlen,user,fname,stat)
 	syslog_DEBUG("> %-10s %d %s\n","RJ_DATA",data_size,data_name);
 	rcc = fread(resp,1,1,fs);
 	syslog_DEBUG("< %-10s %d code=%d\n","RJ_DATA",rcc,resp[0]);
-	fwrite(data,1,data_size,ts);
+	fwrite(pdata,1,data_size,ts);
+	free((char*)pdata);
+
 	fflush(ts);
 	fputc(0,ts);
 	fflush(ts);
@@ -389,15 +394,13 @@ send_lpr(Conn,lphost,lpport,queue,dfp,dlen,user,fname,stat)
 	fclose(ts);
 	fclose(fs);
 
-	stat_lpr(lphost,lpport,queue,"","",stat);
+	stat_lpr(lphost,lpport,queue,"","",AVStr(stat));
 	return data_size;
 }
 
-stat_lpr(lphost,lpport,queue,opt,fname,stat)
-	char *lphost,*queue;
-	char *opt,*fname,*stat;
+int stat_lpr(PCStr(lphost),int lpport,PCStr(queue),PCStr(opt),PCStr(fname),PVStr(stat))
 {	FILE *ts,*fs;
-	char resp[1024];
+	CStr(resp,1024);
 
 	if( open_lpr(lphost,lpport,&ts,&fs) != 0 ){
 		if( stat ) sprintf(stat,"can't connect to the LPR server\n");
@@ -411,7 +414,7 @@ stat_lpr(lphost,lpport,queue,opt,fname,stat)
 	else	fprintf(ts,"%c%s\n",JC_SENDQSTATS,queue);
 	fflush(ts);
 
-	if( stat ) stat[0] = 0;
+	if( stat ) setVStrEnd(stat,0);
 	syslog_DEBUG("STATUS: \n");
 	while( fgets(resp,sizeof(resp),fs) != NULL ){
 		syslog_DEBUG("%s",resp);
@@ -419,21 +422,21 @@ stat_lpr(lphost,lpport,queue,opt,fname,stat)
 	}
 	fclose(ts);
 	fclose(fs);
+	return 0;
 }
-rmjob_lpr(Conn,lphost,lpport,queue,user,jobname,stat)
-	void *Conn;
-	char *lphost,*queue;
-	char *user,*jobname,*stat;
+int rmjob_lpr(DGC*Conn,PCStr(lphost),int lpport,PCStr(queue),PCStr(user),PCStr(jobname),PVStr(stat))
 {	FILE *ts,*fs;
-	char req[1024],resp[1024];
-	char orig_host[512],orig_user[512];
+	CStr(req,1024);
+	CStr(resp,1024);
+	CStr(orig_host,512);
+	CStr(orig_user,512);
 
 	if( open_lpr(lphost,lpport,&ts,&fs) != 0 ){
 		return -1;
 	}
 	if( queue == NULL || *queue == 0 )
 		queue = DEFAULT_QUEUE;
-	getAgentInfo(Conn,user,orig_host,orig_user);
+	getAgentInfo(Conn,user,AVStr(orig_host),AVStr(orig_user));
 
 /*
 sprintf(req,"%c%s %s %s%s\n",JC_REMOVEJOBS,queue,orig_user,jobname,orig_host);
@@ -443,30 +446,29 @@ sprintf(req,"%c%s %s %s\n",JC_REMOVEJOBS,queue,orig_user,jobname);
 	syslog_ERROR("C-S: < REMOVE %s",req+1);
 	fputs(req,ts);
 	fflush(ts);
-	resp[0] = -1;
+	resp[0] = (char)-1;
 	fread(resp,1,1,fs);
 	syslog_ERROR("S-C: > REMOVE %02x\n",resp[0]);
 
 	fclose(ts);
 	fclose(fs);
 
-	stat_lpr(lphost,lpport,queue,"","",stat);
+	stat_lpr(lphost,lpport,queue,"","",AVStr(stat));
 	return resp[0];
 }
 
-lpr_main(ac,av)
-	char *av[];
+int lpr_main(int ac,const char *av[])
 {
-	return send_lpr(NULL,NULL,0,NULL,stdin,0,NULL,NULL,NULL);
+	return send_lpr(NULL,NULL,0,NULL,stdin,0,NULL,NULL,VStrNULL);
 }
-lpq_main(ac,av)
-	char *av[];
-{	char stat[4096];
+int lpq_main(int ac,const char *av[])
+{	CStr(stat,4096);
 
-	stat_lpr(NULL,0,NULL,"","",stat);
+	stat_lpr(NULL,0,NULL,"","",AVStr(stat));
 	fputs(stat,stdout);
+	return 0;
 }
-lprm_main(ac,av)
-	char *av[];
+int lprm_main(int ac,const char *av[])
 {
+	return 0;
 }
