@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1994 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1994-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -20,26 +21,26 @@ History:
 	941225	created
 //////////////////////////////////////////////////////////////////////#*/
 #include <stdio.h>
-#include <string.h>
-extern char *fgetsTimeout();
-extern char *calloc();
+#include <stdlib.h>
+#include "ystring.h"
+#include "dglib.h"
+#include "fpoll.h"
 
 #define MSG_SIZE	0x8000
 #define MSG_TIMEOUT	((double)0.5)
 
-static sv1Vlog(){}
+static void sv1Vlog(PCStr(fmt), ...){}
 
-putPreStatus(dstf,status)
-	FILE *dstf;
-	char *status;
+#define BASE64	"/BASE64"
+
+void putPreStatus(FILE *dstf,PCStr(status))
 {
 	fprintf(dstf,"^%s",status);
 	fflush(dstf);
 }
-putPostStatus(dstf,status)
-	FILE *dstf;
-	char *status;
-{	char *sp,ch;
+void putPostStatus(FILE *dstf,PCStr(status))
+{	const char *sp;
+	char ch;
 
 	fputs("$",dstf);
 	for( sp = status; ch = *sp; sp++ ){
@@ -52,29 +53,48 @@ putPostStatus(dstf,status)
 	}
 	fflush(dstf);
 }
-static unescape_nl(str)
-	char *str;
-{	char *dp,*sp,ch;
+static void unescape_nl(PVStr(str))
+{	refQStr(dp,str); /**/
+	const char *sp;
+	char ch;
 
-	dp = sp = str;
+	sp = str;
+	dp = (char*)sp;
 	while( ch = *sp++ ){
 		if( ch == '\\' ){
 			if( *sp == 'n' )
-				*dp++ = '\n';
-			else	*dp++ = *sp;
+				setVStrPtrInc(dp,'\n');
+			else	setVStrPtrInc(dp,*sp);
 			sp++;
-		}else	*dp++ = ch;
+		}else	setVStrPtrInc(dp,ch);
 	}
-	*dp = 0;
+	setVStrEnd(dp,0);
 }
-putMessage1(dstf,ser,buff,size)
-	FILE *dstf;
-	char *buff;
+static int putmsghead(PCStr(what),int ser,PCStr(buff),int leng,FILE *dst,PCStr(encode),PVStr(xbuff))
+{	int xleng = 0;
+
+	if( buff == NULL )
+		return 0;
+
+	sv1Vlog("%s: @%d %d %s\r\n",what,ser,leng,"");
+	if( encode && strcaseeq(encode,BASE64) )
+		xleng = leng = str_to64(buff,leng,AVStr(xbuff),MSG_SIZE*2,1);
+	else	xleng = 0;
+	fprintf(dst,"@%d %d [%s]\r\n",ser,leng,encode);
+	return xleng;
+}
+
+int putMessage1X(FILE *dstf,int ser,PCStr(buff),int size,PCStr(encode))
 {	int wc,wcc,nw;
+	CStr(xbuff,MSG_SIZE*2);
+	int xsize;
 
 	sv1Vlog("putMessage: got %d\n",size);
-	fprintf(dstf,"@%d %d %s\r\n",ser,size,"");
-	sv1Vlog("@%d %d %s\r\n",ser,size,"");
+
+	if( xsize = putmsghead("putMessage",ser,buff,size,dstf,encode,AVStr(xbuff)) ){
+		buff = xbuff;
+		size = xsize;
+	}
 
 	if( buff != NULL ){
 		nw = 0;
@@ -90,28 +110,27 @@ putMessage1(dstf,ser,buff,size)
 	return 0;
 }
 
-extern double Time();
-static freadIntime(buff,s,n,fp,Timeout)
-	char *buff;
-	FILE *fp;
-	double Timeout;
+static int freadIntime(PVStr(buff),int s,int n,FILE *fp,double Timeout)
 {	int rc,rc0,rc1;
 	double start,end;
 	int nread;
 
 	if( s != 1 )
-		return freadTIMEOUT(buff,s,n,fp);
+		return freadTIMEOUT(BVStr(buff),s,n,fp);
 
 	start = Time();
 	end = 0;
 	rc = 0;
 	nread = 0;
 	while( rc < n ){
+		if( feof(fp) ){
+			break;
+		}
 		rc0 = n - rc;
 		if( 512 < rc0 )
 			rc0 = 512;
 
-		rc1 = freadTIMEOUT(buff+rc,s,rc0,fp);
+		rc1 = freadTIMEOUT(QVStr(buff+rc,buff),s,rc0,fp);
 		end = Time();
 
 		if( rc1 == 0 )
@@ -127,76 +146,77 @@ static freadIntime(buff,s,n,fp,Timeout)
 	return rc;
 }
 
-putMessageF(srcf,dstf,cachefp)
-	FILE *srcf,*dstf,*cachefp;
-{	int ser,rcc,rcc_total;
-	char buff[MSG_SIZE];
+FileSize putMessageFX(FILE *srcf,FILE *dstf,FILE *cachefp,PCStr(encode))
+{
+	return putMessageFX_CB(srcf,dstf,cachefp,encode,NULL,NULL);
+}
+FileSize putMessageFX_CB(FILE *srcf,FILE *dstf,FILE *cachefp,PCStr(encode),msgCBFunc cb,void *cbarg)
+{	int ser,rcc;
+	FileSize rcc_total;
+	CStr(buff,MSG_SIZE);
 
 	rcc_total = 0;
-	for( ser = 1; rcc = freadIntime(buff,1,sizeof(buff),srcf,MSG_TIMEOUT); ser++ ){
+	for( ser = 1; rcc = freadIntime(AVStr(buff),1,sizeof(buff),srcf,MSG_TIMEOUT); ser++ ){
+		if( cb != NULL ){
+			rcc = (*cb)(cbarg,AVStr(buff),rcc);
+			if( rcc < 0 ){
+				break;
+			}
+		}
 		rcc_total += rcc;
 		if( cachefp )
 			fwrite(buff,1,rcc,cachefp);
-		if( putMessage1(dstf,ser,buff,rcc) < 0 )
+		if( putMessage1X(dstf,ser,buff,rcc,encode) < 0 )
 			goto EXIT;
 	}
-	putMessage1(dstf,ser,NULL,0);
+	putMessage1X(dstf,ser,NULL,0,encode);
 	fflush(dstf);
 EXIT:
 	return rcc_total;
 }
 
-getMessage(srcd,func,arg1,arg2)
-	char *(*func)();
-	char *arg1,*arg2;
-{	FILE *src;
-
-	src = fdopen(srcd,"r");
-	return getMessageF(src,NULL,0,func,arg1,arg2);
-}
 
 #define MSG_BUFSIZE (MSG_SIZE *2) /* extra space for pending strings without NL */
-static char *PendingBuff;
-static char *PendingP;
+static struct { defQStr(PendingBuff); } PendingBuff;
+#define PendingBuff PendingBuff.PendingBuff
+static const char *PendingP;
 static int Pending;
 static int DoFill;
 
-static pclear()
+static void pclear()
 {
 	PendingP = PendingBuff;
 	Pending = 0;
 	DoFill = 0;
 }
-static pfill(len,srcf)
-	FILE *srcf;
+static int pfill(int len,FILE *srcf)
 {	int brc;
 
 	if( PendingBuff == 0 )
-		PendingBuff = calloc(MSG_BUFSIZE,1);
+		setQStr(PendingBuff,(char*)calloc(MSG_BUFSIZE,1),MSG_BUFSIZE);
 	if( Pending )
-		bcopy(PendingP,PendingBuff,Pending);
+		Bcopy(PendingP,PendingBuff,Pending);
 	PendingP = PendingBuff;
-	brc = freadTIMEOUT(PendingBuff+Pending,1,len,srcf);
+	brc = freadTIMEOUT(QVStr(PendingBuff+Pending,PendingBuff),1,len,srcf);
 	Pending += brc;
 	sv1vlog("fillPending: +%d = %d bytes\n",brc,Pending);
 	return brc;
 }
-static ppush(buff,len)
-	char *buff;
+static void ppush(PCStr(buff),int len)
 {
 	if( PendingBuff == 0 )
-		PendingBuff = calloc(MSG_BUFSIZE,1);
+		setQStr(PendingBuff,(char*)calloc(MSG_BUFSIZE,1),MSG_BUFSIZE);
 
 	PendingP = PendingBuff;
-	bcopy(buff,PendingBuff+Pending,len);
+	bcopy(buff,(char*)PendingBuff+Pending,len);
 	Pending += len;
 	sv1log("pushPending: +%d = %d bytes\n",len,Pending);
 }
-static pgets(buff,len,srcf)
-	char *buff;
-	FILE *srcf;
+static int pgets(PVStr(buff),int len,FILE *srcf)
 {	int pi;
-	char *sp,*dp,ch;
+	const char *sp;
+	refQStr(dp,buff); /**/
+	char ch;
 	int rc,rrc,EOB = 0;
 
 	if( Pending == 0 || DoFill )
@@ -208,11 +228,13 @@ static pgets(buff,len,srcf)
 	}
 
 	sp = PendingP;
-	dp = buff;
+	cpyQStr(dp,buff);
 	rc = 0;
 	for( pi = 0; pi < Pending; pi++ ){
+		assertVStr(buff,dp+1);
 		rc++;
-		ch = *dp++ = *sp++;
+		ch = *sp++;
+		setVStrPtrInc(dp,ch);
 		if( ch == '\n' )
 			goto GOTNL;
 		if( ch == '\r' && rc < Pending && *sp != '\n' )
@@ -229,16 +251,13 @@ static pgets(buff,len,srcf)
 		return 0;
 	}
 GOTNL:
-	buff[rc] = 0;
+	setVStrEnd(buff,rc);
 	Pending -= rc;
 	PendingP = sp;
 	return rc;
 }
 
-static flushPending(outEOF,ser,func,dstf,arg)
-	char *(*func)();
-	FILE *dstf;
-	char *arg;
+static int flushPending(int outEOF,int ser,siFUNCP func,FILE *dstf,PCStr(arg))
 {	int pending;
 
 	if( pending = Pending ){
@@ -252,18 +271,15 @@ static flushPending(outEOF,ser,func,dstf,arg)
 	return pending;
 }
 
-getMessageF(srcf,cachefp,timeout,func,dstf,arg)
-	FILE *srcf,*cachefp;
-	char *(*func)();
-	FILE *dstf;
-	char *arg;
-{	char head[MSG_SIZE+1];
-	int total,ser,rc,rcc1,rcc,pending,pen;
-	char buff[MSG_SIZE+1];
-	char *op;
+FileSize getMessageFX(FILE *srcf,FILE *cachefp,int timeout,siFUNCP func,FILE *dstf,PCStr(arg),PCStr(encode))
+{	CStr(head,MSG_SIZE+1);
+	FileSize total;
+	int ser,rc,rcc1,rcc,pending,pen;
+	CStr(buff,MSG_SIZE+1);
+	const char *op;
 	int linemode;
-	char ext[128];
 	int outEOF;
+	int decode64,cleng;
 
 	total = 0;
 	head[0] = 0;
@@ -280,7 +296,7 @@ getMessageF(srcf,cachefp,timeout,func,dstf,arg)
 			}
 
 		head[0] = 0;
-		if( fgetsTimeout(head,sizeof(head),srcf,timeout) == NULL ){
+		if( fgetsTimeout(AVStr(head),sizeof(head),srcf,timeout) == NULL ){
 			sv1log("getMessageF: fgets timeout(%d)\n",timeout);
 			break;
 		}
@@ -295,21 +311,22 @@ getMessageF(srcf,cachefp,timeout,func,dstf,arg)
 		}
 		if( head[0] == '$' ){
 			total += flushPending(outEOF,ser,func,dstf,arg);
-			unescape_nl(head);
+			unescape_nl(AVStr(head));
 			(*func)(0,head+1,strlen(head)-1,dstf,arg);
 			sv1Vlog("getMessageF: End of Block\n");
 			break;
 		}
 
-		ext[0] = 0;
-		if( sscanf(head,"@%d %d %[^\r\n]\r\n",&ser,&rcc,ext) < 2 )
+		buff[0] = 0;
+		if( Xsscanf(head,"@%d %d %[^\r\n]\r\n",&ser,&rcc,AVStr(buff)) < 2 )
 		if( sscanf(head,"@%d - %d\r\n",&ser,&rcc) != 2 ){
 			sv1log("getMessageF: Bad block header(%d) %s\n",
 				strlen(head),head);
 			break;
 		}
 
-		sv1Vlog("getMessageF: @%d %d %s\n",ser,rcc,ext);
+		sv1Vlog("getMessageF: @%d %d %s\n",ser,rcc,buff);
+		decode64 = strcasestr(buff,BASE64) != 0;
 
 		if( 0 < rcc ){
 		    pen = pending;
@@ -323,7 +340,7 @@ getMessageF(srcf,cachefp,timeout,func,dstf,arg)
 				}
 
 			if( linemode ){
-				rc = pgets(buff,rcc,srcf);
+				rc = pgets(AVStr(buff),rcc,srcf);
 				if( rc == 0 ){
 					pending = pen+rcc;
 					break;
@@ -338,17 +355,22 @@ getMessageF(srcf,cachefp,timeout,func,dstf,arg)
 
 /*
 951026
-rc = freadIntime(buff,1,rcc1,srcf,(double)1);
+rc = freadIntime(AVStr(buff),1,rcc1,srcf,(double)1);
 timeouting often make the received packet collupted (in back-linemode only ?
 */
-				rc = freadTIMEOUT(buff,1,rcc1,srcf);
+				rc = freadTIMEOUT(AVStr(buff),1,rcc1,srcf);
 				if( rc == 0 ){
 					sv1log("getMessageF: premature EOF\n");
 					break;
 				}
-				op = (*func)(ser,buff,rc,dstf,arg);
+				if( decode64 ){
+					cleng =
+					str_from64(buff,rc,AVStr(head),sizeof(head));
+					Bcopy(head,buff,cleng);
+				}else	cleng = rc;
+				op = (*func)(ser,buff,cleng,dstf,arg);
 				if( cachefp )
-					fwrite(buff,1,rc,cachefp);
+					fwrite(buff,1,cleng,cachefp);
 
 				if( op && streq(op,"back-linemode") ){
 					sv1log("getMessageF: back-linemode\n");
@@ -357,37 +379,62 @@ timeouting often make the received packet collupted (in back-linemode only ?
 					rc = 0;
 				}
 			}
+			if( op && streq(op,"ABORT") ){
+				goto EXIT;
+			}
 			if( op && streq(op,"EOF") ){
 				if( outEOF <= 1 )
 					sv1log("getMessageF: EOF/output\n");
 				outEOF++;
 				break;
 			}
-			total += rc;
+			total += cleng;
 		    }
 		}
 	}
 	if( dstf != NULL ) fflushTIMEOUT(dstf);
 EXIT:
-	sv1vlog("getMessageF: done(%d) + %d\n",total,Pending);
+	if( op && streq(op,"ABORT") ){
+		int len = 0;
+		sv1log("-- ABORT ...\n");
+		for(;;){
+			if(fgetsTimeout(AVStr(head),sizeof(head),srcf,5)==NULL)
+				break;
+			/*
+			sv1log("-- %s",head);
+			*/
+			if( head[0] == '$' ){
+				break;
+			}
+			len += strlen(head);
+		}
+		sv1log("-- ABORT DONE skipped %d\n",len);
+		total += len;
+	}
+	sv1vlog("getMessageF: done(%lld) + %d\n",total,Pending);
 	total += flushPending(outEOF,ser,func,dstf,arg);
 	return total;
 }
-
-static char *cpy1(ser,buff,leng,dst)
-	char *buff;
-	FILE *dst;
+static const char *cpy1(int ser,PCStr(buff),int leng,FILE *dst,PCStr(encode))
 {	int wcc;
+	CStr(xbuff,MSG_SIZE*2);
+	int xsize;
 
 	if( ser <= 0 )
 		return "";
-	sv1Vlog("cpy1: @%d %d %s\r\n",ser,leng,"");
-	fprintf(dst,"@%d %d %s\r\n",ser,leng,"");
+
+	if( xsize = putmsghead("cpy1",ser,buff,leng,dst,encode,AVStr(xbuff)) ){
+		buff = xbuff;
+		leng = xsize;
+	}
+
 	wcc = fwrite(buff,1,leng,dst);
 	return "";
 }
-cpyMessageF(src,dst)
-	FILE *src,*dst;
+FileSize cpyMessageFX(FILE *src,FILE *dst,FILE *cachefp,PCStr(encode))
 {
-	return getMessageF(src,NULL,0,cpy1,dst);
+	/*
+	return getMessageFX(src,cachefp,0,cpy1,dst,"",encode);
+	*/
+	return getMessageFX(src,cachefp,0,cpy1,dst,encode,"");
 }
