@@ -1,11 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1994-1999 Yutaka Sato
-Copyright (c) 1994-1999 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1994-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, and distribute this material for any purpose
-and without fee is hereby granted, provided that the above copyright
-notice and this permission notice appear in all copies.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -17,24 +20,39 @@ History:
 	941008	created
 //////////////////////////////////////////////////////////////////////#*/
 #include <stdio.h>
+#include "yarg.h"
+#include "ystring.h"
+#include "fpoll.h"
 #include "param.h"
 #include "delegate.h"
 #include "filter.h"
-#include "ystring.h"
-extern char *fgetsTIMEOUT();
-extern char *getClientUserMbox();
-extern char *CTX_mount_url_to();
-extern FILE *TMPFILE();
+#include "file.h"
+#include "auth.h"
+#include "proc.h"
+
 #define LNSIZE 1024
 
-static putget(xcode,ts,fs,resp,rsize,fmt,a,b,c,d)
-	FILE *ts,*fs;
-	char *resp;
-	char *fmt,*a,*b,*c,*d;
-{	char req[LNSIZE];
+int POPlistmax = 30;
+static scanListFunc scan1(PCStr(conf),Connection *Conn){
+	CStr(nam,128);
+	CStr(val,2048);
+
+	fieldScan(conf,nam,val);
+	if( streq(nam,"listmax") ){
+		POPlistmax = atoi(val);
+	}
+	return 0;
+}
+void scan_POPCONF(Connection *Conn,PCStr(conf)){
+	scan_commaListL(conf,0,scanListCall scan1,Conn);
+}
+
+static int putget(int xcode,FILE *ts,FILE *fs,PVStr(resp),int rsize,PCStr(fmt),...)
+{	CStr(req,LNSIZE);
+	VARGS(4,fmt);
 
 	if( fmt != NULL ){
-		sprintf(req,fmt,a,b,c,d);
+		sprintf(req,fmt,VA4);
 		strcat(req,"\r\n");
 		if( strncasecmp(req,"PASS",4) == 0 )
 			syslog_ERROR(">> PASS ****\n");
@@ -57,7 +75,7 @@ static putget(xcode,ts,fs,resp,rsize,fmt,a,b,c,d)
 }
 
 typedef struct {
-	char	*name;
+  const	char	*name;
 	int	 resp;
 } CommandSpec;
 
@@ -83,6 +101,8 @@ static CommandSpec pop_command[] = {
 	{ "APOP"},
 	{ "UIDL", WITH_BODY|NULL_BODY_IFARG },
 	{ "TOP",  WITH_BODY|IN_RFC822 },
+	{ "AUTH", WITH_BODY|NULL_BODY_IFARG },
+	{ "CAPA", WITH_BODY },
 
 	/* private extension ? */
 	{ "LAST"},
@@ -90,9 +110,7 @@ static CommandSpec pop_command[] = {
 	0
 };
 
-static openARTICLE(ts,fs,anum,resp)
-	FILE *ts,*fs;
-	char *resp;
+static int openARTICLE(FILE *ts,FILE *fs,int anum,PVStr(resp))
 {
 	fprintf(ts,"ARTICLE %d\r\n",anum);
 	fflush(ts);
@@ -101,9 +119,8 @@ static openARTICLE(ts,fs,anum,resp)
 		return -1;
 	else	return 0;
 }
-static skipmsg(fs)
-	FILE *fs;
-{	char line[LNSIZE];
+static int skipmsg(FILE *fs)
+{	CStr(line,LNSIZE);
 	int size;
 
 	size = 0;
@@ -114,15 +131,12 @@ static skipmsg(fs)
 	}
 	return size;
 }
-static msguid(fs,uid)
-	FILE *fs;
-	char *uid;
+static void msguid(FILE *fs,PVStr(uid))
 {
-	msgMD5(fs,NULL,uid);
+	msgMD5(fs,NULL,(char*)uid);
 }
-static top(fs,tc,lines)
-	FILE *fs,*tc;
-{	char line[LNSIZE];
+static void top(FILE *fs,FILE *tc,int lines)
+{	CStr(line,LNSIZE);
 	int size;
 	int li;
 
@@ -146,33 +160,35 @@ static top(fs,tc,lines)
 		size += strlen(line);
 	}
 	skipmsg(fs);
-	return size;
 }
-static putclient(tc,fmt,a,b,c,d,e,f,g)
-	FILE *tc;
-	char *fmt,*a,*b,*c,*d,*e,*f,*g;
-{	char xfmt[LNSIZE];
+static void putclient(FILE *tc,PCStr(fmt),...)
+{	CStr(xfmt,LNSIZE);
+	VARGS(8,fmt);
 
-	fprintf(tc,fmt,a,b,c,d,e,f,g);
+	fprintf(tc,fmt,VA8);
 	strcpy(xfmt,"D-C ");
 	strcat(xfmt,fmt);
-	Verbose(xfmt,a,b,c,d,e,f,g);
+	Verbose(xfmt,VA8);
 }
-nntp_via_pop(Conn,url,fc,tc)
-	Connection *Conn;
-	char *url;
-	FILE *fc,*tc;
-{	char proto[LNSIZE],host[LNSIZE],port[LNSIZE],group[LNSIZE];
+void nntp_via_pop(Connection *Conn,PCStr(url),FILE *fc,FILE *tc)
+{	CStr(proto,LNSIZE);
+	CStr(host,LNSIZE);
+	CStr(port,LNSIZE);
+	CStr(group,LNSIZE);
 	int ni,porti;
 	FILE *ts,*fs,*tmp;
-	char line[LNSIZE],com[LNSIZE],*ap,arg[LNSIZE];
-	char resp[LNSIZE],*rp;
+	CStr(line,LNSIZE);
+	CStr(com,LNSIZE);
+	const char *ap;
+	CStr(arg,LNSIZE);
+	CStr(resp,LNSIZE);
+	refQStr(rp,resp); /**/
 	int arts,min,max;
 	int anum;
 	int size,lines;
-	char uid[LNSIZE];
+	CStr(uid,LNSIZE);
 
-	ni = scan_protositeport(url,proto,host,port);
+	ni = scan_protositeport(url,AVStr(proto),AVStr(host),AVStr(port));
 	if( ni < 2 ){
 		fprintf(tc,"-ERR\r\n");
 		return;
@@ -183,7 +199,7 @@ nntp_via_pop(Conn,url,fc,tc)
 		if( porti == 0 )
 			porti = 119;
 	}
-	if( sscanf(url,"nntp://%*[^/]/%s",group) <= 0 ){
+	if( Xsscanf(url,"nntp://%*[^/]/%s",AVStr(group)) <= 0 ){
 		fprintf(tc,"-ERR\r\n");
 		return;
 	}
@@ -199,9 +215,10 @@ nntp_via_pop(Conn,url,fc,tc)
 
 	rp = resp;
 	if( fgets(rp,sizeof(resp),fs) == NULL ){
-		fprintf(tc,"+ERR gatewaying to NNTP server: closed\r\n");
+		fprintf(tc,"-ERR gatewaying to NNTP server: closed\r\n");
 		return;
 	}
+
 	rp = strchr(rp,'\r');
 	strcpy(rp," // ");
 	rp += strlen(rp);
@@ -215,12 +232,18 @@ nntp_via_pop(Conn,url,fc,tc)
 	fgets(rp,sizeof(resp),fs);
 
 	if( atoi(rp) != 211 ){
-		putclient(tc,"+ERR gatewaying to NNTP serveri // %s",resp);
+		putclient(tc,"-ERR gatewaying to NNTP serveri // %s",resp);
 		return;
 	}
 	putclient(tc,"+OK gatewaying to NNTP server // %s",resp);
 	sscanf(rp,"211 %d %d %d",&arts,&min,&max);
 	sv1log("GROUP %d %d %d %s\n",arts,min,max,group);
+	if( POPlistmax ){
+		if( POPlistmax < max - min ){
+			min = max - POPlistmax;
+			sv1log("POPlistmax=%d %d-%d\n",POPlistmax,min,max);
+		}
+	}
 
 	tmp = TMPFILE("NNTP/POP");
 
@@ -249,7 +272,7 @@ nntp_via_pop(Conn,url,fc,tc)
 		if( strcaseeq(com,"LIST") ){
 			if( arg[0] ){
 				anum = atoi(arg);
-				if( openARTICLE(ts,fs,anum,resp) == 0 ){
+				if( openARTICLE(ts,fs,anum,AVStr(resp)) == 0 ){
 					size = skipmsg(fs);
 					putclient(tc,"+OK %d %d\r\n",anum,size);
 				}else	putclient(tc,"+ERR %s",resp);
@@ -258,7 +281,7 @@ nntp_via_pop(Conn,url,fc,tc)
 					arts,arts*10000);
 
 				for( anum = min; anum <= max; anum++ )
-				if( openARTICLE(ts,fs,anum,resp) == 0 ){
+				if( openARTICLE(ts,fs,anum,AVStr(resp)) == 0 ){
 					size = skipmsg(fs);
 					putclient(tc,"%d %d\r\n",anum,size);
 					if( anum % 10 == 0 )
@@ -270,8 +293,8 @@ nntp_via_pop(Conn,url,fc,tc)
 		if( strcaseeq(com,"UIDL") ){
 			if( arg[0] ){
 				anum = atoi(arg);
-				if( openARTICLE(ts,fs,anum,resp) == 0 ){
-					msguid(fs,uid);
+				if( openARTICLE(ts,fs,anum,AVStr(resp)) == 0 ){
+					msguid(fs,AVStr(uid));
 					putclient(tc,"+OK %d %s\r\n",anum,uid);
 				}else	putclient(tc,"+ERR %s",resp);
 			}else{
@@ -279,8 +302,8 @@ nntp_via_pop(Conn,url,fc,tc)
 					arts,arts*10000);
 
 				for( anum = min; anum <= max; anum++ )
-				if( openARTICLE(ts,fs,anum,resp) == 0 ){
-					msguid(fs,uid);
+				if( openARTICLE(ts,fs,anum,AVStr(resp)) == 0 ){
+					msguid(fs,AVStr(uid));
 					putclient(tc,"%d %s\r\n",anum,uid);
 					if( anum % 10 == 0 )
 						fflush(tc);
@@ -290,7 +313,7 @@ nntp_via_pop(Conn,url,fc,tc)
 		}else
 		if( strcaseeq(com,"RETR") ){
 			anum = atoi(arg);
-			if( openARTICLE(ts,fs,anum,resp) != 0 )
+			if( openARTICLE(ts,fs,anum,AVStr(resp)) != 0 )
 				putclient(tc,"-ERR %s",resp);
 			else{
 				fseek(tmp,0,0);
@@ -317,7 +340,7 @@ nntp_via_pop(Conn,url,fc,tc)
 			if( sscanf(arg,"%d %d",&anum,&lines) < 1 ){
 				putclient(tc,"-ERR arg count.\r\n");
 			}else
-			if( openARTICLE(ts,fs,anum,resp) != 0 )
+			if( openARTICLE(ts,fs,anum,AVStr(resp)) != 0 )
 				putclient(tc,"-ERR %s",resp);
 			else{
 				fseek(tmp,0,0);
@@ -335,14 +358,15 @@ nntp_via_pop(Conn,url,fc,tc)
 	}
 }
 
-static checkPASS(Conn,opts,epass,seed,fc,tc,resp)
-	Connection *Conn;
-	char *opts,*epass,*seed;
-	FILE *fc,*tc;
-	char *resp;
-{	char *pp,pass[LNSIZE],upass[LNSIZE];
-	char xpass[LNSIZE],mpass[LNSIZE];
-	char line[LNSIZE],com[LNSIZE],*vp;
+static int checkPASS(Connection *Conn,PCStr(opts),PCStr(epass),PCStr(seed),FILE *fc,FILE *tc,PVStr(resp))
+{	const char *pp;
+	CStr(pass,LNSIZE);
+	CStr(upass,LNSIZE);
+	CStr(xpass,LNSIZE);
+	CStr(mpass,LNSIZE);
+	CStr(line,LNSIZE);
+	CStr(com,LNSIZE);
+	const char *vp;
 
 	if( pp = strstr(opts,"apop=") ){
 		if( *epass == 0 ){
@@ -351,7 +375,7 @@ static checkPASS(Conn,opts,epass,seed,fc,tc,resp)
 				return -1;
 			}
 		}else{
-			sscanf(pp+5,"%[^ \t\r\n,]",pass);
+			Xsscanf(pp+5,"%[^ \t\r\n,]",AVStr(pass));
 			sprintf(xpass,"%s%s",seed,pass);
 			toMD5(xpass,mpass);
 			if( strcmp(mpass,epass) != 0 ){
@@ -362,7 +386,7 @@ static checkPASS(Conn,opts,epass,seed,fc,tc,resp)
 		}
 	}
 	if( pp = strstr(opts,"pass=") ){
-		sscanf(pp+5,"%[^ \t\r\n,]",pass);
+		Xsscanf(pp+5,"%[^ \t\r\n,]",AVStr(pass));
 		fprintf(tc,"+OK password required\r\n");
 		fflush(tc);
 		if( fgets(line,sizeof(line),fc) == NULL )
@@ -376,8 +400,7 @@ static checkPASS(Conn,opts,epass,seed,fc,tc,resp)
 	}
 	return 0;
 }
-static mount1(with,src,proto,user,pass,hostn,iport,path,opts)
-	char *with,*src,*proto,*user,*pass,*hostn,*iport,*path,*opts;
+static int mount1(int *with,PCStr(src),PCStr(proto),PCStr(user),PCStr(pass),PCStr(hostn),PCStr(iport),PCStr(path),PCStr(opts))
 {
 	if( strstr(opts,"apop=") ){
 		*with = 1;
@@ -385,40 +408,74 @@ static mount1(with,src,proto,user,pass,hostn,iport,path,opts)
 	}
 	return 0;
 }
-static withAPOP(Conn)
-	Connection *Conn;
+
+void service_pop1(Connection *Conn,PCStr(user),PVStr(nextUSER));
+
+static int withAPOP(Connection *Conn)
 {	int with;
+	CStr(dom,128);
 	with = 0;
 
-	CTX_scan_mtab(Conn,mount1,&with);
+	if( withAuthDigest(Conn,AVStr(dom)) )
+		return 1;
+	CTX_scan_mtab(Conn,(iFUNCP)mount1,&with);
 	return with;
 }
 
-static change_server(Conn,fc,tc,auser,epass,seed,nextUSER,resp)
-	Connection *Conn;
-	FILE *fc,*tc;
-	char *auser,*epass,*seed;
-	char *nextUSER;
-	char *resp;
-{	char *dp,userhost[LNSIZE],user[LNSIZE],hostport[LNSIZE],host[LNSIZE];
-	char tmp[LNSIZE];
-	char *opts;
+const char *POP_USERHOST_DELIMITER = "*%#";
+const char *POP_USERHOST_ESCDELIM = "*%"; /* escaped "@" for chained proxies */
+static int change_server(Connection *Conn,FILE *fc,FILE *tc,PCStr(auser),PCStr(epass),PCStr(seed),PVStr(nextUSER),PVStr(resp))
+{	const char *dp;
+	CStr(user,LNSIZE);
+	CStr(hostport,LNSIZE);
+	CStr(userhost,LNSIZE);
+	CStr(host,LNSIZE);
+	CStr(tmp,LNSIZE);
+	const char *opts;
 	int port;
+	CStr(dom,128);
+	CStr(proto,32);
 
+	strcpy(proto,"pop");
 	strcpy(userhost,auser);
 
 	if( strchr(userhost,'@') == NULL )
-	if( dp = strrchr(userhost,'%') )
-		*dp = '@'; /* from user%host1%host2 to user%host1@host2 */
+	if( dp = strrpbrk(userhost,POP_USERHOST_DELIMITER) )
+		*(char*)dp = '@';  /**//* from user%host1%host2 to user%host1@host2 */
 
-	if( sscanf(userhost,"%[^@]@%s",user,hostport) == 2 ){
-		if( dp = strrchr(user,'%') )
-			*dp = '@';
+	if( dp = strrchr(userhost,'@') ){
+		QStrncpy(user,userhost,dp-userhost+1);
+		wordScan(dp+1,hostport);
+		if( (dp = strstr(hostport,"..")) && isdigits(dp+2) ){
+			strsubst(QVStr(dp,hostport),"..",":");
+		}
+		if( streq(CLNT_PROTO,"pop3s") && (ClientFlags & PF_MITM_DO) ){
+			strcpy(proto,"pop3s");
+			ServerFlags |= (PF_SSL_IMPLICIT | PF_STLS_DO);
+		}
+		if( dp = strrpbrk(user,POP_USERHOST_ESCDELIM) )
+			*(char*)dp = '@'; /**/
 		sprintf(userhost,"//%s/%s",hostport,user);
 	}
-	opts = CTX_mount_url_to(Conn,NULL,"GET",userhost);
+	if( withAuthDigest(Conn,AVStr(dom)) ){
+		CStr(mpass,64);
+		if( authAPOP(Conn,dom,auser,seed,AVStr(mpass)) != 0 ){
+			sprintf(resp,"-ERR wrong username\r\n");
+			return 0;
+		}
+		sv1log("APOP Digest pass = %s : %s\n",epass,mpass);
+		if( !streq(mpass,epass) ){
+			sprintf(resp,"-ERR wrong APOP password\r\n");
+			return 0;
+		}
+	}
+	opts = CTX_mount_url_to(Conn,NULL,"GET",AVStr(userhost));
+	setMountOptions(FL_ARG,Conn,opts);
+	/*
+	MountOptions = opts;
+	*/
 	if( opts ){
-		if( checkPASS(Conn,opts,epass,seed,fc,tc,resp) != 0 )
+		if( checkPASS(Conn,opts,epass,seed,fc,tc,AVStr(resp)) != 0 )
 			return 0;
 	}
 
@@ -431,42 +488,72 @@ static change_server(Conn,fc,tc,auser,epass,seed,nextUSER,resp)
 		strcpy(tmp,userhost+4);
 		strcpy(userhost,tmp);
 	}
-	if( sscanf(userhost,"//%[^/]/%s",hostport,user) == 2 ){
-		port = scan_hostportX("pop",hostport,host,sizeof(host));
+	if( strncasecmp(userhost,"pop3s:",6) == 0 ){
+		strcpy(tmp,userhost+6);
+		strcpy(userhost,tmp);
+		strcpy(proto,"pop3s");
+		ServerFlags |= (PF_SSL_IMPLICIT | PF_STLS_DO);
+	}
+	if( Xsscanf(userhost,"//%[^/]/%s",AVStr(hostport),AVStr(user)) == 2 ){
+		/*
+		port = scan_hostportX("pop",hostport,AVStr(host),sizeof(host));
 		set_realserver(Conn,"pop",host,port);
-		service_pop1(Conn,user,nextUSER);
+		*/
+		port = scan_hostportX(proto,hostport,AVStr(host),sizeof(host));
+		set_realserver(Conn,proto,host,port);
+		service_pop1(Conn,user,AVStr(nextUSER));
 		return 1;
 	} 
 	return 0;
 }
 
-static ProxyPOP(Conn)
-	Connection *Conn;
-{	char myhost[LNSIZE],banner[LNSIZE];
+#define acceptSTLS(Conn) \
+	((ClientFlags&PF_STLS_DO) && (ClientFlags&(PF_STLS_ON|PF_SSL_ON))==0)
+
+static void ProxyPOP(Connection *Conn)
+{	CStr(myhost,LNSIZE);
+	CStr(banner,LNSIZE);
 	FILE *tc,*fc;
-	char req[LNSIZE],com[LNSIZE],*vp;
-	char seed[LNSIZE],user[LNSIZE],pass[LNSIZE],resp[LNSIZE];
+	CStr(req,LNSIZE);
+	CStr(com,LNSIZE);
+	const char *vp;
+	CStr(seed,LNSIZE);
+	CStr(user,LNSIZE);
+	CStr(pass,LNSIZE);
+	CStr(resp,LNSIZE);
 
 	fc = fdopen(FromC,"r");
 	tc = fdopen(ToC,"w");
 
-	ClientIF_name(Conn,FromC,myhost);
+	ClientIF_name(Conn,FromC,AVStr(myhost));
 	sprintf(banner,"+OK Proxy-POP server (%s) at %s starting.",
 		DELEGATE_version(),myhost);
 	if( withAPOP(Conn) ){
 		sprintf(seed,"<%d.%d@%s>",time(0),getpid(),myhost);
-		sprintf(banner+strlen(banner)," %s",seed);
+		Xsprintf(TVStr(banner)," %s",seed);
 	}
 	strcat(banner,"\r\n");
 	fputs(banner,tc);
 	fflush(tc);
 
-	while( fgetsTIMEOUT(req,sizeof(req),fc) != NULL ) GOT: {
+	while( fgetsTIMEOUT(AVStr(req),sizeof(req),fc) != NULL ) GOT: {
 		vp = wordScan(req,com);
 		if( strcasecmp(com,"XECHO") == 0 ){
 			lineScan(vp,resp);
 			fprintf(tc,"%s\r\n",resp);
 		}else
+		if( strcaseeq(com,"CAPA") ){
+			fputs("+OK Capability list follows\r\n",tc);
+			if( !needSTLS(Conn) ){
+				fputs("USER\r\n",tc);
+				if( withAPOP(Conn) )
+					fputs("APOP\r\n",tc);
+			}
+			if( acceptSTLS(Conn) ){
+				fputs("STLS\r\n",tc);
+			}
+			fputs(".\r\n",tc);
+		}else
 		if( strcaseeq(com,"USER") || strcaseeq(com,"APOP") ){
 			sv1log("POP C-S: %s",req);
 			req[0] = 0;
@@ -478,7 +565,7 @@ static ProxyPOP(Conn)
 				wordScan(vp,pass);
 			}
 			resp[0] = 0;
-			if( change_server(Conn,fc,tc,user,pass,seed,req,resp) ){
+			if( change_server(Conn,fc,tc,user,pass,seed,AVStr(req),AVStr(resp)) ){
 				if( req[0] )
 					goto GOT;
 				else	break;
@@ -498,47 +585,66 @@ static ProxyPOP(Conn)
 		}
 		fflush(tc);
 	}
+	/*
 	fclose(tc);
 	fclose(fc);
+	*/
+	CTX_fcloses(FL_ARG,"POPclnt",Conn,tc,fc);
 }
-service_pop(Conn)
-	Connection *Conn;
+int closeFilters(Connection *Conn,PCStr(what));
+int service_pop(Connection *Conn)
 {
+	if( !source_permittedX(Conn) ){
+		CStr(msg,128);
+		sprintf(msg,"-ERR Forbidden\r\n");
+		write(ToC,msg,strlen(msg));
+		return -1;
+	}
 	if( isMYSELF(DST_HOST) )
-		return ProxyPOP(Conn);
-	else	return service_pop1(Conn,NULL,NULL);
+		ProxyPOP(Conn);
+	else	service_pop1(Conn,NULL,VStrNULL);
+
+	closeFilters(Conn,"POP");
+	return 0;
 }
-static closePOPserver(ts,fs)
-	FILE *ts,*fs;
-{	char resp[LNSIZE];
+/*
+static void closePOPserver(FILE *ts,FILE *fs)
+*/
+#define closePOPserver(ts,fs) closePOPserverX(FL_ARG,Conn,ts,fs)
+static void closePOPserverX(FL_PAR,Connection *Conn,FILE *ts,FILE *fs)
+{	CStr(resp,LNSIZE);
 
 	fputs("QUIT\r\n",ts);
 	fflush(ts);
 	if( fgets(resp,sizeof(resp),fs) != NULL )
 		sv1log("%s",resp);
 	else	sv1log("POP S-C EOF on closePOPserver\n");
+	/*
 	fclose(ts);
 	fclose(fs);
+	*/
+	CTX_fcloses(FL_BAR,"POPserv",Conn,ts,fs);
 }
-getPOPcharange(banner,timestamp)
-	char *banner,*timestamp;
-{	char *sp,*dp,ch;
+void getPOPcharange(PCStr(banner),PVStr(timestamp))
+{	const char *sp;
+	refQStr(dp,timestamp); /**/
+	char ch;
 
-	dp = timestamp;
+	cpyQStr(dp,timestamp);
 	if( sp = strchr(banner,'<') )
 	for(; ch = *sp; sp++ ){
-		*dp++ = ch;
+		setVStrPtrInc(dp,ch);
 		if( ch == '>' )
 			break;
 	}
-	*dp = 0;
+	setVStrEnd(dp,0);
 	if( strchr(timestamp,'@') != NULL && strtailchr(timestamp) == '>' )
 		sv1log("#### BANNER TIMESTAMP for APOP: %s\n",timestamp);
-	else	timestamp[0] = 0;
+	else	setVStrEnd(timestamp,0);
 }
-makeAPOPrequest(req,timestamp,user,pass,xpass)
-	char *req,*timestamp,*user,*pass,*xpass;
-{	char argb[LNSIZE],xpassb[LNSIZE];
+void makeAPOPrequest(PVStr(req),PCStr(timestamp),PCStr(user),PCStr(pass),char xpass[])
+{	CStr(argb,LNSIZE);
+	CStr(xpassb,LNSIZE);
 
 	if( xpass == NULL )
 		xpass = xpassb;
@@ -548,22 +654,20 @@ makeAPOPrequest(req,timestamp,user,pass,xpass)
 	sprintf(req,"APOP %s %s\r\n",user,xpass);
 	sv1log("#### %s",req);
 }
-doPopAUTH(Conn,tsp,fsp,timestamp,user,pass,resp,size)
-	Connection *Conn;
-	FILE **tsp,**fsp;
-	char *timestamp,*user,*pass,*resp;
+int doPopAUTH(Connection *Conn,FILE **tsp,FILE **fsp,PCStr(timestamp),PCStr(user),PCStr(pass),PVStr(resp),int size)
 {	FILE *ts,*fs;
-	char req[LNSIZE],*dp;
+	CStr(req,LNSIZE);
+	const char *dp;
 
 	sv1log("AUTH start.\n");
 	ts = *tsp;
 	fs = *fsp;
 
 	if( timestamp[0] ){
-		makeAPOPrequest(req,timestamp,user,pass,NULL);
+		makeAPOPrequest(AVStr(req),timestamp,user,pass,NULL);
 		if( dp = strpbrk(req,"\r\n") )
-			*dp = 0;
-		if( putget(1,ts,fs,resp,size,"%s",req) == 0 ){
+			truncVStr(dp);
+		if( putget(1,ts,fs,AVStr(resp),size,"%s",req) == 0 ){
 			sv1log("APOP OK.\n");
 			return 0;
 		}
@@ -571,28 +675,28 @@ doPopAUTH(Conn,tsp,fsp,timestamp,user,pass,resp,size)
 		ToS = FromS = -1;
 		if( connect_to_serv(Conn,FromC,ToC,0) < 0 )
 			return -1;
+		/*
 		fclose(ts);
 		fclose(fs);
+		*/
+		CTX_fcloses(FL_ARG,"POPserv",Conn,ts,fs);
 		*tsp = ts = fdopen(ToS,"w");
 		*fsp = fs = fdopen(FromS,"r");
-		if( putget(2,ts,fs,resp,size,NULL) != 0 )
+		if( putget(2,ts,fs,AVStr(resp),size,NULL) != 0 )
 			return -2;
 	}
 
-	if( putget(3,ts,fs,resp,size,"USER %s",user) != 0 )
+	if( putget(3,ts,fs,AVStr(resp),size,"USER %s",user) != 0 )
 		return -3;
 
 	if( pass != NULL )
-	if( putget(4,ts,fs,resp,size,"PASS %s",pass) != 0 )
+	if( putget(4,ts,fs,AVStr(resp),size,"PASS %s",pass) != 0 )
 		return -4;
 
 	sv1log("AUTH done.\n");
 	return 0;
 }
-retryPOPopen(Conn,tsp,fsp,user,pass,resp,size)
-	Connection *Conn;
-	FILE **tsp,**fsp;
-	char *user,*pass,*resp;
+int retryPOPopen(Connection *Conn,FILE **tsp,FILE **fsp,PCStr(user),PCStr(pass),PVStr(resp),int size)
 {
 	sv1log("POP-AUTH: APOP for USER+PASS failed: %s",resp);
 	sv1log("POP-AUTH: retrying with USER+PASS.\n");
@@ -622,17 +726,49 @@ retryPOPopen(Conn,tsp,fsp,user,pass,resp,size)
 	return 0;
 }
 
-service_pop1(Conn,user,nextUSER)
-	Connection *Conn;
-	char *user;
-	char *nextUSER;
+static void filterCAPA(Connection *Conn,PCStr(resp),FILE *fs,FILE *tc){
+	CStr(line,LNSIZE);
+	CStr(cap1,LNSIZE);
+	while( 1 ){
+		if( fgets(line,sizeof(line),fs) == 0 )
+			break;
+		Verbose("S-C: CAPA %s",line);
+		wordScan(line,cap1);
+		if( strcaseeq(cap1,"STLS") ){
+			sv1log("S-D: CAPA STLS --- removed\n");
+			continue;
+		}
+		if( line[0] == '.' && (line[1] == '\r' || line[1] == '\n') ){
+			if( acceptSTLS(Conn) ){
+				sv1log("D-C: CAPA STLS --- appended\n");
+				fputs("STLS\r\n",tc);
+			}
+			fputs(line,tc);
+			break;
+		}
+		fputs(line,tc);
+	}
+}
+int POP_STARTTLS_withCL(Connection *Conn,FILE *fc,FILE *tc,PCStr(com),PCStr(arg));
+int POP_STARTTLS_withSV(Connection *Conn,FILE *ts,FILE *fs,PCStr(user));
+
+void service_pop1(Connection *Conn,PCStr(user),PVStr(nextUSER))
 {	FILE *fc,*tc,*ts,*fs;
-	char req[LNSIZE],com[LNSIZE],arg[LNSIZE];
-	char *acom,banner[LNSIZE],resp[LNSIZE];
+	CStr(req,LNSIZE);
+	CStr(com,LNSIZE);
+	CStr(arg,LNSIZE);
+	const char *acom;
+	CStr(banner,LNSIZE);
+	CStr(resp,LNSIZE);
 	int ri,body;
-	char timestamp[LNSIZE],ch,*sp,*dp;
-	char userbuf[LNSIZE],pass[LNSIZE];
+	CStr(timestamp,LNSIZE);
+	char ch;
+	const char *sp;
+	refQStr(dp,req); /**/
+	CStr(userbuf,LNSIZE);
+	CStr(pass,LNSIZE);
 	int APOPforUSERPASS;
+	CStr(dom,128);
 
 	if( connect_to_serv(Conn,FromC,ToC,0) < 0 )
 	{
@@ -643,6 +779,16 @@ service_pop1(Conn,user,nextUSER)
 		return;
 	}
 
+	if( ServerFlags & PF_SSL_IMPLICIT )
+	if( (ServerFlags & PF_SSL_ON) == 0 )
+	if( (ServerFlags & PF_STLS_OPT) != 0 ){
+		sv1log("#### required implicit SSL: %X\n",ServerFlags);
+		sprintf(banner,"-ERR cannot setup SSL with the server\r\n");
+		write(ToC,banner,strlen(banner));
+		set_linger(ToC,DELEGATE_LINGER);
+		return;
+	}
+
 	fc = fdopen(dup(FromC),"r");
 	tc = fdopen(dup(ToC),"w");
 	ts = fdopen(ToS,"w");
@@ -651,6 +797,12 @@ service_pop1(Conn,user,nextUSER)
 	if( fgets(resp,sizeof(resp),fs) == NULL )
 		goto EXIT;
 	sv1log("POP S-D: %s",resp);
+	POP_STARTTLS_withSV(Conn,ts,fs,user);
+	if( needSTLS_SV(Conn) ){
+		sv1log("#### required negotiated STLS: %X\n",ServerFlags);
+		fprintf(tc,"-ERR cannot negotiate TLS with the server\r\n");
+		goto EXIT;
+	}
 
 	if( *resp != '+' ){
 		fputs(resp,tc);
@@ -658,8 +810,16 @@ service_pop1(Conn,user,nextUSER)
 		goto EXIT;
 	}
 
+	/*
+	POP_STARTTLS_withSV(Conn,ts,fs,user);
+	*/
+
 	strcpy(banner,resp);
-	getPOPcharange(banner,timestamp);
+	getPOPcharange(banner,AVStr(timestamp));
+	if( timestamp[0] && MountOptions && isinList(MountOptions,"noapop") ){
+		sv1log("POP ignore APOP timestamp from the server\n");
+		timestamp[0] = 0;
+	}
 
 	if( user != NULL && timestamp[0] ){
 		sprintf(resp,"+OK enter password for APOP/DeleGate: %s",
@@ -674,6 +834,19 @@ service_pop1(Conn,user,nextUSER)
 			if( fgets(resp,sizeof(resp),fs) == NULL )
 				goto EXIT;
 			sv1log("POP S-D: %s",resp);
+
+			if( *resp == '+' )
+			if( withAuthDigest(Conn,AVStr(dom)) ){
+				sprintf(req,"POP D-S: PASS ****\r\n");
+				if( putDigestPass(ts,"PASS %s\r\n",com,user) != 0 ){
+					sprintf(req,"POP D-S: PASS Unknown\n");
+					goto EXIT;
+				}
+				fflush(ts);
+				if( fgets(resp,sizeof(resp),fs) == NULL )
+					goto EXIT;
+				sv1log("POP S-D: %s",resp);
+			}
 		}
 	}
 	if( fputs(resp,tc) == EOF )
@@ -688,11 +861,17 @@ service_pop1(Conn,user,nextUSER)
 		 */
 		/* should be implemented as PollIns([fs,ts]) ... */
 		while( 0 < fPollIn(fs,1) ){
+			int fcode;
 			if( fgets(resp,sizeof(resp),fs) == NULL ){
 				sv1log("POP S-C EOF before client request\n");
 				goto EXIT;
 			}
+			fcode =
 			fputs(resp,tc);
+			if( fcode == EOF ){
+				syslog_ERROR("--POP EOF on fputs >>> client\n");
+				goto EXIT;
+			}
 		}
 		if( fflush(tc) == EOF )
 			goto EXIT;
@@ -715,26 +894,52 @@ service_pop1(Conn,user,nextUSER)
 
 		sp = wordScan(req,com);
 		if( *sp != 0 ){
-			dp = arg;
+			refQStr(ap,arg); /**/
 			for( sp++; ch = *sp; sp++ ){
 				if( ch == '\r' || ch == '\n' )
 					break;
-				*dp++ = ch;
+				setVStrPtrInc(ap,ch);
 			}
-			*dp = 0;
+			setVStrEnd(ap,0);
 		}else	arg[0] = 0;
 
+		if( POP_STARTTLS_withCL(Conn,fc,tc,com,arg) ){
+			continue;
+		}
 		if( strcasecmp(com,"USER") == 0 )
 			sv1log("POP C-S: %s",req);
 		else
 		if( strcasecmp(com,"PASS") == 0 )
 			Verbose("POP C-S: PASS ******\r\n");
+		else
+		if( isWindowsCE() ){
+			sv1log("POP C-S %s",req);
+		}
 		else	Verbose("POP C-S: %s",req);
 
+		if( method_permitted(Conn,"pop",com,1) == 0 ){
+			fprintf(tc,"-ERR (forbidden) %s\r\n",com);
+			continue;
+		}
+
 		if( strcasecmp(com,"XECHO") == 0 ){
 			fprintf(tc,"%s\r\n",arg);
 			continue;
 		}
+		if( needSTLS(Conn) ){
+			if( strcaseeq(com,"CAPA") ){
+				fputs("+OK Capability list follows\r\n",tc);
+				fputs("STLS\r\n",tc);
+				fputs(".\r\n",tc);
+				continue;
+			}
+			if( strcasecmp(com,"STLS") != 0 )
+			if( strcasecmp(com,"QUIT") != 0 )
+			{
+				fprintf(tc,"-ERR Say STLS first\r\n");
+				continue;
+			}
+		}
 
 		if( strcasecmp(com,"USER") == 0 ){
 			if( nextUSER != NULL ){
@@ -760,7 +965,7 @@ service_pop1(Conn,user,nextUSER)
 		if( user != NULL && timestamp[0] ){
 			strcpy(com,"APOP");
 			strcpy(pass,arg);
-			makeAPOPrequest(req,timestamp,user,pass,arg);
+			makeAPOPrequest(AVStr(req),timestamp,user,pass,arg);
 			APOPforUSERPASS = 1;
 		}
 
@@ -778,8 +983,11 @@ service_pop1(Conn,user,nextUSER)
 		}
 
 		if( *resp == '-' && APOPforUSERPASS )
-		if( retryPOPopen(Conn,&ts,&fs,user,pass,resp,sizeof(resp)) < 0 )
+		if( retryPOPopen(Conn,&ts,&fs,user,pass,AVStr(resp),sizeof(resp)) < 0 )
+		{
+			fprintf(tc,"-ERR bad login.\r\n");
 			goto EXIT;
+		}
 
 		body = pop_command[ri].resp;
 		if( body && arg[0] ){
@@ -800,6 +1008,9 @@ service_pop1(Conn,user,nextUSER)
 		else	Verbose("POP S-C[%d:%d]: %s",ri,body,resp);
 
 		if( body && *resp == '+' ){
+			if( strcaseeq(com,"CAPA") ){
+				filterCAPA(Conn,resp,fs,tc);
+			}else
 			switch( body & RESPFLAGS ){
 				case WITH_BODY | IN_RFC822:
 					PGPdecodeMIME(fs,tc,NULL,0xFF,1,0);
@@ -815,12 +1026,13 @@ service_pop1(Conn,user,nextUSER)
 
 		if( user != NULL )
 		if( strcaseeq(com,"PASS") || strcaseeq(com,"APOP") ){
-			char clnt[LNSIZE],serv[LNSIZE];
-			char *cluser,clhost[LNSIZE];
-			char *getClientUserC();
+			CStr(clnt,LNSIZE);
+			CStr(serv,LNSIZE);
+			const char *cluser;
+			CStr(clhost,LNSIZE);
 			int clport;
 
-			clport = getClientHostPort(Conn,clhost);
+			clport = getClientHostPort(Conn,AVStr(clhost));
 			if( (cluser = getClientUserC(Conn)) == NULL )
 				cluser = "-";
 			sprintf(clnt,"%s@%s:%d",cluser,clhost,clport);
@@ -835,24 +1047,26 @@ service_pop1(Conn,user,nextUSER)
 	}
 
 EXIT:
+	/*
 	if( ts != NULL ) fclose(ts);
 	if( fs != NULL ) fclose(fs);
 	if( tc != NULL ) fclose(tc);
 	if( fc != NULL ) fclose(fc);
+	*/
+	CTX_fcloses(FL_ARG,"POPserv",Conn,ts,fs);
+	CTX_fcloses(FL_ARG,"POPclnt",Conn,tc,fc);
 	return;
 }
 
-getlineBlind(out,in,prompt,line,size)
-	FILE *out,*in;
-	char *prompt,*line;
-{	char *dp;
-	char *av[3];
+int getlineBlind(FILE *out,FILE *in,PCStr(prompt),PVStr(line),int size)
+{	const char *dp;
+	const char *av[3]; /**/
 	int pid;
 
 	fprintf(out,"%s",prompt);
 	fflush(out);
 
-	line[0] = 0;
+	setVStrEnd(line,0);
 	av[0] = "stty";
 	av[1] = "-echo";
 	av[2] = 0;
@@ -868,30 +1082,26 @@ getlineBlind(out,in,prompt,line,size)
 	wait(0);
 
 	if( dp = strpbrk(line,"\r\n") )
-		*dp = 0;
+		truncVStr(dp);
 	fprintf(out,"\r\n");
 	return 0;
 }
-getPassword(proto,site,path,pass,size)
-	char *proto,*site,*path,*pass;
-{	char prompt[LNSIZE];
+void getPassword(PCStr(proto),PCStr(site),PCStr(path),PVStr(pass),int size)
+{	CStr(prompt,LNSIZE);
 
-	pass[0] = 0;
+	setVStrEnd(pass,0);
 
 	/* pop up window for password if any Window is available ...*/
 
 	if( isatty(fileno(stdin)) ){
 	sprintf(prompt,"Enter password for \"%s://%s/%s\": ",proto,site,path);
-	getlineBlind(stderr,stdin,prompt,pass,size);
+	getlineBlind(stderr,stdin,prompt,AVStr(pass),size);
 	}
 }
 
-POP_open(Conn,fromC,toC,tsp,fsp,user,pass,resp,rsize)
-	Connection *Conn;
-	FILE **tsp,**fsp;
-	char *user,*pass,*resp;
+int POP_open(Connection *Conn,int fromC,int toC,FILE **tsp,FILE **fsp,PCStr(user),PCStr(pass),PVStr(resp),int rsize)
 {	int sock;
-	char seed[LNSIZE];
+	CStr(seed,LNSIZE);
 	int rcode;
 
 	sock = connect_to_serv(Conn,fromC,toC,0);
@@ -902,35 +1112,47 @@ POP_open(Conn,fromC,toC,tsp,fsp,user,pass,resp,rsize)
 	}
 	*fsp = fdopen(sock,"r");
 	*tsp = fdopen(sock,"w");
-	if( putget(1,*tsp,*fsp,resp,rsize,NULL) != 0 ){
+	if( putget(1,*tsp,*fsp,AVStr(resp),rsize,NULL) != 0 ){
+		/*
 		fclose(*fsp);
 		fclose(*tsp);
+		*/
+		CTX_fcloses(FL_ARG,"POPserv",Conn,*fsp,*tsp);
 		return -1;
 	}
-	getPOPcharange(resp,seed);
-	resp[0] = 0;
-	rcode = doPopAUTH(Conn,tsp,fsp,seed,user,pass,resp,rsize);
+	POP_STARTTLS_withSV(Conn,*tsp,*fsp,"");
+
+	getPOPcharange(resp,AVStr(seed));
+	setVStrEnd(resp,0);
+	rcode = doPopAUTH(Conn,tsp,fsp,seed,user,pass,AVStr(resp),rsize);
 	return rcode;
 }
 
-poprelay_main(ac,av,xConn)
-	char *av[];
-	Connection *xConn;
+int sendmailSMTP(Connection *Conn,PCStr(smtphost),int smtpport,PCStr(mailto),PCStr(mailfrom),FILE *afp);
+
+int poprelay_main(int ac,const char *av[],Connection *xConn)
 {	Connection popConnBuf, *popConn = &popConnBuf;
 	FILE *tsp,*fsp,*tss,*fss,*afp;
-	char *spool;
-	char dest[LNSIZE];
-	char req[LNSIZE],resp[LNSIZE];
+	const char *spool;
+	CStr(dest,LNSIZE);
+	CStr(req,LNSIZE);
+	CStr(resp,LNSIZE);
 	int ai,mi;
-	char *arg;
+	const char *arg;
 	int msize,wcc;
-	char *error;
-	int delete;
-	char owner[LNSIZE],myhost[LNSIZE];
-	char proto[LNSIZE],site[LNSIZE],upath[LNSIZE];
-	char pophost[LNSIZE],user[LNSIZE],pass[LNSIZE];
+	const char *error;
+	int xdelete;
+	CStr(owner,LNSIZE);
+	CStr(myhost,LNSIZE);
+	CStr(proto,LNSIZE);
+	CStr(site,LNSIZE);
+	CStr(upath,LNSIZE);
+	CStr(pophost,LNSIZE);
+	CStr(user,LNSIZE);
+	CStr(pass,LNSIZE);
 	int maxmsg;
-	char mailto[LNSIZE];
+	CStr(mailto,LNSIZE);
+	AuthInfo ident;
 
 	ConnInit(popConn);
 	popConn->from_myself = 1;
@@ -939,40 +1161,43 @@ poprelay_main(ac,av,xConn)
 		fprintf(stderr,"Specify MAILSPOOL=pop://user@host\n");
 		Finish(-1);
 	}
-	decomp_absurl(spool,proto,site,upath,sizeof(upath));
+	decomp_absurl(spool,AVStr(proto),AVStr(site),AVStr(upath),sizeof(upath));
 	if( strcmp(proto,"pop") != 0 ){
 		fprintf(stderr,"Specify MAILSPOOL=pop://user@host\n");
 		Finish(-2);
 	}
 	if( site[0] == 0 )
 		strcpy(site,"localhost");
-	decomp_site(proto,site,user,pass,pophost,NULL);
+	decomp_siteX(proto,site,&ident);
+	wordScan(ident.i_user,user);
+	textScan(ident.i_pass,pass);
+	wordScan(ident.i_Host,pophost);
 
-	getUsername(getuid(),owner);
-	GetHostname(myhost,sizeof(myhost));
+	getUsername(getuid(),AVStr(owner));
+	GetHostname(AVStr(myhost),sizeof(myhost));
 
 	if( user[0] == 0 )
 		strcpy(user,owner);
 	if( pass[0] == 0 )
-		getPassword(proto,site,upath,pass,sizeof(pass));
+		getPassword(proto,site,upath,AVStr(pass),sizeof(pass));
 
 	scan_SERVER(popConn,spool);
 
 	dest[0] = 0;
-	delete = 0;
+	xdelete = 0;
 	maxmsg = 10;
 	strcpy(mailto,"");
 
 	for( ai = 1; ai < ac; ai++ ){
 		arg = av[ai];
 		if( streq(arg,"-DELE") )
-			delete = 1;
+			xdelete = 1;
 		else
 		if( strncasecmp(arg,"-To:",4) == 0 )
 			strcpy(mailto,arg+4);
 	}
 
-	if( POP_open(popConn,-1,-1,&tsp,&fsp,user,pass,resp,sizeof(resp)) ){
+	if( POP_open(popConn,-1,-1,&tsp,&fsp,user,pass,AVStr(resp),sizeof(resp)) ){
 		fprintf(stderr,"%s",resp);
 		Finish(-3);
 	}
@@ -982,10 +1207,10 @@ poprelay_main(ac,av,xConn)
 
 	afp = TMPFILE("POP2SMTP");
 	for( mi = 1; mi <= maxmsg; mi++ ){
-		if( putget(7,tsp,fsp,resp,sizeof(resp),"RETR %d",mi) != 0 )
+		if( putget(7,tsp,fsp,AVStr(resp),sizeof(resp),"RETR %d",mi) != 0 )
 			break;
 		fseek(afp,0,0);
-		relayRESPBODY(fsp,afp,resp,sizeof(resp));
+		relayRESPBODY(fsp,afp,AVStr(resp),sizeof(resp));
 		fflush(afp);
 		Ftruncate(afp,0,1);
 		fseek(afp,0,0);
@@ -1007,21 +1232,20 @@ poprelay_main(ac,av,xConn)
 			syslog_ERROR("DON'T DELETE: %s\n",error);
 			continue;
 		}
-		if( delete )
-		if( putget(8,tsp,fsp,resp,sizeof(resp),"DELE %d",mi) != 0 )
+		if( xdelete )
+		if( putget(8,tsp,fsp,AVStr(resp),sizeof(resp),"DELE %d",mi) != 0 )
 			break;
 	}
 	fclose(afp);
 
-	putget(-9,tsp,fsp,resp,sizeof(resp),"QUIT");
+	putget(-9,tsp,fsp,AVStr(resp),sizeof(resp),"QUIT");
 	Finish(0);
+	return 0;
 }
-sendmailSMTP(Conn,smtphost,smtpport,mailto,mailfrom,afp)
-	Connection *Conn;
-	FILE *afp;
-	char *smtphost,*mailto,*mailfrom;
-{	FILE *fpv[2];
-	char admin[LNSIZE],resp[LNSIZE];
+int sendmailSMTP(Connection *Conn,PCStr(smtphost),int smtpport,PCStr(mailto),PCStr(mailfrom),FILE *afp)
+{	FILE *fpv[2]; /**/
+	CStr(admin,LNSIZE);
+	CStr(resp,LNSIZE);
 
 	Conn->from_myself = 1;
 	if( mailfrom == NULL ){
@@ -1042,8 +1266,11 @@ sendmailSMTP(Conn,smtphost,smtpport,mailto,mailfrom,afp)
 	copyfile1(afp,fpv[1]);
 	fputs(".\r\n",fpv[1]);
 
-	putget(0,fpv[1],fpv[0],resp,sizeof(resp),"QUIT");
+	putget(0,fpv[1],fpv[0],AVStr(resp),sizeof(resp),"QUIT");
+	/*
 	fclose(fpv[0]);
 	fclose(fpv[1]);
+	*/
+	CTX_fcloses(FL_ARG,"POPSMTP",Conn,fpv[0],fpv[1]);
 	return 0;
 }
