@@ -19,14 +19,26 @@ History:
 	960606	created
 //////////////////////////////////////////////////////////////////////#*/
 #include <stdio.h>
+#include <stdlib.h>
 #include <signal.h>
 #include "ystring.h"
-extern char *getenv();
-extern char *gethostaddr();
+#include "dglib.h"
+#include "fpoll.h"
+#include "proc.h"
+#include "file.h"
 #define RHOSTS	".rhosts.delegate"
 
 static int _fromRsh;
-fromRsh()
+
+#define IS_SSH -2
+int fromRsh();
+int fromSSH(){
+	if( fromRsh() )
+		return 0;
+	return _fromRsh == IS_SSH;
+}
+
+int fromRsh()
 {	int hosti,porti,hosto,porto;
 
 	if( 0 < _fromRsh ) return 1;
@@ -40,6 +52,16 @@ fromRsh()
 	if( file_issock(0) < 0 ) return 0;
 	if( file_issock(1) < 0 ) return 0;
 
+	if( getenv("SSH_CONNECTION") != 0 ){
+		if( sock_isAFUNIX(0)
+		 || sockFromMyself(0)
+		 || sock_isconnectedX(0,0)
+		){
+			_fromRsh = IS_SSH;
+			return 0;
+		}
+	}
+
 /*
 	if( file_issock(2) < 0 ) return 0;
 */
@@ -61,8 +83,8 @@ fromRsh()
 	 */
 	if( 0 < PollIn(0,100) ){
 		int rcc,ci;
-		char buf[8];
-		rcc = recvPeekTIMEOUT(0,buf,7);
+		CStr(buf,8);
+		rcc = recvPeekTIMEOUT(0,AVStr(buf),7);
 		if( 0 < rcc )
 		if( buf[rcc-1] == '\0' )
 		if( isdigits(buf) ){
@@ -84,14 +106,18 @@ fromRsh()
 	return 1;
 }
 
-authRsh(bg)
-	int *bg;
-{	char *home,path[1024];
+int authRsh(/*int *bg*/)
+{	const char *home;
+	CStr(path,1024);
 	FILE *fp;
-	char paddr[1024],saddr[1024];
-	char line[1024],hosts[1024];
-	char phost1[1024],*paddr1; /* client's host */
-	char shost1[1024],*saddr1; /* server's host (optional) */
+	CStr(paddr,1024);
+	CStr(saddr,1024);
+	CStr(line,1024);
+	CStr(hosts,1024);
+	CStr(phost1,1024); /* client's host */
+	const char *paddr1;
+	CStr(shost1,1024); /* server's host (optional) */
+	const char *saddr1;
 	int ok;
 
 	if( (home = getenv("HOME")) == NULL ){
@@ -106,8 +132,8 @@ authRsh(bg)
 		return 1;
 	}
 
-	getpeerAddr(0,paddr);
-	gethostAddr(0,saddr);
+	getpeerAddr(0,AVStr(paddr));
+	gethostAddr(0,AVStr(saddr));
 
 	ok = 0;
 	while( fgets(line,sizeof(line),fp) ){
@@ -115,7 +141,7 @@ authRsh(bg)
 		if( hosts[0] == 0 )
 			continue;
 		phost1[0] = shost1[0] = 0;
-		if( sscanf(hosts,"%[^:]:%s",phost1,shost1) < 1 )
+		if( Xsscanf(hosts,"%[^:]:%s",AVStr(phost1),AVStr(shost1)) < 1 )
 			continue;
 
 		if( phost1[0] && strcmp(phost1,"*") != 0 ){
@@ -142,14 +168,15 @@ authRsh(bg)
 	}
 }
 
-RshWatcher(ninvoke,port){
+void serverControl(FILE *in,FILE *out);
+void RshWatcher(int ninvoke,int port){
 	/* echo dynamically allocated PORT number */
 	/* fprintf(stdout,"%d\n",port); */
 
 	/* if( foreground ) */
 	if( ninvoke == 0 ){
-		char host[256];
-		getpeerNAME(0,host);
+		CStr(host,MaxHostNameLen);
+		getpeerNAME(0,AVStr(host));
 		proc_title("delegated/rsh/%s",host);
 
 		if( Fork("RshWatcher") == 0 ){
@@ -168,9 +195,8 @@ RshWatcher(ninvoke,port){
 	*/
 }
 
-serverControl(in,out)
-	FILE *in,*out;
-{	char line[1024];
+void serverControl(FILE *in,FILE *out)
+{	CStr(line,1024);
 
 	fprintf(out,"[enter q to kill the daemon]\r\n");
 	for(;;){
@@ -186,10 +212,9 @@ serverControl(in,out)
 	}
 	Kill(getppid(),SIGTERM);
 }
-stop_server(dst)
-	FILE *dst;
+int stop_server(FILE *dst)
 {	int rcode;
-	char msg[1024];
+	CStr(msg,1024);
 
 	rcode = Kill(getppid(),SIGTERM);
 	sprintf(msg,"kill() = %d\r\n",rcode);
@@ -197,7 +222,7 @@ stop_server(dst)
 	return strlen(msg);
 }
 
-fromInetd()
+int fromInetd()
 {	int iss0,iss1;
 
 	if( isatty(0) || isatty(1) || file_isreg(0) || file_isreg(1) )
@@ -205,6 +230,9 @@ fromInetd()
 
 	if( fromRsh() )
 		return 0;
+	if( _fromRsh == IS_SSH ){
+		return 0;
+	}
 
 	iss0 = file_issock(0);
 	iss1 = file_issock(1);
