@@ -19,17 +19,25 @@ History:
 	961029	extracted from service.c
 //////////////////////////////////////////////////////////////////////#*/
 
-typedef int (*IFUNC)();
+typedef struct DGCtx *DGCP;
+typedef int servFunc(DGCP Conn);
+typedef int (*servFuncP)(DGCP Conn);
+typedef int servFunc2(DGCP Conn,int,int);
+typedef int servFuncX(DGCP Conn,int,int,int fromC,int toC,PCStr(dST_PROTO),PCStr(dST_HOST),int dST_PORT,PCStr(d_SELECTOR));
 
 typedef struct {
 	int	 s_withcache;
 	int	 s_selfack;	/* return ack in the protocol handler */
-	char	*s_name;
+  const	char	*s_name;
 	int	 s_iport;	/* default port */
-	IFUNC	 s_client;
-	char	*s_host;	/* default server host */
+       servFuncP s_client;
+	int	 s_stls;	/* STARTLS */
+  const	char	*s_Host;	/* default server host */
 
-	int	 s_forcerelay;	/* relay to MASTER allways */
-	int	 s_nocallback;
+	int	 s_nocallback;	/* should be obsoleted by
+				 MOUNT="/-_-* * proto=!{protoList}" */
+	int	 s_stats;
 } Service;
 #define VPORT	0x10000
+#define NSERVICES	100
+#define SV_DISABLE	0x00000001
