@@ -1,11 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1999 Yutaka Sato
-Copyright (c) 1999 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1999-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, and distribute this material for any purpose
-and without fee is hereby granted, provided that the above copyright
-notice and this permission notice appear in all copies.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -16,20 +19,45 @@ Description:
 History:
 	991209	extracted from delegated.c
 //////////////////////////////////////////////////////////////////////#*/
+#include "ystring.h"
 #include "vsignal.h"
 #include "delegate.h"
+#include "file.h"
+#include "proc.h"
+#include "fpoll.h"
+#include <errno.h>
 
+/*
 #define SHUTOUT	"${ADMDIR}/shutout"
-int SHUTOUT_TIMEOUT = 900;
+*/
+#define ShutOUT	"${ADMDIR}/shutout"
+static char *_shutout_dir;
+static const char *shutout_dir(){
+	if( _shutout_dir == 0 ){
+		CStr(buf,1024);
+		strcpy(buf,ShutOUT);
+		DELEGATE_substfile(AVStr(buf),"",VStrNULL,VStrNULL,VStrNULL);
+		_shutout_dir = stralloc(buf);
+	}
+	return _shutout_dir;
+}
+#define SHUTOUT shutout_dir()
+void setSHUTOUT(){
+	shutout_dir();
+}
 
-static FILE *stopServ(what,mode,host)
-	char *what,*mode,*host;
-{	char path[1024];
+int SHUTOUT_TIMEOUT = 30*60;
+
+static FILE *stopServ(PCStr(what),PCStr(mode),PCStr(host))
+{	CStr(path,1024);
 	FILE *fp;
 
+	if( host == 0 )
+		return 0;
+
 	sprintf(path,"%s/%s",SHUTOUT,host);
-	DELEGATE_substfile(path,"",NULL,NULL,NULL);
-	fp = dirfopen(what,path,mode);
+	DELEGATE_substfile(AVStr(path),"",VStrNULL,VStrNULL,VStrNULL);
+	fp = dirfopen(what,AVStr(path),mode);
 	if( fp == NULL )
 		return 0;
 	daemonlog("F","#### %s [%s]\n",what,path);
@@ -38,22 +66,29 @@ static FILE *stopServ(what,mode,host)
 
 static struct {
 	int	cl_sock;
-	char   *cl_addr;
+  const	char   *cl_addr;
 	int	cl_port;
     Connection *cl_Conn;
 } curClient;
 
-static FILE *setStopService(addr)
-	char *addr;
+static FILE *setStopService(PCStr(addr))
 {
+	/*
+	daemonlog("F","####! EMERGENCY STOP !####\n");
+	*/
+	daemonlog("F","####! EMERGENCY STOP !#### [%d.%X]%X %d/%d\n",
+		getpid(),getthreadid(),ismainthread(),
+		actthreads(),numthreads());
+
 	return stopServ("setStopService","a",addr);
 }
-static getStopService(Conn,addr)
-	Connection *Conn;
-	char *addr;
+static int getStopService(Connection *Conn,PCStr(addr))
 {	FILE *fp;
 	int mtime,now;
 
+	if( lSINGLEP() ){
+		return 0;
+	}
 	fp = stopServ("getStopService","r",addr); 
 	if( fp == NULL )
 		return 0;
@@ -73,11 +108,11 @@ static getStopService(Conn,addr)
  * hold the connection untill closed by the intruder
  * behaving like /bin/sh ...
  */
-static holder(log)
-	FILE *log;
+static void holder(FILE *log)
 {	FILE *fc,*tc;
-	char line[256],com[256];
-	char date[64];
+	CStr(line,256);
+	CStr(com,256);
+	CStr(date,64);
 
 	fc = fdopen(curClient.cl_sock,"r");
 	tc = fdopen(curClient.cl_sock,"w");
@@ -85,7 +120,7 @@ static holder(log)
 		if( fgets(line,sizeof(line),fc) == NULL )
 			break;
 		if( log ){
-			StrftimeLocal(date,sizeof(date),TIMEFORM_mdHMS,time(0));
+			StrftimeLocal(AVStr(date),sizeof(date),TIMEFORM_mdHMS,time(0),0);
 			fprintf(log,"%s: %s",date,line);
 			fflush(log);
 		}
@@ -96,7 +131,7 @@ static holder(log)
 	}
 }
 
-static char *introFATAL = "\
+static const char *introFATAL = "\
 To administrator:\n\
   Remove this file to restart the service for \"%s\"\n\
 This file was created on a fatal error detected in DeleGate,\n\
@@ -106,16 +141,15 @@ See LOGDIR/abort/%s, and consult with the author if necessary.\n\
 (You can specify the varidity of shutout like TIMEOUT=shutout:60s)\n\
 --\n";
 
-static logFATAL(log,reason,clnt,serv)
-	FILE *log;
-	char *reason,*clnt,*serv;
+static void logFATAL(FILE *log,PCStr(reason),PCStr(clnt),PCStr(serv))
 {	Connection *Conn = curClient.cl_Conn;
-	char date[64],port[256];
+	CStr(date,64);
+	CStr(port,256);
 
-	printPrimaryPort(port);
+	printPrimaryPort(AVStr(port));
 	fprintf(log,introFATAL,clnt,port);
 	fflush(log);
-	StrftimeLocal(date,sizeof(date),TIMEFORM_mdHMS,time(0));
+	StrftimeLocal(AVStr(date),sizeof(date),TIMEFORM_mdHMS,time(0),0);
 	fprintf(log,"%s: [%d] reason=%s src=%s dst=%s\n",date,getpid(),
 		reason,clnt,serv);
 	fflush(log);
@@ -127,20 +161,53 @@ static logFATAL(log,reason,clnt,serv)
 	}
 	fflush(log);
 }
-void exitFATAL(sig)
-{	FILE *clog,*slog;
-	char reason[32];
-	char *clnt,serv[256],*dp;
-	int servport;
+static int exiting;
+void SetStopService(PCStr(reason))
+{	const char *clnt;
+	FILE *clog;
 	Connection *Conn = curClient.cl_Conn;
 
+	clnt = curClient.cl_addr;
+	clog = setStopService(clnt);
+	if( clog )
+		logFATAL(clog,reason,clnt,"*:*");
+	notify_ADMIN(Conn,reason);
+}
+void exitFATAL(int sig)
+{	FILE *clog,*slog;
+	CStr(reason,32);
+	const char *clnt;
+	CStr(serv,256);
+	const char *dp;
+	int servport;
+	Connection *Conn = curClient.cl_Conn;
+	Connection ConnBuf;
+
+	if( exiting != 0 )
+		return;
+	exiting = 1;
+
+	signal(sig,SIG_DFL);
+
 	sprintf(reason,"SIG%s",sigsym(sig));
+	if( streq(reason+3,"???") ){
+		sprintf(reason,"SIG#%d",sig);
+	}
+	if( Conn == NULL ){
+		Conn = &ConnBuf;
+		bzero(Conn,sizeof(Conn));
+	}
+
+	if( sig == SIGSEGV && errno == ENOMEM ){
+		syslog_ERROR("#### EXIT on insufficient memory, exceeded data or stack limitation, or exausted swap space. ####\n");
+		Finish(-1);
+	}
 
 	serv[0] = 0;
-	if( 0 < ToS ){
-		if( getpeerName(ToS,serv,"%A:%P") ){
+	if( 0 < ToS && IsConnected(ToS,NULL) ){
+		if( getpeerName(ToS,AVStr(serv),"%A:%P") ){
 			dp = strchr(serv,':');
-			*dp++ = 0;
+			truncVStr(dp); dp++;
 			servport = atoi(dp);
 		}
 	}
@@ -173,32 +240,54 @@ void exitFATAL(sig)
 	Finish(0);
 }
 
-static putStopInfo(Conn)
-	Connection *Conn;
-{	char mssg[1024];
+static void putStopInfo(Connection *Conn)
+{	CStr(mssg,1024);
 
 	mssg[0] = 0;
 	/* set protocol dependent header */
-	sprintf(mssg+strlen(mssg),
+	Xsprintf(TVStr(mssg),
 "Services for your host[%s] are suspended now on a suspicion of intruder.\r\n",
 		curClient.cl_addr);
 
 	if( source_permitted(Conn) )
-	sprintf(mssg+strlen(mssg),
+	Xsprintf(TVStr(mssg),
 "Ask your administrator <%s> to check %s/%s\r\n",
-		getADMIN1(),SHUTOUT,curClient.cl_addr);
+		getADMIN1(),ShutOUT,curClient.cl_addr);
 
 	write(curClient.cl_sock,mssg,strlen(mssg));
 }
 
-extern int execGeneralist1();
-execGeneralist(Conn,fromC,toC,svsock)
-	Connection *Conn;
-{	char addrport[256],*dp;
+int inShutdown(Connection *Conn,int toC)
+{	CStr(addrport,256);
+	const char *dp;
+
+	if( getpeerName(ClientSock,AVStr(addrport),"%A:%P") ){
+		dp = strchr(addrport,':');
+		truncVStr(dp); dp++;
+		curClient.cl_sock = toC;
+		Strdup((char**)&curClient.cl_addr,addrport);
+		curClient.cl_port = atoi(dp);
+		curClient.cl_Conn = Conn;
+		if( getStopService(Conn,curClient.cl_addr) ){
+			putStopInfo(Conn);
+			return -1;
+		}
+	}else{
+	}
+	return 0;
+}
+void execGeneralist1(Connection *Conn,int fromC,int toC,int svsock);
+int execGeneralist(Connection *Conn,int fromC,int toC,int svsock)
+{	CStr(addrport,256);
+	const char *dp;
 	int port,rcode;
 
 	randenv();
+/*
 	if( getpeerName(fromC,addrport,"%A:%P") ){
+*/
+/*
+	if( getpeerName(ClientSock,addrport,"%A:%P") ){
 		dp = strchr(addrport,':');
 		*dp++ = 0;
 		curClient.cl_sock = toC;
@@ -211,13 +300,47 @@ execGeneralist(Conn,fromC,toC,svsock)
 		}
 	}else{
 	}
+*/
+	if( inShutdown(Conn,toC) != 0 )
+		return -1;
+
 	signal(SIGILL, exitFATAL);
 	signal(SIGTRAP,exitFATAL);
 	signal(SIGEMT, exitFATAL);
 	signal(SIGFPE, exitFATAL);
 	signal(SIGSEGV,exitFATAL);
 	signal(SIGBUS, exitFATAL);
-	rcode = randstack_call(execGeneralist1,Conn,fromC,toC,svsock);
+	rcode = randstack_call(SB_CONN,(iFUNCP)execGeneralist1,Conn,fromC,toC,svsock);
 	curClient.cl_sock = -1;
 	return rcode;
 }
+
+void Abort(int code,PCStr(fmt),...){
+	const char *body = 0;
+	VARGS(8,fmt);
+
+	if( errno == ENOMEM ){
+		body = "#### EXIT on insufficient memory, exceeded data or stack limitation, or exausted swap space. ####";
+		daemonlog("F","%s\n",body);
+	}
+	daemonlog("F",fmt,VA8);
+
+	if( code == 0 ){
+		Connection *Conn = curClient.cl_Conn;
+		Connection ConnBuf;
+		CStr(reason,256);
+		const char *dp;
+
+		if( Conn == NULL ){
+			Conn = &ConnBuf;
+			bzero(Conn,sizeof(Conn));
+		}
+		sprintf(reason,fmt,VA8);
+		if( dp = strpbrk(reason,"\r\n") )
+			truncVStr(dp);
+		notify_ADMINX(Conn,getADMIN1(),reason,body);
+		Finish(-1);
+	}else{
+		abort();
+	}
+}
