@@ -1,11 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1996-1999 Yutaka Sato
-Copyright (c) 1996-1999 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1996-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, and distribute this material for any purpose
-and without fee is hereby granted, provided that the above copyright
-notice and this permission notice appear in all copies.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -23,6 +26,7 @@ History:
 #include <stdio.h>
 #include "ystring.h"
 #include "vsocket.h"
+#include "fpoll.h"
 /*
 #include "vaddr.h"
 */
@@ -39,6 +43,16 @@ void SocksV4_server(DGC*ctx,int _1,int _2,int fromC,int toC);
 int SocksV4_isConnectReuqest(PCStr(pack),int leng,int *ver,PVStr(addr),int *port,const char **user);
 int SocksV4_acceptViaSocks(int sock,PVStr(rhost),int *rport);
 
+int connectToSox(Connection *Conn,PCStr(wh),PCStr(proto),PCStr(host),int port);
+int openServer(Connection *Conn,PCStr(wh),PCStr(proto),PCStr(host),int port){
+	int sock;
+	if( 0 <= (sock = connectToSox(Conn,"SOCKS","socks",host,port)) ){
+		return sock;
+	}else	return OpenServer(wh,proto,host,port);
+}
+#undef OpenServer
+#define OpenServer(wh,proto,host,port) openServer(Conn,wh,proto,host,port)
+
 #define SOCKS_PORT	1080
 #define SOCKS_CONNECT	1
 #define SOCKS_BIND	2
@@ -50,6 +64,9 @@ typedef struct {
 	int	 s_ver;
 	int	 s_rslv; /* V4:delegate resolution, V5:resolv in mylsef */
 	int	 s_sock;
+  const char	*s_dsts;
+	VSAddr	*s_svaddr;
+	VSAddr	*s_svlocal;
 } SocksServer;
 
 typedef struct {
@@ -130,6 +147,9 @@ static void socks_init(DGC*_,PCStr(svhost),PCStr(svportver),PCStr(dsts),PCStr(sr
 	sockservs[sx].s_Host = stralloc(svhost);
 	sockservs[sx].s_port = port;
 	sockservs[sx].s_ver = ver;
+	sockservs[sx].s_svaddr = 0;
+	sockservs[sx].s_svlocal = 0;
+	sockservs[sx].s_dsts = stralloc(dsts?dsts:"*");
 
 	sprintf(key,"SOCKS-%d",sx);
 /*
@@ -146,6 +166,9 @@ static int matchConn(DGC*Conn,int sx)
 	CStr(map,128);
 	int found;
 
+	if( Conn->no_dstcheck & 0x8/*NO_SOCKSDST_CHECK*/ ){
+		return 1;
+	}
 	sprintf(key,"SOCKS-%d",sx);
 	CTX_pushClientInfo(Conn);
 	found = find_CMAP(Conn,key,AVStr(map));
@@ -170,26 +193,75 @@ static int matchConnX(DGC*Conn,int sx,PCStr(dstproto),PCStr(dsthost),int dstport
 	}
 	return found;
 }
+
+static int retServ(DGC*Conn,SocksServer *sv,VSAddr *vserv,VSAddr *vlocal){
+	if( vserv == 0 && vlocal == 0 ){
+		return 0;
+	}
+	if( vserv && sv->s_svaddr == 0 ){
+		const char *addr;
+		if( (addr = gethostaddr(sv->s_Host)) == 0 ){
+			sv1log("##SOCKS-serv. Unknown %s\n",sv->s_Host);
+			bzero(vlocal,sizeof(VSAddr));
+			bzero(vserv,sizeof(VSAddr));
+			return -1;
+		}
+		sv->s_svaddr = (VSAddr*)malloc(sizeof(VSAddr));
+		VSA_atosa(sv->s_svaddr,sv->s_port,addr);
+	}
+	if( vlocal ){
+		if( sv->s_svlocal )
+			*vlocal = *sv->s_svlocal;
+		else	bzero(vlocal,sizeof(VSAddr));
+	}
+	if( vserv ){
+		if( sv->s_svaddr )
+			*vserv = *sv->s_svaddr;
+		else	bzero(vserv,sizeof(VSAddr));
+	}
+	return 0;
+}
+
+/*
 static int tobeViaSocks(DGC*Conn)
+*/
+static SocksServer *tobeViaSocks(DGC*Conn)
 {	int sx;
 
 	for( sx = 0; sockservs[sx].s_Host; sx++ ){
 		if( matchConn(Conn,sx) )
+			return &sockservs[sx];
+			/*
 			return 1;
+			*/
 	}
 	return 0;
 }
 
 int VSA_getViaSocks(DGC*Conn,PCStr(host),int port,VSAddr *vlocal);
+/*
 static int getViaSocks(PCStr(host),int port,VSAddr *vlocal);
+*/
+static SocksServer *getViaSocks(PCStr(host),int port,VSAddr *vlocal);
+int VSA_getViaSocksX(DGC*Conn,PCStr(host),int port,VSAddr *vserv,AuthInfo *au,VSAddr *vlocal);
 
 int GetViaSocks(DGC*Conn,PCStr(host),int port)
 {
 	return VSA_getViaSocks(Conn,host,port,NULL);
 }
 int VSA_getViaSocks(DGC*Conn,PCStr(host),int port,VSAddr *vlocal)
+{
+	return VSA_getViaSocksX(Conn,host,port,NULL,NULL,vlocal);
+}
+int forwardtoSocks(DGC*Conn,VSAddr *sv);
+int VSA_getViaSocksX(DGC*Conn,PCStr(host),int port,VSAddr *vserv,AuthInfo *auth,VSAddr *vlocal)
 {	DGC toConn;
 	int match,sx;
+	SocksServer *sv;
+
+	if( vserv ) bzero(vserv,sizeof(VSAddr));
+	if( vlocal) bzero(vlocal,sizeof(VSAddr));
+	if( auth  ) bzero(auth,sizeof(AuthInfo));
 
 	/*
 	... it should be controlled by tobeViaSocks() ...
@@ -198,15 +270,28 @@ int VSA_getViaSocks(DGC*Conn,PCStr(host),int port,VSAddr *vlocal)
 		return 0;
 	}
 	*/
+	/*
 	if( getViaSocks(host,port,vlocal) ){
+	*/
+	if( sv = getViaSocks(host,port,vlocal) ){
+		retServ(Conn,sv,vserv,vlocal);
 		sv1log("##ViaSocks-A(by cache)## %s:%d\n",host,port);
 		return 1;
 	}
 	toConn = *Conn;
 	set_realserver(&toConn,DST_PROTO,host,port);
+	/*
 	if( tobeViaSocks(&toConn) ){
+	*/
+	if( sv = tobeViaSocks(&toConn) ){
+		retServ(Conn,sv,vserv,vlocal);
 		sv1log("##ViaSocks-B(by rule)## %s:%d\n",host,port);
-		if( vlocal ) bzero(vlocal,sizeof(VSAddr));
+		return 1;
+	}
+	if( forwardtoSocks(&toConn,vserv) ){
+		if( auth && toConn.gw.p_auth) *auth = *toConn.gw.p_auth;
+		if( vlocal) bzero(vlocal,sizeof(VSAddr));
+		sv1log("##ViaSocks-C(by rule)## %s:%d\n",host,port);
 		return 1;
 	}
 	Verbose("##NOT ViaSocks-B## %s:%d\n",host,port);
@@ -221,23 +306,51 @@ void scan_SOCKS(DGC*_,PCStr(socks))
 	stoV(socksb,3,sv,':');
 	socks_init(_,sv[0],sv[1],sv[2],sv[3]);
 }
+extern int SOCKS_LOGIN_TIMEOUT;
 int socks_addservers(){
 	int sx;
+	SocksServer *ss;
 	const char *host;
 	const char *addr;
 
+	/* 9.4.3 this is for UDP but is obsolete
+	 * it's halmful for routing from SOCKS to multiple SOCKSes because
+	 * it disables selecting an appropriate upstream SOCKS server and
+	 * causes forwarding self-generated DNS query to a SOCKS server.
+	 */
+	Verbose("OBSOLETED socks_addservers() for UDP\n");
+	if( streq(iSERVER_PROTO,"socks")
+	){
+		if( sockservs[0].s_Host )
+			return 1;
+		else	return 0;
+	}
+
 	for( sx = 0; host = sockservs[sx].s_Host; sx++ )
 	if( *host )
 	{
+		ss = &sockservs[sx];
 		addr = gethostaddr(host);
 		if( addr == NULL )
 			addr = host;
+		/*
 		SOCKS_addserv("*",0,addr,sockservs[sx].s_port);
+		*/
+		SOCKS_addserv(ss->s_dsts,0,addr,ss->s_port);
 	}
 	return sx;
 }
 
+int WaitCluster(DGC*Conn);
+static int socks_startX(DGC*Conn,SocksServer*sv,int command,PCStr(host),int port,PCStr(user),PVStr(rhost),int *rport,PVStr(xaddr));
 static int socks_start(DGC*Conn,SocksServer*sv,int command,PCStr(host),int port,PCStr(user),PVStr(rhost),int *rport,PVStr(xaddr))
+{
+	int rcode;
+	rcode = socks_startX(Conn,sv,command,host,port,user,BVStr(rhost),rport,BVStr(xaddr));
+	WaitCluster(Conn);
+	return rcode;
+}
+static int socks_startX(DGC*Conn,SocksServer*sv,int command,PCStr(host),int port,PCStr(user),PVStr(rhost),int *rport,PVStr(xaddr))
 {	CStr(obuf,128);
 	char i;
 	int pc,wcc,rep;
@@ -301,6 +414,7 @@ static int socks_start(DGC*Conn,SocksServer*sv,int command,PCStr(host),int port,
 				pass = dp + 1;
 			else	pass = dp;
 		}
+		SOCKS_LOGIN_TIMEOUT = LOGIN_TIMEOUT;
 		return SOCKS_startV5(sock,command,host,port,user,pass,AVStr(rhost),rport);
 	}
 
@@ -361,14 +475,53 @@ static SocksServer *getViaSocksX(unsigned int addr,int port,VSAddr *vlocal)
 	}
 	return 0;
 }
+/*
 static int getViaSocks(PCStr(host),int port,VSAddr *vlocal)
+*/
+static SocksServer *getViaSocks(PCStr(host),int port,VSAddr *vlocal)
 {
+	SocksServer *sv;
+	if( sv = getViaSocksX(gethostint_nboV4(host),port,vlocal) ) return sv;
+	if( sv = getViaSocksX(gethostintMin(host),port,vlocal) ) return sv;
+	if( sv = getViaSocksX(_inet_addrV4(host),port,vlocal) ) return sv;
+	/*
 	if( getViaSocksX(gethostint_nboV4(host),port,vlocal) ) return 1;
 	if( getViaSocksX(gethostintMin(host),port,vlocal) ) return 1;
 	if( getViaSocksX(_inet_addrV4(host),port,vlocal) ) return 1;
+	*/
 	return 0;
 }
 
+int newsockBound(int,int,VSAddr*);
+int connectVSAddr(int sock,VSAddr *vsa);
+int SOCKS_udpassoc0X(DGC*ctx,int ssock,VSAddr *me,VSAddr *rme,AuthInfo *auth);
+/*
+void setUDPviaSocks(int usock);
+*/
+int socksudp_start(DGC*Conn,SocksServer *sv){
+	const char *aaddr;
+	VSAddr me,rme;
+	int rcode;
+	int usock;
+
+	usock = newsockBound(sv->s_sock,1,&me);
+	if( usock < 0 ){
+		return -1;
+	}
+	rcode = SOCKS_udpassoc0X(Conn,sv->s_sock,&me,&rme,GatewayAuth);
+	if( rcode == 0 ){
+		VSA_satoap(&rme,AVStr(Conn->sv.p_SOCKSADDR));
+		connectVSAddr(usock,&rme);
+		Conn->sv.p_SOCKSCTL = dup(sv->s_sock);
+		dup2(usock,sv->s_sock);
+		/*
+		setUDPviaSocks(sv->s_sock);
+		*/
+	}
+	close(usock);
+	return rcode;
+}
+
 static int socks_connect(DGC*Conn,SocksServer *sv,PCStr(host),int port,PCStr(user),PVStr(rhost),int *rport)
 {	CStr(addr,64);
 	VSAddr vlocal;
@@ -376,6 +529,12 @@ static int socks_connect(DGC*Conn,SocksServer *sv,PCStr(host),int port,PCStr(use
 	if( host == NULL || port == 0 )
 		return -1;
 
+	Conn->sv.p_SOCKSCTL = -1;
+	if( Conn->sv.p_flags & PF_UDP ){
+		Conn->sv.p_flags &= ~PF_UDP;
+		return socksudp_start(Conn,sv);
+	}
+
 	*addr = 0;
 	if( socks_start(Conn,sv,SOCKS_CONNECT,host,port,user,AVStr(rhost),rport,AVStr(addr)))
 		return -1;
@@ -389,6 +548,37 @@ static int socks_connect(DGC*Conn,SocksServer *sv,PCStr(host),int port,PCStr(use
 	return 0;
 }
 
+int needSTLS_SVi(Connection *Conn,int server);
+int insertTLS_SV(Connection *Conn,int client,int server);
+int insertCredhy(Connection *Conn,int clsock,int svsock);
+
+int SOCKS_STARTTLS_withSV(Connection *Conn,int sock,PCStr(what)){
+	int fsv;
+
+	if( 0 <= (fsv = insertCredhy(Conn,ClientSock,sock)) ){
+		dup2(fsv,sock);
+		close(fsv);
+		return 1;
+	}
+	/*
+	if( needSTLS_SV(Conn) ){
+	*/
+	if( needSTLS_SVi(Conn,sock) ){
+		if( ServerFlags & PF_STLS_OPT ){
+			/* appliy TLS to the payload */
+		}else{
+			fsv = insertTLS_SV(Conn,ClientSock,sock);
+			if( 0 <= fsv ){
+				sv1log("[%d] SOCKS STLS=fsv (%s)\n",sock,what);
+				dup2(fsv,sock);
+				close(fsv);
+				return 1;
+			}
+		}
+	}
+	return 0;
+}
+
 static int socks_bind(DGC*Conn,SocksServer *sv,PCStr(host),int port,PCStr(user),PVStr(rhost),int *rport)
 {
 	bind_ver = sv->s_ver;
@@ -401,6 +591,8 @@ static int bindsocks(DGC*Conn,SocksServer *sv,PCStr(dsthost),int dstport,PCStr(u
 	if( sock < 0 )
 		return -1;
 
+	SOCKS_STARTTLS_withSV(Conn,sock,"bind");
+
 	sv->s_sock = sock;
 	if( socks_bind(Conn,sv,dsthost,dstport,uname,AVStr(rhost),rport) == 0 )
 		return sock;
@@ -411,6 +603,18 @@ int bindViaSocks(DGC*Conn,PCStr(dsthost),int dstport,PVStr(rhost),int *rport)
 {	int sx,sock;
 	SocksServer *sv;
 
+	/*
+	if( streq(DST_PROTO,"ftp")
+	 || strcaseeq(DST_PROTO,"ftps")
+	 || strcaseeq(DST_PROTO,"ftp-data")
+	)
+	if( IsMyself(dsthost) ){
+		sv1log("##NOT ViaSocks-F(self)## [%s]%s:%d\n",
+			DST_PROTO,dsthost,dstport);
+		return -1;
+	}
+	*/
+
 	getUsernameCached(getuid(),AVStr(Username));
 	sv1log("bindViaSocks(%s:%d)[%s]\n",dsthost,dstport,Username);
 
@@ -452,6 +656,9 @@ int connectViaSocksX(DGC*Conn,PCStr(skhost),int skport,PCStr(opts),PCStr(dsthost
 	sv->s_sock = OpenServer("ConnectViaSocks","socks",skhost,skport);
 	sv->s_Host = stralloc(skhost);
 	sv->s_port = skport;
+	sv->s_svaddr = 0;
+	sv->s_svlocal = 0;
+	sv->s_dsts = stralloc(dsthost);
 	sv->s_rslv = 0;
 	if( strstr(opts,"-4") )
 		sv->s_ver = 4;
@@ -474,6 +681,8 @@ static int connsocks(DGC*Conn,SocksServer *sv,PCStr(dsthost),int dstport,PCStr(u
 	if( sock < 0 )
 		return -1;
 
+	SOCKS_STARTTLS_withSV(Conn,sock,"conn");
+
 	sv->s_sock = sock;
 	if( socks_connect(Conn,sv,dsthost,dstport,uname,AVStr(rhost),rport) == 0 )
 		return sock;
@@ -483,6 +692,9 @@ static int connsocks(DGC*Conn,SocksServer *sv,PCStr(dsthost),int dstport,PCStr(u
 int connectViaSocks(DGC*Conn,PCStr(dsthost),int dstport,PVStr(rhost),int *rport)
 {	int sx,sock;
 	SocksServer *sv;
+	DGC toConn;
+	VSAddr sva;
+	SocksServer svb;
 
 	getUsernameCached(getuid(),AVStr(Username));
 	if( sv = getViaSocksX(gethostint_nboV4(dsthost),dstport,NULL) ){
@@ -503,6 +715,21 @@ int connectViaSocks(DGC*Conn,PCStr(dsthost),int dstport,PVStr(rhost),int *rport)
 		if( 0 <= sock )
 			return sock;
 	}
+
+	toConn = *Conn;
+	if( forwardtoSocks(&toConn,&sva) ){
+		bzero(&svb,sizeof(SocksServer));
+		sv = &svb;
+		sv->s_Host = VSA_ntoa(&sva);
+		sv->s_port = VSA_port(&sva);
+		sock = connsocks(&toConn,sv,dsthost,dstport,Username,
+			AVStr(rhost),rport);
+		if( 0 <= sock ){
+			SOCKS_addserv(dsthost,dstport,sv->s_Host,sv->s_port);
+			Conn->sv.p_SOCKSCTL = toConn.sv.p_SOCKSCTL;
+			return sock;
+		}
+	}
 	return -1;
 }
 int ConnectViaSocks(DGC*Conn,int relay_input)
@@ -517,30 +744,60 @@ int ConnectViaSocks(DGC*Conn,int relay_input)
 	return sock;
 }
 
+
+static int peekVer(DGC*Conn,PVStr(ibuf),int timeout){
+	int rcc;
+	setVStrEnd(ibuf,0);
+	if( PollIn(FromC,timeout*1000) <= 0 ){
+		daemonlog("E","Socks peek timeout (%ds)\n",timeout);
+		return -1;
+	}
+	rcc = recvPeekTIMEOUT(FromC,AVStr(ibuf),1);
+	if( rcc != 1 ){
+		daemonlog("F","Socks can't peek packet %d %d\n",rcc,errno);
+	}
+	return rcc;
+}
+void logReject(Connection *Conn,int self,PCStr(shost),int sport);
+static int permitted_socksVX(Connection *Conn,int ver){
+	if( strcaseeq(iSERVER_PROTO,"socks4") && ver != 4
+	 || strcaseeq(iSERVER_PROTO,"socks5") && ver != 5 ){
+		sprintf(Conn->reject_reason,"SocksV%d is not acceptable",ver);
+		logReject(Conn,1,Client_Host,Client_Port);
+		return 0;
+	}
+	return 1;
+}
+
 /*
  *	SOCKS SERVER
  */
+int service_socksA(DGC*Conn);
 int service_socks(DGC*Conn)
+{
+	int rcode;
+	rcode = service_socksA(Conn);
+	return rcode;
+}
+int service_socksA(DGC*Conn)
 {	CStr(ibuf,16);
-	int rcc;
-
-	/*
-	if( recvPeekTIMEOUT(FromC,AVStr(ibuf),1) != 1 )
-	*/
-	if( (rcc = recvPeekTIMEOUT(FromC,AVStr(ibuf),1)) != 1 )
-	{
-		daemonlog("F","Socks can't peek packet %d %d\n",rcc,errno);
-		return -1;
-	}
 
+	LOGX_appReq++;
 	if( !source_permittedX(Conn) ){
-		CStr(shost,256);
+		CStr(shost,MaxHostNameLen);
 		getClientHostPort(Conn,AVStr(shost));
 		daemonlog("F","E-P: No permission: \"%s\" is not allowed (%s)\n",
 			shost,Conn->reject_reason);
+		if( peekVer(Conn,AVStr(ibuf),2) <= 0 )
+			return -1;
 		/* send appropriate error message depending on client ver. */
 		return -1;
 	}
+	if( peekVer(Conn,AVStr(ibuf),LOGIN_TIMEOUT) <= 0 )
+		return -1;
+	if( permitted_socksVX(Conn,ibuf[0]) == 0 ){
+		return -1;
+	}
 
 	if( ibuf[0] == 5 ){
 		extern int IO_TIMEOUT;
