@@ -1,11 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1996-1999 Yutaka Sato
-Copyright (c) 1996-1999 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1996-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, and distribute this material for any purpose
-and without fee is hereby granted, provided that the above copyright
-notice and this permission notice appear in all copies.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -22,6 +25,8 @@ History:
 #include "fpoll.h"
 #include "file.h"
 #include "dglib.h"
+void tcp_relay2X(DGC*ctx,int timeout,int s1,int d1,int s2,int d2);
+#define tcp_relay2(timeout,s1,d1,s2,d2) tcp_relay2X(ctx,timeout,s1,d1,s2,d2)
 
 extern int IO_TIMEOUT;
 extern int CON_TIMEOUT;
@@ -139,8 +144,15 @@ int fgetstr(PVStr(user),int size,FILE *fc)
 /*
  *	SOCKS SERVER
  */
+void SocksV4_serverA(DGC*ctx,int _1,int _2,int fromC,int toC,FILE *fc);
 void SocksV4_server(DGC*ctx,int _1,int _2,int fromC,int toC)
 {	FILE *fc;
+
+	fc = fdopen(fromC,"r");
+	SocksV4_serverA(ctx,_1,_2,fromC,toC,fc);
+	fcloseFILE(fc);
+}
+void SocksV4_serverA(DGC*ctx,int _1,int _2,int fromC,int toC,FILE *fc){
 	unsigned char ibuf[16]; /**/
 	CStr(obuf,16);
 	int pc,ver,com,ch;
@@ -155,7 +167,6 @@ void SocksV4_server(DGC*ctx,int _1,int _2,int fromC,int toC)
 	CStr(bhost,64);
 	int aport,bport,asock,bsock,csock,nready,socks[2],readyv[2];
 
-	fc = fdopen(fromC,"r");
 	setbuf(fc,NULL);
 	pc = fread(ibuf,8,1,fc);
 	if( pc <= 0 )
