@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1994 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1994-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -30,6 +31,7 @@ History:
 
 extern int IO_TIMEOUT;
 
+int connectToSsh(Connection *Conn,const char *host,int port,PCStr(user),PCStr(pass));
 int makeXproxy(Connection *Conn,PVStr(pxdisplay),PCStr(display),PVStr(pxhost),PCStr(relhost),PCStr(me),int timeo);
 extern int *ccx_global;
 
@@ -38,8 +40,8 @@ static int rewriteTelnet(int direction,PVStr(buf),int cc);
 static void putIAC(FILE *tc,int com,int what);
 
 typedef struct {
-	MStr(	te_Xdisplay,256);
-	MStr(	te_Xproxy,256);
+	MStr(	te_Xdisplay,MaxHostNameLen);
+	MStr(	te_Xproxy,MaxHostNameLen);
 	int	te_Xpid;
 	int	te_THREAD;
 	int	te_THREAD_exiting;
@@ -54,6 +56,10 @@ typedef struct {
 	int	te_docontrol;
 	MStr(	te__opt,8);
 	int	te_keepalive;
+	MStr(	te_ccxbTOCL,64);
+	MStr(	te_ccxbTOSV,64);
+	CCXP	te_ccxTOCL;
+	CCXP	te_ccxTOSV;
 } TelnetEnv;
 static TelnetEnv *telnetEnv;
 #define Xdisplay	telnetEnv->te_Xdisplay
@@ -75,6 +81,10 @@ static TelnetEnv *telnetEnv;
 #define _opt		telnetEnv->te__opt
 /**/
 #define keepalive	telnetEnv->te_keepalive
+#define ccxTOCL		telnetEnv->te_ccxTOCL
+#define ccxTOSV		telnetEnv->te_ccxTOSV
+#define ccxbTOCL	telnetEnv->te_ccxbTOCL
+#define ccxbTOSV	telnetEnv->te_ccxbTOSV
 
 void scan_TELNETCONF(Connection*_,PCStr(conf))
 {	CStr(what,32);
@@ -118,6 +128,7 @@ static void sigTERM(int sig){
 
 #define SE	240
 #define NOP	241
+#define IP	244	/* Interrupt Process */
 #define EC	247
 #define EL	248
 #define SB	250
@@ -128,6 +139,7 @@ static void sigTERM(int sig){
 #define IAC	255
 
 #define SYNCH	242
+			/* DM DataMark */
 
 #define IS	0
 #define VAR	0
@@ -136,6 +148,7 @@ static void sigTERM(int sig){
 #define O_ECHO		 1
 #define O_SUPAHEAD	 3	/* Suppress Go Ahead */
 #define O_STATUS	 5	/* Give Status */
+#define O_TM		 6	/* Timing-Mark */
 #define O_NAOL		 8	/* Negotiate About Output Line width */
 #define O_NAOP		 9	/* Negotiate About Output Page size */
 #define O_TERMTYPE	24	/* Teminal Type */
@@ -144,6 +157,8 @@ static void sigTERM(int sig){
 #define O_LFLOW		33	/* Remote Flow Control */
 #define O_XDISPLOC	35	/* X Display Location */
 #define O_ENVIRON	36	/* Environment Variables */
+#define O_AUTH		37	/* Authentication */
+#define O_NENVIRON	39	/* New Environment Option RFC1572 */
 
 static char NOPstr[2] = { IAC, NOP };
 
@@ -171,6 +186,32 @@ setRecvSIGURG(sock)
 }
 */
 
+int recvPEEK(int sock,PVStr(buf),int size);
+void CCXcounts(CCXP ccx);
+int CCXsize();
+static int withTM(int sock){
+	int rcc;
+	CStr(buf,3);
+
+	/* reset error status of CCX on break */
+	if( ccxTOSV ){
+		bcopy(ccxbTOSV,ccxTOSV,CCXsize());
+	}
+	if( ccxTOCL ){
+		bcopy(ccxbTOCL,ccxTOCL,CCXsize());
+	}
+
+	/* 9.4.0 withTM() is since 9.0.3-pre18, but recvPEEK() breaks the
+	 * relay of relayOOB()
+	rcc = recvPEEK(sock,AVStr(buf),sizeof(buf));
+	if( rcc == 3 )
+	if( (0xFF&buf[0]) == IAC && (0xFF&buf[1]) == WILL && buf[2] == O_TM ){
+		sv1log("relay TimingMark before OOB DataMark...\n");
+		return 1;
+	}
+	*/
+	return 0;
+}
 static int Read(PCStr(what),int sock,int dst,PVStr(buf),int fsbsize,int timeout)
 {	int cnt,rcc,ci,cj;
 	const char *sdir;
@@ -182,6 +223,9 @@ static int Read(PCStr(what),int sock,int dst,PVStr(buf),int fsbsize,int timeout)
 		timeout = keepalive;
 	}
 	for( cnt = 0; cnt < 10; cnt++ ){
+		if( timeout == -1 )
+			nready = 1;
+		else
 		if( THREAD )
 			nready = thread_PollIn(sock,timeout*1000);
 		else	nready = PollIn(sock,timeout*1000);
@@ -219,7 +263,17 @@ static int Read(PCStr(what),int sock,int dst,PVStr(buf),int fsbsize,int timeout)
 			continue;
 		}
 		errno = 0;
-		rcc = read(sock,(char*)buf,fsbsize);
+
+		if( gotOOB(sock) )
+		if( !withTM(sock) )
+		if( 0 < withOOB(sock) ){
+			if( relayOOB(sock,dst) ){
+				/* should skip inputs until SYNCH */
+			}
+			continue;
+		}
+
+		rcc = read(sock,(char*)buf,QVSSize(buf,fsbsize));
 		if( rcc <= 0 ){
 			sv1log("sock=%d read=%d, errno=%d\n",sock,rcc,errno);
 			break;
@@ -380,6 +434,7 @@ static const char *option(int ch)
 {
 	switch( ch ){
 		case O_ECHO:     return "Echo";
+		case O_TM:     return "Timingmark";
 		case O_SUPAHEAD: return "SuppressGoAhead";
 		case O_STATUS:	 return "GiveStatus";
 		case O_TERMTYPE: return "TerminalType";
@@ -389,6 +444,7 @@ static const char *option(int ch)
 		case O_LFLOW:	 return "RemoteFlowControl";
 		case O_XDISPLOC: return "XDisplayLocation";
 		case O_ENVIRON:  return "EnvironmentValiables";
+		case O_NENVIRON: return "NewEnvironment";
 	}
 	sprintf(_opt,"%d",ch);
 	return _opt;
@@ -492,9 +548,17 @@ static int scanCommands(int direction,PVStr(buf),int cc)
 			sdir,i,ch,ch,(' '<ch&&ch<0x7F)?ch:' ');
 		if( direction == StoC ){
 			/* code conversion */
+			if( ccxTOCL ){
+				cc = ccx_telnet((int*)ccxTOCL,AVStr(buf),&i,cc);
+				if( (0xFF & buf[i]) == IAC ) i--;
+			}else
 			if( ccx_global ) cc = ccx_telnet(ccx_global,AVStr(buf),&i,cc);
 		}else
 		if( direction == CtoS ){
+			if( ccxTOSV ){
+				cc = ccx_telnet((int*)ccxTOSV,AVStr(buf),&i,cc);
+				if( (0xFF & buf[i]) == IAC ) i--;
+			}
 			if( ccx_global ){
 				/* conversion switch command ... */
 				/*
@@ -518,16 +582,42 @@ static int scanCommands(int direction,PVStr(buf),int cc)
 	return cc;
 }
 static int ccx_telnet(int*ccx,PVStr(buf),int *ip,int cc)
-{	CStr(out,4096);
-	CStr(tmp,4096);
+{	CStr(out,0x8000);
+	CStr(tmp,0x8000);
 	int i,j,k,l,tc,oc;
+	int fc;
 
 	i = *ip;
+	if( (0xFF & buf[i]) == SYNCH ){
+		return cc;
+	}
 	for( j = i; j < cc; j++ )
 		if( (buf[j] & 0xFF) == IAC )
 			break;
 
+	if( j == i ){
+		return cc;
+	}
+
+/*
+ fprintf(stderr,
+"-- %3d/%3d [%2X [%3d][%3d] %2X] %2X/%3d %2X/%3d %2X %2X %2X %2X %2X %2X ",
+j-i,cc,
+0<j?0xFF&buf[j-1]:0,i,j,j<cc?0xFF&buf[j]:0,
+0xFF&buf[i+0],0xFF&buf[i+0],
+0xFF&buf[i+1],0xFF&buf[i+1],
+0xFF&buf[i+2],0xFF&buf[i+3],
+0xFF&buf[i+4],0xFF&buf[i+5],
+0xFF&buf[i+6],0xFF&buf[i+7]
+ ); CCXcounts((CCXP)ccx);
+*/
+
 	oc = CCXexec((CCXP)ccx,buf+i,j-i,AVStr(out),sizeof(out));
+	/*
+	fc = CCXexec((CCXP)ccx,"",0,DVStr(out,oc),sizeof(out)-oc);
+	oc += fc;
+	*/
+
 	tc = 0;
 	for( k = j; k < cc; k++ ){
 		if( elnumof(tmp) <= tc ){
@@ -745,12 +835,13 @@ static int xgetline(void *visible,FILE *fc,FILE *tc,UTag *Tline,UTag *Tcbuf)
 	return ci;
 }
 
+#define addIbuf(v) setVStrElemInc(ibuf,ii,v)
 static void putIAC(FILE *tc,int com,int what)
-{	CStr(ibuf,32);
+{	JStr(ibuf,32);
 	char ii;
 
 	ii = 0;
-	ibuf[ii++] = IAC; ibuf[ii++] = com; ibuf[ii++] = what; /**/
+	addIbuf(IAC); addIbuf(com); addIbuf(what);
 	ii = scanCommands(DtoC,AVStr(ibuf),ii);
 	fwrite(ibuf,1,ii,tc);
 }
@@ -793,7 +884,7 @@ static void ConnectedMessage(Connection *Conn,PCStr(serv),FILE *tc,PCStr(comline
 
 static int get_hostname(Connection *Conn,FILE *fc,FILE *tc,UTag *Tline,UTag *Tcbuf,int ncom)
 {	int ci;
-	CStr(clhost,256);
+	CStr(clhost,MaxHostNameLen);
 
 /*
 	putIAC(tc,WILL,O_ECHO);
@@ -839,7 +930,7 @@ static void relayClientsWills(FILE *ts)
 		relayClientsWill(ts,op);
 }
 
-static int CS(Connection *Conn,int fcbsize,int omask)
+static int telnetCS(Connection *Conn,int fcbsize,int omask)
 {	CStr(buf,0x4000);
 	int gotsig;
 	int rcc;
@@ -899,7 +990,7 @@ static int CS(Connection *Conn,int fcbsize,int omask)
 	}
 	return total;
 }
-static int SC(Connection *Conn,int fsbsize,int omask)
+static int telnetSC(Connection *Conn,int fsbsize,int omask)
 {	CStr(buf,0x4000);
 	int gotsig;
 	int rcc,wcc;
@@ -921,10 +1012,10 @@ static int SC(Connection *Conn,int fsbsize,int omask)
 
 			count += 1;
 			total += rcc;
-			if( (dump_commands & S_C) || CCX ){
+			if( (dump_commands & S_C) || CCX0 ){
 				rcc = scanCommands(StoC,AVStr(buf),rcc);
 				if( rcc == 0 ){
-					/* ESC seq. pending in CCX */
+					/* ESC seq. pending in CCX0 */
 					continue;
 				}
 			}
@@ -979,7 +1070,7 @@ static int SC1(Connection *Conn,int fsbsize,int omask)
 	/*
 	setRecvSIGURG(FromS);
 	*/
-	cc = SC(Conn,fsbsize,omask);
+	cc = telnetSC(Conn,fsbsize,omask);
 	THREADexit(Conn);
 	return cc;
 }
@@ -990,11 +1081,170 @@ static int CS1(Connection *Conn,int fcbsize,int omask)
 	/*
 	setRecvSIGURG(FromC);
 	*/
-	cc = CS(Conn,fcbsize,omask);
+	cc = telnetCS(Conn,fcbsize,omask);
 	THREADexit(Conn);
 	return cc;
 }
 
+static int relayCS0(Connection *Conn,int bsize){
+	CStr(buf,0x4000);
+	int rcc;
+	int icc;
+
+	rcc = Read("CS",FromC,ToS,AVStr(buf),bsize,-1);
+	icc = rcc;
+	if( rcc <= 0 )
+		return -1;
+	/*
+	if( dump_commands && C_S ){
+	*/
+	if( (dump_commands && C_S) || ccxTOSV ){
+		rcc = scanCommands(CtoS,AVStr(buf),rcc);
+	}
+	if( docontrol ){
+		docontrol = 0;
+		controlcommand(Conn,ToC,FromC);
+		if( rcc == 0 )
+			return 0;
+	}
+	if( ToS < 0 )
+		return 0;
+	if( write(ToS,buf,rcc) <= 0 )
+		return -1;
+	return rcc;
+}
+static int relaySC0(Connection *Conn,int bsize){
+	CStr(buf,0x4000);
+	int rcc;
+	int icc;
+
+	rcc = Read("SC",FromS,ToC,AVStr(buf),bsize,-1);
+	icc = rcc;
+	if( rcc <= 0 )
+		return -1;
+/*
+	if( dump_commands && S_C ){
+*/
+	/*
+	if( dump_commands & S_C ){
+	*/
+	if( (dump_commands & S_C) || ccxTOCL ){
+		rcc = scanCommands(StoC,AVStr(buf),rcc);
+	}
+	if( docontrol ){
+		docontrol = 0;
+		controlcommand(Conn,ToS,FromS);
+		if( rcc == 0 )
+			return 0;
+	}
+	if( ToC < 0 )
+		return 0;
+	if( write(ToC,buf,rcc) <= 0 )
+		return -1;
+	return rcc;
+}
+static int bidirectional_relay2(Connection *Conn,int fcbsize,int fsbsize)
+{	register int ppid,cpid;
+	int omask;
+	int total;
+	void (*opipe)(int);
+	void (*oterm)(int);
+	void (*ointr)(int);
+
+	CStr(snoop,64);
+	int sf;
+	if( 0 <= find_CMAP(Conn,"DUMP",AVStr(snoop)) ){
+		sf = 0;
+		if( strcasestr(snoop,"tosv") ) sf |= C_S|(CtoS<<8);
+		if( strcasestr(snoop,"tocl") ) sf |= S_C|(StoC<<8);
+		dump_commands |= sf;
+	}
+
+	Verbose("buffer: CS=%d[%d>%d] SC=%d[%d>%d] (%s)\n",
+		fcbsize,FromC,ToS,fsbsize,FromS,ToC,
+		"Polling");
+
+	omask = sigblock(sigmask(SIGPIPE)|sigmask(SIGTERM));
+	opipe = Vsignal(SIGPIPE,sigPIPE);
+	oterm = Vsignal(SIGTERM,sigTERM);
+	ointr = Vsignal(SIGINT, sigTERM);
+
+	if( FromC < 0 ){
+		total = telnetSC(Conn,fsbsize,omask);
+	}else
+	if( FromS < 0 ){
+		total = telnetCS(Conn,fcbsize,omask);
+	}else{
+		int fds[2];
+		int rds[2];
+		int fi;
+		int nready;
+		int rcc;
+		int count[2];
+		int total[2];
+
+		fds[0] = FromC;
+		fds[1] = FromS;
+		count[0] = 0; count[1] = 0;
+		total[0] = 0; total[1] = 0;
+
+		if( CCXactive(CCX_TOCL) ){
+			Bcopy(CCX_TOCL,ccxbTOCL,sizeof(ccxbTOCL));
+			ccxTOCL = CCX_TOCL;
+		}else	ccxTOCL = 0;
+		if( CCXactive(CCX_TOSV) ){
+			Bcopy(CCX_TOSV,ccxbTOSV,sizeof(ccxbTOSV));
+			ccxTOSV = CCX_TOSV;
+		}else	ccxTOSV = 0;
+
+		for(;;){
+			nready = PollIns(IO_TIMEOUT*1000,2,fds,rds); 
+			if( nready <= 0 )
+				break;
+
+			if( nready == 0 && errno == 0 || gotOOB(-1) ){
+				int fi,sync;
+				sync = 0;
+				for( fi = 0; fi < 2; fi++ ){
+					if( withOOB(fds[fi]) )
+					if( !withTM(fds[fi]) )
+					sync += relayOOB(fds[fi],fds[(fi+1)%2]);
+				}
+				if( 0 < sync )
+					continue;
+			}
+			if( 0 < rds[0] ){
+				rcc = relayCS0(Conn,fcbsize);
+				if( rcc < 0 ){
+					sv1log("CS-EOF\n");
+					break;
+				}
+				count[0] += 1;
+				total[0] += rcc;
+			}
+			if( 0 < rds[1] ){
+				rcc = relaySC0(Conn,fcbsize);
+				if( rcc < 0 ){
+					sv1log("SC-EOF\n");
+					break;
+				}
+				count[1] += 1;
+				total[1] += rcc;
+			}
+
+		}
+		sv1log("CS-RELAY[%d>%d]: %dBytes %dI/O buf=%d\n",
+			FromC,ToS,total[0],count[0],fcbsize);
+		sv1log("SC-RELAY[%d<%d]: %dBytes %dI/O buf=%d\n",
+			ToC,FromS,total[1],count[1],fsbsize);
+	}
+
+	signal(SIGPIPE,opipe);
+	signal(SIGTERM,oterm);
+	signal(SIGTERM,ointr);
+	sigsetmask(omask);
+	return total;
+}
 static int bidirectional_relay(Connection *Conn,int fcbsize,int fsbsize)
 {	register int ppid,cpid;
 	int omask;
@@ -1024,15 +1274,15 @@ static int bidirectional_relay(Connection *Conn,int fcbsize,int fsbsize)
 	ointr = Vsignal(SIGINT, sigTERM);
 
 	if( FromC < 0 ){
-		total = SC(Conn,fsbsize,omask);
+		total = telnetSC(Conn,fsbsize,omask);
 	}else
 	if( FromS < 0 ){
-		total = CS(Conn,fcbsize,omask);
+		total = telnetCS(Conn,fcbsize,omask);
 	}else
 	if( useTHREAD ){
 		THREAD = 1;
 		THREAD_exiting = 0;
-		thread_fork(0,(IFUNCP)CS1,Conn,fsbsize,omask);
+		thread_fork(0,STX_tid,"CS1",(IFUNCP)CS1,Conn,fsbsize,omask);
 		SC1(Conn,fcbsize,omask /*,ppid*/);
 		THREAD = 0;
 	}else{
@@ -1073,7 +1323,7 @@ static void getServersWill(int fromS,FILE *tc)
 }
 
 static int putXdisplay(FILE *ts,PCStr(display))
-{	CStr(ibuf,128);
+{	JStr(ibuf,128);
 	char ch;
 	int ii,ij;
 	int opt;
@@ -1089,37 +1339,37 @@ if( opt == O_ENVIRON )
 	return 0;
 
 	ii = 0;
-	ibuf[ii++] = IAC; ibuf[ii++] = WILL; ibuf[ii++] = opt; /**/
-	ibuf[ii++] = IAC; ibuf[ii++] = SB;   ibuf[ii++] = opt; /**/
-	ibuf[ii++] = IS;
+	addIbuf(IAC); addIbuf(WILL); addIbuf(opt);
+	addIbuf(IAC); addIbuf(SB);   addIbuf(opt);
+	addIbuf(IS);
 	if( opt == O_ENVIRON ){
-		ibuf[ii++] = VAR;
+		addIbuf(VAR);
 		for( ij = 0; ch = "DISPLAY"[ij]; ij++ ){
-			ibuf[ii++] = ch; /**/
+			addIbuf(ch);
 		}
-		ibuf[ii++] = VALUE;
+		addIbuf(VALUE);
 	}
 	for( ij = 0; ch = display[ij]; ij++ ){
 		if( elnumof(ibuf) <= ii+3 )
 			break;
-		ibuf[ii++] = ch; /**/
+		addIbuf(ch);
 	}
-	ibuf[ii++] = IAC; ibuf[ii++] = SE;
+	addIbuf(IAC); addIbuf(SE);
 	ii = scanCommands(DtoC,AVStr(ibuf),ii);
 	fwrite(ibuf,1,ii,ts);
 	return 1;
 }
 
 static int makeXproxy1(Connection *Conn,PVStr(myname),PCStr(peername),int peerport)
-{	CStr(me,256);
-	CStr(myhp,256);
+{	CStr(me,MaxHostNameLen);
+	CStr(myhp,MaxHostNameLen);
 	int xpid;
 
 /*
 Socks can accept only one X connection ...
 
 	if( Conn->sv_viaSocks ){
-		CStr(rhost,32);
+		CStr(rhost,64);
 		int rport,rsock;
 
 		rsock = bindViaSocks(Conn,DST_HOST,DST_PORT,rhost,&rport);
@@ -1133,7 +1383,7 @@ Socks can accept only one X connection ...
 	xpid = makeXproxy(Conn,AVStr(Xproxy),Xdisplay,AVStr(myname),peername,me,0);
 
 	/* if SRCIF="proxyDisplay:*:X" is given */{
-		CStr(xdisp,256);
+		CStr(xdisp,MaxHostNameLen);
 		const char *dp;
 		int xport;
 
@@ -1155,13 +1405,13 @@ static void proxy_telnet(Connection *Conn)
 	CStr(comline,256);
 	CStr(cbuf,256);
 	CStr(command,256);
-	CStr(hostport,256);
-	CStr(serv,256);
-	CStr(clnt,256);
+	CStr(hostport,MaxHostNameLen);
+	CStr(serv,MaxHostNameLen);
+	CStr(clnt,MaxHostNameLen);
 	const char *addr;
 	CStr(auth,256);
 	CStr(auser,256);
-	CStr(ahost,256);
+	CStr(ahost,MaxHostNameLen);
 	const char *iuser;
 	int port;
 	int csize;
@@ -1232,7 +1482,7 @@ static void proxy_telnet(Connection *Conn)
 			continue;
 		}else
 		if( streq(command,"a") || streq(command,"accept") ){
-			CStr(myname,256);
+			CStr(myname,MaxHostNameLen);
 
 			if( hostport[0] == 0 ){
 				fprintf(tc,"???? Usage: accept hostname\r\n");
@@ -1258,6 +1508,12 @@ static void proxy_telnet(Connection *Conn)
 		}else
 		if( streq(command,"j") ){
 			CStr(stat,256);
+			CCXcreate("*",hostport,CCX_TOCL);
+			if( CCXactive(CCX_TOCL) ){
+				Bcopy(CCX_TOCL,ccxbTOCL,sizeof(ccxbTOCL));
+				ccxTOCL = CCX_TOCL;
+			}else	ccxTOCL = 0;
+
 			global_setCCX(Conn,AVStr(hostport),AVStr(stat));
 			fprintf(tc,"-- charcode conversion [%s]%s\r\n",
 				hostport,stat);
@@ -1272,6 +1528,23 @@ static void proxy_telnet(Connection *Conn)
 		}
 		serv[0] = 0;
 		port = DFLT_PORT;
+		if( strneq(hostport,"-ssh.",5) ){
+			strcpy(REAL_PROTO,"ssh");
+			ovstrcpy(hostport,hostport+5);
+			REAL_PORT = 22;
+			port = 22;
+		}
+		if( strstr(hostport,"@-ssh.") ){
+			IStr(auth,128);
+			const char *dp;
+			dp = wordScanY(hostport,auth,"^@");
+			if( strneq(dp,"@-ssh.",6) ){
+				strcpy(REAL_PROTO,"ssh");
+				sprintf(hostport,"%s@%s",auth,dp+6);
+				REAL_PORT = 22;
+				port = 22;
+			}
+		}
 		Xsscanf(hostport,"%s %d",AVStr(serv),&port);
 
 		if( serv[0] == 0 ){
@@ -1307,6 +1580,23 @@ fputLog(Conn,"Login","TELNET; from=%s%s; to=%s\n",clnt,auth,serv);
 		fflush(tc);
 
 		ConnError = 0;
+
+		if( streq(DST_PROTO,"ssh") || DST_PORT == 22 ){
+			int tos;
+			IStr(user,128);
+			if( strchr(serv,'@') ){
+				wordScanY(serv,user,"^@");
+			}
+			tos = connectToSsh(Conn,DST_HOST,DST_PORT,user,"");
+			sv1log("SSHGW[%d] %s:%d\n",tos,DST_HOST,DST_PORT);
+			if( tos != -1 ){
+				ToS = FromS = tos;
+				bidirectional_relay2(Conn,0x2000,0x2000);
+			}else{
+				cantConnMessage(Conn,serv,tc);
+				fflush(tc);
+			}
+		}else
 		if( connect_to_serv(Conn,FromC,ToC,0) < 0 ){
 			cantConnMessage(Conn,serv,tc);
 			fflush(tc);
@@ -1322,8 +1612,8 @@ fputLog(Conn,"Login","TELNET; from=%s%s; to=%s\n",clnt,auth,serv);
 				fflush(ts);
 			}
 			if( Xdisplay[0] ){
-				CStr(myname,256);
-				CStr(peername,256);
+				CStr(myname,MaxHostNameLen);
+				CStr(peername,MaxHostNameLen);
 				int port;
 
 				gethostNAME(ToS,AVStr(myname));
@@ -1377,7 +1667,10 @@ fputLog(Conn,"Login","TELNET; from=%s%s; to=%s\n",clnt,auth,serv);
 			}
 
 			fclose(ts);
+			/*
 			bidirectional_relay(Conn,1024,1024);
+			*/
+			bidirectional_relay2(Conn,1024,1024);
 			fprintf(tc,"\r\n");
 
 			if( 0 < Xpid ){
@@ -1402,7 +1695,7 @@ static void AsServer(Connection *Conn)
 	CStr(myhost,128);
 	CStr(user,128);
 	CStr(pass,128);
-	CStr(ibuf,128);
+	JStr(ibuf,128);
 	int ii;
 	CStr(line,1024);
 	const char *dp;
@@ -1420,8 +1713,8 @@ static void AsServer(Connection *Conn)
 	fputs("\r\n",tc);
 
 	ii = 0;
-	ibuf[ii++] = IAC; ibuf[ii++] = WONT; ibuf[ii++] = O_ECHO;
-	ibuf[ii++] = IAC; ibuf[ii++] = DO;   ibuf[ii++] = O_ECHO;
+	addIbuf(IAC); addIbuf(WONT); addIbuf(O_ECHO);
+	addIbuf(IAC); addIbuf(DO);   addIbuf(O_ECHO);
 	fwrite(ibuf,1,ii,tc);
 	fflush(tc);
 	scanCommands(DtoC,AVStr(ibuf),ii);
@@ -1436,8 +1729,8 @@ static void AsServer(Connection *Conn)
 	sv1log("USER: %s\n",user);
 
 	ii = 0;
-	ibuf[ii++] = IAC; ibuf[ii++] = WILL; ibuf[ii++] = O_ECHO;
-	ibuf[ii++] = IAC; ibuf[ii++] = DONT; ibuf[ii++] = O_ECHO;
+	addIbuf(IAC); addIbuf(WILL); addIbuf(O_ECHO);
+	addIbuf(IAC); addIbuf(DONT); addIbuf(O_ECHO);
 	fwrite(ibuf,1,ii,tc);
 	fflush(tc);
 	scanCommands(DtoC,AVStr(ibuf),ii);
@@ -1454,8 +1747,8 @@ static void AsServer(Connection *Conn)
 		goto EXIT;
 
 	ii = 0;
-	ibuf[ii++] = IAC; ibuf[ii++] = WONT; ibuf[ii++] = O_ECHO;
-	ibuf[ii++] = IAC; ibuf[ii++] = DO;   ibuf[ii++] = O_ECHO;
+	addIbuf(IAC); addIbuf(WONT); addIbuf(O_ECHO);
+	addIbuf(IAC); addIbuf(DO);   addIbuf(O_ECHO);
 	fwrite(ibuf,1,ii,tc);
 	fflush(tc);
 	scanCommands(DtoC,AVStr(ibuf),ii);
@@ -1513,6 +1806,59 @@ static int telnetonly(Connection *Conn)
 	Conn->sv = sv;
 	return !relayany;
 }
+
+static struct { 
+	char *ne_USER;
+} clenv;
+static char doNewEnviron[] = {IAC,SB,O_NENVIRON,1,IAC,SE};
+static int getUSERenv(FILE *fc,FILE *tc,AuthInfo *au){
+	int rcc,nrcc;
+	int timeout = 15*1000;
+	IStr(buf,1024+1);
+	IStr(nbuf,1024+1);
+	const unsigned char *up = (const unsigned char*)buf;
+	const unsigned char *vp = (const unsigned char*)nbuf;
+	IStr(userb,MaxHostNameLen);
+
+	putIAC(tc,DO,O_NENVIRON); fflush(tc); /* ask USER */
+	write(fileno(tc),doNewEnviron,sizeof(doNewEnviron));
+
+	for(;;){
+		if( PollIn(fileno(fc),timeout) == 0 ){
+			break;
+		}
+		rcc = read(fileno(fc),buf,sizeof(buf)-1);
+		if( 0 < rcc ){
+			Bcopy(buf,nbuf,rcc);
+			nrcc = scanCommands(CtoS,AVStr(nbuf),rcc);
+		}
+		if( 3 <= rcc && up[0] == 0xFF ){
+			if( up[1] == WILL && up[2] == O_AUTH ){
+				putIAC(tc,DONT,O_AUTH);
+				fflush(tc);
+			}
+			if( up[1] == WILL || up[1] == SB )
+			if( up[2] == O_NENVIRON ){
+			    if( clenv.ne_USER ){
+				strcpy(userb,clenv.ne_USER);
+				if( strpbrk(userb,":@") ){
+					bzero(au,sizeof(AuthInfo));
+					decomp_siteX("sftp",userb,au);
+ sv1log("--USER[%s]:%d @ HOST[%s]:%d\n",
+ au->i_user,strlen(au->i_pass),au->i_Host,au->i_Port);
+				}else{
+					bzero(au,sizeof(AuthInfo));
+					strcpy(au->i_user,clenv.ne_USER);
+				}
+				return 1;
+			    }else{
+				timeout = 300;
+			    }
+			}
+		}
+	}
+	return 0;
+}
 int service_telnet(Connection *Conn)
 {
 	if( telnetEnv == 0 )
@@ -1521,12 +1867,41 @@ int service_telnet(Connection *Conn)
 	if( LOG_VERBOSE )
 		dump_commands = C_S | S_C;
 
+	if( strncaseeq(DFLT_HOST,"-ssh",4)
+	 && (DFLT_HOST[4] == '.' || DFLT_HOST[4] == 0) ){
+		ovstrcpy(DFLT_PROTO,"ssh");
+		if( DFLT_HOST[4] == 0 )
+			strcpy(DFLT_HOST,"-");
+		else	ovstrcpy(DFLT_HOST,DFLT_HOST+5);
+		DFLT_PORT = 22;
+		sv1log(">>>> ssh://%s:%d\n",DFLT_HOST,DFLT_PORT);
+	}
+
 	if( BORN_SPECIALIST )
 	if( strcmp(iSERVER_PROTO,"telnet") == 0 )
 	if( strcmp(DFLT_HOST,"-.-") == 0 ){
 		AsServer(Conn);
 		return 0;
 	}
+
+	/* -l user[@host] */
+	if( strcaseeq(DFLT_PROTO,"ssh") )
+	if( DFLT_AUTH == 0 ){
+		FILE *tc = fdopen(dup(ToC),"w");
+		FILE *fc = fdopen(dup(FromC),"r");
+		AuthInfo au;
+		if( getUSERenv(fc,tc,&au) ){
+			if( au.i_Host[0] ){
+				strcpy(DFLT_HOST,au.i_Host);
+			}
+			if( DFLT_AUTH == 0 )
+				DFLT_AUTH = (AuthInfo*)malloc(sizeof(AuthInfo));
+			*DFLT_AUTH = au;
+		}
+		fclose(tc);
+		fclose(fc);
+	}
+
 	if( isMYSELF(DFLT_HOST) ){
 		ImProxy = 1;
 		if( PollIn(FromC,10) == 0 ){
@@ -1548,7 +1923,7 @@ int service_telnet(Connection *Conn)
 		FILE *fc = fdopen(FromC,"r");
 		FILE *tc = fdopen(ToC,"w");
 		CStr(auser,256);
-		CStr(ahost,256);
+		CStr(ahost,MaxHostNameLen);
 		int dport = DFLT_PORT;
 
 		write(ToC,NOPstr,2);
@@ -1563,6 +1938,20 @@ int service_telnet(Connection *Conn)
 		fcloseFILE(fc);
 		fcloseFILE(tc);
 	}
+	if( streq(DST_PROTO,"ssh") || DST_PORT == 22 ){
+		int tos;
+		IStr(user,128);
+		IStr(pass,128);
+		strcpy(user,DFLT_USER);
+		strcpy(pass,DFLT_PASS);
+		/* see MYAUTH? */
+
+		tos = connectToSsh(Conn,DST_HOST,DST_PORT,user,pass);
+		sv1log("SSHGW[%d] %s:%d\n",tos,DST_HOST,DST_PORT);
+		if( tos != -1 ){
+			ToS = FromS = tos;
+		}
+	}
 	if( ToC < 0 || ToS < 0 )
 		connect_to_serv(Conn,FromC,ToC,0);
 
@@ -1587,7 +1976,10 @@ int service_telnet(Connection *Conn)
 		scanCommands(DtoC,AVStr(buf),3);
 	}
 	*/
+	/*
 	bidirectional_relay(Conn,0x2000,0x2000);
+	*/
+	bidirectional_relay2(Conn,0x2000,0x2000);
 	return 0;
 }
 
@@ -1603,15 +1995,20 @@ int service_telnet(Connection *Conn)
 #define RW_TERM		0x104
 #define RW_XDISPLOC	0x105
 #define RW_ENVIRON	0x106
+#define RW_NENVIRON	0x107
 
 static short Trans[][32] = {
  { IAC, SB, O_ENVIRON,  IS, START, VALUE, STR, VAR, STR, RW_ENVIRON, LOOP },
+ { IAC, SB, O_NENVIRON, IS, START, VAR, STR, VALUE, STR, RW_NENVIRON, LOOP },
  { IAC, SB, O_TERMTYPE, IS, START, STR, RW_TERM,    EOA },
  { IAC, SB, O_XDISPLOC, IS, START, STR, RW_XDISPLOC, EOA },
  0
 };
 
+/*
 static int getString(PCStr(buf),int cc,int c0,PVStr(str))
+*/
+static int getString(PCStr(buf),int cc,int c0,int cx,PVStr(str))
 {	int ci,si;
 	unsigned char ch;
 
@@ -1619,6 +2016,11 @@ static int getString(PCStr(buf),int cc,int c0,PVStr(str))
 	for(ci = c0; ci < cc; ci++ ){
 		assertVStr(str,str+si+1);
 		ch = buf[ci];
+		if( ch == cx ){
+			/* the next literal char. in Trans[] */
+			if( ch == VAR || ch == VALUE )
+				break;
+		}
 		setVStrElemInc(str,si,ch); /**/
 		if( ch < ' ' || 0x7F < ch )
 			break;
@@ -1668,6 +2070,7 @@ static int rewriteTelnet(int direction,PVStr(buf),int cc)
 	int inc = 0,inc1;
 	ACStr(str,4,2048);
 	int strx,start_sjp,start_sji;
+	int str_sji = 0;
 
 	for( si = 0; ; si++ ){
 	    sb1 = Trans[si];
@@ -1695,7 +2098,11 @@ static int rewriteTelnet(int direction,PVStr(buf),int cc)
 			break;
 
 		  case STR:
+			str_sji = sji; /* the beginning of STR */
+			sji = getString(buf,cc,sji,sb1[sjp+1],EVStr(str[strx]));
+			/*
 			sji = getString(buf,cc,sji,EVStr(str[strx]));
+			*/
 			strx++;
 			break;
 
@@ -1723,6 +2130,25 @@ static int rewriteTelnet(int direction,PVStr(buf),int cc)
 			}
 			break;
 
+		  case RW_NENVIRON: /* rewriting VALUE STR for DISPLAY */
+			sv1log("NEW-ENVIRON [%s=%s]\n",str[0],str[1]);
+			if( streq(str[0],"USER") )
+			if( direction == CtoS || direction == CtoD )
+			{
+				clenv.ne_USER = stralloc(str[1]);
+				clenv.ne_USER[strlen(str[1])-1] = 0;/*bug*/
+			}
+			if( streq(str[0],"DISPLAY") )
+			if( direction == CtoS )
+			if( Xproxy[0] && !streq(Xproxy,str[1]) ){
+				sv1log("DISPLAY [%s] -> [%s]\n",str[1],Xproxy);
+				inc1 = replaceString(AVStr(buf),cc,str_sji,sji,
+					Xproxy);
+				inc += inc1;
+				sji += inc1;
+			}
+			break;
+
 		  default:
 			if( chp != chi )
 				goto NEXTALT;
@@ -1732,3 +2158,360 @@ static int rewriteTelnet(int direction,PVStr(buf),int cc)
 	}
 	return inc;
 }
+
+/*////////////////////////////////////////////////////////////////////////
+Copyright (c) 2007 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
+
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use,
+without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
+OR IMPLIED WARRANTIES.
+//////////////////////////////////////////////////////////////////////////
+Content-Type:	program/C; charset=US-ASCII
+Program:	telnetgw.c
+Author:		Yutaka Sato <y.sato@delegate.org>
+Description:
+History:
+	071126	created
+//////////////////////////////////////////////////////////////////////#*/
+#if defined(__KURO_BOX__)
+#include <pty.h>
+#define Forkpty(pty,name) forkpty(pty,name,NULL,NULL)
+#else
+int Forkpty(int *pty,char *name);
+#endif
+int Stty(int fd,const char *mode);
+int Gtty(int fd,const char *mode);
+
+char *fgetsTO(PVStr(b),int z,FILE *f,int t1,int t2);
+FILE *fopenTO(PCStr(path),PCStr(mode),int timeout);
+static char *ssh_com = "ssh -p %d %s@%s";
+static int sshPty = -1;
+static int sshPid = 0;
+static int sshThread = 0;
+static int badPass = 0;
+
+static void relay_resp(FILE *fs,FILE *tc,int timeout,PVStr(resp),PCStr(com)){
+	CStr(res,1024);
+	refQStr(rp,resp);
+	refQStr(dp,res);
+	int tout0;
+	int tout;
+
+	if( 0 < timeout )
+		tout0 = tout = timeout;
+	else	tout0 = tout = 3*1000;
+
+	if( resp )
+		setVStrEnd(resp,0);
+	if( fPollIn(fs,tout) <= 0 ){
+		sv1log("--SSH >> response timeout [%s]%d\n",com,tout);
+		return;
+	}
+
+	for(;;){
+		if( 0 < fgetsBuffered(AVStr(res),sizeof(res),fs) ){
+		}else
+		if( fgetsTO(AVStr(res),sizeof(res),fs,tout,1) == NULL ){
+			break;
+		}
+		if( strchr(res,'\n') )
+			sv1log("--SSH >>(%s) %s",com,res);
+		else	sv1log("--SSH >>(%s/NO-NL)[%s]\n",com,res);
+		if( dp = strpbrk(res,"\r\n") )
+			strcpy(dp,"\r\n");
+		if( streq(res,"\r\n") || streq(res,"\n") ){
+			/* ignore empty lines */
+		}else
+		if( rp ){
+			strcpy(rp,res);
+			rp += strlen(rp);
+		}
+		tout = 1;
+	}
+}
+static void closeAll(){
+	int fi;
+	int rcode;
+	int lfd = curLogFd();
+	for( fi = 3; fi < 64; fi++ ){
+		if( fi == lfd )
+			continue;
+		rcode = close(fi);
+	}
+}
+static int badpass(PCStr(resp)){
+	if( badPass ){
+		if( resp[0] == '\r'
+		 || resp[0] == '\n'
+		){
+			sv1log("--SSH: SKIP in BAD PASS [%s]\n",resp);
+			return badPass;
+		}
+	}
+	if( strtailstr(resp,"Password:")
+	 || strtailstr(resp,"Password: ")
+	 || strtailstr(resp,"password: ")
+	 || strstr(resp,"Enter passphrase for key")
+	){
+		sv1log("--SSH: BAD PASS [%s]\n",resp);
+		badPass = 1;
+	}else{
+		sv1log("--SSH: PASS OK [%s]\n",resp);
+		badPass = 0;
+	}
+	return badPass;
+}
+
+int dupLogFd();
+void execsystem(PCStr(what),PCStr(pathcom));
+static int forkSsh(PCStr(host),int port,PCStr(user),PCStr(pass),int tf[2],FILE *fc,FILE *tc){
+	int pid;
+	int pty;
+	IStr(name,128);
+
+	if( curLogFd() == fileno(stderr) ){
+		dupLogFd(); /* 9.8.2 not to send log-output to pty */
+	}
+	pid = Forkpty(&pty,(char*)name);
+	if( 0 < pid ){
+		CStr(resp,0x10000);
+		FILE *fs;
+
+		sshPid = pid;
+		sshPty = pty;
+		sv1log("--SSH: pid=%d pty master %d %s\n",sshPid,pty,name);
+		tf[0] = pty;
+		tf[1] = pty;
+
+		msleep(100);
+		fs = fdopen(pty,"r");
+		relay_resp(fs,tc,15*1000,AVStr(resp),"FORK-1");
+		if( badpass(resp) && *pass ){
+			IStr(req,128);
+			sprintf(req,"%s\r\n",pass);
+			write(pty,req,strlen(req));
+			relay_resp(fs,tc,15*1000,AVStr(resp),"FORK-2");
+			badpass(resp);
+		}
+		fprintf(tc,"%s",resp);
+		fflush(tc);
+		return 0;
+	}else{
+		CStr(com,1024);
+		int rcode;
+		sprintf(com,"");
+		Xsprintf(TVStr(com),ssh_com,port,user,host);
+		sv1log("--SSH: %s\n",com);
+
+		closeAll();
+		rcode = Stty(0,"-echo,-echonl");
+		execsystem("ssh",com);
+		printf("ssh exec(%s) failed\n",com);
+		_exit(0);
+		return 0;
+	}
+}
+static void setpty(int pty,FILE *tc){
+	IStr(com,128);
+	sv1log("----STTY I%X O%X L%X C%X\n",
+		Gtty(pty,"iflags"),Gtty(pty,"oflags"),
+		Gtty(pty,"lflags"),Gtty(pty,"cflags"));
+
+	Stty(pty,"-echo"); // BSD
+
+	sprintf(com,"stty echo\n");
+	write(pty,com,strlen(com)); // for FedoraCore and SSH/Cygwin
+	/* shoul skip the response */
+
+	Stty(pty,"icrnl");
+	putIAC(tc,WILL,O_SUPAHEAD); fflush(tc);
+	putIAC(tc,DONT,O_ECHO); fflush(tc);
+	putIAC(tc,WILL,O_ECHO); fflush(tc);
+	putIAC(tc,DO,O_NAWS); /* initiate negotiation for WindowSize */
+}
+int getwinsize(int fd,int *row,int *col);
+int setwinsize(int fd,int row,int col);
+static void syncpty(int pty,FILE *tc){
+	/*
+	int row,col;
+	row = col = 0;
+	getwinsize(pty,&row,&col);
+	fprintf(stderr,"----STTY I%X O%X L%X C%X WinSize=%d,%d\n",
+		Gtty(pty,"iflags"),Gtty(pty,"oflags"),
+		Gtty(pty,"lflags"),Gtty(pty,"cflags"),
+		row,col);
+	*/
+}
+static int setWinSize(int pty,FILE *tc,unsigned PCStr(ub),int rcc){
+	int i,ocol,orow,col,row,nrow,ncol;
+	if( ub[1] == WILL || ub[1] == SB )
+	if( ub[2] == O_NAWS ){
+		if( ub[1] == SB ){
+			col = ub[4];
+			row = ub[6];
+		}else{
+			col = ub[7];
+			row = ub[9];
+		}
+		getwinsize(pty,&orow,&ocol);
+		setwinsize(pty,row,col);
+		//setwinsize(pty,row-1,col-1); // for SSH/Cygwin??
+		nrow = ncol = 0;
+		getwinsize(pty,&nrow,&ncol);
+		/*
+		for(i = 0; i < rcc; i++)
+			fprintf(stderr,"[%2d] %02X/%3d\n",i,ub[i],ub[i]);
+		*/
+		sv1log("----WinSize [%d,%d][%d,%d][%d,%d]\n",
+			ocol,orow,col,row,ncol,nrow);
+		return 1;
+	}
+	return 0;
+}
+static int relay1(FILE *tc,int fs,int ts,int cl){
+	int fdv[2];
+	int rdv[2];
+	int rdy,rcc,wcc;
+	CStr(buf,1024+1);
+	const unsigned char *ub = (const unsigned char*)buf;
+	int xpid;
+	int rcode;
+
+	fdv[0] = cl;
+	fdv[1] = fs;
+
+	if( badPass ){
+		Stty(sshPty,"-echo");
+		putIAC(tc,WILL,O_ECHO);
+		putIAC(tc,DONT,O_ECHO);
+		fflush(tc);
+	}else{
+		setpty(sshPty,tc);
+	}
+	for(;;){
+		rdy = PollIns(IO_TIMEOUT*1000,2,fdv,rdv);
+		syncpty(sshPty,tc);
+
+		if( rdy <= 0 ){
+			break;
+		}
+		if( rdv[0] ){ /* from client */
+			rcc = read(fdv[0],buf,sizeof(buf)-1);
+			if( rcc <= 0 ) break;
+			if( 2 <= rcc && ub[rcc-2] == '\r' && ub[rcc-1] == 0 ){
+				sv1log("C-S CR %d [%X][%d]\n",rcc,ub[0],ub[1]);
+				rcc--;
+			}
+			if( ub[0] == 0xFF ){
+				if( setWinSize(sshPty,tc,ub,rcc) ){
+				}else
+				if( ub[1] == DO && ub[2] == O_SUPAHEAD ){
+					sv1log("C-S %d DO SUPGA\n",rcc);
+					putIAC(tc,WILL,O_SUPAHEAD);
+				}else
+				if( ub[1] == DO && ub[2] == O_ECHO ){
+					sv1log("C-S %d DO ECHO\n",rcc);
+					/*
+					Stty(sshPty,"echo,echonl");
+					*/
+					putIAC(tc,WILL,O_ECHO);
+				}else{
+					fprintf(stderr,"C-S %d [%X][%d][%d]\n",
+						rcc,ub[0],ub[1],ub[2]);
+				}
+				fflush(tc);
+				continue;
+			}
+			/* ... scan Telnet command and erase ... */
+			wcc = write(ts,buf,rcc);
+			if( wcc <= 0 ) break;
+		}
+		if( rdv[1] ){ /* from ssh */
+			rcc = read(fdv[1],buf,sizeof(buf)-1);
+			if( rcc <= 0 ) break;
+			wcc = write(cl,buf,rcc);
+			if( wcc <= 0 ) break;
+			if( 1 <= rcc && ub[rcc-1] == '\n' )
+			if( rcc == 1 || ub[rcc-2] != '\r' )
+			{
+				sv1log("S-C NL to LF-CR %d\n",rcc);
+				write(cl,"\r",1);
+			}
+			if( 1 <= rcc && ub[rcc-1] == '\r' ){
+				sv1log("S-C CR to NL %d\n",rcc);
+				write(cl,"\n",1);
+			}
+			if( badPass ){
+				setVStrEnd(buf,rcc);
+				if( !badpass(buf) ){
+					setpty(sshPty,tc);
+				}
+			}
+		}
+	}
+	close(fs);
+	if( fs != ts ) close(ts);
+	close(cl);
+	xpid = NoHangWait();
+	return 0;
+}
+static int relay0(PCStr(host),int port,PCStr(user),PCStr(pass),int cl){
+	int sshv[2]; /* to the pty of ssh */
+	FILE *fc;
+	FILE *tc;
+	AuthInfo au;
+
+	fc = fdopen(cl,"r");
+	tc = fdopen(cl,"w");
+	if( user[0] == 0 || host[0] == 0 || streq(host,"-") ){
+		if( getUSERenv(fc,tc,&au) ){
+			if( au.i_user[0] ) user = au.i_user;
+			if( au.i_pass[0] ) pass = au.i_pass;
+			if( au.i_Host[0] ) host = au.i_Host;
+			if( au.i_Port    ) port = au.i_Port;
+		}else{
+ sv1log("--USER not given\n");
+ fprintf(tc,"*** Send the user name as:\r\n");
+ fprintf(tc,"*** % telnet -l UsrName delegate-host\r\n");
+			fclose(tc);
+			fclose(fc);
+			return 0;
+		}
+	}
+	forkSsh(host,port,user,pass,sshv,fc,tc);
+	relay1(tc,sshv[0],sshv[1],cl);
+	return 0;
+}
+int connectToSsh(Connection *Conn,PCStr(host),int port,PCStr(user),PCStr(pass)){
+	int sockv[2];
+	int nready;
+	int err;
+
+	if( sshThread ){
+		err = thread_wait(sshThread,1000);
+		Verbose("--PrevThread %X err=%d\n",sshThread,err);
+		sshThread = 0;
+	} 
+	if( sshPty != -1 ){
+		err = close(sshPty);
+		Verbose("--PrevPty %d err=%d\n",sshPty,err);
+		sshPty = -1;
+	}
+	if( sshPid ){
+		int xpid;
+		xpid = NoHangWait();
+		Verbose("--PrevPid %d %d\n",sshPid,xpid);
+		sshPid = 0;
+	}
+	badPass = 0;
+	Socketpair(sockv);
+	sshThread = thread_fork(0x40000,STX_tid,"relay0",(IFUNCP)relay0,host,port,user,pass,sockv[0]);
+	nready = PollIn(sockv[1],15*1000);
+	return sockv[1];
+}
