@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1996 Electrotechnical Laboratry (ETL), AIST, MITI
+Copyright (c) 1996-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -21,25 +22,35 @@ History:
 #include <errno.h>
 #include <stdio.h>
 #include "ystring.h"
+#include "file.h"
+#include "log.h"
 #include "vsocket.h"
+#include "dglib.h"
 
-extern FILE *dirfopen();
+int IsOS2EMX();
+void UnixSocketDir(PVStr(path));
+int connectTO(int sock,SAP addr,int leng,int timeout);
+void Myhostport(PVStr(myhp),int size);
+int SocketPair(int d,int type,int protocol,int sv[2]);
+int free_un(PCStr(what),PCStr(lpath),PVStr(spath),int size,int if_owner);
 
 int AF_UNIX_DISABLE = 0;
 int AF_UNIX_SOCKETPAIR_DISABLE = 0;
 static int un_seqno;
 
 #define errlog	sv1log
-extern int LOG_VERBOSE;
-#define dbglog	LOG_VERBOSE==0 ? 0 : sv1vlog
+#define dbglog	LOGLEVEL<2?0:Verbose
 
 #if defined(_SOCKADDR_LEN)/*OSF1*/ || defined(SUN_LEN)/*AIX*/
 /* must set sockaddr_un.sun_len ? */
 #else
 #endif
 
-Socketpair(sv)
-	int sv[];
+int INET_Socketpair(int sv[])
+{
+	return SocketPair(AF_INET,SOCK_STREAM,0,sv);
+}
+int Socketpair(int sv[])
 {
 	sv[0] = sv[1] = -1;
 
@@ -59,31 +70,33 @@ Socketpair(sv)
 	else	return socketpair(AF_UNIX,SOCK_STREAM,0,sv);
 }
 
-UDP_Socketpair(sv)
-	int sv[];
+int UDP_Socketpair(int sv[])
 {
 	/*
 	some system (Solaris) don't support SOCK_DGRAM on AF_UNIX sockets
 	return socketpair(AF_UNIX,SOCK_DGRAM,0,sv);
 	*/
+	/* MacOSX only supports socketpair(AF_UNIX) */
+	if( !isWindows() )
+	if( socketpair(AF_UNIX,SOCK_DGRAM,0,sv) == 0 )
+		return 0;
 	return socketpair(AF_INET,SOCK_DGRAM,0,sv);
 }
 
-server_open_localhost(what,path,nlisten)
-	char *what,*path;
+int server_open_localhost(PCStr(what),PVStr(path),int nlisten)
 {	int sock;
 	FILE *sockfp;
-	char sockname[256];
+	CStr(sockname,256);
 
-	sock = server_open(what,"localhost",0,nlisten);
+	sock = server_open(what,CVStr("localhost"),0,nlisten);
 	if( sock < 0 )
-	sock = server_open(what,NULL,0,nlisten);
+	sock = server_open(what,VStrNULL,0,nlisten);
 
 	if( sock < 0 ){
 		errlog("####LS cannot create local socket\n");
 	}else{
-		gethostName(sock,sockname,"%A:%P");
-		sockfp = dirfopen(what,path,"w");
+		gethostName(sock,AVStr(sockname),"%A:%P");
+		sockfp = dirfopen(what,AVStr(path),"w");
 		if( sockfp == NULL ){
 			errlog("####LS cannot create %s\n",path);
 			close(sock);
@@ -94,8 +107,8 @@ server_open_localhost(what,path,nlisten)
 			else{
 				fprintf(sockfp,"%s\n",sockname);
 /* DEBUG */{
-char myhp[256];
-Myhostport(myhp,sizeof(myhp));
+CStr(myhp,MaxHostNameLen);
+Myhostport(AVStr(myhp),sizeof(myhp));
 fprintf(sockfp,"# %s\n",myhp);
 fflush(sockfp);
 }
@@ -109,11 +122,11 @@ fflush(sockfp);
 	}
 	return sock;
 }
-client_open_localhost(what,path,timeout)
-	char *what,*path;
+int client_open_localhost(PCStr(what),PCStr(path),int timeout)
 {	int sock;
 	FILE *sockfp;
-	char sockname[256],host[256];
+	CStr(sockname,MaxHostNameLen);
+	CStr(host,MaxHostNameLen);
 	int port;
 
 	sockfp = fopen(path,"r");
@@ -128,10 +141,13 @@ client_open_localhost(what,path,timeout)
 	}
 
 	sock = -1;
-	if( fscanf(sockfp,"%[^\r\n]",sockname) != 1 )
+/*
+	if( fscanf (sockfp,"%[^\r\n]",sockname) != 1 )
+*/
+	if( Fgets(AVStr(sockname),sizeof(sockname),sockfp) == NULL )
 		errlog("####LS cannot read %s\n",path);
 	else
-	if( sscanf(sockname,"%[^:]:%d",host,&port) != 2 )
+	if( Xsscanf(sockname,"%[^:]:%d",AVStr(host),&port) != 2 )
 		errlog("####LS cannot parse [%s] %s\n",sockname,path);
 	else{
 		sock = client_open(what,what,host,port);
@@ -143,15 +159,16 @@ client_open_localhost(what,path,timeout)
 	return sock;
 }
 
-server_open_un(what,path,nlisten)
-	char *what,*path;
+/**/
+int server_open_un(PCStr(what),PVStr(path),int nlisten)
 {	struct sockaddr_un una;
 	int sock;
 	int rcode;
-	char dir[1024],*dp;
+	CStr(dir,1024);
+	const char *dp;
 
 	if( AF_UNIX_DISABLE )
-		return server_open_localhost(what,path,nlisten);
+		return server_open_localhost(what,AVStr(path),nlisten);
 
 	if( sizeof(una.sun_path) <= strlen(path) ){
 		errlog("#### bind_un: too long path: %s\n",path);
@@ -161,7 +178,7 @@ server_open_un(what,path,nlisten)
 	if( !IsOS2EMX() ){
 		strcpy(dir,path);
 		if( dp = strrpbrk(dir,"/\\") )
-			*dp = 0;
+			truncVStr(dp);
 		if( !fileIsdir(dir) ){
 			mkdirRX(dir);
 			if( !fileIsdir(dir) )
@@ -169,26 +186,34 @@ server_open_un(what,path,nlisten)
 		}
 	}
 
+	if( nlisten < 0 )
+		sock = socket(AF_UNIX,SOCK_DGRAM,0);
+	else
 	sock = socket(AF_UNIX,SOCK_STREAM,0);
 	una.sun_family = AF_UNIX;
-	strcpy(una.sun_path,path);
+	Xstrcpy(EVStr(una.sun_path),path);
 
 	rcode = bind(sock,(SAP)&una,sizeof(una));
 
 	if( rcode == 0 ){
+		if( 0 < nlisten ){
 		rcode = listen(sock,nlisten);
 		if( rcode != 0 )
 			errlog("#### bind_un: cannot listen, errno=%d\n",errno);
+		}
 		return sock;
 	}else{
 		close(sock);
 		return -1;
 	}
 }
-client_open_un(what,path,timeout)
-	char *what,*path;
+int client_open_unX(PCStr(what),int sock,PCStr(path),int timeout);
+int client_open_un(PCStr(what),PCStr(path),int timeout)
+{
+	return client_open_unX(what,-1,path,timeout);
+}
+int client_open_unX(PCStr(what),int sock,PCStr(path),int timeout)
 {	struct sockaddr_un una;
-	int sock;
 	int rcode;
 
 	if( AF_UNIX_DISABLE )
@@ -197,11 +222,15 @@ client_open_un(what,path,timeout)
 	if( sizeof(una.sun_path) <= strlen(path) )
 		return -1;
 
+	if( sock < 0 )
 	sock = socket(AF_UNIX,SOCK_STREAM,0);
-	una.sun_family = AF_UNIX;
-	strcpy(una.sun_path,path);
 
-	rcode = connectTO(sock,&una,sizeof(una),timeout);
+	bzero(&una,sizeof(una));
+	una.sun_family = AF_UNIX;
+	Xstrcpy(EVStr(una.sun_path),path);
+
+	errno = 0;
+	rcode = connectTO(sock,(struct sockaddr*)&una,sizeof(una),timeout);
 
 	if( rcode == 0 )
 		return sock;
@@ -212,15 +241,15 @@ client_open_un(what,path,timeout)
 	}
 }
 
-static put_link(what,lpath,spath)
-	char *what,*lpath,*spath;
+static int put_link(PCStr(what),PVStr(lpath),PCStr(spath))
 {	FILE *lfp,*vfp;
-	char xspath[2048],msg[2048];
+	CStr(xspath,2048);
+	CStr(msg,2048);
 	int eof;
 	int rcode = -1;
 
 	msg[0] = 0;
-	if( (lfp = dirfopen(what,lpath,"w")) == NULL ){
+	if( (lfp = dirfopen(what,AVStr(lpath),"w")) == NULL ){
 		strcpy(msg,"can't create");
 		goto EXIT;
 	}
@@ -260,13 +289,12 @@ EXIT:
 	else	dbglog("#### put_link %s %s\n",lpath,msg);
 	return rcode;
 }
-static get_link(what,lpath,spath,size,ntry)
-	char *what,*lpath,*spath;
+static int get_link(PCStr(what),PCStr(lpath),PVStr(spath),int size,int ntry)
 {	FILE *lfp = NULL;
-	int try = 0;
+	int xtry = 0;
 	int opened = 0;
 
-	spath[0] = 0;
+	setVStrEnd(spath,0);
 	for(;;){
 		if( lfp == NULL )
 			lfp = fopen(lpath,"r");
@@ -283,13 +311,13 @@ static get_link(what,lpath,spath,size,ntry)
 				lock_unlock(fileno(lfp));
 			}
 		}
-		if( spath[0] || ntry <= ++try )
+		if( spath[0] || ntry <= ++xtry )
 			break;
 		msleep(100);
 	}
 
-	if( 1 < try )
-	errlog("#### get_link [%d/%d] %x %s <%s>\n",opened,try,lfp,lpath,spath);
+	if( 1 < xtry )
+	errlog("#### get_link [%d/%d] %x %s <%s>\n",opened,xtry,lfp,lpath,spath);
 
 	if( lfp != NULL)
 		fclose(lfp);
@@ -298,9 +326,8 @@ static get_link(what,lpath,spath,size,ntry)
 	else	return -1;
 }
 
-static is_owner(spath)
-	char *spath;
-{	char *dp;
+static int is_owner(PCStr(spath))
+{	const char *dp;
 	int ctime,owner;
 
 	if( dp = strrchr(spath,'/') )
@@ -311,25 +338,25 @@ static is_owner(spath)
 			return 1;
 	return 0;
 }
-bind_un(what,lpath,backlog,spath,size)
-	char *what,*lpath,*spath;
-{	char file[1024],stime[128];
+int bind_un(PCStr(what),PVStr(lpath),int backlog,PVStr(spath),int size)
+{	CStr(file,1024);
+	CStr(stime,128);
 	int pid;
 	int sock;
-	int try;
+	int xtry;
 
-	if( free_un(what,lpath,spath,size,0) == 0 )
+	if( free_un(what,lpath,AVStr(spath),size,0) == 0 )
 		errlog("#### bind_un: salvaged %s <%s>\n",lpath,spath);
 
-	UnixSocketDir(spath);
+	UnixSocketDir(AVStr(spath));
 	pid = getpid();
-	sprintf(spath+strlen(spath),"%02d/",pid % 32);
-	StrftimeLocal(stime,sizeof(stime),"%m%d%H%M%S",time(0));
-	sprintf(spath+strlen(spath),"%s.%05d.%02d",stime,pid,++un_seqno);
+	Xsprintf(TVStr(spath),"%02d/",pid % 32);
+	StrftimeLocal(AVStr(stime),sizeof(stime),"%m%d%H%M%S",time(0),0);
+	Xsprintf(TVStr(spath),"%s.%05d.%02d",stime,pid,++un_seqno);
 
 	errno = 0;
-	for( try = 0; try++ < 10; ){
-		sock = server_open_un(what,spath,backlog);
+	for( xtry = 0; xtry++ < 10; ){
+		sock = server_open_un(what,AVStr(spath),backlog);
 		if( 0 <= sock )
 			break;
 		if( sock < -1 )
@@ -337,15 +364,15 @@ bind_un(what,lpath,backlog,spath,size)
 		msleep(100);
 	}
 
-	if( 1 < try || sock < 0 )
+	if( 1 < xtry || sock < 0 )
 		errlog("#### bind_un: %d * bind() = %d, errno=%d, %s\n",
-			try,sock,errno,spath);
+			xtry,sock,errno,spath);
 
 	if( 0 <= sock ){
 		if( 0 < File_size(lpath) )
 			errlog("#### bind_un FAILED: snatched %s\n",lpath);
 		else
-		if( put_link(what,lpath,spath) != 0 )
+		if( put_link(what,AVStr(lpath),spath) != 0 )
 			errlog("#### bind_un FAILED: can't link %s\n",lpath);
 		else{
 			dbglog("#### bind_un: bound [%s] %s\n",spath,lpath);
@@ -357,16 +384,15 @@ bind_un(what,lpath,backlog,spath,size)
 	unlink(spath);
 	return -1;
 }
-free_un(what,lpath,spath,size,if_owner)
-	char *what,*lpath,*spath;
+int free_un(PCStr(what),PCStr(lpath),PVStr(spath),int size,int if_owner)
 {	int rcode;
 	int do_unlink;
 
-	spath[0] = 0;
+	setVStrEnd(spath,0);
 	rcode = 0;
 	do_unlink = 1;
 
-	if( get_link(what,lpath,spath,size,1) == 0 ){
+	if( get_link(what,lpath,AVStr(spath),size,1) == 0 ){
 		if( if_owner && !is_owner(spath) ){
 			do_unlink = 0;
 			rcode = -1;
@@ -386,13 +412,12 @@ free_un(what,lpath,spath,size,if_owner)
 
 	return rcode;
 }
-connect_un(what,lpath,timeout)
-	char *what,*lpath;
-{	char spath[1024];
+int connect_un(PCStr(what),PCStr(lpath),int timeout)
+{	CStr(spath,1024);
 	FILE *lfp;
 	int sock;
 
-	if( get_link(what,lpath,spath,sizeof(spath),20) == 0 ){
+	if( get_link(what,lpath,AVStr(spath),sizeof(spath),20) == 0 ){
 		sock = client_open_un(what,spath,timeout);
 		if( 0 <= sock ){
 			dbglog("#### connect_un: connected %s\n",spath);
