@@ -17,15 +17,19 @@ Author:		Yutaka Sato <ysato@etl.go.jp>
 Description:
 History:
 	940824	created
+TODO:
+	must be merged into dget.c
+	must check recursion using MD5
 //////////////////////////////////////////////////////////////////////#*/
 #include <stdio.h>
 #include "ystring.h"
+#include "dglib.h"
 
-static char *usage = "\
+static const char *usage = "\
 Usage: urlfind URL\n\
     -- Find recursively in URL space.\n\
 ";
-static char *arg1spec = "\
+static const char *arg1spec = "\
 Argument specification error:\n\
    The first argument should be URL as follows:\n\
       protocol://host\n\
@@ -34,28 +38,31 @@ Argument specification error:\n\
       protocol://host:port/path\n\
 ";
 
-urlfind_main(ac,av)
-	char *av[];
+static void pull_url1(PCStr(url));
+int urlfind_main(int ac,const char *av[])
 {
 	if( ac < 2 ){
 		fprintf(stderr,"%s",usage);
 		exit(1);
 	}
 	pull_url1(av[1]);
+	return 0;
 }
 
-static int http1();
-static char lastURL[1024];
+static struct { defQStr(lastURL); } lastURL;
+static void http1(int svsock,PCStr(proto),PCStr(host),int port,PCStr(path));
 
-static pull_url1(url)
-	char *url;
-{	char proto[1024],hostport[1024],host[1024],path[1024];
+static void pull_url1(PCStr(url))
+{	CStr(proto,1024);
+	CStr(hostport,1024);
+	CStr(host,1024);
+	CStr(path,1024);
 	int port;
 	int svsock;
-	char url1[1024];
+	CStr(url1,1024);
 
 	path[0] = 0;
-	if( sscanf(url,"%[^:]://%[^/]%s",proto,hostport,path) < 2 ){
+	if( Xsscanf(url,"%[^:]://%[^/]%s",AVStr(proto),AVStr(hostport),AVStr(path)) < 2 ){
 		fprintf(stderr,"%s",arg1spec);
 		exit(1);
 	}
@@ -66,13 +73,16 @@ static pull_url1(url)
 		fprintf(stderr,"Only protocol `http' is supported, sorry.\n");
 		exit(1);
 	}
-	if( sscanf(hostport,"%[^:]:%d",host,&port) != 2 )
+	if( Xsscanf(hostport,"%[^:]:%d",AVStr(host),&port) != 2 )
 		port = serviceport(proto);
 
 	sprintf(url1,"%s://%s/%s",proto,hostport,path[0]=='/'?path+1:path);
-	if( strcmp(url1,lastURL) == 0 )
+	if( lastURL.lastURL == 0 )
+		setQStr(lastURL.lastURL,(char*)StructAlloc(1024),1024);
+
+	if( strcmp(url1,lastURL.lastURL) == 0 )
 		return;
-	strcpy(lastURL,url1);
+	strcpy(lastURL.lastURL,url1);
 
 	fprintf(stderr,"%s ",url1);
 	fflush(stderr);
@@ -85,12 +95,10 @@ static pull_url1(url)
 	http1(svsock,proto,host,port,path);
 }
 
-static url1(url,base,urlfp)
-	char *url;
-	char *base;
-	FILE *urlfp;
-{	char xurl[4096],*bp;
-	char *dp;
+static void url1(PCStr(url),PCStr(base),FILE *urlfp)
+{	CStr(xurl,4096);
+	const char *bp;
+	const char *dp;
 
 	if( strncmp(url,"http:",5) == 0 )
 	if( url[5] != '/' )
@@ -107,25 +115,26 @@ static url1(url,base,urlfp)
 	if( strchr(url,':') == 0 ){
 		strcpy(xurl,base);
 		if( bp = strrchr(xurl,'/') )
-			bp[1] = 0;
+			((char*)bp)[1] = 0;
 	}
 	strcat(xurl,url);
 
 	if( strncmp(xurl,base,strlen(base)) == 0 )
 		fprintf(urlfp,"%s\n",xurl);
 }
-static http1(svsock,proto,host,port,path)
-	char *proto,*host,*path;
+static void http1(int svsock,PCStr(proto),PCStr(host),int port,PCStr(path))
 {	FILE *fs,*ts;
 	FILE *urlfp;
-	char request[1024];
-	char resp[1024],xline[4096];
-	char *url;
-	char type[256];
+	CStr(request,1024);
+	CStr(resp,1024);
+	CStr(xline,4096);
+	const char *url;
+	CStr(type,256);
 	int leng;
-	char hostport[256],base[1024];
+	CStr(hostport,MaxHostNameLen);
+	CStr(base,1024);
 
-	HostPort(hostport,proto,host,port);
+	HostPort(AVStr(hostport),proto,host,port);
 	sprintf(base,"%s://%s/%s",proto,hostport,path[0]=='/'?path+1:path);
 
 	ts = fdopen(svsock,"w");
@@ -137,26 +146,26 @@ static http1(svsock,proto,host,port,path)
 
 	if( fgets(resp,sizeof(resp),fs) == NULL ){
 		fprintf(stderr,"[NULL]\n");
-		goto ERR;
+		goto xERR;
 	}
 
-	if( strncmp(resp,"HTTP/1.0",8) != 0 ){
-		fprintf(stderr,"[NON-HTTP/1.0]\n");
-		goto ERR;
+	if( strncmp(resp,"HTTP/1.",7) != 0 ){
+		fprintf(stderr,"[NON-HTTP/1.X]\n");
+		goto xERR;
 	}
 
 	type[0] = 0;
 	leng = 0;
 	while( fgets(resp,sizeof(resp),fs) != NULL ){
-		char *dp;
+		const char *dp;
 
 		if( dp = strpbrk(resp,"\r\n") )
-			*dp = 0;
+			truncVStr(dp);
 		if( dp == resp )
 			break;
 
 		if( strncasecmp("Content-Type:",resp,12) == 0 )
-			sscanf(resp,"%*s %[^;\r\n]",type);
+			Xsscanf(resp,"%*s %[^;\r\n]",AVStr(type));
 		else
 		if( strncasecmp("Content-Length:",resp,13) == 0 )
 			sscanf(resp,"%*s %d",&leng);
@@ -164,28 +173,29 @@ static http1(svsock,proto,host,port,path)
 	fprintf(stderr,"[%s][%d]\n",type,leng);
 	fflush(stderr);
 	if( strcmp(type,"text/html") != 0 )
-		goto ERR;
+		goto xERR;
 
 	urlfp = (FILE*)tmpfile();
 	while( fgets(resp,sizeof(resp),fs) != NULL ){
-		url_absolute("-.-",proto,host,port,"",resp,xline);
-		scan_url(xline,url1,base,urlfp);
+		url_absolute("-.-",proto,host,port,"",resp,AVStr(xline),VStrNULL);
+		scan_url(xline,(iFUNCP)url1,base,urlfp);
 	}
 	fclose(ts);
 	fclose(fs);
 	fflush(urlfp);
 	fseek(urlfp,0,0);
-	{	char url[1024],*np;
+	{	CStr(url,1024);
+		const char *np;
 
 		while( fgets(url,sizeof(url),urlfp) != NULL ){
 			if( np = strpbrk(url,"\r\n") )
-				*np = 0;
+				truncVStr(np);
 			pull_url1(url);
 		}
 	}
 	fclose(urlfp);
 	return;
-ERR:
+xERR:
 	fclose(ts);
 	fclose(fs);
 }
