@@ -1,11 +1,13 @@
 /*////////////////////////////////////////////////////////////////////////
 Copyright (c) 1994-2000 Yutaka Sato and ETL,AIST,MITI
-Copyright (c) 2001-2005 National Institute of Advanced Industrial Science and Technology (AIST)
+Copyright (c) 2001-2008 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
 
-Permission to use this material for evaluation, copy this material for
-your own use, and distribute the copies via publically accessible on-line
-media, without fee, is hereby granted provided that the above copyright
-notice and this permission notice appear in all copies.
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
 AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
@@ -16,17 +18,21 @@ OR IMPLIED WARRANTIES.
 #include "log.h"
 
 #define NAME	"DeleGate"
-#define VERSION	"9.0.2"
-#define DATE	"May 18, 2005"
+#define VERSION	"9.9.1"
+#define DATE	"January 21, 2009"
+#define DSTATUS	"STABLE"
 #define AUTHOR	"Yutaka Sato"
 #define A_ORG	"National Institute of Advanced Industrial Science and Technology"
 #define A_EMAIL	"ysato AT delegate DOT org"
 #define A_SITE	"delegate.org"
 
 #define COPYRIGHT "\
-AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165\r\n\
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443\r\n\
 Copyright (c) 1994-2000 Yutaka Sato and ETL,AIST,MITI\r\n\
-Copyright (c) 2001-2005 National Institute of Advanced Industrial Science and Technology (AIST)"
+Copyright (c) 2001-2009 National Institute of Advanced Industrial Science and Technology (AIST)"
+
+#ifndef _VERSION_H
+extern const char *MyVer;
 
 static char _VERSION[32] = VERSION;
 static char _DATE[32] = DATE;
@@ -82,15 +88,36 @@ const char *DELEGATE_copyright(){
 	}
 	return copyright;
 }
+const char *DELEGATE_licensee(){
+	if( LICENSEE && *LICENSEE )
+		return LICENSEE;
+	else	return "non-commercial and/or evaluation users";
+}
 
 const char *DELEGATE_ver()
 {
+	MyVer = VERSION;
 	return VERSION;
 }
 const char *DELEGATE_date()
 {
 	return DATE;
 }
+int getExeVer(PVStr(ver)){
+	int v[4];
+	int n;
+	v[0] = v[1] = v[2] = v[3] = 0;
+	if( sscanf(VERSION,"%d.%d.%d-pre%d",&v[0],&v[1],&v[2],&v[3]) == 4 ){
+	}else{
+		sscanf(VERSION,"%d.%d.%d-fix%d",&v[0],&v[1],&v[2],&v[3]);
+		v[3] = 0x80 | v[3];
+	}
+	setVStrElem(ver,0,v[0]);
+	setVStrElem(ver,1,v[1]);
+	setVStrElem(ver,2,v[2]);
+	setVStrElem(ver,3,v[3]);
+	return (v[0] << 24) | (v[1] << 16) | (v[2] << 8) | v[3];
+}
 static const char *verdate;
 const char *DELEGATE_verdate(){
 	CStr(buf,128);
@@ -137,6 +164,9 @@ const char *DELEGATE_Distribution(){
 const char *DELEGATE_homepage(){
 	return "http://www.delegate.org/delegate/";
 }
+const char *DELEGATE_builtout(){
+	return "http://www.delegate.org/delegate/ext/builtin/*";
+}
 
 void put_identification(FILE *out)
 {
@@ -157,3 +187,45 @@ iaDXL4z+x+oRvaQulEFUFOdZxFMYDL7AgXN/wFoRktkbrd6I0HdMoPPdekJC6216\r\n\
 m5E5y+hrNTxYSOMP/QIDAQAB\r\n\
 -----END PUBLIC KEY-----\r\n\
 ";
+
+const char *DELEGATE_srcsign();
+int checkVer(){
+	const char *ver;
+	CStr(xver,128);
+	CStr(xdate,128);
+	const char *xsign;
+
+	ver = DELEGATE_ver();
+	truncVStr(xver);
+	truncVStr(xdate);
+	Xsscanf(DELEGATE_srcsign(),"%[^:]:%[^:]",AVStr(xver),AVStr(xdate));
+	if( xsign = strrchr(DELEGATE_srcsign(),':') )
+		xsign++;
+	else	xsign = "";
+	if( strcmp(ver,xver) != 0 ){
+		fprintf(stderr,"!! inconsistent version (DeleGate/%s) !!\n",
+			ver);
+		return -1;
+	}
+	return 0;
+}
+unsigned int verFpi(int crc){
+	char oct;
+	const char *fp;
+	int ovf;
+	int bi;
+
+	for( fp = DELEGATE_srcsign(); oct = *fp; fp++ ){
+		for( bi = 0; bi < 8; bi++ ){
+			/*
+			ovf = (crc < 0) ^ (oct < 0);
+			*/
+			ovf = (crc < 0) ^ ((oct & 0x80) != 0);
+			crc <<= 1;
+			oct <<= 1;
+			if( ovf ) crc ^= 0x04C11DB7;
+		}
+	}
+	return crc;
+}
+#endif
