@@ -1,13 +1,16 @@
-/*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1997 Electrotechnical Laboratry (ETL), AIST, MITI
+char *SIGN_vsap_c="{FILESIGN=vsap.c:20090121161131+0900:b43e6b203501214f:Author@DeleGate.ORG:DFTOoIAyWGTjbcvsXnlgM3/G9AjffyY7WcXNwVlY0UnwtU6k2vjrFGbG6UPAcD+j5YztUXsJG3+9HCI07rK0TQRFvgBsiyL4jdyLEKLEiUbEzF6g5aT+iyLB53wVvWUz+0BcL/gU3Ts4d2Zf2glsbMoRlRa9L55/7ndQ9EOoZJM=}";
 
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+/*////////////////////////////////////////////////////////////////////////
+Copyright (c) 1997-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2008 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
+
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -269,11 +272,11 @@ static void closeSvSocks()
 }
 
 static int do_connect(int lsock,PCStr(rport),int toC)
-{	CStr(host,256);
+{	CStr(host,MaxHostNameLen);
 	int port;
 	int clsock;
-	CStr(sockname,256);
-	CStr(peername,256);
+	CStr(sockname,MaxHostNameLen);
+	CStr(peername,MaxHostNameLen);
 	int wcc;
 	int sockid;
 
@@ -353,7 +356,7 @@ static int NewPort(PCStr(lport),PVStr(host),int port,int toC,int nlisten)
 }
 static int do_bind(int sock,PVStr(lport),PCStr(opts),int *sharedp,PVStr(sockname),int toC)
 {	int svsock;
-	CStr(host,256);
+	CStr(host,MaxHostNameLen);
 	CStr(ports,256);
 	int port;
 	int nlisten;
@@ -396,6 +399,11 @@ static int do_bind(int sock,PVStr(lport),PCStr(opts),int *sharedp,PVStr(sockname
 		daemonlog("D","## RESV_PORT %d\n",svsock);
 		*sharedp = 1;
 	}else
+	if( lSINGLEP()
+	 && 0 <= (svsock = findopen_port("VSAP",AVStr(host),port,nlisten)) ){
+		daemonlog("D","## SHARED_PORT %d\n",svsock);
+		*sharedp = 1;
+	}else
 	if( 0 <= (svsock = NewPort(lport,AVStr(host),port,toC,nlisten)) ){
 		daemonlog("D","## NEW_PORT %d\n",svsock);
 		*sharedp = 0;
@@ -424,8 +432,8 @@ static int do_accept(PCStr(lport),PCStr(opts),int shared,int priobase,int fromC,
 	int rcode;
 	int clsock;
 	int wcc;
-	CStr(sockname,256);
-	CStr(peername,256);
+	CStr(sockname,MaxHostNameLen);
+	CStr(peername,MaxHostNameLen);
 	int fi,fdv[32],rfv[32],nready;
 	int svsock;
 	int priority,timeout;
@@ -599,7 +607,7 @@ static int intcom(PCStr(arg),int src,int dst,int *rccp,int *wccp)
 }
 static void do_forward(Connection *Conn,PCStr(lport),PCStr(peername),int shared,int svsock,int priority,int fromC,int toC)
 {	int cls,svs;
-	CStr(host,256);
+	CStr(host,MaxHostNameLen);
 	int port;
 	CStr(opt,256);
 	CStr(clntname,1024);
@@ -688,8 +696,8 @@ int service_vsap(Connection *Conn)
 	const char *req;
 	int svsock,shared,clsock,rcode;
 	CStr(myport,256);
-	CStr(sockname,256);
-	CStr(peername,256);
+	CStr(sockname,MaxHostNameLen);
+	CStr(peername,MaxHostNameLen);
 	int wcc,rcc;
 	CStr(com,1024);
 	CStr(arg,1024);
@@ -766,7 +774,7 @@ daemonlog("E","CLIENT-SAYS: %s",request);
 		lineScan(argp,arg);
 
 		if( strcasecmp(com,"AUTH") == 0 ){
-			CStr(ahost,256);
+			CStr(ahost,MaxHostNameLen);
 			ahost[0] = 0;
 			if( doAUTH(Conn,NULL,authout,"vsap","-",0,AVStr(arg),AVStr(ahost),NULL,NULL) == EOF ){
 			}else{
@@ -897,6 +905,11 @@ EXIT:
 #define FOR_CONNECT	1
 #define FOR_ACCEPT	2
 
+#define FOR_HTCONNECT	0x10
+#define FOR_HTBIND	0x20
+#define FOR_HTACCEPT	0x40
+#define FOR_HTALL	(FOR_HTCONNECT|FOR_HTBIND|FOR_HTACCEPT)
+
 typedef struct {
   const	char   *r_host;
 	int	r_port;
@@ -921,7 +934,7 @@ void minit_vsapcl(){
 }
 
 static scanListFunc vsap1(PCStr(vsap))
-{	CStr(host,256);
+{	CStr(host,MaxHostNameLen);
 	int port;
 	CStr(methods,256);
 	int ri;
@@ -949,6 +962,8 @@ static scanListFunc vsap1(PCStr(vsap))
 			serverV[serverX].r_methods |= FOR_CONNECT;
 		if( strstr(methods,"ACCEPT") )
 			serverV[serverX].r_methods |= FOR_ACCEPT;
+		if( isinListX(methods,"HTTP","c") )
+			serverV[serverX].r_methods |= FOR_HTALL;
 	}
 	serverX++;
 	return 0;
@@ -987,6 +1002,8 @@ static int resp_OK(PCStr(resp))
 	return 0;
 }
 
+int connectViaUpper(Connection *Conn,PCStr(where),PCStr(proto),PCStr(host),int port);
+int connect2server(Connection *Conn,PCStr(proto),PCStr(host),int port);
 static int open_vsap(Connection *Conn,int method)
 {	int rsock;
 	int ri;
@@ -1001,12 +1018,24 @@ static int open_vsap(Connection *Conn,int method)
 		if( acceptedViaVSAP == 0 )
 			if( (serverV[ri].r_methods & method) == 0 )
 				continue;
+		if( method == FOR_HTACCEPT )
+		if( serverV[ri].r_methods & FOR_HTACCEPT ){
+			return 1;
+		}
 		host = serverV[ri].r_host;
 		port = serverV[ri].r_port;
+		rsock = -1;
+		if( method & (FOR_HTBIND|FOR_HTCONNECT) ){
+			rsock = connectViaUpper(Conn,"VSAP","http",host,port);
+		}
+		if( rsock < 0 )
 		rsock = Socket1("VSAP",NEWSOCK,VStrANYPORT,host,port,0,NULL,0);
 		if( 0 <= rsock )
 			break;
 	}
+	if( method & (FOR_HTCONNECT|FOR_HTBIND|FOR_HTACCEPT) ){
+		return rsock;
+	}
 	if( 0 <= rsock ){
 		const char *auth;
 		CStr(authb,256);
@@ -1023,8 +1052,28 @@ static int open_vsap(Connection *Conn,int method)
 	return rsock;
 }
 
-int VSAPgetsockname(int rsock,PVStr(sockname))
+int gettelesockname(DGC*Conn,PVStr(name));
+int getteleportname(DGC*Conn,PVStr(name));
+int isNotClientSock(DGC*Conn,int sock){
+	if( sock == ServerSock || sock == ToS || sock == FromS
+	 || sock != ClientSock && sock != ToC && sock != FromC
+	){
+		return 1;
+	}
+	return 0;
+}
+int VSAPgetsockname(DGC*Conn,int rsock,PVStr(sockname))
 {
+	if( AccViaHTMUX == 0 ){
+		/* 9.9.1 don't use non-HTMUX RIDENT to bind FTP data-conn. */
+	}else
+	if( isNotClientSock(Conn,rsock) ){
+		/* don't use client side RIDENT as a server RIDENT */
+	}else{
+		if( gettelesockname(Conn,BVStr(sockname)) == 0 ){
+			return 0;
+		}
+	}
 	setVStrEnd(sockname,0);
 	if( elnumof(Socknames) <= rsock )
 		return -1;
@@ -1034,8 +1083,17 @@ int VSAPgetsockname(int rsock,PVStr(sockname))
 	}
 	return -1;
 }
-int VSAPgetpeername(int rsock,PVStr(peername))
+int VSAPgetpeername(DGC*Conn,int rsock,PVStr(peername))
 {
+	if( AccViaHTMUX == 0 ){
+		/* 9.9.1 don't use non-HTMUX RIDENT to bind FTP data-conn. */
+	}else
+	if( isNotClientSock(Conn,rsock) ){
+	}else{
+		if( getteleportname(Conn,BVStr(peername)) == 0 ){
+			return 0;
+		}
+	}
 	setVStrEnd(peername,0);
 	if( elnumof(Peernames) <= rsock )
 		return -1;
@@ -1046,12 +1104,24 @@ int VSAPgetpeername(int rsock,PVStr(peername))
 	return -1;
 }
 
+int ConnectViaHTTP(Connection *Conn,int sock,int timeout,PVStr(sockname),PVStr(peername));
+int AcceptViaHTTP(Connection *Conn,int ba,int sock,int timeout,int priority,PCStr(rport),PVStr(sockname),PVStr(peername));
+
 int CTX_VSAPconnect(Connection *Conn,PVStr(sockname),PVStr(peername))
 {	CStr(resp,1024);
 	int rsock;
 	int cid;
 	int acode;
 
+	rsock = open_vsap(Conn,FOR_HTCONNECT);
+	if( 0 <= rsock ){
+		rsock = ConnectViaHTTP(Conn,rsock,0,
+			BVStr(sockname),BVStr(peername));
+		if( 0 <= rsock ){
+			goto EXIT;
+		}
+		return -1;
+	}
 	rsock = open_vsap(Conn,FOR_CONNECT);
 	if( rsock < 0 )
 		return -1;
@@ -1080,99 +1150,13 @@ int CTX_VSAPconnect(Connection *Conn,PVStr(sockname),PVStr(peername))
 	SockPrintf(rsock,"RELAY\r\n");
 	Xsscanf(resp,"%*s %*d %d %s %s connected.",&cid,AVStr(sockname),AVStr(peername));
 
+EXIT:
+	sv1log("-- VSAPconnect[%d][%s][%s]\n",rsock,sockname,peername);
 	Socknames[rsock] = stralloc(sockname);
 	Peernames[rsock] = stralloc(peername);
 	return rsock;
 }
-int CTX_VSAPbind(Connection *Conn,PVStr(sockname),int nlisten)
-{	int rsock;
-	CStr(resp,1024);
-	int cid;
-	int acode;
 
-	rsock = open_vsap(Conn,FOR_ACCEPT);
-	if( rsock < 0 )
-		return -1;
 
-	SockPrintf(rsock,"%s BIND %s -l=%d\r\n",VER,sockname,nlisten);
-	if( RecvLine(rsock,resp,sizeof(resp)) <= 0 ){
-		close(rsock);
-		return -1;
-	}
-	if( !resp_OK(resp) ){
-		daemonlog("E","VSAPd says: %s",resp);
-		close(rsock);
-		return -1;
-	}
-	daemonlog("D","VSAPd says: %s",resp);
+/* '"DIGEST-OFF"' */
 
-	Xsscanf(resp,"%*s %*d %d %s",&cid,AVStr(sockname));
-	return rsock;
-}
-int VSAPaccept(int timeout,int rsock,int priority,PVStr(sockname),PVStr(peername))
-{	CStr(resp,1024);
-	int cid;
-	int acode;
-
-	setVStrEnd(peername,0);
-	setVStrEnd(sockname,0);
-
-	if( serverX == 0 )
-		return -1;
-	if( rsock < 0 )
-		return -1;
-
-	if( elnumof(Socknames) <= rsock ){
-		return -1;
-	}
-
-	SockPrintf(rsock,"ACCEPT -p=%d -t=%d\r\n",priority,timeout);
-	if( RecvLine(rsock,resp,sizeof(resp)) <= 0 ){
-		daemonlog("E","## accept failed: VSAPd closed.\n");
-		return -1;
-	}
-
-	daemonlog("D","VSAPd says: %s",resp);
-	if( resp_OK(resp) ){
-		Xsscanf(resp,"%*s %*d %d %s %s",&cid,AVStr(sockname),AVStr(peername));
-/*
-		if( streq(iSERVER_PROTO,"ftp")
-		 || streq(iSERVER_PROTO,"http")
-		){
-daemonlog("E","############ PROXY http\n");
-			SockPrintf(rsock,"PROXY %s\r\n",iSERVER_PROTO);
-		}else
-*/
-/*
--t timeout
-*/
-		SockPrintf(rsock,"RELAY\r\n");
-	}
-
-	if( *sockname == 0 ){
-		if( resp_OK(resp) )
-			daemonlog("E","## NO SOCKNAME GOT\n");
-		return -1;
-	}
-	acceptedViaVSAP = 1;
-	Socknames[rsock] = stralloc(sockname);
-	Peernames[rsock] = stralloc(peername);
-	return 0;
-}
-int CTX_VSAPbindaccept(Connection *Conn,int timeout,int priority,PVStr(sockname),PVStr(peername))
-{	CStr(rport,256);
-	int rsock,rcode;
-
-	setVStrEnd(peername,0);
-	setVStrEnd(sockname,0);
-
-	printPrimaryPort(AVStr(rport));
-	if( (rsock = CTX_VSAPbind(Conn,AVStr(rport),1)) < 0 )
-		return -1;
-
-	if( VSAPaccept(timeout,rsock,priority,AVStr(sockname),AVStr(peername)) < 0 ){
-		close(rsock);
-		rsock = -1;
-	}
-	return rsock;
-}
