@@ -18,15 +18,14 @@ History:
 //////////////////////////////////////////////////////////////////////#*/
 #include <stdio.h>
 #include "delegate.h"
-#include "http.h" /* putHttpHeader */
-extern char *malloc();
-extern FILE *openHttpResponseFilter();
+#include <fcntl.h>
+int codeconv_bufsize(int ccode,int size);
 
 #define STRLEN(a)	strlen((char*)a)
 #define STRCHR(a,b)	strchr((char*)a,b)
-#define STRCAT(a,b)	strcat((char*)a,(char*)b)
-#define STRCPY(a,b)	strcpy((char*)a,(char*)b)
-#define STRNCPY(a,b,n)	strncpy((char*)a,(char*)b,n)
+#define STRCAT(a,b)	Xstrcat(QVStr((char*)a,a),(char*)b)
+#define STRCPY(a,b)	Xstrcpy(QVStr((char*)a,a),(char*)b)
+#define STRNCPY(a,b,n)	Xstrncpy(QVStr((char*)a,a),(char*)b,n)
 
 #define MAX_REC_DOC	100
 
@@ -41,7 +40,7 @@ extern FILE *openHttpResponseFilter();
 typedef struct {
 	int	t_tag;
 	int	t_type;
-	char   *t_name;
+  const	char   *t_name;
 } Tags;
 
 /*
@@ -126,9 +125,7 @@ static Tags TagTab[] = {
 		val = (val << 7) | (buf[idx++] & 0x7F); \
 	val = (val << 7) | buf[idx++]; \
 }
-static relay_taglen(src,dst,rem)
-	FILE *src,*dst;
-	int *rem;
+static int relay_taglen(FILE *src,FILE *dst,int *rem)
 {	int val,ch;
 
 	val = 0;
@@ -142,68 +139,74 @@ static relay_taglen(src,dst,rem)
 	return val;
 }
 
-static scan_decF(buf,len)
-	unsigned char *buf;
+static int scan_decF(unsigned char *buf,int len)
 {	int iv,ix;
 	iv = 0;
 	for( ix = 0; ix < len; ix++ )
 		iv = (iv << 8) | buf[ix];
 	return iv;
 }
-static unsigned char *dstrncpy(dst,src,len)
-	unsigned char *dst,*src;
-{	int l1,ch;
+static char *dstrncpy(xPVStr(dst),PCStr(src),int len)
+{	int l1;
+	unsigned char ch;
 
 	for( l1 = 0; l1 < len; l1++ ){
+		assertVStr(dst,dst+1);
 		ch = *src++;
 		if( ch<=0x20||0x7F<=ch || ch=='%'||ch==';'||ch=='"' ){
-			sprintf((char*)dst,"%%%02x",ch & 0xFF);
+			sprintf(dst,"%%%02x",ch & 0xFF);
 			dst += 3;
-		}else	*dst++ = ch;
+		}else	setVStrPtrInc(dst,ch);
 	}
-	*dst = 0;
-	return dst;
+	setVStrEnd(dst,0);
+	return (char*)dst;
 }
-static encode_did(pbuf,buf,len)
-	unsigned char *pbuf,*buf;
+static void encode_did(PVStr(pbuf),unsigned char *buf,int len)
 {	int idx;
-	unsigned char *pp;
-	int tag,l1;
+	refQStr(pp,pbuf); /**/
+	int l1,tag;
 
-	pp = pbuf;
 	for( idx = 0; idx < len; ){
+		assertVStr(pbuf,pp+1);
 		if( buf[idx] < 0x20 ){
 			tag = buf[idx++];
-			sprintf((char*)pp,"%d=",tag);
+			sprintf(pp,"%d=",tag);
 			pp += STRLEN(pp);
 			scan_int(buf,idx,l1);
-			pp = dstrncpy(pp,&buf[idx],l1);
+			pp = dstrncpy(AVStr(pp),(char*)&buf[idx],l1);
 			idx += l1;
-			*pp++ = ';';
-		}else	*pp++ = buf[idx++];
+			setVStrPtrInc(pp,';');
+		}else	setVStrPtrInc(pp,buf[idx++]);
 	}
-	*pp = 0;
+	setVStrEnd(pp,0);
 }
 
-static decode_did(edid,ddid)
-	unsigned char *edid,*ddid;
+static int decode_did(unsigned char *edid,unsigned char *ddid)
 {	int tag,l1;
-	unsigned char ch,*ep,*fp,*dp,*tp,*np,ebuf[0x4000],elem[0x1000];
+	unsigned char ch;
+	const unsigned char *ep;
+	const unsigned char *fp;
+	unsigned char *dp;
+	unsigned char *tp;
+	const unsigned char *np;
+	unsigned CStr(ebuf,0x4000);
+	CStr(elem,0x1000);
 
 	STRCPY(ebuf,edid);
 	dp = ddid;
 	for( ep = ebuf; *ep; ep = np ){
-		if( np = (unsigned char*)STRCHR(ep,';') )
-			*np++ = 0;
-		if( sscanf((char*)ep,"%d=%s",&tag,elem) != 2 )
+		if( np = (unsigned char*)STRCHR(ep,';') ){
+			truncVStr(np); np++;
+		}
+		if( Xsscanf((char*)ep,"%d=%s",&tag,AVStr(elem)) != 2 )
 			break;
 
 		dp[0] = tag;
 		dp[1] = 0;
 		tp = &dp[2]; 
-		for( fp = elem; ch = *fp++; ){
+		for( fp = (unsigned char*)elem; ch = *fp++; ){
 			if( ch == '%' ){
-				char xb[3];
+				CStr(xb,3);
 				int xv;;
 				xb[0] = *fp++;
 				xb[1] = *fp++;
@@ -221,11 +224,9 @@ static decode_did(edid,ddid)
 	return dp - ddid;
 }
 	
-static print_data(pbuf,tag,len,buf)
-	char *pbuf;
-	unsigned char *buf;
+static int print_data(xPVStr(pbuf),int tag,int len,unsigned char *buf)
 {	int ti,iv,ttype;
-	char *tname;
+	const char *tname;
 	Tags *ttab;
 
 	ttype = 0;
@@ -243,23 +244,23 @@ static print_data(pbuf,tag,len,buf)
 		pbuf += STRLEN(pbuf);
 		switch( ttype ){
 		    case _DI:
-			encode_did(pbuf,buf,len);
+			encode_did(AVStr(pbuf),buf,len);
 			break;
 		    case _ST:
 			break;
 		    case _TX:
 
 if( tag == PD_DatabaseNames ){
-	char tb[1024];
+	CStr(tb,1024);
 	STRNCPY(tb,buf,len);
-	tb[len] = 0;
+	setVStrEnd(tb,len);
 	sv1log("Database-Names: %s\n",tb);
 }else
 if( tag == PD_Headline ){
 	STRNCPY(pbuf,buf,len);
-	pbuf[len] = 0;
+	setVStrEnd(pbuf,len);
 }else
-			dstrncpy(pbuf,buf,len);
+			dstrncpy(AVStr(pbuf),(char*)buf,len);
 			break;
 		    case _BM:
 			iv = scan_decF(buf,len);
@@ -285,11 +286,9 @@ if( tag == PD_Headline ){
 	return ttype;
 }
 
-static dump_record(buf,buflen,out)
-	unsigned char *buf,*out;
-{	unsigned char *bp;
-	int tag,len,idx,idx0;
-	char pbuf[1024];
+static void dump_record(unsigned char *buf,int buflen,PVStr(out))
+{	int tag,len,idx,idx0;
+	CStr(pbuf,1024);
 	int ttype;
 
 	idx = 0;
@@ -297,12 +296,12 @@ static dump_record(buf,buflen,out)
 		idx0 = idx;
 		scan_int(buf,idx,tag);
 		scan_int(buf,idx,len);
-		ttype = print_data(pbuf,tag,len,&buf[idx]);
+		ttype = print_data(AVStr(pbuf),tag,len,&buf[idx]);
 		Verbose("%5d [tag=%3d;len=%3d]%d %s\n",
 			idx0,tag,len,ttype,pbuf);
 
 		if( ttype == _ST )
-			dump_record(&buf[idx],len,out);
+			dump_record(&buf[idx],len,AVStr(out));
 		else
 		if( out != NULL ){
 			STRCAT(out,pbuf);
@@ -312,23 +311,23 @@ static dump_record(buf,buflen,out)
 	}
 }
 
-extern char *findFieldValue();
-static char *curDB = 0;
+static const char *curDB = 0;
 static int byDID = 0;
 
-static dhg2html(dst,dhg)
-	char *dst,*dhg;
+static void dhg2html(PVStr(dst),PCStr(dhg))
 {	int score = 0;
 	int lines = 0;
 	int bytes = 0;
-	char *date = "",	dateb[256];
-	char *type = "TEXT",	typeb[256];
-	char *did = "-",	didb[0x10000];
-	char *subj = "",	subjb[0x1000],subjbe[0x1000];
-	char *hfp;
-	char *dp;
-
-	dp = dst;
+	const char *date = "";
+	CStr(dateb,256);
+	const char *type = "TEXT";
+	CStr(typeb,256);
+	const char *did = "-";
+	CStr(didb,0x10000);
+	const char *subj = "";
+	CStr(subjb,0x1000);
+	CStr(subjbe,0x1000);
+	const char *hfp;
 
 	if( hfp = findFieldValue(dhg,"Score") ) score = atoi(hfp);
 	if( hfp = findFieldValue(dhg,"Lines") ) lines = atoi(hfp);
@@ -340,8 +339,9 @@ static dhg2html(dst,dhg)
 	if( hfp = findFieldValue(dhg,"Document-ID") )
 		{ wordScan(hfp,didb); did = didb; }
 	if( hfp = findFieldValue(dhg,"Headline") ){
-		if( sscanf(hfp,"%[^\r\n]",subjb) ){
-			encodeEntities(subjb,subjbe);
+		lineScan(hfp,subjb);
+		if( subjb[0] ){
+			encodeEntitiesX(subjb,AVStr(subjbe),sizeof(subjbe));
 			subj = subjbe;
 		}
 	}
@@ -352,18 +352,17 @@ static dhg2html(dst,dhg)
 		type,bytes,did,subj);
 }
 
-static relay_userinfo(up,src,dst,out,recleng,trace)
-	FILE *src,*dst;
-	char *out;
-{	char *buf;
+static void relay_userinfo(Connection *Conn,int up,FILE *src,FILE *dst,PVStr(out),int recleng,int trace)
+{
+	defQStr(buf); /*alloc*/
 	int rem0,rem,tag,len,rcc;
 	int cconv;
-	char *what;
+	const char *what;
 
 	what = up ? "C>S" : "S>C";
 	rem = recleng;
 
-	cconv = check_codeconv(0);
+	cconv = CTX_check_codeconv(Conn,0);
 	if( codeconv_bufsize(cconv,10) > 10 )
 		cconv = 0;
 
@@ -378,30 +377,31 @@ static relay_userinfo(up,src,dst,out,recleng,trace)
 				len = rem;
 			else	continue;
 		}
-if( tag == PD_DocumentTextGroup ){
+ if( tag == PD_DocumentTextGroup ){
 	Verbose("Document-Text-Group: %d\n",len);
 	continue;
-}
+ }
 
 		if( rem < len ){
 			ERRMSG("**** ERROR **** tag:%d len:%d > rem:%d\n",
 				tag,len,rem);
 			len = rem;
 		}
-		buf = malloc(len+1);
-		rcc = fread(buf,1,len,src);
-		buf[rcc] = 0;
+		setQStr(buf,(char*)malloc(len+1),len+1);
 
-{
-int ttype;
-char pbuf[2048];
-ttype = print_data(pbuf,tag,len,buf);
-Verbose("%5d[TAG=%3d;LEN=%3d]%d %s\n",recleng-rem0,tag,len,ttype,pbuf);
-}
+		rcc = fread((char*)buf,1,len,src);
+		setVStrEnd(buf,rcc);
+
+ {
+ int ttype;
+ CStr(pbuf,2048);
+ ttype = print_data(AVStr(pbuf),tag,len,(unsigned char*)buf);
+ Verbose("%5d[TAG=%3d;LEN=%3d]%d %s\n",recleng-rem0,tag,len,ttype,pbuf);
+ }
 		if( tag == 106 ){
-			char ebuf[1024];
+			CStr(ebuf,1024);
 
-TO_EUC(buf,ebuf,"text/plain");
+TO_EUC((const char*)buf,AVStr(ebuf),"text/plain");
 
 			sv1log("%s: SeedWords=[%s]%d [%s]%d\n",what,
 				buf,strlen(buf),ebuf,strlen(ebuf));
@@ -409,17 +409,17 @@ TO_EUC(buf,ebuf,"text/plain");
 		}else
 		if( tag == PD_DocumentHeaderGroup ){
 			if( out ){
-				char tmp[4096];
+				CStr(tmp,4096);
 				tmp[0] = 0;
-				dump_record(buf,len,tmp);
-				dhg2html(&out[strlen(out)],tmp);
-			}else	dump_record(buf,len,NULL);
+				dump_record((unsigned char*)buf,len,AVStr(tmp));
+				dhg2html(TVStr(out),tmp);
+			}else	dump_record((unsigned char*)buf,len,VStrNULL);
 		}else
 		if( tag == PD_DocumentText ){
 			if( cconv ){
-				check_codeconv(1);
+				CTX_check_codeconv(Conn,1);
 				sv1log("%s: %d bytes\n",what,len);
-				line_codeconv(buf,buf,"text/plain");
+				CTX_line_codeconv(Conn,buf,AVStr(buf),"text/plain");
 			}
 			if( byDID )
 			strcat(out,buf);
@@ -429,28 +429,25 @@ if( tag == PD_Date )
 	Verbose("Date: [%d] %s\n",len,buf);
 
 		fwrite(buf,rcc,1,dst);
-		free(buf);
+		free((char*)buf);
 		rem -= rcc;
 	}
 	Verbose("DONE\n");
 }
 
 #define int3(b,i)	( ((b[i])<<16) | ((b[i+1])<<8) | (b[i+2]) )
-dump_searchPDU(ptype,buf)
-	char *buf;
+void dump_searchPDU(int ptype,unsigned char *buf)
 {
 Verbose("Type: %d (Search); Upper=%d; Lower=%d; Present=%d Replace=%d\n",
 	ptype, int3(buf,1), int3(buf,4), int3(buf,7), buf[10]);
 }
 
-static relay_record(up,fs,tc,head,body)
-	FILE *fs,*tc;
-	char *head,*body;
+static int relay_record(Connection *Conn,int up,FILE *fs,FILE *tc,xPVStr(head),PVStr(body))
 {	int ptype,tag,len;
 	int rcc,wcc,rem;
 	int bsize,hsize;
-	unsigned char buf[0x10000];
-	char *what;
+	unsigned CStr(buf,0x10000);
+	const char *what;
 
 	what = up ? "C>S" : "S>C";
 	Verbose("%s:\n",what);
@@ -461,7 +458,7 @@ static relay_record(up,fs,tc,head,body)
 		return 0;
 
 	buf[25] = 0;
-	bsize = atoi(buf);
+	bsize = atoi((char*)buf);
 	fwrite(buf,25,1,tc);
 	Verbose("%s: %d [%s]\n",what,bsize,buf);
 
@@ -484,15 +481,15 @@ static relay_record(up,fs,tc,head,body)
 
 	if( ptype == P_InitAPDU ){
 		Verbose("Type: %d (Init);\n",ptype);
-		dump_record(&buf[1],hsize-1,NULL);
+		dump_record(&buf[1],hsize-1,VStrNULL);
 	}
 	if( ptype == P_InitRespAPDU ){
 		Verbose("Type: %d (InitResp); Result=%d;\n",buf[1]);
-		ptype,dump_record(&buf[2],hsize-2,NULL);
+		ptype,dump_record(&buf[2],hsize-2,VStrNULL);
 	}
 	if( ptype == P_SearchAPDU ){
 		dump_searchPDU(ptype,buf);
-		dump_record(&buf[11],hsize-11,NULL);
+		dump_record(&buf[11],hsize-11,VStrNULL);
 	}
 	if( ptype == P_SearchRespAPDU ){
 Verbose("Type: %d (SearchResp); Status=%d; Result=%d; Returned=%d; Next=%d\n",
@@ -501,27 +498,25 @@ Verbose("Type: %d (SearchResp); Status=%d; Result=%d; Returned=%d; Next=%d\n",
 			sprintf(head,"Search-Returned: %d\n",int3(buf,5));
 			head += strlen(head);
 		}
-		dump_record(&buf[11],hsize-11,NULL);
+		dump_record(&buf[11],hsize-11,VStrNULL);
 	}
 
 	if( 0 < rem ){
 		tag = relay_taglen(fs,tc,&rem);
 		len = relay_taglen(fs,tc,&rem);
 		Verbose("User-Information: [%d/%x %d] rem=%d\n",tag,tag,len,rem);
-		relay_userinfo(up,fs,tc,body,rem,ptype!=P_SearchRespAPDU);
+		relay_userinfo(Conn,up,fs,tc,AVStr(body),rem,ptype!=P_SearchRespAPDU);
 	}
 	fflush(tc);
 	return bsize;
 }
 
-static proxyWAIS(Conn)
-	Connection *Conn;
+static void proxyWAIS(Connection *Conn)
 {
 	Verbose("proxy-WAIS\n");
 }
 
-service_wais(Conn)
-	Connection *Conn;
+int service_wais(Connection *Conn)
 {	int cc,sctotal,cstotal;
 	FILE *ts,*fs,*tc,*fc;
 
@@ -542,8 +537,8 @@ service_wais(Conn)
 
 	sctotal = 0;
 	Verbose("WAIS START\n");
-	while( cc = relay_record(1,fc,ts,NULL,NULL) ){
-		if( (cc = relay_record(0,fs,tc,NULL,NULL)) == 0 )
+	while( cc = relay_record(Conn,1,fc,ts,VStrNULL,VStrNULL) ){
+		if( (cc = relay_record(Conn,0,fs,tc,VStrNULL,VStrNULL)) == 0 )
 			break;
 		sctotal += cc;
 	}
@@ -556,232 +551,153 @@ service_wais(Conn)
 	return sctotal;
 }
 
-WAIS_serverOpen(Conn,name,port)
-	Connection *Conn;
-	char *name;
+void WAIS_serverOpen(Connection *Conn,PCStr(name),int port)
 {	int svsock;
 
 	svsock = connect_to_serv(Conn,FromC,ToC,0);
 }
 
-alwais(ac,av)
-	char *av[];
-{	int svsock;
-	char *host;
-	int port;
-	Connection *Conn;
-	FILE *ts,*fs,*null;
-	int len;
-	char *pdu;
-
-	fprintf(stderr,"ALWAIS\n");
-	Conn = (Connection*)calloc(1,sizeof(Connection));
-	scan_SERVER(Conn,"wais://etlport:8888/");
-
-	svsock = connect_to_serv(Conn,0,1,0);
-
-	fprintf(stderr,"ALWAIS [%d]\n",svsock);
-	ts = fdopen(svsock,"w");
-	fs = fdopen(svsock,"r");
-	null = fopen("/dev/null","w");
-
-/*
-pdu = "\
-\0\26\26\0\1\0\1\0\0\0\1\0\1\21\0\22\4jitr\24\0011\
-\143\10\
-\55\015cs=(A-BDenso)\162\1\1";
-len = 41;
-
-	fprintf(ts,"%010dz2wais        0",len);
-	fwrite(pdu,1,len,ts);
-	fflush(ts);
-	relay_record(0,fs,null,NULL,NULL);
-*/
-
-
-pdu = "\
-\0\26\26\0\1\0\1\0\0\0\1\0\1\21\0\22\4jitr\24\0013\
-\143\10\
-\152\003euc\162\1\1";
-len = 34;
-
-	fprintf(ts,"%010dz2wais        0",len);
-	fwrite(pdu,1,len,ts);
-	fflush(ts);
-	relay_record(0,fs,null,NULL,NULL);
-
-	fclose(ts);
-	fclose(fs);
-	fclose(null);
-	free(Conn);
-}
-
-static wais_init(Conn,sv)
-	Connection *Conn;
-{	char ipdu[2048];
-	int len,la;
-	char *auth;
-
-FILE *ts,*fs,*null;
-ts = fdopen(sv,"w");
-fs = fdopen(sv,"r");
-null = fopen("/dev/null","w");
-
-	len = 0;
-	ipdu[len++] =  0; ipdu[len++] = 0;
-	ipdu[len++] = 20;
-	ipdu[len++] =  3; ipdu[len++] = 1; ipdu[len++] = 0x80;
-	ipdu[len++] =  4; ipdu[len++] = 1; ipdu[len++] = 0x80;
-	ipdu[len++] =  5; ipdu[len++] = 3; ipdu[len++] = 2; ipdu[len++] = 0; ipdu[len++] = 0;
-	ipdu[len++] =  6; ipdu[len++] = 3; ipdu[len++] = 2; ipdu[len++] = 0; ipdu[len++] = 0;
-	ipdu[1] = len - 2;
-
-dump_record(&ipdu[3],len-3,NULL);
-
-	fprintf(ts,"%010dz2wais        0",len);
-	fwrite(ipdu,1,len,ts);
-	fflush(ts);
-	relay_record(0,fs,null,NULL,NULL);
-}
-static wais_search(Conn,sv,head,out,dbname,words,did,type,leng)
-	Connection *Conn;
-	char *dbname,*words,*did,*type;
-	char *head,*out;
+#define addQbuf(v) setVStrElemInc(qpdu,len,v)
+static void wais_search(Connection *Conn,int sv,PVStr(head),PVStr(out),PCStr(dbname),PCStr(words),PCStr(did),PCStr(type),int leng)
 {
 FILE *ts,*fs,*null;
-	char qpdu[2048];
+	JStr(qpdu,2048);
 	int len,ulen,blen;
+int tfd;
 int vtop;
 static int serno = 50;
 
 ts = fdopen(sv,"w");
 fs = fdopen(sv,"r");
+/*
 null = fopen("/dev/null","w");
+*/
+tfd = open("/dev/null",1); null = fdopen(tfd,"w");
 
 	len = 0;
-	qpdu[len++] =  0; len++;
-	qpdu[len++] = 22;
+	addQbuf( 0); len++;
+	addQbuf(22);
 
-if( words ){
-	qpdu[len++] =  0; qpdu[len++] = 0; qpdu[len++] = 50;
-	qpdu[len++] =  0; qpdu[len++] = 10; qpdu[len++] = 0;
-	qpdu[len++] =  0; qpdu[len++] = 0; qpdu[len++] = 50;
-}else{
-	qpdu[len++] =  0; qpdu[len++] = 0; qpdu[len++] = 10;
-	qpdu[len++] =  0; qpdu[len++] = 0; qpdu[len++] = 16;
-	qpdu[len++] =  0; qpdu[len++] = 0; qpdu[len++] = 15;
-}
-	qpdu[len++] =  1;
+ if( words ){
+	addQbuf( 0); addQbuf( 0); addQbuf(50);
+	addQbuf( 0); addQbuf(10); addQbuf( 0);
+	addQbuf( 0); addQbuf( 0); addQbuf(50);
+ }else{
+	addQbuf( 0); addQbuf( 0); addQbuf(10);
+	addQbuf( 0); addQbuf( 0); addQbuf(16);
+	addQbuf( 0); addQbuf( 0); addQbuf(15);
+ }
+	addQbuf( 1);
 	vtop = len;
 
 	if( words ){
-		qpdu[len++] = 17; qpdu[len++] = 0;
+		addQbuf(17); addQbuf( 0);
 	}else{
-		qpdu[len++] = 17; qpdu[len++] = 3;
-		strcpy(&qpdu[len],"FOO");
+		addQbuf(17); addQbuf( 3);
+		Xstrcpy(DVStr(qpdu,len),"FOO");
 		len += 3;
 	}
-	qpdu[len++] = 18; qpdu[len++] = strlen(dbname);
-		strcpy(&qpdu[len],dbname);
+	addQbuf(18); addQbuf(strlen(dbname));
+		Xstrcpy(DVStr(qpdu,len),dbname);
 		len += strlen(dbname);
 
 	if( words ){
-		qpdu[len++] =  20; qpdu[len++] = 1; qpdu[len++] = '3';
+		addQbuf(20); addQbuf( 1); addQbuf('3');
 	}else{
-		qpdu[len++] =  20; qpdu[len++] = 1; qpdu[len++] = '1';
-		qpdu[len++] =  19; qpdu[len++] = 15;
-		strcpy(&qpdu[len]," \037Document Text");
+		addQbuf(20); addQbuf( 1); addQbuf('1');
+		addQbuf(19); addQbuf(15);
+		Xstrcpy(DVStr(qpdu,len)," \037Document Text");
 		len += 15;
 	}
-	qpdu[len++] =   2; qpdu[len++] = 1; qpdu[len++] = ++serno;
-	qpdu[1] = len - 2;
+	addQbuf( 2); addQbuf( 1); addQbuf(++serno);
+	setVStrElem(qpdu,1,len - 2);
 
-if( words ){
-	qpdu[len++] = 99; ulen = len; qpdu[len++] = 0x80; qpdu[len++] = 0;
-}else{
-	qpdu[len++] = 21; ulen = len; qpdu[len++] = 0x80; qpdu[len++] = 0;
-}
+ if( words ){
+	addQbuf(99); ulen = len; addQbuf(0x80); addQbuf( 0);
+ }else{
+	addQbuf(21); ulen = len; addQbuf(0x80); addQbuf( 0);
+ }
 
 	if( words != NULL ){
-		char ewords[1024];
+		CStr(ewords,1024);
 
-TO_EUC(words,ewords,"text/plain");
+TO_EUC(words,AVStr(ewords),"text/plain");
 
-		qpdu[len++] = 111;qpdu[len++] = 1; qpdu[len++] = 1;
-		qpdu[len++] = 114;qpdu[len++] = 1; qpdu[len++] = MAX_REC_DOC;
-		qpdu[len++] = 106;qpdu[len++] = strlen(ewords);
-		strcpy(&qpdu[len],ewords);
+		addQbuf(111);addQbuf( 1); addQbuf( 1);
+		addQbuf(114);addQbuf( 1); addQbuf(MAX_REC_DOC);
+		addQbuf(106);addQbuf(strlen(ewords));
+		Xstrcpy(DVStr(qpdu,len),ewords);
 		len += strlen(ewords);
 	}
 	if( did != NULL ){
 		int dl;
-		char tmp[128];
+		CStr(tmp,128);
 
-		qpdu[len++] = PD_AttributeList;
-		qpdu[len++] = 17;
-		strcpy(&qpdu[len],"un re ig ig ig ig");
+		addQbuf(PD_AttributeList);
+		addQbuf(17);
+		Xstrcpy(DVStr(qpdu,len),"un re ig ig ig ig");
 		len += 17;
-		dl = decode_did(did,&qpdu[len+3]);
-		qpdu[len++] = PD_Term;
-		qpdu[len++] = 0x80 | 0x7F & (dl>>7);
-		qpdu[len++] = 0x7F & dl;
+		dl = decode_did((unsigned char*)did,(unsigned char*)&qpdu[len+3]);
+		addQbuf(PD_Term);
+		addQbuf(0x80 | 0x7F & (dl>>7));
+		addQbuf(0x7F & dl);
 		len += dl;
 
-		qpdu[len++] = PD_AttributeList;
-		qpdu[len++] = 17;
-		strcpy(&qpdu[len],"wt re ig ig ig ig");
+		addQbuf(PD_AttributeList);
+		addQbuf(17);
+		Xstrcpy(DVStr(qpdu,len),"wt re ig ig ig ig");
 		len += 17;
-		qpdu[len++] = PD_Term;
-		qpdu[len++] = strlen(type);
-		strcpy(&qpdu[len],type);
+		addQbuf(PD_Term);
+		addQbuf(strlen(type));
+		Xstrcpy(DVStr(qpdu,len),type);
 		len += strlen(type);
 
-		qpdu[len++] = PD_Operator;
-		qpdu[len++] = 1;
-		qpdu[len++] = 'a';
+		addQbuf(PD_Operator);
+		addQbuf( 1);
+		addQbuf('a');
 
-		qpdu[len++] = PD_AttributeList;
-		qpdu[len++] = 17;
-		strcpy(&qpdu[len],"wb ro ig ig ig ig");
+		addQbuf(PD_AttributeList);
+		addQbuf(17);
+		Xstrcpy(DVStr(qpdu,len),"wb ro ig ig ig ig");
 		len += 17;
-		qpdu[len++] = PD_Term;
-		qpdu[len++] = 1;
-		qpdu[len++] = '0';
+		addQbuf(PD_Term);
+		addQbuf( 1);
+		addQbuf('0');
 
-		qpdu[len++] = PD_Operator;
-		qpdu[len++] = 1;
-		qpdu[len++] = 'a';
+		addQbuf(PD_Operator);
+		addQbuf( 1);
+		addQbuf('a');
 
-		qpdu[len++] = PD_AttributeList;
-		qpdu[len++] = 17;
-		strcpy(&qpdu[len],"wb rl ig ig ig ig");
+		addQbuf(PD_AttributeList);
+		addQbuf(17);
+		Xstrcpy(DVStr(qpdu,len),"wb rl ig ig ig ig");
 		len += 17;
 		sprintf(tmp,"%d",leng);
-		qpdu[len++] = PD_Term;
-		qpdu[len++] = strlen(tmp);
-		strcpy(&qpdu[len],tmp);
+		addQbuf(PD_Term);
+		addQbuf(strlen(tmp));
+		Xstrcpy(DVStr(qpdu,len),tmp);
 		len += strlen(tmp);
 
-		qpdu[len++] = PD_Operator;
-		qpdu[len++] = 1;
-		qpdu[len++] = 'a';
+		addQbuf(PD_Operator);
+		addQbuf( 1);
+		addQbuf('a');
 	}
 
 	blen = len - (ulen+2);
-	qpdu[ulen+0] = 0x80 | 0x7F & (blen>>7);
-	qpdu[ulen+1] = 0x7F & blen;
+	setVStrElem(qpdu,ulen+0,0x80 | 0x7F & (blen>>7));
+	setVStrElem(qpdu,ulen+1,0x7F & blen);
 
 /*
 Verbose(">>>>>>>> %010dz2wais        0\n",len);
 Verbose("HederLength: %d / %d\n",((qpdu[0]<<8)|qpdu[1]),len);
 dump_searchPDU(22,qpdu+2);
-dump_record(&qpdu[vtop],len-vtop,NULL);
+dump_record(&qpdu[vtop],len-vtop,VStrNULL);
 */
 
 	fprintf(ts,"%010dz2wais        0",len);
 	fwrite(qpdu,1,len,ts);
 	fflush(ts);
-	relay_record(0,fs,null,head,out);
+	relay_record(Conn,0,fs,null,AVStr(head),AVStr(out));
 }
 
 /*
@@ -791,12 +707,9 @@ dump_record(&qpdu[vtop],len-vtop,NULL);
  *    /servername/database?word+word+word
  */
 
-static putIndex(Conn,tc,vno,dbname,words,head,body)
-	Connection *Conn;
-	FILE *tc;
-	char *dbname,*words,*head,*body;
+static int putIndex(Connection *Conn,FILE *tc,int vno,PCStr(dbname),PCStr(words),PCStr(head),PCStr(body))
 {	int totalc,returned;
-	char *hfp;
+	const char *hfp;
 
 	totalc = putHttpHeaderV(Conn,tc,vno,NULL,"text/html",NULL,0,0,0);
 	fprintf(tc,"<TITLE> WAIS Search in %s database</TITLE>",dbname);
@@ -825,16 +738,19 @@ static putIndex(Conn,tc,vno,dbname,words,head,body)
 	return totalc;
 }
 
-HttpWais(Conn,vno,sv,server,iport,path)
-	Connection *Conn;
-	char *server,*path;
-{	char xpath[0x10000],*dbname,*docid,*words;
+int HttpWais(Connection *Conn,int vno,int sv,PCStr(server),int iport,PCStr(path))
+{	CStr(xpath,0x10000);
+	const char *dbname;
+	const char *docid;
+	refQStr(words,xpath); /**/
 	FILE *tc,*tcx;
 	int totalc;
-	char *dp,*type;
+	const char *dp;
+	const char *type;
 	int leng;
 	int svsock;
-	char head[0x2000],body[0x40000]; /* large enough to receive catalog... X-< */
+	CStr(head,0x2000);
+	CStr(body,0x40000); /* large enough to receive catalog... X-< */
 
 	head[0] = body[0] = 0;
 	strcpy(xpath,path);
@@ -843,20 +759,20 @@ HttpWais(Conn,vno,sv,server,iport,path)
 		dbname++;
 
 	if( words = strchr(dbname,'?') ){
-		*words++ = 0;
-		nonxalpha_unescape(words,words,1);
+		truncVStr(words); words++;
+		nonxalpha_unescape(words,AVStr(words),1);
 		docid = NULL;
 		type = NULL;
 		leng = 0;
 		goto search;
 	}else
 	if( type = strchr(dbname,'/') ){
-		*type++ = 0;
+		truncVStr(type); type++;
 		if( dp = strchr(type,'/') ){
-			*dp++ = 0;
+			truncVStr(dp); dp++;
 			leng = atoi(dp);
 			if( docid = strchr(dp,'/') ){
-				*docid++ = 0;
+				truncVStr(docid); docid++;
 				words = NULL;
 				goto search;
 			}
@@ -877,7 +793,7 @@ search:
 	if( words && *words || docid ){
 		curDB = dbname;
 		byDID = docid != NULL;
-		wais_search(Conn,svsock,head,body,dbname,words,docid,type,leng);
+		wais_search(Conn,svsock,AVStr(head),AVStr(body),dbname,words,docid,type,leng);
 		byDID = 0;
 		curDB = NULL;
 	}
@@ -899,113 +815,3 @@ search:
 	fflush(tc);
 	return totalc;
 }
-
-/*=============== obsoleted ===============
-#include <signal.h>
-#include <setjmp.h>
-static jmp_buf wais_env;
-
-static SC(Conn,omask)
-	Connection *Conn;
-{	int cc,total;
-	FILE *fs,*tc;
-
-	fs = fdopen(FromS,"r");
-	tc = fdopen(dup(ToC),"w");
-	total = 0;
-	if( setjmp(wais_env) == 0 ){
-		sigsetmask(omask);
-		while( cc = relay_record(0,fs,tc,NULL,NULL) )
-			total += cc;
-	}
-	fclose(fs);
-	fclose(tc);
-	return total;
-}
-static CS(Conn,peer,omask)
-	Connection *Conn;
-{	char buf[0x4000];
-	int gotsig;
-	int cc,total;
-	FILE *ts,*fc;
-
-	total = 0;
-	ts = fdopen(ToS,"w");
-	fc = fdopen(dup(FromC),"r");
-	if( (gotsig = setjmp(wais_env)) == 0 ){
-		sigsetmask(omask);
-		while( cc = relay_record(1,fc,ts,NULL,NULL) )
-			total += cc;
-	}
-	fclose(ts);
-	fclose(fc);
-	if( gotsig == SIGPIPE ){
-		sv1log("CS-SIGPIPE\n");
-		sleep(1);
-	}
-	return total;
-}
-static void sigPIPE(sig){
-	signal(SIGPIPE,SIG_IGN);
-	longjmp(wais_env,SIGPIPE);
-}
-static void sigTERM(sig){
-	signal(SIGTERM,SIG_IGN);
-	longjmp(wais_env,SIGTERM);
-}
-service_wais(Conn)
-	Connection *Conn;
-{	register int ppid,cpid;
-	int omask;
-	int total;
-
-	ppid = getpid();
-	omask = sigblock(sigmask(SIGPIPE)|sigmask(SIGTERM));
-	signal(SIGPIPE,sigPIPE);
-	signal(SIGTERM,sigTERM);
-
-	if( (cpid = Fork("service_wais")) == 0 ){
-		total = SC(Conn,omask);
-		signal(SIGPIPE,SIG_IGN);
-		signal(SIGTERM,SIG_IGN);
-		sv1log("SC-RELAY[%d<%d]: %dBytes\n",ToC,FromS,total);
-		Kill(ppid,SIGTERM);
-		Finish(0);
-	}else{
-		total = CS(Conn,cpid,omask);
-		signal(SIGPIPE,SIG_IGN);
-		signal(SIGTERM,SIG_IGN);
-		sv1log("CS-RELAY[%d>%d]: %dBytes\n",FromC,ToS,total);
-		Kill(cpid,SIGTERM);
-		wait(0);
-		sigsetmask(omask);
-	}
-	return total;
-}
-static dumpb(buf,rcc)
-	char *buf;
-{	int i,ch;
-
-	for( i = 0; i < rcc; i++ ){
-		ch = buf[i] & 0xFF;
-
-if( ch == '\n' ){
-	int j;
-	for( j = i+1; j < rcc; j++ ){
-		if( buf[j] == '\n' )
-			break;
-
-		if( (buf[j] & 0x80) && (buf[j+1] & 0x80) ){
-			buf[j] &= 0x7F;
-			buf[j+1] &= 0x7F;
-			j++;
-		}
-	}
-}
-		if( ch == '\n' || ' ' <= ch && ch < 0x7F )
-			putc(ch,stderr);
-		else	fprintf(stderr,"\\%03o",ch);
-	}
-	fprintf(stderr,"\n");
-}
-*/
