@@ -1,13 +1,14 @@
 /*////////////////////////////////////////////////////////////////////////
-Copyright (c) 1995 Electrotechnical Laboratry (ETL), AIST, MITI
-
-Permission to use, copy, modify, and distribute this material for any
-purpose and without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies, and
-that the name of ETL not be used in advertising or publicity pertaining
-to this material without the specific, prior written permission of an
-authorized representative of ETL.
-ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
+Copyright (c) 1995-2000 Yutaka Sato and ETL,AIST,MITI
+Copyright (c) 2001-2006 National Institute of Advanced Industrial Science and Technology (AIST)
+AIST-Product-ID: 2000-ETL-198715-01, H14PRO-049, H15PRO-165, H18PRO-443
+        
+Permission to use this material for noncommercial and/or evaluation
+purpose, copy this material for your own use, and distribute the copies
+via publicly accessible on-line media, without fee, is hereby granted
+provided that the above copyright notice and this permission notice
+appear in all copies.
+AIST MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
 MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
 OR IMPLIED WARRANTIES.
 /////////////////////////////////////////////////////////////////////////
@@ -38,7 +39,7 @@ int TELEPORT_BUFFERED = 1;
 int TELEPORT_loglev = TPD_DBG;
 
 #define LOG(lv)	(lv <= TELEPORT_loglev)
-#define ERR	(!LOG(TPD_ERR)) ? 0 : sv1tlog
+#define xERR	(!LOG(TPD_ERR)) ? 0 : sv1tlog
 #define DBG	(!LOG(TPD_DBG)) ? 0 : sv1log
 #define VRB	(!LOG(TPD_VRB)) ? 0 : sv1vlog
 
@@ -402,7 +403,7 @@ static int enRead(int fd,PCStr(prefix),PVStr(buf),int size)
 {	int rcc;
 	CStr(ibuf,IMSIZE);
 
-	rcc = read(fd,ibuf,sizeof(ibuf));
+	rcc = read(fd,(char*)ibuf,sizeof(ibuf));
 	if( rcc <= 0 )
 		return rcc;
 	return QZencode(fd,prefix,AVStr(buf),ibuf,rcc);
@@ -537,7 +538,7 @@ fclose(fp);
 
 	sprintf(helo,"HELO\r\n");
 	write(clsock,helo,strlen(helo));
-	if( RecvLine(clsock,resp,sizeof(resp)) <= 0 )
+	if( RecvLine(clsock,(char*)resp,sizeof(resp)) <= 0 )
 		return -1;
 	DBG("{T}.Teleport-SAYS: %s",resp);
 
@@ -597,13 +598,13 @@ int bindTeleportVehicle(int tx,int clsocks[],PCStr(host),int port,PCStr(tunnel),
 	minit_vehicle();
 	svsock = server_open("teleport",VStrNULL,0,50);
 	if( svsock < 0 ){
-		ERR("{T}.TeleportVehicle can't open server socket.\n");
+		xERR("{T}.TeleportVehicle can't open server socket.\n");
 		return 0;
 	}
 	if( (pid = Fork("bindTeleportVehicle")) != 0 ){
 		sockHostport(svsock,&TeleportPorts[tx]);
 		close(svsock);
-		ERR("{T}.TeleportPorts[%d]: %d\n",tx,TeleportPorts[tx]);
+		xERR("{T}.TeleportPorts[%d]: %d\n",tx,TeleportPorts[tx]);
 		if( strcmp(host,"tty7") == 0 ){
 			/* "stty -echo" must be done in this parent process to
 			 * make resetting "stty echo" is done in thie process
@@ -621,7 +622,7 @@ int bindTeleportVehicle(int tx,int clsocks[],PCStr(host),int port,PCStr(tunnel),
 	wfp = NULL;
 	while( setjmp(jmpenv) != 0 ){
 		if( rfp == NULL ){
-			ERR("!!!!!!!! Teleport setjmp ERROR !!!!!!!!\n");
+			xERR("!!!!!!!! Teleport setjmp ERROR !!!!!!!!\n");
 			break;
 		}
 		clearZomb(1);
@@ -635,7 +636,7 @@ int bindTeleportVehicle(int tx,int clsocks[],PCStr(host),int port,PCStr(tunnel),
 	for( wtotal = 0; ; wtotal += wsecs ){
 		if( connectTeleport(clsocks,host,port,tunnel,telesocks) == 0 )
 			break;
-		ERR("{T}.Teleport: %s:%d open failed.\n",host,port);
+		xERR("{T}.Teleport: %s:%d open failed.\n",host,port);
 
 		if( strcmp(host,"tty7") == 0 )
 			exit(0);
@@ -740,7 +741,7 @@ DBG("{T}.GOT HARD-EOF local, INFORM remote of it: %s\n",path);
 			}
 		    }else
 		    if( rdv[fpx] < 0 ){
-			ERR("!!!! poll result in error #%d !!!!\n",ap->a_id);
+			xERR("!!!! poll result in error #%d !!!!\n",ap->a_id);
 			ap->a_zomb = 1;
 		    }
 		}
@@ -756,10 +757,10 @@ DBG("{T}.GOT HARD-EOF local, INFORM remote of it: %s\n",path);
 		if( rdv[0] <= 0 )
 			continue;
 
-		msg[0] = 0;
+		setVStrEnd(msg,0);
 		/*if( RecvLine(fileno(rfp),msg,sizeof(msg)) <= 0 )*/
 		if( fgets(msg,sizeof(msg),rfp) == NULL ){
-			ERR("{T}.!!!!!!!! EOF from backbone !!!!!!!!\n");
+			xERR("{T}.!!!!!!!! EOF from backbone !!!!!!!!\n");
 			longjmp(jmpenv,-1);
 		}
 		tracemsg("B>A",msg);
@@ -768,7 +769,7 @@ DBG("{T}.GOT HARD-EOF local, INFORM remote of it: %s\n",path);
 		if( msg[0] == '=' )
 			ovstrcpy(msg,msg+1);
 		else{
-			ERR("{T}.Teleport ERROR:[%d] %s\n",strlen(msg),msg);
+			xERR("{T}.Teleport ERROR:[%d] %s\n",strlen(msg),msg);
 		}
 */
  if( msg[0] != '=' )
@@ -786,7 +787,7 @@ DBG("{T}.GOT HARD-EOF local, INFORM remote of it: %s\n",path);
 
 /*
 			if( head[0]=='=' && head[1]=='=' ){
-				ERR("NO SUCH AGENT: [%s] (SOFT-EOF)\n",path);
+				xERR("NO SUCH AGENT: [%s] (SOFT-EOF)\n",path);
 				continue;
 			}
 			pathReverse(path,rpath);
@@ -796,9 +797,9 @@ DBG("{T}.GOT HARD-EOF local, INFORM remote of it: %s\n",path);
 			{
 				if( strcmp(path,Ppath) == 0 ){
 					if( ++pcount % 10 == 0 )
-					ERR("NO SUCH AGENT: [%s] (%d)\n",path,pcount);
+					xERR("NO SUCH AGENT: [%s] (%d)\n",path,pcount);
 				}else{
-					ERR("NO SUCH AGENT: [%s]\n",path);
+					xERR("NO SUCH AGENT: [%s]\n",path);
 					strcpy(Ppath,path);
 					pcount = 1;
 				}
@@ -813,7 +814,7 @@ DBG("{T}.GOT HARD-EOF local, INFORM remote of it: %s\n",path);
 		}
 
 if( strchr(msg,'\n') == 0 ){
-	ERR("ERROR: WITHOUT NEWLINE\n");
+	xERR("ERROR: WITHOUT NEWLINE\n");
 	strcat(msg,"\n");
 }
 
@@ -904,7 +905,7 @@ static int openTeleport(PCStr(host),int port,int closefd)
 					qzput = 1;
 					fprintf(wfp,"=Q%s=QZ\r\n",MY_QZ);
 					Fflush(wfp);
-					ERR("sent =Q%s after %s",MY_QZ,buf);
+					xERR("sent =Q%s after %s",MY_QZ,buf);
 				}
 */
 			}
@@ -940,7 +941,7 @@ int teleportOpen(int mx,PCStr(master),int mport,PCStr(target_server),int closefd
 
 			sprintf(msg,"%s\r\n",target_server);
 			Write(tsock,msg,strlen(msg));
-			rcc = read(tsock,msg,sizeof(msg));
+			rcc = read(tsock,(char*)msg,sizeof(msg));
 			if( 0 < rcc ){
 				setVStrEnd(msg,rcc);
 				DBG("{T}.Teleport[%d]: got: %s",tsock,msg);
@@ -1055,7 +1056,7 @@ static int connectMyself(PCStr(telehost),int teleport)
 			return -1;
 		}
 		afp = fdopen(dup(clsock),"r");
-		ackmsg[0] = 0;
+		setVStrEnd(ackmsg,0);
 		fgets(ackmsg,sizeof(ackmsg),afp);
 		fclose(afp);
 		sv1log("%d CPORT[%s:%d] %s",clsock,telehost,teleport,ackmsg);
@@ -1120,7 +1121,7 @@ static void callbackDelegate(int ifd,int cbsock,PCStr(telehost),int teleport) /*
 	}
 */
 ifp = NULL;
-host[0] = 0;
+setVStrEnd(host,0);
 port = 0;
 
 	clsock = client_open("callbackDelegate","teleport",host,port);
@@ -1143,7 +1144,7 @@ static int bindVehicle(int telesock,PCStr(server))
 	if( PollIn(telesock,TELEPORT_TIMEOUT*1000) <= 0 )
 		return -1;
 
-	rcc = RecvLine(telesock,msg,sizeof(msg));
+	rcc = RecvLine(telesock,(char*)msg,sizeof(msg));
 	if( rcc <= 0 )
 		return -1;
 
@@ -1152,7 +1153,7 @@ static int bindVehicle(int telesock,PCStr(server))
 
 	if( msg[0] == '=' && msg[1] == '>' ){
 		wordScan(msg+1,mpath);
-		mpath[0] = '<';
+		setVStrElem(mpath,0,'<');
 		for( mp = mpath+1; *mp; mp++ )
 			if( *mp == '>' )
 				*(char*)mp = '!';
@@ -1163,7 +1164,7 @@ static int bindVehicle(int telesock,PCStr(server))
 		if( PollIn(telesock,TELEPORT_TIMEOUT*1000) <= 0 )
 			return -1;
 
-		rcc = RecvLine(telesock,msg,sizeof(msg));
+		rcc = RecvLine(telesock,(char*)msg,sizeof(msg));
 		setVStrEnd(msg,rcc);
 
 		DBG("[%d] GOT RESPONSE: %s",telesock,msg);
